/*=============================================================================
    Skymedi corporation

    All Rights Reserved
    
    SK2850

    ARC700

=============================================================================*/
#include <linux/config.h>
#include <asm/uaccess.h>
#include <asm/serial.h>
#include "NorFlash.h"

extern void* memset(void* s, int c, size_t n);
extern void* memcpy(void* __dest, __const void* __src, size_t __n);

unsigned int	gdwTimeLimite;
unsigned int	gdwCount;
unsigned char	gbVendCmdIn;
unsigned char	gbStorageType;
unsigned char	NorReadCtl1;
unsigned char	NorReadCtl2;
unsigned char	gbFlashID[8];
unsigned int	gdwTmpAddr;
unsigned int	gdwTmp;
unsigned int	gdwNorCE;


#define SKY_CODE_One_BUFFER_WRITE       0xFC
#define SKY_CODE_One_BUFFER_READ        0xFD


void NorReadID(unsigned char bCE, unsigned char* buf);
void NorRead( unsigned int addr, unsigned int BlockNo, unsigned char* buffer, unsigned char bDMAEnable );

void Nor_Reginit(void)
{
	int _PLL_REG;

    // set flash back-end module clock
    // FPGA 70M/4 = 17.5MHz
    
    _PLL_REG = PLL_CTL1;
    _PLL_REG &= ~0xF0000000;
    _PLL_REG |=  0xE0000000;
    
//    _PLL_REG &= ~0x30000;		// clock/4
	_PLL_REG |= 0x30000;		// clock/10
    
    PLL_CTL1 = _PLL_REG;
    PLL_CTL1 = _PLL_REG;
    
	FDM_BLK_MASK_H_REG          = 0x00000000;
    FDM_BLK_MASK_M_REG          = 0x0000001f;
    FDM_MP_FACTOR_REG           = 0x00000000;
    FDM_DIE_FACTOR_REG          = 0x00000000;
    FDM_CHIP_FACTOR_REG         = 0x00000000;
    FDM_PARTI_SHIFT_REG         = 0x00000000;
    FDM_BLK_SHIFT_REG           = 0x0000000d;
    FDM_BANK_SHIFT_REG          = 0x00000000;
    FDM_PAGE_SHIFT_REG          = 0x00000005;
    FDM_ADDR_CYCLE_REG          = 0x00000002;
    FDM_FLASH_CFG_REG           = 0x00000030;
    FDM_FLASH_WR_CFG_REG        = 0x00000051;
    FDM_FLASH_RD_CFG_REG        = 0x00000093;
    FDM_FLASH_RD_CFG2_REG       = 0x00000003;
    FDM_FLASH_ALE_CLE_CTRL_REG  = 0x00000003;
    FDM_FLASH_CTRL0_REG         = 0x00000011;
    FDM_FLASH_CTRL1_REG         = 0x0000009f;
    FDM_FLASH_CTRL2_REG         = 0x00000089;
    FDM_FLASH_CTRL3_REG         = 0x00000004;
    FDM_FLASH_CTRL_REG          = 0x00000000;
}

/**********************************************************************************

**********************************************************************************/
//unsigned int NorInit(void)
int NorInit(void)
{ 		
	Nor_Reginit();

    /* Reset Controller */
    FDM_FW_RST_REG = (unsigned char)(~( FW_RST_BE_RSTb | FW_RST_ECC_RSTb ));       //  backend and ECC reset                               
    FDM_FW_RST_REG =    FW_RST_BE_RSTb | FW_RST_ECC_RSTb;        //  backend and ECC default value
	 
    FDM_FLASH_CTRL0_REG &= ~FSH_CTRL0_BUS_INV;
    FDM_FLASH_CTRL0_REG &= ~FSH_CTRL0_WORD_MODE;

    NorReadID(0, gbFlashID);
    if ( ((gbFlashID[0]==0)&&(gbFlashID[1]==0)) || ((gbFlashID[0]==0xFF)&&(gbFlashID[1]==0xFF)) ) {			
    	NorReadID(1, gbFlashID);					// CE0 no flash, check CE1
    	if ( ((gbFlashID[0]==0)&&(gbFlashID[1]==0)) || ((gbFlashID[0]==0xFF)&&(gbFlashID[1]==0xFF)) ) {
    		NorReadID(2, gbFlashID);				// CE1 no flash, check CE2
    		if ( ((gbFlashID[0]==0)&&(gbFlashID[1]==0)) || ((gbFlashID[0]==0xFF)&&(gbFlashID[1]==0xFF)) ) {
    			NorReadID(3, gbFlashID);			// CE2 no flash, check CE3
    			if ( ((gbFlashID[0]==0)&&(gbFlashID[1]==0)) || ((gbFlashID[0]==0xFF)&&(gbFlashID[1]==0xFF)) )
    				return -1;//return 0;						// CE3 no flash, return error
    			else
    				gdwNorCE = 3;					// CE3 with flash
    		} else
    			gdwNorCE = 2;						// CE2 with flash
    	} else
    		gdwNorCE = 1;							// CE1 with flash    
    } else 
    	gdwNorCE = 0;
     
    return 0;        
}


/**********************************************************************************

**********************************************************************************/
unsigned char Nor_ManualReadByte(void)
{
    unsigned char DummyByte;
    
    // check back end is ready
    while( !(FDM_BACKEND_STS_REG & 0x20) );
    
    DummyByte = FDM_FDIO_L_REG;
    
    // check back end is ready
    while( !(FDM_BACKEND_STS_REG & 0x20) );

    return FDM_FALE_L_REG;
}


/**********************************************************************************

**********************************************************************************/
void Nor_ManualWriteByte( unsigned char data )
{
    // check back end is ready
    while( !(FDM_BACKEND_STS_REG & 0x20) );

    FDM_FDIO_L_REG = data;

    while( !(FDM_BACKEND_STS_REG & 0x20) );
}


/**********************************************************************************

**********************************************************************************/

extern unsigned char NorReadCtl1;
extern unsigned char NorReadCtl2;

void NorSetTimming(void)
{
	Nor_Reginit();
    NorReadCtl1 = FDM_FLASH_RD_CFG_REG;
    NorReadCtl2 = FDM_FLASH_RD_CFG2_REG;
    
    FDM_FLASH_RD_CFG_REG  = 0x30;
    FDM_FLASH_RD_CFG2_REG = 0x03;
 
    FDM_FLASH_CTRL0_REG &= ~FSH_CTRL0_BUS_INV;
    FDM_FLASH_CTRL0_REG &= ~FSH_CTRL0_WORD_MODE;   
}

/**********************************************************************************

**********************************************************************************/
void NorRestoreTimming(void)
{
    FDM_FLASH_RD_CFG_REG  = NorReadCtl1;
    FDM_FLASH_RD_CFG2_REG = NorReadCtl2;
}

/**********************************************************************************

**********************************************************************************/
void NorRead( unsigned int addr, unsigned int BlockNo, unsigned char* buffer, unsigned char bDMAEnable )
{
    unsigned int BufferAddr;
    unsigned int add;
    
    BufferAddr = (unsigned int)buffer;
    add        = (addr*512);
    
    NorSetTimming();
    
    NOR_ENABLE;
    
    // block length 512Bytes =  256Words
    FDM_LENGTH_HIGH_REG = 0x01;
    FDM_LENGTH_LOW_REG  = 0x00;
    
    do
    {
        NOR_CS_LOW;
    
        // Read instruction
        Nor_ManualWriteByte( 0x03 );
        while( !(FDM_BACKEND_STS_REG & 0x20) );
    
        // address
        Nor_ManualWriteByte( (unsigned char)(add >> 16) );
        while( !(FDM_BACKEND_STS_REG & 0x20) );
        Nor_ManualWriteByte( (unsigned char)(add >>  8) );
        while( !(FDM_BACKEND_STS_REG & 0x20) );
        Nor_ManualWriteByte( (unsigned char)(add >>  0) );
        while( !(FDM_BACKEND_STS_REG & 0x20) );
        
        // read block count    
        FDM_DAT_BLK_CNT_H_REG = 0;
        FDM_DAT_BLK_CNT_M_REG = 0;
        FDM_DAT_BLK_CNT_L_REG = 1;
/*    
        CRCTL_BUF_CTL ^= 1;
        CRCTL_BUF_CTL ^= 1;
*/		// reset toggle bit to change operation mode --------- CRReset()
	    CRCTL_BUF_CTL ^= 1;
	    CRCTL_BUF_CTL ^= 1;	    
	    CRCTL_BUF_CTL &= ~0x3;	        
	    CRCTL_BUF_DMA_BLK_LENGTH = 0;
	    //-----------------------------------------------------
            
        CRCTL_BUF_CTL  |= CR_XFER_FROM_CARD + CR_FLASH_MODE + CR_AUTO_MODE + CR_ENABLE;
        CRCTL_BUF_DMA_BLK_LENGTH = 1;
        
        if (bDMAEnable) {
	        DMA_CONFIG   &=  DMA_CLEAR_DIRECTION;
	        DMA_CONFIG   |= (DMA_SOURCE_CR_BANK + DMA_TARGET_SRAM);
	        DMA_SRC_ADDR  = DMA_CR_BUF_ADD;
	        DMA_DEST_ADDR = BufferAddr;
	        DMA_BYTE_SIZE = 512;
        }
        
        FDM_FLASH_CMD_REG = SKY_CODE_One_BUFFER_READ;
        
        while( !(FDM_BACKEND_STS_REG & BACKEND_STS_B_READY) );

		if (bDMAEnable) {
	        while( !(DMA_CONFIG & DMA_CHK_DMA_XFER_DONE) );		// DMA move data to buffer
	        // clear finish flag
	        DMA_CONFIG |= DMA_CHK_DMA_XFER_DONE;
		} else 
			memcpy( (void*)buffer, (void*)DMA_CR_BUF_ADD, 512 );		// CPU move data to buffer

        NOR_CS_HIGH;

        BlockNo--;
        BufferAddr += 512;
        add        += 512;        
    } while( BlockNo );

    NOR_DISABLE;
    
    NorRestoreTimming();   
}

void NorRleaseFromPowerDown(unsigned char bCE)
{
	gdwTmp = gdwNorCE;		// save current CE
    gdwNorCE = bCE;			// new CE
    
    NorSetTimming();
    NOR_ENABLE;

    NOR_CS_LOW;
    
    CRCTL_BUF_CTL ^= 1;
    CRCTL_BUF_CTL ^= 1;	    
    CRCTL_BUF_CTL &= ~0x3;        
    CRCTL_BUF_CTL  |= CR_XFER_FROM_CARD + CR_FLASH_MODE + CR_AUTO_MODE + CR_ENABLE;
    
    Nor_ManualWriteByte( 0xAB );
    while( !(FDM_BACKEND_STS_REG & 0x20) );
    
    NOR_CS_HIGH;
    NOR_DISABLE;
    
    gdwNorCE = gdwTmp;			// restore old CE
}

void NorReadID(unsigned char bCE, unsigned char* buf)
{
    int i;

	gdwTmp = gdwNorCE;		// save current CE
    gdwNorCE = bCE;			// new CE
    
    NorRleaseFromPowerDown(bCE);
    NorSetTimming();    
    NOR_ENABLE;
    NOR_CS_LOW;
    
    CRCTL_BUF_CTL ^= 1;
    CRCTL_BUF_CTL ^= 1;	    
    CRCTL_BUF_CTL &= ~0x3;        
    CRCTL_BUF_CTL  |= CR_XFER_FROM_CARD + CR_FLASH_MODE + CR_AUTO_MODE + CR_ENABLE;
        
    Nor_ManualWriteByte( 0x9F );
    while( !(FDM_BACKEND_STS_REG & 0x20) );
    
    for( i = 0; i < NOR_ID_BYTE; i++ )
    {
        buf[i] = (unsigned char)(Nor_ManualReadByte());
        while( !(FDM_BACKEND_STS_REG & 0x20) );
    }
    
    NOR_CS_HIGH;
    NOR_DISABLE;
    
    gdwNorCE = gdwTmp;			// restore old CE   
}


/*****************************************************************************/
//	Flowbee - 2007-08-25	Find the brightness and contrast value.	--
/*****************************************************************************/
/*
	Boot sector data structure.
*/
struct _fat_boot_sector {
	char			ignored[3];		/* Boot strap short or near jump */
	char			system_id[8];	/* Name - can be used to special case
				   						partition manager volumes */
	unsigned char	sector_size[2];	/* bytes per logical sector */
	unsigned char	cluster_size;	/* sectors/cluster */
	unsigned short	reserved;		/* reserved sectors before fat table */
	unsigned char	fats;			/* number of FATs */
	unsigned char	dir_entries[2];	/* root directory entries */
	unsigned char	sectors[2];		/* number of sectors */
	unsigned char	media;			/* media code (unused) */
	unsigned short	fat_length;		/* sectors/FAT */
	unsigned short	secs_track;		/* sectors per track */
	unsigned short	heads;			/* number of heads */
	unsigned long	hidden;			/* hidden sectors (unused) */
	unsigned long	total_sect;		/* number of sectors (if sectors == 0) */

	//char 			res_bytes[476];
	char 			res_bytes[474];
	unsigned short  end_id;	
} boot_sec;

/*
	File description block
*/
struct _fdb {
	char fname[11];
	char fattr;
	char res[6];
	unsigned short l_date;
	unsigned short h_clu_num;
	unsigned short b_time;
	unsigned short b_date;
	unsigned short clu_num;
	unsigned long fsize;
} fdb;

#define CONFIG_BLOCK_OFFSET			0x3f0000
#define BYTES_PER_SECTOR			(boot_sec.sector_size[0] + (boot_sec.sector_size[1] << 8))
#define BYTES_PER_CLUSTER			(boot_sec.cluster_size * BYTES_PER_SECTOR)

#define ROOT_DIR_OFFSET				((1 + (boot_sec.fats * boot_sec.fat_length)) * BYTES_PER_SECTOR)
#define FDBS_PER_ROOT_DIR			(boot_sec.dir_entries[0] + (boot_sec.dir_entries[1] << 8))
#define BYTES_PER_FDB				32
#define BYTES_PER_ROOT_DIR			(FDBS_PER_ROOT_DIR * BYTES_PER_FDB)

#define FILE_NAME1					"USER"
#define FILE_NAME2					"user"
#define STRING_NAME1				"VOLUME"
#define STRING_NAME2				"volume"
#define DELIMITER					'#'

//-- Flowbee 2008-06-23
#define BC_FILE_NAME1				"BC"
#define BC_FILE_NAME2				"bc"
#define CHAR_NUM					5

#define DATA_AREA_OFFSET			(ROOT_DIR_OFFSET + BYTES_PER_ROOT_DIR)

#define FAT_TABLE_START_OFFSET		2

#define STOP_NUM					0xAAAAAAAA
#define NOR_BUF_SIZE				512

typedef enum
{
	USER_SETUP_LANGUAGE				= 0,
	USER_SETUP_POWERSAVING			= 1,
	USER_SETUP_SYS_ALPHA			= 2,
	USER_SETUP_HOMEPAGE				= 3,
	USER_SETUP_BRIGHTNESS			= 4,
	USER_SETUP_CONTRAST				= 5,
	USER_SETUP_HDTV					= 6,
	USER_SETUP_CAL_ALPHA			= 7,
	USER_SETUP_ALARM				= 8,
	USER_SETUP_ALARM_HOUR			= 9,
	USER_SETUP_ALARM_MIN			= 10,
	USER_SETUP_SS_REPEAT			= 11,
	USER_SETUP_SS_INTERVAL			= 12,
	USER_SETUP_SS_EFFECT			= 13,
	USER_SETUP_AUDIO_REPEAT			= 14,
	USER_SETUP_VIDEO_REPEAT			= 15,
	USER_SETUP_ATOPWR				= 16,
	USER_SETUP_ATOPWR_ON_HOUR		= 17,
	USER_SETUP_ATOPWR_ON_MIN		= 18,
	USER_SETUP_ATOPWR_OFF_HOUR		= 19,
	USER_SETUP_ATOPWR_OFF_MIN		= 20,
	USER_SETUP_VOLUME				= 21
}USER_SETUP_ITEM;//define: NUM_OF_USER_SETUP_ITEM = 22, instance: SavedUserSetupItem

//== Brightness and Contrast tables
#if (CONFIG_SKY_IO_FS_COMPANY_V1 || CONFIG_SKY_IO_FS_COMPANY_V2 || CONFIG_SKY_IO_FS_COMPANY_V3)
//-- Fashion Star 
#define BRTNSS_CNTRST_ITEM_NUM		9
enum
{
	SETUP_BRTNSS_PRCNT_10		= 40,
	SETUP_BRTNSS_PRCNT_20		= 64,
	SETUP_BRTNSS_PRCNT_30		= 88,
	SETUP_BRTNSS_PRCNT_40		= 112,
	SETUP_BRTNSS_PRCNT_50		= 136,
	SETUP_BRTNSS_PRCNT_60		= 152,
	SETUP_BRTNSS_PRCNT_70		= 168,
	SETUP_BRTNSS_PRCNT_80		= 184,
	SETUP_BRTNSS_PRCNT_90		= 200
};

enum
{
	SETUP_CNTRST_PRCNT_10		= 32,
	SETUP_CNTRST_PRCNT_20		= 56,
	SETUP_CNTRST_PRCNT_30		= 80,
	SETUP_CNTRST_PRCNT_40		= 104,
	SETUP_CNTRST_PRCNT_50		= 128,
	SETUP_CNTRST_PRCNT_60		= 144,
	SETUP_CNTRST_PRCNT_70		= 160,
	SETUP_CNTRST_PRCNT_80		= 176,
	SETUP_CNTRST_PRCNT_90		= 192
};

static unsigned char SETUP_BRIGHTNESS[BRTNSS_CNTRST_ITEM_NUM] = 
{
	SETUP_BRTNSS_PRCNT_10,
	SETUP_BRTNSS_PRCNT_20,
	SETUP_BRTNSS_PRCNT_30,
	SETUP_BRTNSS_PRCNT_40,
	SETUP_BRTNSS_PRCNT_50,
	SETUP_BRTNSS_PRCNT_60,
	SETUP_BRTNSS_PRCNT_70,
	SETUP_BRTNSS_PRCNT_80,
	SETUP_BRTNSS_PRCNT_90
};

static unsigned char SETUP_CONTRAST[BRTNSS_CNTRST_ITEM_NUM] = 
{
	SETUP_CNTRST_PRCNT_10,
	SETUP_CNTRST_PRCNT_20,
	SETUP_CNTRST_PRCNT_30,
	SETUP_CNTRST_PRCNT_40,
	SETUP_CNTRST_PRCNT_50,
	SETUP_CNTRST_PRCNT_60,
	SETUP_CNTRST_PRCNT_70,
	SETUP_CNTRST_PRCNT_80,
	SETUP_CNTRST_PRCNT_90
};

#define DEFAULT_BRIGHTNESS_INDEX		(4)
#define DEFAULT_CONTRAST_INDEX			(4)

#else	// CONFIG_FASHION_STAR
//-- Others
#define BRTNSS_CNTRST_ITEM_NUM		5

enum
{
	SETUP_BRTNSS_PRCNT_10		= 32,
	SETUP_BRTNSS_PRCNT_30		= 64,
	SETUP_BRTNSS_PRCNT_50		= 96,
	SETUP_BRTNSS_PRCNT_70		= 128,
	SETUP_BRTNSS_PRCNT_90		= 176
};

enum
{
	SETUP_CNTRST_PRCNT_30		= 75,
	SETUP_CNTRST_PRCNT_40		= 100,
	SETUP_CNTRST_PRCNT_50		= 137,
	SETUP_CNTRST_PRCNT_60		= 150,
	SETUP_CNTRST_PRCNT_70		= 175
};

static unsigned char SETUP_BRIGHTNESS[BRTNSS_CNTRST_ITEM_NUM] = 
{
	SETUP_BRTNSS_PRCNT_10,
	SETUP_BRTNSS_PRCNT_30,
	SETUP_BRTNSS_PRCNT_50,
	SETUP_BRTNSS_PRCNT_70,
	SETUP_BRTNSS_PRCNT_90
};

static unsigned char SETUP_CONTRAST[BRTNSS_CNTRST_ITEM_NUM] = 
{
	SETUP_CNTRST_PRCNT_30,
	SETUP_CNTRST_PRCNT_40,
	SETUP_CNTRST_PRCNT_50,
	SETUP_CNTRST_PRCNT_60,
	SETUP_CNTRST_PRCNT_70
};

//-- Define default brightness and contrast index
#if CONFIG_SKY_IO_LT_COMPANY_V1
#define DEFAULT_BRIGHTNESS_INDEX		(3)
#define DEFAULT_CONTRAST_INDEX			(2)
#else
#define DEFAULT_BRIGHTNESS_INDEX		(2)
#define DEFAULT_CONTRAST_INDEX			(2)
#endif


#endif	// CONFIG_FASHION_STAR

#define DEFAULT_BRIGHTNESS_VALUE		(SETUP_BRIGHTNESS[DEFAULT_BRIGHTNESS_INDEX])
#define DEFAULT_CONTRAST_VALUE			(SETUP_CONTRAST[DEFAULT_CONTRAST_INDEX])
unsigned char brightness;
unsigned char contrast;

void set_default_brightness_contrast(void)
{
	//brightness = DEFAULT_BRIGHTNESS_VALUE; //Mel mark it
    brightness = 0xc8; //Mel new value
	contrast = DEFAULT_CONTRAST_VALUE;
}

int find_brightness_contrast(unsigned char *_bright, unsigned char *_cont)
{
	unsigned int v_i=0;
	unsigned int v_j=0;
	unsigned char tmp_c;
	unsigned char tmp[NOR_BUF_SIZE*2];
	unsigned char _b = 0, _c = 0;;
	char fname1[10] = FILE_NAME1;
	char fname2[10] = FILE_NAME2;
	char sname1[10] = STRING_NAME1;
	char sname2[10] = STRING_NAME2;	

	set_default_brightness_contrast();
	//--Init Nor Flash controller.
	if(NorInit() == -1){
		return -1;
	}
	//--Read boot sector and get BPB(bios parameter block) information.
	NorRead( (CONFIG_BLOCK_OFFSET / NOR_BUF_SIZE), 1, (unsigned char *)&boot_sec, 0);
	//-- Check BPB
	if(boot_sec.cluster_size == 0x00){
		return 1;
	}
	// Check bytes per sector (should be 512 / 1024 / 2048 / 4096)
	if((BYTES_PER_SECTOR != 512) && (BYTES_PER_SECTOR != 1024) 
	&& (BYTES_PER_SECTOR != 2048) && (BYTES_PER_SECTOR != 4096)){
		return 1;
	}
	// Check root entry count (FAT16 = 512)
	if(FDBS_PER_ROOT_DIR > 512){
		return 1;
	}
	// Check fat table num
	if(boot_sec.fats != 2){
		return 1;
	}
	// Check end id
	if(boot_sec.end_id != 0xaa55){
		return 1;
	}
	
	for(v_i = 0; v_i < (BYTES_PER_ROOT_DIR); v_i += NOR_BUF_SIZE){
		NorRead( ((CONFIG_BLOCK_OFFSET + ROOT_DIR_OFFSET + v_i) / NOR_BUF_SIZE), 1, tmp, 0 );
		//--Parsing FDB
		for(v_j = 0; v_j < NOR_BUF_SIZE; v_j += BYTES_PER_FDB){
			memcpy(&fdb, (tmp + v_j), BYTES_PER_FDB);
			if( (fdb.fname[0] == fname1[0]) || (fdb.fname[0] == fname2[0]) ){
				if( (fdb.fname[1] == fname1[1]) || (fdb.fname[1] == fname2[1]) ){
					if( (fdb.fname[2] == fname1[2]) || (fdb.fname[2] == fname2[2]) ){					
						v_i = STOP_NUM;
						break;
					}
				}
			}
		}
		if(v_i == STOP_NUM){
			break;
		}
	}	

	if(v_i != STOP_NUM){
		return 2;
	}
	//--Get the cluster num and the file data.
	NorRead( ((CONFIG_BLOCK_OFFSET + DATA_AREA_OFFSET + ((fdb.clu_num - FAT_TABLE_START_OFFSET) * BYTES_PER_CLUSTER)) / NOR_BUF_SIZE), 1, tmp, 0 );
	NorRead( ((NOR_BUF_SIZE + CONFIG_BLOCK_OFFSET + DATA_AREA_OFFSET + ((fdb.clu_num - FAT_TABLE_START_OFFSET) * BYTES_PER_CLUSTER)) / NOR_BUF_SIZE), 1, &tmp[NOR_BUF_SIZE], 0 );
	//--Get brightness and contrast value
	//--Find the first keyword "volume"
	for(v_i = 0; v_i < (2*NOR_BUF_SIZE); v_i ++){
		if( (tmp[v_i] == sname1[0]) || (tmp[v_i] == sname2[0]) ){
			v_i ++;
			if( (tmp[v_i] == sname1[1]) || (tmp[v_i] == sname2[1]) ){
				v_i ++;
				if( (tmp[v_i] == sname1[2]) || (tmp[v_i] == sname2[2]) ){
					//--Find keyword '#'
					v_i += 3;
					v_j = 0;
					do{
						if(tmp[v_i] == DELIMITER){
							v_j ++;
						}
						v_i ++;
					}while( (v_j < USER_SETUP_BRIGHTNESS) && (v_i < (2*NOR_BUF_SIZE)) );
					//--Transfer from ascii to integer.
					_b = (tmp[v_i] - 0x30);
					v_i += 2;
					_c = (tmp[v_i] - 0x30);
					// Flowbee 2008-05-26 Check brightness and contrast index
					if(_b >= BRTNSS_CNTRST_ITEM_NUM){
						_b = BRTNSS_CNTRST_ITEM_NUM;
					}
					if(_c >= BRTNSS_CNTRST_ITEM_NUM){
						_c = BRTNSS_CNTRST_ITEM_NUM;
					}
					*_bright = SETUP_BRIGHTNESS[_b];
					*_cont = SETUP_CONTRAST[_c];
					return 0;
				}
			}
		}
	}	
	return 3;
}

int atoi(char *_s)
{
	int i = 0, j = 0, ret_val = 0;

	while(i < CHAR_NUM){
		if( (_s[i] != '\0') && (_s[i] >= '0') && (_s[i] <= '9') ){
			ret_val = ret_val * 10;
			ret_val += (_s[i] - '0');
		}
		else if(_s[i] == '\0'){
			break;
		}
		else{
			return -1;
		}
		i ++;
	}
	//printk("\n val: %d", ret_val);
	return ret_val;
}

int new_find_brightness_contrast(unsigned char *_bright, unsigned char *_cont)
{
	unsigned int v_i=0;
	unsigned int v_j=0;
	char tmp[NOR_BUF_SIZE*2];
	char fname1[10] = BC_FILE_NAME1;
	char fname2[10] = BC_FILE_NAME1;
	char b[CHAR_NUM], c[CHAR_NUM];
	struct _fat_boot_sector boot_sec;
	struct _fdb fdb;

	set_default_brightness_contrast();
	//--Init Nor Flash controller.
	if(NorInit() == -1){
		return -1;
	}
	//--Read boot sector and get BPB(bios parameter block) information.
	NorRead( (CONFIG_BLOCK_OFFSET / NOR_BUF_SIZE), 1, (unsigned char *)&boot_sec, 0);
	//-- Check BPB
	if(boot_sec.cluster_size == 0x00){
		return 1;
	}
	// Check bytes per sector (should be 512 / 1024 / 2048 / 4096)
	if((BYTES_PER_SECTOR != 512) && (BYTES_PER_SECTOR != 1024) 
	&& (BYTES_PER_SECTOR != 2048) && (BYTES_PER_SECTOR != 4096)){
		return 1;
	}
	// Check root entry count (FAT16 = 512)
	if(FDBS_PER_ROOT_DIR > 512){
		return 1;
	}
	// Check fat table num
	if(boot_sec.fats != 2){
		return 1;
	}
	// Check end id
	if(boot_sec.end_id != 0xaa55){
		return 1;
	}
	
	for(v_i = 0; v_i < (BYTES_PER_ROOT_DIR); v_i += NOR_BUF_SIZE){
		NorRead( ((CONFIG_BLOCK_OFFSET + ROOT_DIR_OFFSET + v_i) / NOR_BUF_SIZE), 1, tmp, 0 );
		//--Parsing FDB
		for(v_j = 0; v_j < NOR_BUF_SIZE; v_j += BYTES_PER_FDB){
			memcpy(&fdb, (tmp + v_j), BYTES_PER_FDB);
			if( (fdb.fname[0] == fname1[0]) || (fdb.fname[0] == fname2[0]) ){
				if( (fdb.fname[1] == fname1[1]) || (fdb.fname[1] == fname2[1]) ){
					v_i = STOP_NUM;
					break;
				}
			}
		}
		if(v_i == STOP_NUM){
			break;
		}
	}	

	if(v_i != STOP_NUM){
		return 2;
	}
	//--Get the cluster num and the file data.
	NorRead( ((CONFIG_BLOCK_OFFSET + DATA_AREA_OFFSET + ((fdb.clu_num - FAT_TABLE_START_OFFSET) * BYTES_PER_CLUSTER)) / NOR_BUF_SIZE), 1, tmp, 0 );
	NorRead( ((NOR_BUF_SIZE + CONFIG_BLOCK_OFFSET + DATA_AREA_OFFSET + ((fdb.clu_num - FAT_TABLE_START_OFFSET) * BYTES_PER_CLUSTER)) / NOR_BUF_SIZE), 1, &tmp[NOR_BUF_SIZE], 0 );

	//-- Get brightness
	v_j = 0;
	for(v_i = 0; v_i < (2*NOR_BUF_SIZE); v_i ++){
		if(tmp[v_i] != DELIMITER){
			b[v_j ++] = tmp[v_i];
		}
		else{
			b[v_j ++] = '\0';
			v_i ++;
			v_j = 0xaa;
			break;
		}
	}
	if(v_j != 0xaa){
		return 2;
	}

	//-- Get contrast
	v_j = 0;
	for( ; v_i < (2*NOR_BUF_SIZE); v_i ++){
		if(tmp[v_i] != DELIMITER){
			c[v_j ++] = tmp[v_i];
		}
		else{
			c[v_j ++] = '\0';
			v_i ++;
			v_j = 0xaa;
			break;
		}
	}
	if(v_j != 0xaa){
		return 2;
	}

	*_bright = 0xc8; //Mel Modified //*_bright = (unsigned char)(atoi(b));
	*_cont = (unsigned char)(atoi(c));
	return 3;
}


/*****************************************************************************/
//	Flowbee - 2007-08-25	Find the brightness and contrast value.	--
/*****************************************************************************/





