/************************************************************************
 *		This file include primitive functions which can control the
 *		TeraWins T-CON.
 ************************************************************************/
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include "arc_pgu.h"
#include "arc_tcon.h"
#include "arc_disp_param.h"


#if CONFIG_TERAWINS_TCON

extern int *TerawinsCmdTbl[TERAWINS_END*2];

//== Global variable
static unsigned long val = 0;
static volatile unsigned long vtval; 	
unsigned long tval;
extern asmlinkage void __init __puts(const char *s);
/*******************************************************************************
			Terawins I2C control
*******************************************************************************/
/* -------------------------------------------------------------------------- */
inline void terawins_delay(unsigned long count)
{
	unsigned long _x; 
	do { 
		for(_x=0;_x< count;_x++){
			__asm__ __volatile__("nop");
			//__builtin_arc_nop();
		}
	} while(0);
}

/* -------------------------------------------------------------------------- */
void terawins_i2c_reset(void)
{
	SCL_HI;
	SCL_OUTPUT;
	SDA_HI;
	SDA_OUTPUT;	

	terawins_delay(TERAWINS_TIME_1);
}

/* -------------------------------------------------------------------------- */
void terawins_i2c_start(void)
{
	SDA_HI;
	SDA_OUTPUT;
	SCL_HI;
	SCL_OUTPUT;
	terawins_delay(TERAWINS_TIME_1);	
	SDA_LO;
	terawins_delay(TERAWINS_TIME_1);	
	SCL_LO;
}

/* -------------------------------------------------------------------------- */
void terawins_i2c_stop(void)
{
	SDA_LO;
	SCL_HI;
	terawins_delay(TERAWINS_TIME_1);
	SDA_HI;
}

/* -------------------------------------------------------------------------- */
void terawins_i2c_ack(int bit)
{
	/* Here's the deal. The slave is slow, and sometimes needs to wait
       before it can receive the acknowledge. Therefore it forces the clock
       low until it is ready. We need to poll the clock line until it goes
       high before we release the ack. */
    
	SCL_LO;      /* Set the clock low */
	if ( bit )
	{
		SDA_HI;
	}
	else
	{
		SDA_LO;
	}
    
	SCL_INPUT;   /* Set the clock to input */
	#if 0
	while(!SCL);  /* and wait for the slave to release it */      
	#else
	do{
		SCL;
	}while(!vtval);
	#endif
	terawins_delay(TERAWINS_TIME_1);
	SCL_OUTPUT;
	SCL_LO;
	terawins_delay(TERAWINS_TIME_2);	
}

/* -------------------------------------------------------------------------- */
int terawins_i2c_getack(void)
{
	int ret = 1;

	SDA_INPUT;
	terawins_delay(TERAWINS_TIME_2);
	SCL_HI;
	#if 0
	if (SDA)
	    ret = 0;
	#else
	SDA;
	if (vtval)
	    ret = 0;

	#endif
	
	terawins_delay(TERAWINS_TIME_2);

	/* pull up SDA first, and set SDA to OUTPUT */
	//Flowbee
//	SDA_LO;   
	SDA_HI;
	SDA_OUTPUT;  
	  
	SCL_LO;

	return ret;
}

/* -------------------------------------------------------------------------- */
void terawins_i2c_outb(unsigned char byte)
{
    int i;
   /* clock out each bit, MSB first */
   for ( i=0x80; i; i>>=1 ) {
      	if ( i & byte ){
        	SDA_HI;
      	}
      	else{
        	SDA_LO;
      	}
	terawins_delay(TERAWINS_TIME_1);
      	SCL_HI;
	terawins_delay(TERAWINS_TIME_1);
	SCL_LO;
   }

   SDA_HI;
}

/* -------------------------------------------------------------------------- */
unsigned char terawins_i2c_inb(int msb, int ack)
{
	int i;
	unsigned char byte = 0;

	if(msb){
	/* clock in each bit, MSB first */
		for ( i=0x80; i; i>>=1 ) {
			terawins_delay(TERAWINS_TIME_1);
			SDA_HI;
			SDA_INPUT;   /* And set to input */
			SCL_HI;
			terawins_delay(TERAWINS_TIME_1);
			#if 0
			if ( SDA )
				byte |= i;

			#else
			SDA;
			if (vtval)
				byte |= i;

			#endif

			SCL_LO;
			SDA_OUTPUT;
			terawins_delay(TERAWINS_TIME_1);
		}
	}else{
	/* clock in each bit, LSB first */
		for(i=0; i<8; i++){
			SDA_HI;
			SDA_INPUT;   /* And set to input */
			SCL_HI;
			terawins_delay(TERAWINS_TIME_1);		
			#if 0
			if ( SDA )
				byte |= (1 << i);

			#else
			SDA;
			if (vtval)
				byte |= (1 << i);

			#endif

			SCL_LO;
			SDA_OUTPUT;
			terawins_delay(TERAWINS_TIME_1);			
		}
	}
   
	terawins_i2c_ack(ack);
	return byte;
}

/* -------------------------------------------------------------------------- */
#if 0 //Mel original
int terawins_i2c_write(int address, const unsigned char* buf, int count)
{
	int i,x=0;	
	terawins_i2c_start();
	terawins_i2c_outb(address & 0xFE);
	if (terawins_i2c_getack()){
	    for (i=0; i<count; i++){
	    	terawins_i2c_outb(buf[i]);
	        if (!terawins_i2c_getack()){
	        	x=2;
	                break;
	        }
	    }
	}else{
		x=1;
	}
	
	terawins_i2c_stop();
	return x;
}
#else

int terawins_i2c_write(int address, const unsigned char* buf, int count)
{
	int i,x=0;	
    int retry=0;

_AddrRetry:    
	terawins_i2c_start();
	terawins_i2c_outb(address & 0xFE);
	if (terawins_i2c_getack())
	{
	    for (i=0; i<count; i++)
	    {
	    	terawins_i2c_outb(buf[i]);
	        if (!terawins_i2c_getack()) {
	        	x=2;
	            break;
	        }
	    }
	}else {
		x=1;
	}
	terawins_i2c_stop();

    if(x==1 && retry==0)
    {
        x=0;
        retry=1;
        address += 0x10;
        //No printk("Retry I2C_Addr(0x%x).\n", address);
        goto _AddrRetry;
    }
	return x;
}
#endif

static spinlock_t sk_i2c_lock = SPIN_LOCK_UNLOCKED;
static unsigned long sk_i2c_flag;
/* -------------------------------------------------------------------------- */
int terawins_set_cmd(uint8 dev_addr, uint8 reg_addr, uint8 reg_val)
{
	unsigned char _buf[2] = { reg_addr, reg_val};
	int i;

	i = terawins_i2c_write(dev_addr, _buf, 2);
	if(i != 0){
		//TW_MSG("Terawins I2C write error !!\n");
	}
	return i;
}

/* -------------------------------------------------------------------------- */
int terawins_set_page(unsigned char _page_num, int *_page)
{
	int ret_val = 0;
	while(*(_page) != END_CMD){
		ret_val = terawins_set_cmd(_page_num, (unsigned char)(*(_page)), (unsigned char)(*(_page + 1)));
		if((ret_val == 2) || (ret_val == 1)){
			TW_MSG("Write to TeraWins Error!! page:%x %x %x\n", _page_num, (unsigned char)(*(_page)), (unsigned char)(*(_page + 1)));
			return ret_val;
		}
		_page += 2;
		terawins_delay(0x10);
	}
	return ret_val;
}

#if 0	//-- unused function
/* -------------------------------------------------------------------------- */
static int terawins_i2c_read(int address, unsigned char* buf, int count, int msb)
{
    int i,x=1;

	terawins_i2c_start();
	terawins_i2c_outb(address | 0x01);
	if (terawins_i2c_getack()) {
		for (i=0; i<count; i++)
			buf[i] = terawins_i2c_inb(msb, 0);
	}
	else
        x=0;
	
	terawins_i2c_stop();
	return x;
}

/* -------------------------------------------------------------------------- */
static unsigned char terawins_get_cmd(uint8 dev_addr, uint8 reg_addr)
{
	unsigned char _buf = reg_addr;
    int i,x=1;	
	int count = 1;
	int msb = 1;
	/* Step1: write */
	terawins_i2c_start();
	terawins_i2c_outb(dev_addr & 0xFE);
	if (terawins_i2c_getack()){
    	terawins_i2c_outb(_buf);
        if (!terawins_i2c_getack()){
        	x=2;
        }
	}else{
		x=1;
	}
    /* Step2: restart and read. */
	terawins_i2c_start();
	terawins_i2c_outb(dev_addr | 0x01);
	if (terawins_i2c_getack()) {
		for (i=0; i<count; i++)
			_buf = terawins_i2c_inb(msb, 0);
	}
	else
        x=0;
	
	terawins_delay(TERAWINS_TIME_2);	
	terawins_i2c_stop();
	terawins_delay(TERAWINS_TIME_2);
	return _buf;
}
#endif

/*******************************************************************************
			Terawins functions
*******************************************************************************/
/* -------------------------------------------------------------------------- */
void terawins_on_off(uint8 _on)
{
	if(_on == 1){
		//-- turn on
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xC2, 0x00);
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x91, 0x07);
		terawins_delay(0x200000);
	}
	else{
		//-- turn off
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xC2, 0x12);
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x91, 0x87);
	}
}

/* -------------------------------------------------------------------------- */
int terawins_set_resolution(uint8 idx)
{
	int ret_val = 0;

	/* Set command to terawins T-CON. */
	//-- Setting page0 registers.
	ret_val = terawins_set_page(TERAWINS_PAGE0_ADDR, TerawinsCmdTbl[idx * 2]);
	//-- Setting page1 registers.
	ret_val |= terawins_set_page(TERAWINS_PAGE1_ADDR, TerawinsCmdTbl[idx * 2 + 1]);
	return ret_val;
}

/* -------------------------------------------------------------------------- */
void terawins_pwm_on_off(uint8 _on)
{
	int i = 0;
	if(_on == 1){
		//-- Turn on PWM
        //Mel: original
		//i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe8, 0x10);
        i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe8, 0x13);
	}
	else{
		//-- Turn off PWM
        //Mel: original
		//i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe8, 0x00);
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe8, 0x03);
	}
	if(i != 0){
		TW_MSG("Set PWM on/off fail !\n");
	}
}

/* -------------------------------------------------------------------------- */
void terawins_backlight_on_off(uint8 _on)
{
	int i = 0;
	if(_on == 1){
		//-- Turn on terawins backlight
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xec, 0xe2);
	}
	else{
		//-- Turn off terawins backlight
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xec, 0xc0);			
	}
	if(i != 0){
		TW_MSG("Terawins backlight on/off fail !\n");
	}
}

/* -------------------------------------------------------------------------- */
void terawins_set_pwm(uint8 _val)
{
	int i = 0;

    //__puts("Set PWM[booting]\n");
	//TW_MSG("\nPWM !!\n");
	//-- Set pwm duty cycle.
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe9, _val);	
	if(i != 0){
		TW_MSG("Set PWM fail !\n");
	}
}

#if CONFIG_SKY_IO_QMI_COMPANY_V1
/* -------------------------------------------------------------------------- */
int set_qmi_brightness(uint8 dev_addr, uint8 reg_val)
{
	int i;
	
	spin_lock_irqsave(&sk_i2c_lock, sk_i2c_flag);
	i = terawins_i2c_write(dev_addr, &reg_val, 1);
	spin_unlock_irqrestore(&sk_i2c_lock, sk_i2c_flag);
	
	return i;
}

/* -------------------------------------------------------------------------- */
void set_qmi_blue_led(void)
{
     //GPIO BIT13=0 BIT14=1 (Blue LED lightest)
     QMI_CT1_OUTPUT;
     QMI_CT1_LO;
     QMI_CT2_OUTPUT;
     QMI_CT2_HI;
}

/* -------------------------------------------------------------------------- */
void set_lcd_on(void)
{     
     QMI_LCD_OUTPUT;
     QMI_LCD_HI;
}
#endif // CONFIG_SKY_IO_QMI_COMPANY_V1

/* -------------------------------------------------------------------------- */
void terawins_set_brightness(uint8 _val)
{
	int i = 0;
	//TW_MSG("\nBrightness !!\n");
	//-- Set pwm freq.
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x69, _val);
	if(i != 0){
		TW_MSG("Set Brightness fail !\n");
	}
}

/* -------------------------------------------------------------------------- */
void terawins_set_contrast(uint8 _val)
{
	int i = 0;
	//TW_MSG("\nContrast !!\n");
	//-- Set pwm freq.
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x68, _val);
	if(i != 0){
		TW_MSG("Set Contrast fail !\n");
	}
}

/* -------------------------------------------------------------------------- */
int init_terawins_tcon(void)
{
	int *p0, *p1;

	TW_MSG("\nInit Terawins T-CON !!\n");
	/* Set default commands to terawins T-CON. */
	//-- Setting page0 registers.
	p0 = TerawinsCmdTbl[TERAWINS_INIT * 2];
	terawins_set_page(TERAWINS_PAGE0_ADDR, p0);
	//-- Setting page1 registers.
	p1 = TerawinsCmdTbl[TERAWINS_INIT * 2 + 1];
	terawins_set_page(TERAWINS_PAGE1_ADDR, p1);
	return 0;
}

/*******************************************************************************
			Terawins OSD functions.
*******************************************************************************/
/* -------------------------------------------------------------------------- */
static int terawins_set_osd_cmd(unsigned char cfg_idx, unsigned char cfg_data)
{
	int ret_val = 0;
	//-- set osd control register address.
	ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_INDEX, cfg_idx);
	
	//-- set osd control register data.
	ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_DATA, cfg_data);

	if(ret_val != 0){
		TW_MSG("\nTeraWins set osd cmd error!\n");
	}
	
	return ret_val;
}


/* -------------------------------------------------------------------------- */
void terawins_enable_osd(unsigned char _en)
{
	if(_en == 1){
		terawins_set_osd_cmd(OSD_CTRL, 0x80);
	}
	else{
		terawins_set_osd_cmd(OSD_CTRL, 0x00);
	}
}

#endif

