/*
 * hdjpeg.c
 *
 * Copyright (C) 1991-1997, Thomas G. Lane.
 * This file is part of the Independent JPEG Group's software.
 * For conditions of distribution and use, see the accompanying README file.
 *
 * This file contains a command-line user interface for the JPEG decompressor.
 * It should work on any system with Unix- or MS-DOS-style command lines.
 *
 * Two different command line styles are permitted, depending on the
 * compile-time switch TWO_FILE_COMMANDLINE:
 *	hdjpeg [options]  inputfile 
 * In the second style, output is always to standard output, which you'd
 * normally redirect to a file or pipe to some other program.  Input is
 * either from a named file or from standard input (typically redirected).
 * The second style is convenient on Unix but is unhelpful on systems that
 * don't support pipes.  Also, you MUST use the first style if your system
 * doesn't do binary I/O to stdin/stdout.
 * To simplify script writing, the "-outfile" switch is provided.  The syntax
 *	djpeg [options]  -outfile outputfile  inputfile
 * works regardless of which command line style is used.
 */

#include "hal.h"
#include "arc_pgu.h"
#include "scalereg.h"
#include <linux/autoconf.h>

#define IMAGE_SCALE_ENABLE 1

/*
 * This list defines the known output image formats
 * (not all of which need be supported by a given version).
 * You can change the default output format by defining DEFAULT_FMT;
 * indeed, you had better do so if you undefine PPM_SUPPORTED.
 */
 /* Flowbee@960228 */
//#define DEFAULT_XRES	640
//#define DEFAULT_YRES	480
#define DEFAULT_ALIGN_SIZE 32
//-- flowbee 2007-10-02
#define BUF_ALIGN(x, alignment) ((x) & ~alignment)
//#define BUF_ALIGN(x, alignment) 	((x&alignment) > 0): ((x + alignment) & ~alignment) ? ((x) & ~alignment)


#define JCS_YCbCr	1
#define JCS_RGB		2
#define READ_BINARY	"rb"
static int out_color_space;
int DispNo;

unsigned int jpeg_size=0;
//unsigned short OUTPUT_XRES=640, OUTPUT_YRES=480;
extern unsigned short OUTPUT_XRES;
extern unsigned short OUTPUT_YRES;
extern unsigned int FBBaseAddr;
extern unsigned int logo_output;
extern unsigned long YUVBase;
extern unsigned long YUVBufSize;
extern unsigned long YBufSize;
extern unsigned long UVBufSize;


/*
 * Argument-parsing code.
 * The switch parser is designed to be useful with DOS-style command line
 * syntax, ie, intermixed switches and file names, where only the switches
 * to the left of a given file name affect processing of that file.
 * The main program in this file doesn't actually use this capability...
 */
unsigned short TurnOnPGU, HWScale, NoScale;

void YUVBuf2RGBBuf(unsigned char *YBuf, unsigned short *RGBBuf, unsigned short xres, unsigned short yres);

#ifdef IMAGE_SCALE_ENABLE
void ScaleLine(unsigned char *srcline, unsigned char *dstline, unsigned short srcWidth, unsigned short dstWidth, unsigned short xdiv);
#endif

//extern void __puts(const char *s);
extern int arc_pgufb_mode_yuv(void);
extern int PGU_Enable(void);
extern int set_rgb_pointer(void);
extern int set_yuv_pointer(void);
extern void *malloc(int size);
extern void* __memcpy(void* __dest, __const void* __src, size_t __n);
extern void halJPEGDInit_i(void);
extern unsigned int halJPEGD_GetIntMask(void);
extern void force_reset(void);
extern void flush_all_dcache(void);
extern void __puts(const char *s); //Mel

int pgu_get_yuv_base(int no)
{
	/* Flowbee@960301 */
	return (YUVBase + (no * YUVBufSize));
}

void set_config (void)
{
	/* Flowbee@960301 */
    out_color_space = JCS_YCbCr;
	NoScale = 0;
    HWScale = 0;
    TurnOnPGU = 1;
	/* Flowbee */
    DispNo = 1;
//    DispNo = 0;

    /* PGU output format. */
	out_color_space = JCS_YCbCr;
}

#define ScaleColor(col0, col1, part, all, div)	\
	((part) == 0) ? (col0) : ((((col1*part) + (col0 *(all-part)))*div) >> SCALE_SHIFT)


unsigned int SetRegister(unsigned int addr, unsigned int val)
{
	*((unsigned int *)addr) = val;
	val = *((unsigned int *)addr);
	return val;
}

unsigned int GetRegister(unsigned int addr)
{
	unsigned int val;
	val = *((unsigned int *)addr);
	return val;
}

#ifdef IMAGE_SCALE_ENABLE
static short lastidx;
void GetScaledLine(unsigned short sidx, unsigned char** ScaleBuf, unsigned char** NextBuf, unsigned char* srcline, unsigned short srcWidth, unsigned short dstWidth, unsigned short xdiv)
{
	unsigned char *tmp;
	if (lastidx == sidx)
		return;
	else if ((lastidx+1) == sidx)
	{
		tmp = *ScaleBuf;
		*ScaleBuf= *NextBuf;
		*NextBuf = tmp;
		ScaleLine(&srcline[srcWidth*(sidx+1)], *NextBuf, srcWidth, dstWidth, xdiv);
	}
	else
	{
		ScaleLine(&srcline[srcWidth*sidx], *ScaleBuf, srcWidth, dstWidth, xdiv);
		ScaleLine(&srcline[srcWidth*(sidx+1)], *NextBuf, srcWidth, dstWidth, xdiv);
	}
	lastidx = sidx;
}

void ScaleImage(unsigned int srcAddr, unsigned int dstAddr, unsigned char blank)
{
	unsigned int cmd;

	SetRegister(HAL_REG_SCALE_SRC_ADDR, srcAddr);
	SetRegister(HAL_REG_SCALE_DST_ADDR, dstAddr);
	cmd = 0xc0000001;
	if (blank)
		cmd |= 0x04;	// CbCr blank 0x80
	SetRegister(HAL_REG_SCALE_CTRL, cmd);	// start scaling
	while ((GetRegister(HAL_REG_SCALE_INT) & 0x01) == 0);	// wait for HW done
	SetRegister(HAL_REG_SCALE_INT, 0x01);	// clear interrupt
}


void SetScaleInfo(unsigned int srcWidth, unsigned int srcHeight, unsigned int dstWidth, unsigned int dstHeight)
{
	unsigned xdiv, ydiv, dispx, dispy, xblank, yblank;
        SetRegister(HAL_REG_SCALE_CTRL, 0xc0000040);    //reset Scaler
	SetRegister(HAL_REG_SCALE_CTRL, 0xc0000040);    //reset Scaler
	SetRegister(HAL_REG_SCALE_SRC_RES, (srcWidth << 16) + srcHeight);
	if ((srcWidth <= dstWidth) && (srcHeight <= dstHeight))
	{
		dispx = ((srcWidth + 1) >> 1) << 1;
		dispy = ((srcHeight + 1) >> 1) << 1;
		xblank = (dstWidth - srcWidth) >> 1;
		yblank = (dstHeight - srcHeight) >> 1;
	}
	else
	{
		dispx = (srcWidth * dstHeight) / srcHeight;
		dispx = ((dispx + 1)  >> 1) << 1;	// must be even
		if (dispx < dstWidth)
		{
			xblank = (dstWidth - dispx) >> 1;
			yblank = 0;
			dispy = dstHeight;
		}
		else
		{
			dispx = dstWidth;
			dispy = (srcHeight * dstWidth) / srcWidth;
			if (dstHeight > dispy)
				dispy = ((dispy + 1) >> 1) << 1;	// must be even
			yblank = (dstHeight - dispy) >> 1;
			xblank = 0;
		}
	}
	dispx = ((dispx + 7) >> 3) << 3;
	xblank = (dstWidth - dispx) >> 1;

	//-- Flowbee 2008-04-17
	dispx = dstWidth;
	xblank = (dstWidth - dispx) >> 1;
	dispy = dstHeight;
	yblank = (dstHeight - dispy) >> 1;

	SetRegister(HAL_REG_SCALE_DST_RES, (dispx << 16) + dispy);
	xdiv = (1<<SCALE_SHIFT)/dispx;
	ydiv = (1<<SCALE_SHIFT)/dispy;
	SetRegister(HAL_REG_SCALE_DST_BLANK, (xblank << 16) + yblank);
	SetRegister(HAL_REG_SCALE_DIV, (xdiv << 16) + ydiv);
//	printf("srcWidth %d srcHeight %d dstWidth %d dstHeight %d\n", srcWidth, srcHeight, dispx, dispy);
}

void SWScaleImage(unsigned int srcAddr, unsigned int dstAddr, unsigned char blank)
{
	unsigned char *srcline, *dstline, *ScaleBuf, *NextBuf;
	unsigned short xblank, yblank, sidx;
	unsigned short srcWidth, srcHeight, dstWidth, dstHeight, xdiv, ydiv;
	unsigned int regValue, i, j;
	int party;

	dstline = (unsigned char*)dstAddr;
	srcline = (unsigned char*)srcAddr;
	regValue = GetRegister(HAL_REG_SCALE_DST_BLANK);
	xblank = regValue >> 16;
	yblank = regValue & 0xFFFF;
	regValue = GetRegister(HAL_REG_SCALE_DIV);
	xdiv = regValue >> 16;
	ydiv = regValue & 0xFFFF;
	regValue = GetRegister(HAL_REG_SCALE_SRC_RES);
	srcWidth = regValue >> 16;
	srcHeight = regValue & 0xFFFF;
	regValue = GetRegister(HAL_REG_SCALE_DST_RES);
	dstWidth = regValue >> 16;
	dstHeight = regValue & 0xFFFF;
	blank = (blank == 0) ? 0x10 : 0x80;	// Y(0x10) or CbCr(0x80)
	if (yblank > 0)
	{
		unsigned short xdim;
		xdim = dstWidth + (xblank << 1);
		memset(dstline, blank, yblank * xdim);
		memset(&dstline[(yblank+dstHeight)*xdim], blank, yblank * xdim);
		dstline += yblank*xdim;
	}
	ScaleBuf = malloc(dstWidth<<1);
	NextBuf = ScaleBuf + dstWidth;
	party = sidx = 0; lastidx = -2;
	for (j=0; j<dstHeight; j++)
	{
		if (xblank > 0)		// fill blank first
		{
			memset(dstline, blank, xblank);
			memset(&dstline[xblank+dstWidth], blank, xblank);
		}
		while (party >= dstHeight)
		{
			sidx ++;
			party -= dstHeight;
		}
		GetScaledLine(sidx, &ScaleBuf, &NextBuf, srcline, srcWidth, dstWidth, xdiv);
		for (i=0; i<dstWidth; i++)
			dstline[i+xblank] = ScaleColor(ScaleBuf[i], NextBuf[i], party, dstHeight, ydiv);
		dstline += (dstWidth + (xblank<<1));
		party += srcHeight;
	}
#if 0
	if (NextBuf < ScaleBuf)
		ScaleBuf = NextBuf;
	free(ScaleBuf);
#endif
}


void ScaleLine(unsigned char *srcline, unsigned char *dstline, unsigned short srcWidth, unsigned short dstWidth, unsigned short xdiv)
{
	unsigned short i, sidx, partx;

	partx = sidx = 0;
	for (i=0; i<dstWidth; i++)
	{
		while (partx >= dstWidth)
		{
			sidx++;
			partx -= dstWidth;
		}
		dstline[i] = ScaleColor(srcline[sidx], srcline[sidx+1], partx, dstWidth, xdiv);
		partx += srcWidth;
	}
}
#endif	// end if IMAGE_SCALE_ENABLE

unsigned int srcwidth, srcheight, srcAddr, yuvLogAddr;

unsigned int DispMemAlloc(unsigned int memsize, unsigned char start)
{
	/* Flowbee@960301 */
	srcAddr = pgu_get_yuv_base(start);;	// start must be an even number
	return (FBBaseAddr + srcAddr);
}

/*
 * The main program.
 */
int hdjpeg(int enablePGU)
{
	halRawImgFmt_E fmt;
	unsigned int YSize=0, UVSize=0;
	unsigned int YAddr, CbAddr, CrAddr, DispAddr;
	unsigned char *DBuf, *VBuf, fold, uvxfold, uvyfold;
	unsigned int x, y, xdim, ydim, uvdim=0;
	unsigned int origwidth, origheight;
	unsigned char* framebuf;
	int i;

	set_config();
	framebuf = (unsigned char *)FBBaseAddr;
	srcAddr = FBBaseAddr;

	//copy decompressed logo picture to framebuffer
	__memcpy( (void *)(srcAddr), (void *)logo_output, jpeg_size);

	/* test jpeg decoder register 10 times for power on testing */
	for (i=0; i<10; i++)
	{
		halJPEGDInit_i();
		if (halJPEGD_GetIntMask() != 0x7)
		{
			force_reset();
		}
	}

	// Decode Phase 1, flush dcache before DecodeP1
	flush_all_dcache();
	fmt = halJPEGDDecodeP1(srcAddr, &srcwidth, &srcheight);
	flush_all_dcache();

	origwidth = srcwidth;
	origheight = srcheight;
	if (fmt == halcRawImgFmt_Unknown)
	{
		//__puts("BUG! can't parse logo!\n");
		return 0;
	}
	else
	{
		fold = 0;
		if ((srcwidth != OUTPUT_XRES) || (srcheight != OUTPUT_YRES))
		{
			while ((srcwidth > (OUTPUT_XRES<<fold)) || (srcheight > (OUTPUT_YRES<<fold)))
				fold++;
			fold = (fold > 0) ? fold-1 : 0;
		}
		uvxfold = uvyfold = fold;	// assume YUV420
		if (fmt == halcRawImgFmt_GrayLevel)
		{
			srcwidth = ((srcwidth + 7) >> 3) << 3;		// srcwidth must be 8 multiple
			srcheight = ((srcheight + 7) >> 3) << 3;	// srcheight must be 8 multiple
			YSize = srcwidth * srcheight;
			UVSize = YSize >> 2;
			YSize = BUF_ALIGN((YSize + DEFAULT_ALIGN_SIZE), DEFAULT_ALIGN_SIZE); 
			UVSize = BUF_ALIGN((UVSize + DEFAULT_ALIGN_SIZE), DEFAULT_ALIGN_SIZE);
			uvdim = 0;
		}
		else if (fmt == halcRawImgFmt_YCbCr420)
		{
			srcwidth = ((srcwidth + 15) >> 4) << 4;		// srcwidth must be 16 multiple
			srcheight = ((srcheight + 15) >> 4) << 4;	// srcheight must be 16 multiple
			YSize = srcwidth * srcheight;
			UVSize = YSize >> 2;
			YSize = BUF_ALIGN((YSize + DEFAULT_ALIGN_SIZE), DEFAULT_ALIGN_SIZE); 
			UVSize = BUF_ALIGN((UVSize + DEFAULT_ALIGN_SIZE), DEFAULT_ALIGN_SIZE);
			uvdim = srcwidth >> 1;
		}
		else if (fmt == halcRawImgFmt_YCbCr422h)
		{
			srcwidth = ((srcwidth + 15) >> 4) << 4;		// srcwidth must be 16 multiple
			srcheight = ((srcheight + 7) >> 3) << 3;	// srcheight must be 8 multiple
			YSize = srcwidth * srcheight;
			UVSize = YSize >> 1;
			YSize = BUF_ALIGN((YSize + DEFAULT_ALIGN_SIZE), DEFAULT_ALIGN_SIZE); 
			UVSize = BUF_ALIGN((UVSize + DEFAULT_ALIGN_SIZE), DEFAULT_ALIGN_SIZE);
			uvyfold++;
			uvdim = srcwidth >> 1;
		}
		else if (fmt == halcRawImgFmt_YCbCr422v)
		{
			srcwidth = ((srcwidth + 7) >> 3) << 3;		// srcwidth must be 8 multiple
			srcheight = ((srcheight + 15) >> 4) << 4;	// srcheight must be 16 multiple
			YSize = srcwidth * srcheight;
			UVSize = YSize >> 1;
			YSize = BUF_ALIGN((YSize + DEFAULT_ALIGN_SIZE), DEFAULT_ALIGN_SIZE); 
			UVSize = BUF_ALIGN((UVSize + DEFAULT_ALIGN_SIZE), DEFAULT_ALIGN_SIZE);
			uvxfold++;
			uvdim = srcwidth;
		}
		else if (fmt == halcRawImgFmt_YCbCr444)
		{
			srcwidth = ((srcwidth + 7) >> 3) << 3;		// srcwidth must be 8 multiple
			srcheight = ((srcheight + 7) >> 3) << 3;	// srcheight must be 8 multiple
			YSize = srcwidth * srcheight;
			UVSize = YSize;
			YSize = BUF_ALIGN((YSize + DEFAULT_ALIGN_SIZE), DEFAULT_ALIGN_SIZE); 
			UVSize = BUF_ALIGN((UVSize + DEFAULT_ALIGN_SIZE), DEFAULT_ALIGN_SIZE);
			uvxfold++; uvyfold++;
			uvdim = srcwidth;
		}
		if ((srcwidth == OUTPUT_XRES) && (srcheight == OUTPUT_YRES) && (fmt == halcRawImgFmt_YCbCr420)) {
			DispAddr = DispMemAlloc(YSize + (UVSize << 1), DispNo);	// direct to display
            __puts("DispMemAlloc[YUV1]\n");
        }
		else {
			DispAddr = DispMemAlloc(YSize + (UVSize << 1), (DispNo + 1));
            __puts("DispMemAlloc[YUV2]\n");
        }
		YAddr = DispAddr;
		CbAddr = YAddr + YSize;
		CrAddr = CbAddr + UVSize;
	}
	if ( (YSize+UVSize*2) > (YUVBase + YUVBufSize))
	{
		//__puts("BUG! Image too large to decode\n");
//		return 0;
	}

	// Decode Phase 2, flush dcache before DecodeP2
	flush_all_dcache();

	if (!halJPEGDDecodeP2(YAddr, CbAddr, CrAddr))
	{
		//__puts("BUG! can't decode\n");
		return 0;
	}

#ifdef IMAGE_SCALE_ENABLE
	if ((srcwidth != OUTPUT_XRES) || (srcheight != OUTPUT_YRES) || (fmt != halcRawImgFmt_YCbCr420))
	{
		unsigned short orig;
		unsigned short goodxdim;

		flush_all_dcache();

		//__puts("Start to scaling...\n");
		// Integer Scale
		xdim = srcwidth >> fold;
		ydim = srcheight >> fold;
		goodxdim = xdim;
		orig = origwidth >> fold;
		if (fold > 0)
		{
			goodxdim = ((xdim + 15) >> 4) << 4;		// 16 boundary
			DBuf = (unsigned char*)DispAddr;
			for (y=0; y<ydim; y++)
				for (x=0; x<goodxdim; x++)
					DBuf[x + (y*goodxdim)] = (x >= orig) ? 0x10 : DBuf[(x<<fold)+(y<<fold)*srcwidth];
		}
		srcwidth = goodxdim;
		srcheight = ydim;
		DBuf = (unsigned char*)(DispAddr + YSize);
		VBuf = DBuf + UVSize;
		if (fmt == halcRawImgFmt_GrayLevel)
		{
			memset(DBuf, 0x80, (srcwidth*srcheight)>>2);
			memset(VBuf, 0x80, (srcwidth*srcheight)>>2);
		}
		else
		{
			unsigned int src, dst;
			goodxdim = goodxdim >> 1;
			xdim = xdim >> 1;
			ydim = ydim >> 1;
			orig = orig >> 1;
			for (y=0, dst=0; y<ydim; y++)
				for (x=0; x<goodxdim; x++)
				{
					src = (x<<uvxfold)+(y<<uvyfold)*uvdim;
					DBuf[dst] = (x >= orig) ? 0x80 : DBuf[src];
					VBuf[dst++] = (x >= orig) ? 0x80 : VBuf[src];
				}
			if ((goodxdim & 7) != 0) 
				HWScale = 0;
		}
		// Scale to fit display
		if (HWScale)
		{
			DispAddr = pgu_get_yuv_base(DispNo) + FBBaseAddr;
		}
		else
		{
			YAddr = DispAddr;	// change to logical address
			CbAddr = YAddr + YSize;
			CrAddr = CbAddr + UVSize;
			DispAddr = FBBaseAddr + YUVBase + (DispNo * YUVBufSize);
		}
//		SetScaleInfo(srcwidth, origheight, OUTPUT_XRES, OUTPUT_YRES);
		SetScaleInfo(srcwidth, srcheight, OUTPUT_XRES, OUTPUT_YRES);
		HWScale ? ScaleImage(YAddr, DispAddr, 0) : SWScaleImage(YAddr, DispAddr, 0);
//		SetScaleInfo(srcwidth>>1, origheight>>1, OUTPUT_XRES>>1, OUTPUT_YRES>>1);
		SetScaleInfo(srcwidth>>1, srcheight>>1, OUTPUT_XRES>>1, OUTPUT_YRES>>1);
		//DispAddr += (OUTPUT_XRES*OUTPUT_YRES);
		DispAddr += YBufSize;
		HWScale ? ScaleImage(CbAddr, DispAddr, 1) : SWScaleImage(CbAddr, DispAddr, 1);
//		SetScaleInfo(srcwidth>>1, origheight>>1, OUTPUT_XRES>>1, OUTPUT_YRES>>1);
		SetScaleInfo(srcwidth>>1, srcheight>>1, OUTPUT_XRES>>1, OUTPUT_YRES>>1);
		//DispAddr += (OUTPUT_XRES*OUTPUT_YRES)>>2;
		DispAddr += UVBufSize;
		HWScale ? ScaleImage(CrAddr, DispAddr, 1) : SWScaleImage(CrAddr, DispAddr, 1);
		//__puts("Scale Done\n");
		flush_all_dcache();
	}	
#endif

	// Color Convertion, Default : RGB
	if (out_color_space == JCS_RGB)
	{
		unsigned short *rgbbuf;
		__puts("JCS_RGB: Change to RGB\n");

		flush_all_dcache();

		rgbbuf = (unsigned short*)&framebuf[OUTPUT_XRES*OUTPUT_YRES*2*DispNo];
		YUVBuf2RGBBuf(&framebuf[pgu_get_yuv_base(DispNo)], rgbbuf, OUTPUT_XRES, OUTPUT_YRES);

		if (enablePGU)
		{
			flush_all_dcache();
			set_rgb_pointer();
			PGU_Enable();
		}
	}
#if 1
	else
	{
		flush_all_dcache();
		set_yuv_pointer();
		PGU_Enable();
	}
#endif

	return 1;
}
