/* ld script to make ARC Linux kernel
 * taken from the armnommu/ version by Martin Mares
 *
 * Copyright ARC International
 *
 */

OUTPUT_ARCH(arc)
ENTRY(stext)

SECTIONS
{
	. = 0x80600000;		/* TEXTBASE, vector table area */

	.vector : {
/*
		_svector = .;
		*(.vector)
*/
	}
	
	. = 0x80600200;          /* config data structure ( 512 bytes + 7168 bytes) */
	
	.skycfg : {
          . = ALIGN(512);
		skycfg_start = .;	
		skyconfig.o (.data.skyconfig)
		skycfg_end = .;
	}
	
	. = 0x80602000;		/* TEXTADDR, _start address */

	.init : {			/* Init code and data		*/
		_stext = .;
		_init_begin = .;
			*(.text.init)
			*(.data.init)
		. = ALIGN(16);
/*		. = ALIGN(128);*/
		_init_end = .;
	}

	/DISCARD/ : {			/* Exit code and data		*/
		*(.text.exit)
		*(.data.exit)
	}


/*	. = ALIGN(1024);*/
	. = ALIGN(128);
	.bss : {
		_bss_start = .;	/* BSS				*/
		*(.bss)
		_bss_end = .;
		*(COMMON)
		_end = . ;
	}
/*	. = ALIGN(1024);*/
	. = ALIGN(128);

	.data : {
		/*
		 * first, the init task union, aligned
		 * to an 8192 byte boundary.
		 */
		*(.init.task)

		/*
		 * then the cacheline aligned data
		 */
		. = ALIGN(32);
		*(.data.cacheline_aligned)

		/*
		 * and the usual data section
		 */


		*(.data)

		CONSTRUCTORS
		_edata = .;
	}

	. = ALIGN(1024);

	_start_cp_text = .;

	.text : {			/* Real text segment		*/
		_text = .;		/* Text and read-only data	*/
		. = ALIGN(32);
			*(.text)
			*(.fixup)
			*(.gnu.warning)
			*(.text.lock)	/* out-of-line lock text */
			*(.rodata)
			*(.rodata.*)
			*(.glue_7)
			*(.glue_7t)
			*(.kstrtab)
		. = ALIGN(16);
		_start___ex_table = .;	/* Exception table		*/
			*(__ex_table)
		_stop___ex_table = .;

		_start___ksymtab = .;	/* Kernel symbol table		*/
			*(__ksymtab)
		_stop___ksymtab = .;

		_start___kallsyms = .; /* All kernel symbols           */
			*(__kallsyms)
		_stop___kallsyms = .;

		*(.got)			/* Global offset table		*/

	    	input_data = .;
		piggy.o (.data.piggy)
		input_data_end = .;

		. = ALIGN(1024);
		audio_start = .;	
		audio.o (.data.audio)
		audio_end = .;

		. = ALIGN(1024);
		ramdisk2_start = .;	
		ramdisk2.o (.data.ramdisk2)
		ramdisk2_end = .;

		. = ALIGN(1024);
		logo_start = .;	
		logo.o (.data.logo)
		logo_end = .;

		. = ALIGN(1024);
		ramd_start = .;	
		ramdisk.o (.data.ramdisk)
		. = ALIGN(512);
		ramd_end = .;

		_etext = .;		/* End of text section		*/
	}

	. = ALIGN(1024);
	end_kernel = .;

	/* Stabs debugging sections.	*/
/* Robert 2007-11-22, remove debugging sections. */
/**
	.stab 0 : { *(.stab) }
	.stabstr 0 : { *(.stabstr) }
	.stab.excl 0 : { *(.stab.excl) }
	.stab.exclstr 0 : { *(.stab.exclstr) }
	.stab.index 0 : { *(.stab.index) }
	.stab.indexstr 0 : { *(.stab.indexstr) }
	.comment 0 : { *(.comment) }
	.debug_aranges 0 : { *(.debug_aranges) }
	.debug_pubnames 0 : { *(.debug_pubnames) }
	.debug_info 0 : { *(.debug_info) }
	.debug_abbrev 0 : { *(.debug_abbrev) }
	.debug_line 0 : { *(.debug_line) }
	.debug_frame 0 : { *(.debug_frame) }
	.debug_str 0 : { *(.debug_str) }
	.debug_loc 0 : { *(.debug_loc) }
	.debug_macinfo 0 : { *(.debug_macinfo) }
**/
}
