#ifndef SKY_8850_I2S_H_
#define SKY_8850_I2S_H_

#define REG_R32(reg)						(*((volatile unsigned int*)(reg)))
#define REG_W32(reg, data)					(*((volatile unsigned int*)(reg)) = (data))

#define I2S_BASE_ADDR						0xc1fc0400
#define I2S_R32(reg)						(REG_R32((I2S_BASE_ADDR+reg)))
#define I2S_W32(reg, val)					(REG_W32((I2S_BASE_ADDR+reg), val))

#define I2S_FIFO_OVERFLOW_MASK				0x200
#define I2S_FIFO_UNDERFLOW_MASK				0x400
#define I2S_FIFO_DEPTH						2048

#define MCD_SHIFT							5
#define MSD_SHIFT							11
#define I2S_MCD_MASK						(0x3f<<MCD_SHIFT)
#define I2S_SCD_MASK						(0x1f<<MSD_SHIFT)
#define I2S_MSCD_MASK						(I2S_MCD_MASK|I2S_SCD_MASK)
#define I2S_GET_MCD							((I2S_R32(PCM_CTL)&I2S_MCD_MASK)>>MCD_SHIFT)
#define I2S_GET_SCD							((I2S_R32(PCM_CTL)&I2S_SCD_MASK)>>MSD_SHIFT)
#define I2S_MAKE_MCD(val)					(val<<MCD_SHIFT)
#define I2S_MAKE_MSD(val)					(val<<MSD_SHIFT)
#define I2S_SET_SAMPLE_RATE(mcd, scd)		(I2S_W32(PCM_CTL, \
											((I2S_R32(PCM_CTL)&~I2S_MSCD_MASK)|\
											(I2S_MAKE_MCD(mcd)|I2S_MAKE_MSD(scd)))))
#define I2S_SCLK_INV_ON						I2S_W32(PCM_CTL, I2S_R32(PCM_CTL) | 0x8)
#define I2S_SCLK_INV_OFF					I2S_W32(PCM_CTL, I2S_R32(PCM_CTL) & 0xffffff7)

#define I2S_IF_ENABLE						I2S_W32(PCM_CTL, I2S_R32(PCM_CTL) | 0x1)
#define I2S_IF_DISABLE						I2S_W32(PCM_CTL, I2S_R32(PCM_CTL) & 0xfffffffe)
#define I2S_WORD_LEN(val)					I2S_W32(PCM_CTL, (I2S_R32(PCM_CTL)&0xfff8ffff)|(val<<16))
#define WORD_LEN_16							0x0
#define WORD_LEN_24							0x1
#define WORD_LEN_32							0x2

#define I2S_INT_CLEAR						I2S_W32(PCM_FIFO_INT, 0x0)
#define I2S_DATA_INT_ON						I2S_W32(PCM_FIFO_INT, I2S_R32(PCM_FIFO_INT) | 0x100)
#define I2S_DATA_INT_OFF					I2S_W32(PCM_FIFO_INT, I2S_R32(PCM_FIFO_INT) & 0xfffff8ff)
#define I2S_DATA_TRAN_ON					I2S_W32(PCM_CTL, I2S_R32(PCM_CTL) | 0x4)
#define I2S_DATA_TRAN_OFF					I2S_W32(PCM_CTL, I2S_R32(PCM_CTL) & 0xfffffffb)

#define I2S_OVERFLOW						((I2S_R32(PCM_FIFO_STATUS) & I2S_FIFO_OVERFLOW_MASK)>>9)
#define I2S_OVERFLOW_CLEAR					(I2S_W32(PCM_FIFO_STATUS, I2S_R32(PCM_FIFO_STATUS))
#define I2S_UNDERFLOW						((I2S_R32(PCM_FIFO_STATUS) & I2S_FIFO_UNDERFLOW_MASK)>>10)
#define I2S_UNDERFLOW_CLEAR 				(I2S_W32(PCM_FIFO_STATUS, I2S_R32(PCM_FIFO_STATUS)))
#define I2S_FIFO_WRITE_P					((I2S_R32(PCM_FIFO_POINT)>>14) & 0x1ff)
#define I2S_FIFO_READ_P						((I2S_R32(PCM_FIFO_POINT)>>23) & 0x1ff)
#define I2S_FIFO_AVAIL						(I2S_FIFO_DEPTH - I2S_R32(PCM_FIFO_AVAIL)&0xffff)

#define I2S_8000HZ_MCD						0x18	// 49152000/64/4/24 = 8000
#define I2S_8000HZ_SCD						0x4
#define I2S_11025HZ_MCD						0x10	// 45185700/16/4/16 = 11031.6650390625
#define I2S_11025HZ_SCD						0x4
#define I2S_12000HZ_MCD						0x10	// 49152000/64/4/16 = 12000
#define I2S_12000HZ_SCD						0x4
#define I2S_16000HZ_MCD						0xC		// 49152000/64/4/12 = 16000
#define I2S_16000HZ_SCD						0x4
#define I2S_22050HZ_MCD						0x8		// 45185700/64/4/8 = 22063.330078125
#define I2S_22050HZ_SCD						0x4
#define I2S_24000HZ_MCD						0x8		// 49152000/64/4/8 = 24000
#define I2S_24000HZ_SCD						0x4
#define I2S_32000HZ_MCD						0x6		// 49152000/64/4/6 = 32000
#define I2S_32000HZ_SCD						0x4
#define I2S_44100HZ_MCD						0x4		// 45185700/64/4/4 = 44126.66015625
#define I2S_44100HZ_SCD						0x4
#define I2S_48000HZ_MCD						0x4		// 49152000/64/4/4 = 48000
#define I2S_48000HZ_SCD						0x4	
#define I2S_88200HZ_MCD						0x2		// 45185700/64/4/2 = 88253.3203125
#define I2S_88200HZ_SCD						0x4			
#define I2S_96000HZ_MCD						0x2		// 49152000/64/4/2 = 96000
#define I2S_96000HZ_SCD						0x4

#define AUDIO_PLL_ADDR						0xc0fc183c
#define AUDIO_PLL_UNLOCK
#define AUDIO_PLL_SET(data)					REG_W32(AUDIO_PLL_ADDR, data)
#define AUDIO_PLL_49MHZ						0x80030056	//148.5MHz 	// 49.155972 MHz
#define AUDIO_PLL_45MHZ						0x80030495	//148.5MHz 	// 45.186538 MHz

typedef enum _I2S_REGS{
	PCM_CTL = 0x00,
	PCM_READ_PORT = 0x04,
	PCM_WRITE_PORT = 0x08,
	PCM_FIFO_STATUS = 0x0C,
	PCM_FIFO_INT = 0x10,
	PCM_FIFO_POINT = 0x14,
	PCM_FIFO_AVAIL = 0x18,
}I2S_REGS;

#endif /*2850_I2S_H_*/
