#ifndef SKY_IR_H
#define SKY_IR_H

#define PFX "SKY_IR"
#define info(format, arg...) printk(KERN_INFO PFX ": " format "\n" , ## arg)

#define SKY_IR_DEBUG 1

#ifdef SKY_IR_DEBUG
#define dbg(fn, format, arg...) printk(KERN_DEBUG "%s : " format "\n" , fn, arg)
#define DBG_TRACE(arg...) printk(arg)
#else
#define dbg(format, arg...) do {} while (0)
#define DBG_TRACE(arg...)
#endif

#define IRBaseIoAddress 0xc1fc2004
#define IRBaseIoAddress_len 32

enum IR_Register{
	IR3_CON = 0x0,
	IR3_CAN = 0x4,
	IR3_CBN = 0x8,
	IR3_CCN = 0xC,
	IR3_CD1 = 0x10,
	IR3_CD2 = 0x14,
	IR3_CD3 = 0x18,
	IR3_CD4 = 0x1C
};

void *ir_mem;


#define IR_W32(reg, val32)	writel(val32, ir_mem+(reg))
#define IR_R32(reg)		readl(ir_mem+(reg))


#define IR_INT_MASK			0x00000004
#define IR_STATUS_MASK		0x00000002
#define IR_MODE_MASK		0x00000001

#define IR_INT_ENABLE		IR_W32(IR3_CON, IR_R32(IR3_CON) | IR_INT_MASK)
#define IR_INT_DISABLE	IR_W32(IR3_CON, IR_R32(IR3_CON) & ~IR_INT_MASK)
#define IR_INT_RESET		IR_W32(IR3_CON, IR_R32(IR3_CON) | IR_STATUS_MASK )

#define IR3CON			IR_R32(IR3_CON)
#define IR3CD1			IR_R32(IR3_CD1)
#define IR3CD2			IR_R32(IR3_CD2)
#define IR3CD3			IR_R32(IR3_CD3)
#define IR3CD4			IR_R32(IR3_CD4)

#define IR_INT_NUM	18

#define IR_CAN_INIT		IR_W32(IR3_CAN, 0x4d28)
#define IR_CBN_INIT		IR_W32(IR3_CBN, 0x67)
#define IR_CCN_INIT		IR_W32(IR3_CCN, 0x10)

int ir_init(void);
void ir_close(void);

#endif