/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

  /* arch/arcnommu/kernel/ptrace.c */

#include <linux/kernel.h>
#include <linux/ptrace.h>
#include <linux/sched.h>
#include <linux/user.h>
#include <linux/mm.h>
#include <linux/sys.h> /* For NR_syscalls */

#include <asm/uaccess.h>
#include <asm/arcdefs.h>


/* OFFSET(x,y) is the offset of y in x */
#define OFFSET(x,y) ( &((x *)0)->y )

//#define PTRACE_DEBUG 1

void ptrace_disable(struct task_struct *child)
{
  /* DUMMY: dummy function to resolve undefined references */
}


unsigned long
getreg(struct pt_regs *moreregs, struct callee_regs *childregs, unsigned int offset,struct task_struct *child)
{
	if(offset < sizeof(*moreregs))
		return *(unsigned long *)(((char *)moreregs) + offset);


	if((offset-sizeof(*moreregs)) < sizeof(*childregs) )
		return *(unsigned long *)(((char *)childregs) + offset - sizeof(*moreregs));
	
	if(offset ==(int) OFFSET(struct user_regs_struct,stop_pc))
	{
		/* orig_r8 contains either syscall num or NR_syscalls+1 incase of an exception */
		if(moreregs->orig_r8 > NR_syscalls + 1)
		{
#ifdef PTRACE_DEBUG
			printk ("Reading stop_pc(efa) %08lx\n",child->thread.fault_address);
#endif
			return child->thread.fault_address;
		}
		/* orig_r8 -1 for int1 and -2 for int2 and NR_SYSCALLS + 1
		   is for a non-trap_s exception*/
		else
		{
#ifdef PTRACE_DEBUG
		        printk ("Reading stop_pc(ret) %08lx\n",moreregs->ret);	
#endif
		  return moreregs->ret;
		}
	}
	if(offset ==(int) OFFSET(struct user_regs_struct,efa))
	  {
#ifdef PTRACE_DEBUG
		printk ("Reading efa %08lx\n",child->thread.fault_address);
#endif		
		return child->thread.fault_address;
	  }
		
	return 0;			     /* can't happen */
}


void
setreg(struct pt_regs *moreregs, struct callee_regs *childregs, int offset, int data)
{
	if(offset ==(int) OFFSET(struct user_regs_struct, reserved1) ||
	   offset == (int) OFFSET(struct user_regs_struct, reserved2) || offset == (int) OFFSET(struct user_regs_struct,efa))
		return;
	
	if(offset ==(int) OFFSET(struct user_regs_struct,stop_pc))
	{
#ifdef PTRACE_DEBUG
	        printk("[setreg]: writing to stop_pc(ret) %08lx\n",data);
#endif
		moreregs->ret = data;
		return ;
	}

	if(offset < sizeof(*moreregs))
	{
		*(unsigned long *)(((char *)moreregs) + offset) = data;
		return;
	}

	offset -= sizeof(*moreregs);

	if(offset < sizeof(*childregs))
		*(unsigned long *)(((char *)childregs) + offset) = data;

	return;	

	/* cant happen */
}


asmlinkage int sys_ptrace(long request, long pid, long addr, long data)
{
	struct task_struct *child;
	int ret;
	/*lock_kernel();*/        /* for SMP */
#ifdef PTRACE_DEBUG
	printk("---- ptrace(%ld,%ld,%lx,%lx) ----\n", request, pid, addr, data);
#endif
	ret = -EPERM;
	if (request == PTRACE_TRACEME) {
		/* are we already being traced? */
		if (current->ptrace & PT_PTRACED)
			goto out;
		/* set the ptrace bit in the process flags. */
		current->ptrace |= PT_PTRACED;
		ret = 0;
		goto out;
	}
	ret = -ESRCH;
	read_lock(&tasklist_lock);
	child = find_task_by_pid(pid);
	if (child)
		get_task_struct(child);
	read_unlock(&tasklist_lock);
	if (!child)
		goto out;

	ret = -EPERM;
	if (pid == 1)		/* you may not mess with init */
		goto out_tsk;

	/* Must do this before ptrace_check_attach() */
	if (request == PTRACE_ATTACH) {
		ret = ptrace_attach(child);
		goto out_tsk;
	}

	ret = ptrace_check_attach(child, request == PTRACE_KILL);
	if (ret < 0)
		goto out_tsk;
	
	switch (request) {
	case PTRACE_PEEKTEXT: /* read word at location addr. */
	case PTRACE_PEEKDATA: {
		unsigned long tmp;
		int copied;

		copied = access_process_vm(child, addr, &tmp, sizeof(tmp), 0);
		ret = -EIO;
		if (copied != sizeof(tmp))
			break;
		ret = put_user(tmp,(unsigned long *) data);
		break;
		}

	case PTRACE_POKETEXT: /* write the word at location addr. */
	case PTRACE_POKEDATA:
		ret = 0;
		if (access_process_vm(child, addr, &data, sizeof(data), 1) == sizeof(data))
			break;
		ret = -EIO;
		break;

	case PTRACE_SYSCALL:
	case PTRACE_CONT: { /* restart after signal. */
		ret = -EIO;
		if ((unsigned long) data > _NSIG)
			break;
		if (request == PTRACE_SYSCALL)
			child->ptrace |= PT_TRACESYS;
		else
			child->ptrace &= ~PT_TRACESYS;
		child->exit_code = data;
		wake_up_process(child);
		ret = 0;
		break;
		}

	case PTRACE_GETREGS: {
		struct pt_regs *moreregs;
		struct callee_regs *childregs;
		int i;
		unsigned long tmp;
		moreregs = (struct pt_regs *)child->thread.usp;
		childregs = (struct callee_regs *)child->thread.pc;
		
		for(i=0; i< sizeof(struct user_regs_struct);i++)
		{
			/* getreg wants a byte offset */
			tmp = getreg(moreregs,childregs, i << 2,child);
			ret = put_user(tmp,((unsigned long *)data) + i);
			if(ret < 0)
				goto out_tsk;
		}
		break;
	        }
		
	case PTRACE_SETREGS: {
		struct callee_regs *childregs;
		struct pt_regs *moreregs;
		int i;

		moreregs = (struct pt_regs *)child->thread.usp;
		childregs = (struct callee_regs *)child->thread.pc;

		for(i=0; i< sizeof(struct user_regs_struct)/4; i++)
		{
			unsigned long tmp;
			ret = get_user(tmp, (((unsigned long *)data) + i));
			if(ret < 0)
				goto out_tsk;
			setreg(moreregs, childregs, i << 2, tmp);
		}
		
		ret = 0;
		break;
	        }


		/*
		 * make the child exit.  Best I can do is send it a sigkill. 
		 * perhaps it should be put in the status that it wants to 
		 * exit.
		 */
	case PTRACE_KILL: {
		ret = 0;
		if (child->state == TASK_ZOMBIE)	/* already dead */
			break;
		child->exit_code = SIGKILL;
		wake_up_process(child);
		break;
	        }

		
	case PTRACE_DETACH:
		/* detach a process that was attached. */
		ret = ptrace_detach(child, data);
		break;


	case PTRACE_SETOPTIONS:
		if (data & PTRACE_O_TRACESYSGOOD)
			child->ptrace |= PT_TRACESYSGOOD;
		else
			child->ptrace &= ~PT_TRACESYSGOOD;
		ret = 0;
		break;
	        

		
	case PTRACE_PEEKUSR: {
		struct pt_regs *moreregs;
		struct callee_regs *childregs;
		int tmp;
		
		moreregs = (struct pt_regs *)child->thread.usp;
		childregs = (struct callee_regs *)child->thread.pc;
		
		/* user regs */
		if(addr > (unsigned)OFFSET(struct user,regs) &&
		   addr < (unsigned)OFFSET(struct user,regs) + sizeof(struct user_regs_struct))
		{
			
#ifdef PTRACE_DEBUG
			printk ("break point addr %08lx\n",child->thread.fault_address);
#endif
			tmp = getreg(moreregs, childregs, addr,child);
			ret = put_user(tmp, ((unsigned long *)data));
		}
		/* signal */
		else if(addr == (unsigned)OFFSET(struct user, signal))
		{
			tmp = current->exit_code; /* set by syscall_trace */
			ret = put_user(tmp, ((unsigned long *)data));
		}
		
		/* nothing else is interesting yet*/
		else if(addr > 0 && addr < sizeof(struct user))
			return -EIO;
		/* out of bounds */
		else
			return -EIO;

		break;
		}


	case PTRACE_POKEUSR: {
		struct pt_regs *moreregs;
		struct callee_regs *childregs;
		
		moreregs = (struct pt_regs *)child->thread.usp;
		childregs = (struct callee_regs *)child->thread.pc;

		ret = 0;
		if(addr ==(int) OFFSET(struct user_regs_struct,efa))
			return -EIO;
	
		if(addr > (unsigned)OFFSET(struct user,regs) &&
		   addr < (unsigned)OFFSET(struct user,regs) + sizeof(struct user_regs_struct))
		{
			setreg(moreregs, childregs, addr, data);
		}
		else
			return -EIO;

		break;
		}
		
	
		/*
		 * Single step not supported.  ARC's single step bit won't
		 * work for us.  It is in the DEBUG register, which the RTI
		 * instruction does not restore.
		 */
	case PTRACE_SINGLESTEP:	
	default:
		ret = -EIO;
		break;
	}
out_tsk:
	free_task_struct(child);
out:
	/*unlock_kernel();*/      /* for SMP */
	return ret;
}



asmlinkage void syscall_trace(void)
{
	if ((current->ptrace & (PT_PTRACED|PT_TRACESYS)) !=
			(PT_PTRACED|PT_TRACESYS))
		return;
	/* the 0x80 provides a way for the tracing parent to distinguish
	   between a syscall stop and SIGTRAP delivery */
	current->exit_code = SIGTRAP | ((current->ptrace & PT_TRACESYSGOOD)
					? 0x80 : 0);
	current->state = TASK_STOPPED;
	notify_parent(current, SIGCHLD);
	schedule();
	/*
	 * this isn't the same as continuing with a signal, but it will do
	 * for normal use.  strace only continues with a signal if the
	 * stopping signal is not SIGTRAP.  -brl
	 */
	if (current->exit_code) {
		send_sig(current->exit_code, current, 1);
		current->exit_code = 0;
	}
}
