/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) ARC International
 * Authors: Rahul Trivedi
 */

#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/bootmem.h>
#ifdef CONFIG_BLOCK_DEV_RAM
#include <linux/blk.h>
#endif

#include <asm/page.h>
#include <asm/pgalloc.h>

pgd_t swapper_pg_dir[PTRS_PER_PGD]__attribute__((aligned(PAGE_SIZE)));
char empty_zero_page[PAGE_SIZE]__attribute__((aligned(PAGE_SIZE)));

struct pgtable_cache_struct quicklists;

int do_check_pgt_cache(int low, int high)
{
	int freed = 0;
 
	if(pgtable_cache_size > high) {
		do {
			if(pgd_quicklist) {
				free_pgd_slow(get_pgd_fast());
				freed++;
			}
			if(pmd_quicklist) {
				pmd_free_slow(pmd_alloc_one_fast(NULL, 0));
				freed++;
			}
			if(pte_quicklist) {
				pte_free_slow(pte_alloc_one_fast(NULL, 0));
				freed++;
			}
		} while(pgtable_cache_size > low);
	}
	return freed;
}

static unsigned long totalram_pages = 0;
extern char _etext, _text, _edata, _end, _init_end, _init_begin;


void __init pagetable_init(void) 
{
	pgd_init((unsigned long)swapper_pg_dir);
	pgd_init((unsigned long)swapper_pg_dir +
			 sizeof(pgd_t) * USER_PTRS_PER_PGD);
}

void paging_init(void)
{
	unsigned long zones_size[MAX_NR_ZONES] = {0, 0, 0};

	pagetable_init();

#if 0	
	zones_size[ZONE_DMA] = max_low_pfn;

	free_area_init(zones_size);
#else
	zones_size[ZONE_DMA] = ((unsigned long)(END_MEM) - PHYS_SRAM_OFFSET) >> PAGE_SHIFT;
	free_area_init_node(0, 0, 0, zones_size, PHYS_SRAM_OFFSET, 0);
#endif
} 

void mem_init(void)
{
	int codesize, datasize, initsize, reserved_pages;
	int tmp;
  
	high_memory = (void *) __va(((max_low_pfn - min_low_pfn) * PAGE_SIZE) + PHYS_SRAM_OFFSET);
	max_mapnr = num_mappedpages = num_physpages = max_low_pfn - min_low_pfn;
  
	totalram_pages = free_all_bootmem();
  
	reserved_pages = 0;
	for(tmp = 0 ; tmp < num_physpages ; tmp++)
		if(PageReserved(mem_map + tmp))
			reserved_pages++;

	codesize = (unsigned long)&_etext - (unsigned long)&_text;
	datasize = (unsigned long)&_end - (unsigned long)&_etext;
	initsize = (unsigned long)&_init_end - (unsigned long)&_init_begin; 

	printk(KERN_NOTICE "Memory: %luKB available (%dK code,%dK data, %dK init)\n",
		   (unsigned long) nr_free_pages() << (PAGE_SHIFT-10),
		   codesize >> 10, datasize >> 10, initsize >> 10);
}

// PS 21.12.2004
void show_mem(void)
{
  printk(KERN_NOTICE "Show_mem not implemented\n");
}
// end PS

void free_initmem(void)
{
	extern char _init_begin, _init_end;
	unsigned long addr;

	addr = (unsigned long)(&_init_begin);
	for(; addr < (unsigned long)(&_init_end) ; addr += PAGE_SIZE) {
		ClearPageReserved(virt_to_page(addr));
		set_page_count(virt_to_page(addr), 1);
		free_page(addr);
		totalram_pages++;
	}
	printk(KERN_INFO "Freeing unused kernel memory: %luk freed\n", 
					(&_init_end - &_init_begin) >> 10);
}

#ifdef CONFIG_BLK_DEV_INITRD
void free_initrd_mem(unsigned long start, unsigned long end)
{
	
	int pages = 0;
	for (; start < end; start += PAGE_SIZE) {
		ClearPageReserved(virt_to_page(start));
		set_page_count(virt_to_page(start), 1);
		free_page(start);
		totalram_pages++;
		pages++;
	}
	printk("Freeing initrd memory: %luk freed\n", (pages * PAGE_SIZE) >> 10);

}
#endif

void si_meminfo(struct sysinfo *val)
{
	val->totalram  = totalram_pages;
	val->sharedram = 0;
	val->freeram   = nr_free_pages();
	val->bufferram = atomic_read(&buffermem_pages);
	val->totalhigh = 0;
	val->freehigh  = 0;
	val->mem_unit  = PAGE_SIZE;
}
