/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 * arch/arc/kernel/entry.S
 *
 * Copyright (C)
 *
 * This program is free software you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Author(s) : Amit Bhor, Rahul Trivedi
 */


/*
      ------------------------------------------------------------------
      Function                            OLD GCC ABI     Metaware ABI
      ------------------------------------------------------------------
      Arguments                           r0 - r7         r0 - r7
   *  Caller Saved Registers              r0 - r15        r0 - r12
                                          r24, r25
   *  Callee Saved Registers              r16 - r23       r13- r25
   *  Stack chain pointer                 r24             r11
   *  Register used for temporary         r25             r12
      calculation               
      Global Pointer (gp)                 r26             r26
      Frame Pointer (fp)                  r27             r27
      Stack Pointer (sp)                  r28             r28
      Interrupt link register (ilink1)    r29             r29
      Interrupt link register (ilink2)    r30             r30
      Branch link register (blink)        r31             r31
      ------------------------------------------------------------------
GCC for A5, ARC600 and ARC700 now confirms to the Metware High C ABI

*/
	.cpu A7
	
#include <linux/config.h>

	.section .vector, "ax",@progbits
	.align 4

/* Each entry in the vector table must occupy 2 words. Since it is a jump
 * across sections (.vector to .text) we are gauranteed that 'j somewhere'
 * will use the 'j limm' form of the intrsuction as long as somewhere is in
 * a section other than .vector. 
 */
	
j	res_service		; 0x0,	restart vector
j	mem_service		; 0x8,	memory exception
j	instr_service		; 0x10,	instruction error
.rept	29			
j	handle_interrupt	; top level interrupt handler
.endr
j	EV_MachineCheck		; 0x100, fatal Machine check exception
j	EV_TLBMissI		; 0x108, Intruction TLB miss
j	EV_TLBMissD		; 0x110, Data TLB miss
j	EV_TLBProtV		; 0x118, Protection Violation
j	EV_PrivilegeV		; 0x120, Privilege Violation
j	EV_Trap			; 0x128, Trap exception
j	EV_Extension		; 0x130, Extension intruction exception
.rept	24
j	reserved		; Reserved Exceptions
.endr


#include <linux/sys.h>		 /* contains the declaration of NR_syscalls */
#include <linux/linkage.h>	 /* contains declarations of SYMBOL_NAME, etc */
#include <asm/entry.h>		 /* For the SAVE_* macros */
#include <asm/ptrace.h>		 /* For the PT_* offsets */
#include <asm/unistd.h>		 /* For SYSCALL_IRQ  */
#include <asm/errno.h>		 
#include <asm/system.h>
#include <asm/arcregs.h>
#include <asm/arcdefs.h>

; the bit of ECR which should be set if EV_TRAP was caused beacuse a break point was hit
#define TRAP_S_BRKPT	1	

/* Simulator fails to disable interrupts in exceptions */
#ifdef	CONFIG_ARC700_SIM_RUN	
//#define SIM_EXCEPTION_BUG
#endif

; the ENTRY macro in linux/linkage.h does not work for us ';' is treated as a comment
; and I could not find a way to put a new line in a #define

.macro ARC_ENTRY name
  .globl SYMBOL_NAME(\name) 
  .align 4		
  SYMBOL_NAME_LABEL(\name)
.endm  
  
#ifdef CONFIG_ARC_TRACE_UNIT

#ifndef CONFIG_ARC_TRACE_DEPTH
#define CONFIG_ARC_TRACE_DEPTH 16
#endif

.macro STORE_TRACE_INDEX index
	/* get entry defined by index */
	mov %r0,0x00000000 | (\index << 2) /* source */
	sr %r0,[0xad] 
	lr %r0,[0xae]
	st %r0,[%r1]
	add %r1,%r1,4
	mov %r0,0x00000001 | (\index << 2) /* dest */
	sr %r0,[0xad] 
	lr %r0,[0xae]
	st %r0,[%r1]
	add %r1,%r1,4
	mov %r0,0x00000002 | (\index << 2) /* other */
	sr %r0,[0xad] 
	lr %r0,[0xae]
	st %r0,[%r1]
	add %r1,%r1,4
.endm

.macro FILL_TRACE from=0,to=CONFIG_ARC_TRACE_DEPTH
	STORE_TRACE_INDEX(\from)
.if \to-\from
	FILL_TRACE "(\from+1)",\to
.endif
.endm

.macro STORE_TRACE 
	sub %r0,%r0,%r0
	sub %r1,%r1,%r1
	mov %r1,trace_buffer
	FILL_TRACE
.endm
#endif


; ---------------------------------------------
	.section .text, "ax",@progbits

SYMBOL_NAME_LABEL(res_service)	; processor restart
	flag	0x1		; not implemented
	nop
	nop

SYMBOL_NAME_LABEL(reserved)	; processor restart
		rtie       	; jump to processor initializations
	
ARC_ENTRY handle_interrupt
	st	r9, [SYMBOL_NAME(int1_saved_reg)] ;  save r9
	st	r9, [SYMBOL_NAME(int2_saved_reg)] 
	lr	r9, [status32]
	and.f	0, r9, STATUS_A1_MASK ;  find out what level interrupt occurred
	bnz	handle_interrupt_level1	;  level 1 interrupt
	; fall through for a level 2 interrupt
	
SYMBOL_NAME_LABEL(handle_interrupt_level2)
	lr	r9, [status32_l2]
#if 0
	and.f	0, r9, STATUS_AE_MASK
	bz	1f
	flag	0x1	; Interrupt taken in exception, insane !!! halt
	nop
	nop
1:
#endif
	; find out in which mode was the system running when the interrupt 
	; occurred
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	bz	1f		
	; interrupt occurred in user mode hence load sp kernel stack pointer 
#if 1 //wschen 2008-02-15
2:
#endif

	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, (0x2000 - 4)
1:
#if 1 //wschen 2008-02-15
	and.f	0, sp, 0x80000000
	jz	2b
#endif

	SAVE_ALL_INT2

	mov	r8, SYMBOL_NAME(irq_stat)	 
	ld	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]
	add	r9, r9, 1
	st	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]

	lr	r0, [icause2]
	and	r0, r0, 0x1f
	
	mov	r1, sp

	bl	SYMBOL_NAME(process_interrupt)

	b	ret_from_interrupt
	
	
ARC_ENTRY handle_interrupt_level1
	lr	r9, [status32_l1]
#if 0
	and.f	0, r9, STATUS_AE_MASK
	bz	1f
	flag	0x1	; Interrupt taken in exception, insane !!! halt
	nop
	nop
1:
#endif
	; find out in which mode was the system running when the interrupt 
	; occurred
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	bz	1f		
	; interrupt occurred in user mode hence load sp kernel stack pointer 
#if 1 //wschen 2008-02-15
2:
#endif

	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, (0x2000 - 4)
1:
#if 1 //wschen 2008-02-15
	and.f	0, sp, 0x80000000
	jz	2b
#endif

	SAVE_ALL_INT1

	mov	r8, SYMBOL_NAME(irq_stat)	 
#if 1 //wschen 2008-02-29
	; We want the increment operation over here to be atomic
	cli_level_12 r9
#endif

	ld	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]
	add	r9, r9, 1
	st	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]

#if 1 //Robert 2008-05-22
	sti_level_2	r9
#endif

	lr	r0, [icause1]
	and	r0, r0, 0x1f
		
	mov	r1, sp

	bl	SYMBOL_NAME(process_interrupt)

	b	ret_from_interrupt

/* Instruction Error Exception Handler */	
ARC_ENTRY instr_service
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
1:
	SAVE_ALL_SYS

#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif

#if 1 //Robert 2008-05-22 Apply PATCH from ARC
	lr	r0, [ecr]    /* load ecr in order to get the cause code and parameter */
	lr	r1, [efa]       /* faulting address */
#endif
; We dont want exceptions to be disabled while the fault is handled
; Now that we have saved the context we return from exception hence re-enable
; exceptions and remain in kernel mode. Interrupts remain disabled.
; Also this is required because the current process is going die, and this status32 will be inherited by the next process
; that will be scheduled.
	ld	r9, [sp, PT_status32]	;read erstatus so we return to the same state of interrupt enables.  
	; set mode we are returning to as kernel mode
	and	r9, r9, ~(STATUS_U_MASK|STATUS_L_MASK|STATUS_DE_MASK)
	sr	r9, [erstatus]
	mov	r9, 2f
	sr	r9, [eret]
	rtie
2:
#if 0 //Robert 2008-05-22
;	lr	r0, [ecr]    /* load ecr in order to get the cause code and parameter */
;	lr	r1, [efa]       /* faulting address */
#endif
	mov	r2, sp 
	bl	instruction_error
	j	ret_from_exception

/* Memory Error Exception Handler */	
ARC_ENTRY mem_service
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
1:
	SAVE_ALL_SYS

#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif

	lr	r0, [ecr]    /* load ecr in order to get the cause code and parameter */
	lr	r1, [efa]       /* faulting address */
	mov	r2, sp 
	bl	memory_error
	j	ret_from_exception

/* Protection Violation Exception Handler */	
ARC_ENTRY EV_MachineCheck
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
	
1:
	SAVE_ALL_SYS
#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif


	lr	r0, [ecr]    /* load ecr in order to get the cause code and parameter */
	lr	r1, [efa]       /* faulting address */
	mov	r2, sp 
#if 1	//ARC Fix 20071114
#if 0 //wschen 2008-02-12
        and.f   0, r0, 0x200100 /* Is this an overlapping TLB ? */
        bz      not_tlb_overlap
        bl      tlb_overlap
        j       ret_from_exception

not_tlb_overlap:
	bl	machine_check

#else	//Robert 2008-05-21, ARC Fix 20080521
        cmp     r0, 0x200100 /* Is this an overlapping TLB ? */
        bne     not_tlb_overlap
        bl      tlb_overlap
        j       ret_from_exception

not_tlb_overlap:
/* FIXME: Maybe a hack, to solve bug 1384 */
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	bnz	check_hardware_bug
	bl	machine_check

; Check if an access to above 0x8000000 in user-mode caused the MC
; If so, do not panic
check_hardware_bug:
	and.f	0, r1, 0x80000000
        bnz     ignore_hw_bug
        bl      machine_check
        j       ret_from_exception
;	bz	machine_check

ignore_hw_bug:

	RESTORE_ALL_SYS
;	flag 	0x1		
; fall through to TLBProtV
#endif
#else
	bl	machine_check
	j	ret_from_exception
#endif

//Robert 2008-05-22 Apply new patch from ARC
/* Protection Violation Exception Handler */	
ARC_ENTRY EV_TLBProtV
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
#ifdef SIM_EXCEPTION_BUG
	cli_level_1	r9
#endif

    ; -----(1) Pre-Excp Mode (User/Kernel)? --------
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
;#if 1 //wschen 2008-02-19
;5:
;#endif
    ; -----(2) Switch from user to kernel stack -----

	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
#if 0 //Robert 2008-05-22
;#if 1 //wschen 2008-02-19
;	and.f	0, sp, 0x80000000
;	jz	5b
;#endif
#endif
1:
	SAVE_ALL_SYS
#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif

    ;---------(3b) Save some more regs-----------------
    ;  vineetg: Mar 6th: Random Seg Fault issue #1
    ;  ecr and efa were not saved in case an Intr sneaks in 
    ;  after fake rtie
    ;
    lr  r4, [ecr]
    lr  r5, [efa]

    ;TAKE_SNAP_ASM r8, r9, SNAP_EXCP_IN

    ; --------(4) Return from CPU Exception Mode --------- 
    ;  Fake a rtie, but rtie to next label 
    ;  so do_page_fault ( ) executes in pure kernel 
    ;  mode with further Exceptions enabled

; We dont want exceptions to be disabled while the fault is handled
; Now that we have saved the context we return from exception hence re-enable
; exceptions and remain in kernel mode. Interrupts remain disabled.
	ld	r9, [sp, PT_status32]	;read erstatus so we return to the same state of interrupt enables.  
	; set mode we are returning to as kernel mode
#if 0 //wschen 2008-02-25
	and	r9, r9, ~(STATUS_U_MASK|STATUS_L_MASK|STATUS_DE_MASK)
#else
	and	r9, r9, ~(STATUS_U_MASK | STATUS_E1_MASK | STATUS_E2_MASK | STATUS_DE_MASK | STATUS_L_MASK)
#endif

	sr	r9, [erstatus]
	mov	r9, 2f
	sr	r9, [eret]
    ;---------------------------------------------------
    ;  vineetg: Mar 6th: Random Seg Fault issue #2
    ;  IRQ were **NOT DISABLED** before calling rtie 

	rtie

    ;------ (5) Type of Protection Violation? ----------
2:
;	lr	r0, [ecr]	/* load ecr in order to get the cause code and parameter */
	cmp	r0, 0x230400	/* Misaligned data acces ?*/
;	bne	3f
	beq	4f

;	lr	r1, [efa]
    ;========= (6a) Protection Violation Processing ========
	mov	r1, 0x0
	breq r4, 0x230100, 3f	; if LD exception ? write = 0
	mov	r1, 0x1		        ; else write = 1

3:	
    mov	r2, r5              ; faulting address
	mov	r0, sp              ; pt_regs
	bl	do_page_fault
	j	ret_from_exception

    ;========== (6b) Non aligned access ============
4:
    mov r0, r4              ; cause code
    mov	r1, r5              ; faulting address
	mov	r2, sp              ; pt_regs
	bl	protection_fault
	j	ret_from_exception

/* Privilege Violation Exception Handler */	
ARC_ENTRY EV_PrivilegeV
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
#ifdef SIM_EXCEPTION_BUG
	cli_level_1	r9
#endif
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
	
1:
	SAVE_ALL_SYS
#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif
	lr	r0, [ecr] 
	lr	r1, [efa]
	mov	r2, sp 

; We dont want exceptions to be disabled while the fault is handled
; Now that we have saved the context we return from exception hence re-enable
; exceptions and remain in kernel mode. Interrupts remain disabled.
	ld	r9, [sp, PT_status32]	;read erstatus so we return to the same state of interrupt enables.  
	; set mode we are returning to as kernel mode
	and	r9, r9, (~STATUS_U_MASK)
	sr	r9, [erstatus]
	mov	r9, 2f
	sr	r9, [eret]
	rtie
2:
;	lr	r0, [ecr]    /* load ecr in order to get the cause code and parameter */
;	lr	r1, [efa]       /* faulting address */
;	mov	r2, sp 
	bl	privilege_fault
	j	ret_from_exception
	
/* Extension Instruction Exception Handler */	
ARC_ENTRY EV_Extension
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
	
1:
	SAVE_ALL_SYS
#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif


	lr	r0, [ecr]    /* load ecr in order to get the cause code and parameter */
	lr	r1, [efa]       /* faulting address */
	mov	r2, sp 
	bl	extension_fault
	j	ret_from_exception
	
	
; trace system call if PT_TRACESYS enabled
tracesys:

	jl	SYMBOL_NAME(syscall_trace)
	mov	r0, -ENOSYS
	ld	r8, [sp, PT_r8]
	sub.f	r9, r8, NR_syscalls
	nop
	jz	tracesys_exit
	ld	r0, [sp, PT_r0]
	ld	r1, [sp, PT_r1]
	ld	r2, [sp, PT_r2]	
	ld	r3, [sp, PT_r3]
	ld	r4, [sp, PT_r4]	
	ld	r5, [sp, PT_r5]	
	ld	r6, [sp, PT_r6]
	ld	r7, [sp, PT_r7]
	mov 	r9, SYMBOL_NAME(sys_call_table)
	ld.as	r9, [r9, r8]
	jl.nd	[r9]
	
	;fall through
tracesys_exit:
	jl	SYMBOL_NAME(syscall_trace)
	;nop
	j	ret_from_system_call
	
; System call handler. parameters in r0-r7. r0 has the system call number
; EV_Trap caused by TRAP_S and TRAP0 instructions.
; We manipulate ERET to return from exception and stay in kernel mode and
; proceed.

ARC_ENTRY EV_Trap
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
#ifdef SIM_EXCEPTION_BUG
	cli_level_1	r9
#endif
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
1:
	SAVE_ALL_SYS_TRAP	; save all registers required for system call handling

; We dont want exceptions to be disabled while system calls are handled. 
; Now that we have saved the context we return from exception hence re-enable
; exceptions and remain in kernel mode.
	ld	r9, [sp, PT_status32]	;read erstatus so we return to the same state of interrupt enables.  
	; set mode we are returning to as kernel mode
	and	r9, r9, (~STATUS_U_MASK)
	sr	r9, [erstatus]
	mov	r9, sys_trap_ret
	sr	r9, [eret]
	rtie
	 
sys_trap_ret:	

	; check if a brk pt caused the exception
	lr	r10, [ecr]
	and.f	0,r10,TRAP_S_BRKPT
	jz	not_traps_brkpt
	
	;make sure orig_r8 is a positive value
	add	r10, sp, PT_orig_r8
#if 0 //wschen 2008-02-12
	st	NR_syscalls + 1, [r10] 
#else
	st	NR_syscalls + 2, [r10] 
#endif

	lr	r0, [ecr]    /* load ecr in order to get the cause code and parameter */
	lr	r1, [efa]       /* faulting address */
	mov	r2, sp 

	ld	r9, [SYMBOL_NAME(_current_task)]	; current task in r9	
	add	r10, r9, (TASK_THREAD + THREAD_USP)
	st	sp, [r10]
	
	;Save callee regs so that the tracer gets the correct value.
	SAVE_CALLEE_SAVED
	add	r10, r9, (TASK_THREAD + THREAD_PC)
	st	sp, [r10]
	bl	trap_s_brkpt
	
	;sp should point to pt_regs 
	ld	r9, [SYMBOL_NAME(_current_task)]	; current task in r9	
	add	r10, r9, (TASK_THREAD + THREAD_USP)
	ld	sp,[r10]
	j	ret_from_exception	 

not_traps_brkpt:	
	; check for ptrace, and if set notify parent
	ld	r9, [r9, TASK_PTRACE]
	and.f 	0,r9, PT_TRACESYS
	jnz	tracesys
	
	; Check if the system call number does not exceed the total system calls available 
	sub.f	r9, r8, NR_syscalls
	nop
	jz	badsys

	; Offset into the syscall_table and call corresponding function
	mov	r9,SYMBOL_NAME(sys_call_table)		 
	ld.as	r9,[r9, r8]	; using the scaled address mode	
	
	jl.nd	[r9]		
	; fall through 


ARC_ENTRY ret_from_system_call
	; put the return value of system call 
	st	r0, [sp, PT_r0]
	jal	ret_from_exception

#if 0	//Robert 2008-05-02 Reserved feature
;################### Return from Intr/Excp ##########################

ARC_ENTRY ret_from_interrupt_level2
	; 	 check if we need to do software interrupts 
	; FIXME :: this needs protection for level 2 interrupts
	mov	r8, SYMBOL_NAME(irq_stat)
	ld	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]
	sub	r9, r9, 1
	st	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]

	ld	r9, [r8, CPUSTAT_SOFTIRQ_PENDING]
	mov r8,0x2
	sr r8, [0x43] 		; clear bit in aux_irqlv12

	mov.f	0, r9
	nop 
	bz	ret_from_exception

	jl	SYMBOL_NAME(do_softirq)				
	b	ret_from_exception

ARC_ENTRY ret_from_interrupt_level1
	; 	 check if we need to do software interrupts 
	; FIXME :: this needs protection for level 2 interrupts
	mov	r8, SYMBOL_NAME(irq_stat)	 
	ld	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]
	sub	r9, r9, 1
	st	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]

	ld	r9, [r8, CPUSTAT_SOFTIRQ_PENDING]
	
	mov r8,0x1
	sr r8, [0x43] 		; clear bit in aux_irqlv12
	mov.f	0, r9
	nop 
	bz	ret_from_exception

	jl	SYMBOL_NAME(do_softirq)				

	;  fall through
#endif

ARC_ENTRY ret_from_interrupt
	; 	 check if we need to do software interrupts 
	; FIXME :: this needs protection for level 2 interrupts
	mov	r8, SYMBOL_NAME(irq_stat)	 
	ld	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]
	sub	r9, r9, 1
	st	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]

	ld	r9, [r8, CPUSTAT_SOFTIRQ_PENDING]
	mov.f	0, r9
	nop
	bz	ret_from_exception

	jl	SYMBOL_NAME(do_softirq)				
	;  fall through
	
ARC_ENTRY ret_from_exception
	; check if we are returning to kernel mode
	ld	r8, [sp, PT_status32]	; sp points to pt_regs structure on stack
	and.f	0, r8, STATUS_U_MASK	; test
	nop
	bz	skip_check
	; else returning to user mode
	; load current task
	ld	r8, [SYMBOL_NAME(_current_task)]
#if 1 //wschen 2008-02-29
	mov	r8, sp
	and	r8, r8, ~(0x2000-1)
#endif
	; check for rescheduling
	ld	r9, [r8, TASK_NEED_RESCHED]
	mov.f	0, r9		; test
	nop
	bnz	reschedule	; uses flags set by the mov.f 0, r9 intruction 
	; check for signals
	ld	r9, [r8, TASK_SIGPENDING]
	mov.f	0, r9
	nop
	bz	skip_check
	;  there are pending signals 
	mov	r0, sp		;  pass pointer tp the pt_regs structure
	mov	r1, 0           ;  pass offset = NULL

	ld	r9, [SYMBOL_NAME(_current_task)]	; current task in r9	
	add	r10, r9, (TASK_THREAD + THREAD_USP)
	st	sp,[r10]
	
	; save callee regs so that incase of a core dump, the correct regs are available
	SAVE_CALLEE_SAVED
	;FIXME - dont use KSP, use PC instead
	add	r9, r8, (TASK_THREAD + THREAD_PC) 
	st	sp, [r9]
	
	jl	SYMBOL_NAME(do_signal)

	;sp pointing back to pt_regs
	ld	r9, [SYMBOL_NAME(_current_task)]	; current task in r9	
	add	r10, r9, (TASK_THREAD + THREAD_USP)
	ld	sp,[r10]
	
	; fall through 	
skip_check : 	
	; Exceptions cannot be disabled. We work with the understanding that
	; from this point on till the rtie at the end, no exceptions will occur.
	; We are always restoring from kernel stack which will be in
	; untranslated region
	; Restore the saved sys context.
	lr	r10, [status32]
	and.f	0, r10, STATUS_AE_MASK
	nop
	bz	not_exception
	; Disable interrupts, do_softirq would have enabled them !
	cli_level_12 r9
	RESTORE_ALL_SYS
	rtie
not_exception:	
	and.f	0, r10, STATUS_A1_MASK
	nop
	bz	not_level1_interrupt
	; Disable level 2 interrupts, do_softirq would have enabled them !
	cli_level_12 r9
	RESTORE_ALL_INT1
	rtie
not_level1_interrupt:
	and.f	0, r10, STATUS_A2_MASK
	nop
	bz	not_level2_interrupt
	; Disable interrupts, do_softirq would have enabled them !
	cli_level_12 r9
	RESTORE_ALL_INT2
	rtie
not_level2_interrupt:	
	; Disable interrupts, do_softirq would have enabled them !
	cli_level_12 r9
	;this case is for syscalls for which the exception active bit is not
	;set
	RESTORE_ALL_SYS
	; FIXME :: might not be required, waiting for further info
	; Ensure all pending memory operations have physically completed before
	; returning.
	; sync 
	; return to interrupted pc
	rtie
	
reschedule :
	jl	schedule
	b	ret_from_exception

badsys :
	st	-ENOSYS, [sp, PT_r0]
	jal	ret_from_system_call


ARC_ENTRY ret_from_fork
	; when the forked child comes here from the __switch_to function
	; r0 has the last task pointer. 
	; put last task in scheduler queue
	jl	SYMBOL_NAME(schedule_tail)
	jal	SYMBOL_NAME(ret_from_exception)

	
ARC_ENTRY __switch_to
	/* save return address onto stack */
	st.a	blink, [sp, -4]
	/* save frame pointer stack */
	st.a	fp, [sp, -4]
	SAVE_CALLEE_SAVED
	/* save kernel stack pointer */
	add	r9, r0, (TASK_THREAD + THREAD_KSP) 
	st	sp, [r9]
	/* return last task in r0 ( return registor) */
	ld	r0, [SYMBOL_NAME(_current_task)]
	
	/* wschen 2008-02-12 Ensure all pending memory operations have physically completed before
	   switching contexts. This is to make sure that the right process is killed */
	sync

	/* switch to new task , second parameter to this function , ie r1*/
	st	r1, [SYMBOL_NAME(_current_task)]
	/* load kernel stack pointer from the new task struct */
	add	r9, r1, (TASK_THREAD + THREAD_KSP) 
	ld	sp, [r9]
	/* restore the registers for the new task */
	RESTORE_CALLEE_SAVED
	/* restore the frame pointer */
	ld.ab	fp, [sp, 4]
	/* restore blink (return address) */
	ld.ab	blink, [sp, 4]
	/* return to the point where this thread "slept" */
	j	[blink]
	/* FIXME : unused delay slot */
	
ARC_ENTRY sys_execve_wrapper
	; copy pointer to pt_regs as a parameter
	mov	 r3, sp
	SAVE_CALLEE_SAVED
	jl	SYMBOL_NAME(sys_execve)
	RESTORE_CALLEE_SAVED
	jal	ret_from_system_call

ARC_ENTRY sys_fork_wrapper
	; copy pointer to pt_regs as a parameter
	mov	 r0, sp
	SAVE_CALLEE_SAVED
	jl	SYMBOL_NAME(sys_fork)
	RESTORE_CALLEE_SAVED
	jal	ret_from_system_call

ARC_ENTRY sys_vfork_wrapper
	; copy pointer to pt_regs as a parameter
	mov	 r0, sp
	SAVE_CALLEE_SAVED
	jl	SYMBOL_NAME(sys_vfork)
	RESTORE_CALLEE_SAVED
	jal	ret_from_system_call
		
ARC_ENTRY sys_clone_wrapper
	; copy parameter( struct pt_regs *) for sys_clone
	mov	r2, sp
	SAVE_CALLEE_SAVED	
	jl	SYMBOL_NAME(sys_clone)
	RESTORE_CALLEE_SAVED
	jal	ret_from_system_call

ARC_ENTRY sys_rt_sigreturn_wrapper
	; copy pointer to pt_regs as a parameter
	mov	 r0, sp
	SAVE_CALLEE_SAVED
	jl	SYMBOL_NAME(sys_rt_sigreturn)
	RESTORE_CALLEE_SAVED
	jal	ret_from_system_call
	
ARC_ENTRY sys_sigreturn_wrapper
	; copy pointer to pt_regs as a parameter
	mov	 r0, sp
	SAVE_CALLEE_SAVED
	jl	SYMBOL_NAME(sys_sigreturn)
	RESTORE_CALLEE_SAVED
	jal	ret_from_system_call

ARC_ENTRY sys_rt_sigsuspend_wrapper
	; copy parameter( struct pt_regs *) for sys_rt_sigsuspend
	mov	r2, sp
	SAVE_CALLEE_SAVED	
	jl	SYMBOL_NAME(sys_rt_sigsuspend)
	RESTORE_CALLEE_SAVED
	jal	ret_from_system_call
	
ARC_ENTRY sys_sigsuspend_wrapper
	; copy parameter( struct pt_regs *) for sys_sigsuspend
	mov	r3, sp
	SAVE_CALLEE_SAVED	
	jl	SYMBOL_NAME(sys_sigsuspend)
	RESTORE_CALLEE_SAVED
	jal	ret_from_system_call

ARC_ENTRY sys_sigaltstack_wrapper
	; copy pointer to pt_regs as a parameter
	mov	 r2, sp
	SAVE_CALLEE_SAVED
	jl	SYMBOL_NAME(sys_sigaltstack)
	RESTORE_CALLEE_SAVED
	jal	ret_from_system_call
	
; The syscall table starts here...
#ifndef NR_syscalls
#define NR_syscalls 256
#endif
.data
ARC_ENTRY sys_call_table
__syscall_start:
  /* 0 */	.long	SYMBOL_NAME(sys_ni_syscall)
  		.long	SYMBOL_NAME(sys_exit)
   		.long	SYMBOL_NAME(sys_fork_wrapper) 
		.long	SYMBOL_NAME(sys_read)
  		.long	SYMBOL_NAME(sys_write)
  /* 5 */	.long	SYMBOL_NAME(sys_open)
  		.long	SYMBOL_NAME(sys_close)
 		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_waitpid */
  		.long	SYMBOL_NAME(sys_creat)
  		.long	SYMBOL_NAME(sys_link)
  /* 10 */	.long	SYMBOL_NAME(sys_unlink)
   		.long	SYMBOL_NAME(sys_execve_wrapper)
		.long	SYMBOL_NAME(sys_chdir)
  		.long	SYMBOL_NAME(sys_time)			/* used by libc4 */
  		.long	SYMBOL_NAME(sys_mknod)
  /* 15 */	.long	SYMBOL_NAME(sys_chmod)
  		.long	SYMBOL_NAME(sys_lchown16)
  		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_break */
  		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_stat */
  		.long	SYMBOL_NAME(sys_lseek)
  /* 20 */	.long	SYMBOL_NAME(sys_getpid)
  		.long	SYMBOL_NAME(sys_mount)
  		.long	SYMBOL_NAME(sys_oldumount)		/* used by libc4 */
  		.long	SYMBOL_NAME(sys_setuid16)
  		.long	SYMBOL_NAME(sys_getuid16)
  /* 25 */	.long	SYMBOL_NAME(sys_stime)
;		.long	SYMBOL_NAME(sys_ni_syscall) 
		.long	SYMBOL_NAME(sys_ptrace)
		.long	SYMBOL_NAME(sys_alarm)	/* used by libc4 */
	  	.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_fstat */
   		.long	SYMBOL_NAME(sys_pause)	
  /* 30 */	.long	SYMBOL_NAME(sys_utime)			/* used by libc4 */
  		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_stty */
  		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_getty */
  		.long	SYMBOL_NAME(sys_access)
  		.long	SYMBOL_NAME(sys_nice)
  /* 35 */	.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_ftime */
 		.long	SYMBOL_NAME(sys_sync)
  		.long	SYMBOL_NAME(sys_kill)
  		.long	SYMBOL_NAME(sys_rename)
  		.long	SYMBOL_NAME(sys_mkdir)
  /* 40 */	.long	SYMBOL_NAME(sys_rmdir)
  		.long	SYMBOL_NAME(sys_dup)
   		.long	SYMBOL_NAME(sys_pipe)
  		.long	SYMBOL_NAME(sys_times)
  		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_prof */
  /* 45 */	.long	SYMBOL_NAME(sys_brk)
  		.long	SYMBOL_NAME(sys_setgid16)
  		.long	SYMBOL_NAME(sys_getgid16)
  		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_signal */
  		.long	SYMBOL_NAME(sys_geteuid16)
  /* 50 */	.long	SYMBOL_NAME(sys_getegid16)
  		.long	SYMBOL_NAME(sys_acct)
  		.long	SYMBOL_NAME(sys_umount)
  		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_lock */
  		.long	SYMBOL_NAME(sys_ioctl)
  /* 55 */	.long	SYMBOL_NAME(sys_fcntl)
  		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_mpx */
  		.long	SYMBOL_NAME(sys_setpgid)
  		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_ulimit */
  		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_olduname */
  /* 60 */	.long	SYMBOL_NAME(sys_umask)
  		.long	SYMBOL_NAME(sys_chroot)
  		.long	SYMBOL_NAME(sys_ustat)
  		.long	SYMBOL_NAME(sys_dup2)
  		.long	SYMBOL_NAME(sys_getppid)
  /* 65 */	.long	SYMBOL_NAME(sys_getpgrp)
  		.long	SYMBOL_NAME(sys_setsid)
   		.long	SYMBOL_NAME(sys_sigaction)
		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_sgetmask */
  		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_ssetmask */
  /* 70 */	.long	SYMBOL_NAME(sys_setreuid16)
  		.long	SYMBOL_NAME(sys_setregid16)
   		.long	SYMBOL_NAME(sys_sigsuspend_wrapper)
  		.long	SYMBOL_NAME(sys_sigpending)
  		.long	SYMBOL_NAME(sys_sethostname)
  /* 75 */	.long	SYMBOL_NAME(sys_setrlimit)
  		.long	SYMBOL_NAME(sys_old_getrlimit)		/* used by libc4 */
  		.long	SYMBOL_NAME(sys_getrusage)
  		.long	SYMBOL_NAME(sys_gettimeofday)
  		.long	SYMBOL_NAME(sys_settimeofday)
 /* 80 */	.long	SYMBOL_NAME(sys_getgroups16)
  		.long	SYMBOL_NAME(sys_setgroups16)
   		.long	SYMBOL_NAME(old_select)		/* used by libc4 */
 		.long	SYMBOL_NAME(sys_symlink)
 		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_lstat */
  /* 85 */	.long	SYMBOL_NAME(sys_readlink)
  		.long	SYMBOL_NAME(sys_uselib)
  		.long	SYMBOL_NAME(sys_swapon)
  		.long	SYMBOL_NAME(sys_reboot)
  		.long	SYMBOL_NAME(old_readdir)		/* used by libc4 */
  /* 90 */	.long	SYMBOL_NAME(old_mmap)			/* used by libc4 */
  		.long	SYMBOL_NAME(sys_munmap)
  		.long	SYMBOL_NAME(sys_truncate)
  		.long	SYMBOL_NAME(sys_ftruncate)
  		.long	SYMBOL_NAME(sys_fchmod)
  /* 95 */	.long	SYMBOL_NAME(sys_fchown16)
  		.long	SYMBOL_NAME(sys_getpriority)
  		.long	SYMBOL_NAME(sys_setpriority)
  		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_profil */
  		.long	SYMBOL_NAME(sys_statfs)
  /* 100 */	.long	SYMBOL_NAME(sys_fstatfs)
  		.long	SYMBOL_NAME(sys_ni_syscall)
   		.long	SYMBOL_NAME(sys_socketcall)
 		.long	SYMBOL_NAME(sys_syslog)
  		.long	SYMBOL_NAME(sys_setitimer)
  /* 105 */	.long	SYMBOL_NAME(sys_getitimer)
  		.long	SYMBOL_NAME(sys_newstat)
  		.long	SYMBOL_NAME(sys_newlstat)
  		.long	SYMBOL_NAME(sys_newfstat)
  		.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_uname */
  /* 110 */	.long	SYMBOL_NAME(sys_ni_syscall)		/* was sys_iopl */
  		.long	SYMBOL_NAME(sys_vhangup)
  		.long	SYMBOL_NAME(sys_ni_syscall)
		.long	SYMBOL_NAME(sys_ni_syscall)     /* vm86old for i386 */
  		.long	SYMBOL_NAME(sys_wait4)
  /* 115 */	.long	SYMBOL_NAME(sys_swapoff)
  		.long	SYMBOL_NAME(sys_sysinfo)
   		.long	SYMBOL_NAME(sys_ipc)
  		.long	SYMBOL_NAME(sys_fsync)
   		.long	SYMBOL_NAME(sys_sigreturn_wrapper)
/* 120 */	.long	SYMBOL_NAME(sys_clone_wrapper)
 		.long	SYMBOL_NAME(sys_setdomainname)
 		.long	SYMBOL_NAME(sys_newuname)
 		.long	SYMBOL_NAME(sys_ni_syscall)
 		.long	SYMBOL_NAME(sys_adjtimex)
 /* 125 */	.long	SYMBOL_NAME(sys_mprotect)
 		.long	SYMBOL_NAME(sys_sigprocmask)
 		.long	SYMBOL_NAME(sys_create_module)
 		.long	SYMBOL_NAME(sys_init_module)
 		.long	SYMBOL_NAME(sys_delete_module)
 /* 130 */	.long	SYMBOL_NAME(sys_get_kernel_syms)
 		.long	SYMBOL_NAME(sys_quotactl)
 		.long	SYMBOL_NAME(sys_getpgid)
 		.long	SYMBOL_NAME(sys_fchdir)
 		.long	SYMBOL_NAME(sys_bdflush)
 /* 135 */	.long	SYMBOL_NAME(sys_sysfs)
 		.long	SYMBOL_NAME(sys_personality)
 		.long	SYMBOL_NAME(sys_ni_syscall)	/* .long	_sys_afs_syscall */
 		.long	SYMBOL_NAME(sys_setfsuid16)
 		.long	SYMBOL_NAME(sys_setfsgid16)
 /* 140 */	.long	SYMBOL_NAME(sys_llseek)
 		.long	SYMBOL_NAME(sys_getdents)
 		.long	SYMBOL_NAME(sys_select)
 		.long	SYMBOL_NAME(sys_flock)
 		.long	SYMBOL_NAME(sys_msync)
 /* 145 */	.long	SYMBOL_NAME(sys_readv)
 		.long	SYMBOL_NAME(sys_writev)
 		.long	SYMBOL_NAME(sys_getsid)
 		.long	SYMBOL_NAME(sys_fdatasync)
 		.long	SYMBOL_NAME(sys_sysctl)
 /* 150 */	.long	SYMBOL_NAME(sys_mlock)
 		.long	SYMBOL_NAME(sys_munlock)
 		.long	SYMBOL_NAME(sys_mlockall)
 		.long	SYMBOL_NAME(sys_munlockall)
 		.long	SYMBOL_NAME(sys_sched_setparam)
 /* 155 */	.long	SYMBOL_NAME(sys_sched_getparam)
 		.long	SYMBOL_NAME(sys_sched_setscheduler)
 		.long	SYMBOL_NAME(sys_sched_getscheduler)
 		.long	SYMBOL_NAME(sys_sched_yield)
 		.long	SYMBOL_NAME(sys_sched_get_priority_max)
 /* 160 */	.long	SYMBOL_NAME(sys_sched_get_priority_min)
 		.long	SYMBOL_NAME(sys_sched_rr_get_interval)
 		.long	SYMBOL_NAME(sys_nanosleep)
 		.long	SYMBOL_NAME(sys_mremap)
 		.long	SYMBOL_NAME(sys_setresuid16)
 /* 165 */	.long	SYMBOL_NAME(sys_getresuid16)
 		.long	SYMBOL_NAME(sys_ni_syscall)
 		.long	SYMBOL_NAME(sys_query_module)
 		.long	SYMBOL_NAME(sys_poll)
 		.long	SYMBOL_NAME(sys_nfsservctl)
 /* 170 */	.long	SYMBOL_NAME(sys_setresgid16)
 		.long	SYMBOL_NAME(sys_getresgid16)
 		.long	SYMBOL_NAME(sys_prctl)
  		.long	SYMBOL_NAME(sys_rt_sigreturn_wrapper)
 		.long	SYMBOL_NAME(sys_rt_sigaction)
 /* 175 */	.long	SYMBOL_NAME(sys_rt_sigprocmask)
 		.long	SYMBOL_NAME(sys_rt_sigpending)
 		.long	SYMBOL_NAME(sys_rt_sigtimedwait)
 		.long	SYMBOL_NAME(sys_rt_sigqueueinfo)
  		.long	SYMBOL_NAME(sys_rt_sigsuspend_wrapper)
 /* 180 */	.long	SYMBOL_NAME(sys_pread)
 		.long	SYMBOL_NAME(sys_pwrite)
 		.long	SYMBOL_NAME(sys_chown16)
 		.long	SYMBOL_NAME(sys_getcwd)
 		.long	SYMBOL_NAME(sys_capget)
 /* 185 */	.long	SYMBOL_NAME(sys_capset)
;		.long	SYMBOL_NAME(sys_ni_syscall)
  		.long	SYMBOL_NAME(sys_sigaltstack_wrapper)
 		.long	SYMBOL_NAME(sys_sendfile)
 		.long	SYMBOL_NAME(sys_ni_syscall)
 		.long	SYMBOL_NAME(sys_ni_syscall)
  /* 190 */	.long	SYMBOL_NAME(sys_vfork_wrapper)
 		.long	SYMBOL_NAME(sys_getrlimit)
  		.long	SYMBOL_NAME(sys_mmap2)
 		.long	SYMBOL_NAME(sys_truncate64)
 		.long	SYMBOL_NAME(sys_ftruncate64)
 /* 195 */	.long	SYMBOL_NAME(sys_stat64)
 		.long	SYMBOL_NAME(sys_lstat64)
 		.long	SYMBOL_NAME(sys_fstat64)
 		.long	SYMBOL_NAME(sys_lchown)
 		.long	SYMBOL_NAME(sys_getuid)
 /* 200 */	.long	SYMBOL_NAME(sys_getgid)
 		.long	SYMBOL_NAME(sys_geteuid)
 		.long	SYMBOL_NAME(sys_getegid)
 		.long	SYMBOL_NAME(sys_setreuid)
 		.long	SYMBOL_NAME(sys_setregid)
 /* 205 */	.long	SYMBOL_NAME(sys_getgroups)
 		.long	SYMBOL_NAME(sys_setgroups)
 		.long	SYMBOL_NAME(sys_fchown)
 		.long	SYMBOL_NAME(sys_setresuid)
 		.long	SYMBOL_NAME(sys_getresuid)
 /* 210 */	.long	SYMBOL_NAME(sys_setresgid)
 		.long	SYMBOL_NAME(sys_getresgid)
 		.long	SYMBOL_NAME(sys_chown)
 		.long	SYMBOL_NAME(sys_setuid)
 		.long	SYMBOL_NAME(sys_setgid)
 /* 215 */	.long	SYMBOL_NAME(sys_setfsuid)
 		.long	SYMBOL_NAME(sys_setfsgid)
 		.long	SYMBOL_NAME(sys_pivot_root)
 		.long	SYMBOL_NAME(sys_mincore)
		.long	SYMBOL_NAME(sys_madvise)
 /* 220 */	.long	SYMBOL_NAME(sys_getdents64)
 		.long	SYMBOL_NAME(sys_fcntl64)
		.long	SYMBOL_NAME(sys_ni_syscall)
		.long	SYMBOL_NAME(sys_ni_syscall)
		.long 	SYMBOL_NAME(sys_gettid)
 /* 225 */	.long	SYMBOL_NAME(sys_readphy)

__syscall_end:

		.rept	NR_syscalls - (__syscall_end - __syscall_start) / 4
				.long	SYMBOL_NAME(sys_ni_syscall)
		.endr

