/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 * Copyright (C)
 *
 * This program is free software you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Author(s) : Rahul Trivedi
 * Exception handlers for the arc700
 */
#include <linux/config.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/ptrace.h>
#include <linux/timer.h>
#include <linux/mm.h>
#include <linux/smp.h>
#include <linux/smp_lock.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/module.h>
#include <asm/tlb.h>

#include <asm/uaccess.h>
#include <asm/arcregs.h>
#ifdef CONFIG_ARC_TRACE_UNIT
#include <asm/trace_unit.h>
#endif

spinlock_t die_lock = SPIN_LOCK_UNLOCKED;

void show_trace_task(struct task_struct *tsk)
{
  /* TODO: rtee: write the show_regs function */
}

void show_regs(struct pt_regs *regs)
{
	unsigned int i = 0;
	printk ("sp: 0x%08lx\tfp: 0x%08lx\n", regs->sp, regs->fp);
	printk ("ret: 0x%08lx\tblink: 0x%08lx\n", regs->ret, regs->blink);
	printk ("status32: 0x%08lx\n", regs->status32);
	printk("r%02u: %08lx\t", i++, regs->r0);
	printk("r%02u: %08lx\t", i++, regs->r1);
	printk("r%02u: %08lx\t", i++, regs->r2);
	printk("\n");
	printk("r%02u: %08lx\t", i++, regs->r3);
	printk("r%02u: %08lx\t", i++, regs->r4);
	printk("r%02u: %08lx\t", i++, regs->r5);
	printk("\n");
	printk("r%02u: %08lx\t", i++, regs->r6);
	printk("r%02u: %08lx\t", i++, regs->r7);
	printk("r%02u: %08lx\t", i++, regs->r8);
	printk("\n");
	printk("r%02u: %08lx\t", i++, regs->r9);
	printk("r%02u: %08lx\t", i++, regs->r10);
	printk("r%02u: %08lx\t", i++, regs->r11);
	printk("\n");
	printk("r%02u: %08lx\t", i++, regs->r12);
	printk("\n");

#ifdef CONFIG_ARC_TRACE_UNIT
	//PB
	for (i=0; i<CONFIG_ARC_TRACE_DEPTH; i++)
	{
	  printk ("\ntrace[%u] src: 0x%.8X, dest: 0x%.8X, asid: 0x%.2X, mode: %c, repeat %c, valid %c",i,trace_buffer[i].source,trace_buffer[i].dest,trace_buffer[i].asid,(trace_buffer[i].mode==0x01)?'U':'K',(trace_buffer[i].repeat==0x01)?'Y':'N',(trace_buffer[i].valid==0x01)?'Y':'N');
	}
	printk("\n");
	//!PB
#endif
}

void die(const char *str, struct pt_regs *regs, unsigned long cause)
{
  int i;
	struct task_struct *tsk = current;
	unsigned int efa = read_new_aux_reg(0x404);  /* Excepn Fault Addr */

//PB
#ifdef CONFIG_ARC_TRACE_UNIT
	for (i=0; i<CONFIG_ARC_TRACE_DEPTH; i++)
	{
	  printk ("\ntrace[%u] src: 0x%.8X, dest: 0x%.8X, asid: 0x%.2X, mode: %c, repeat %c, valid %c",i,trace_buffer[i].source,trace_buffer[i].dest,trace_buffer[i].asid,(trace_buffer[i].mode==0x01)?'U':'K',(trace_buffer[i].repeat==0x01)?'Y':'N',(trace_buffer[i].valid==0x01)?'Y':'N');
	}
	printk("\n");
#endif
   if (regs != NULL)
     show_regs(regs);
   else
     printk("NULL regs value in die");
//!PB


        console_verbose();
        spin_lock_irq(&die_lock);
	bust_spinlocks(1);
//	printk("%s: address: %08lx\tecause: %08lx\n", str, regs->ret,cause);
	printk("%s: \n   While (%s): 0x%08lx\n   Faulting Instn : 0x%08lx\n",
            str, (cause?"Write to":"Read from"), efa, regs->ret);

//PB
	if (tsk)
        {
	  if (tsk->mm)
            printk("Current task = %s, PID = %u, ASID = %lu\n", tsk->comm, tsk->pid, tsk->mm->context.asid);
          else
            printk("Current task = %s, PID = %u, MM is NULL\n", tsk->comm, tsk->pid);
        }
        //show_regs(regs);
//!PB
	bust_spinlocks(0);
        spin_unlock_irq(&die_lock);
        do_exit(SIGSEGV);
}
void do_trap(unsigned long cause, char *str, struct pt_regs *regs,
	     siginfo_t *info)
{
  if(regs->status32 & STATUS_U_MASK) {
    struct task_struct *tsk = current;
#ifdef CONFIG_ARC_FAULTS_DBG
	printk("%s: address = %08lx ecause = %08lx\n", str, regs->ret,cause);
	printk("Sending signal %u to %s\n", info->si_signo, tsk->comm);
        show_regs(regs);
#endif	/* CONFIG_ARC_FAULTS_DBG */
    tsk->thread.fault_address = (unsigned int) info->si_addr;
    tsk->thread.error_code = cause;

    if(info) 
      force_sig_info(info->si_signo, info, tsk);
    else
      force_sig(info->si_signo,tsk);
    return;
  }
  else {
    unsigned long fixup;
    /* Are we prepared to handle this kernel fault?
     *
     * (The kernel has valid exception-points in the source 
     *  when it acesses user-memory. When it fails in one
     *  of those points, we find it in a table and do a jump
     *  to some fixup code that loads an appropriate error
     *  code)
     */
    if((fixup = search_exception_table(regs->ret)) != 0) {
      regs->ret = fixup;
      return;
    }

    /*
     * Oops. The kernel tried to access some bad page. We'll have to
     * terminate things with extreme prejudice.
     */
    die(str, regs, cause);
  }
}

#define DO_ERROR_INFO(signr, str, name, sicode) \
asmlinkage void name(unsigned long cause, unsigned long address, \
		     struct pt_regs *regs) \
{ \
  siginfo_t info;\
  info.si_signo = signr;\
  info.si_errno = 0;\
  info.si_code = sicode;\
  info.si_addr = (void *)address;\
  do_trap(cause,str,regs,&info);\
}

DO_ERROR_INFO(SIGSEGV, "Access Violation/Misaligned Access", 
	      protection_fault, SEGV_ACCERR)
DO_ERROR_INFO(SIGSEGV, "Trying to load a duplicate entry", 
	      duplicate_tlb_fault, SEGV_ACCERR)
DO_ERROR_INFO(SIGILL,"Privileged Operation/Disabled Extension/Actionpoint Hit",
	      privilege_fault, ILL_PRVOPC)
DO_ERROR_INFO(SIGILL,"Extenion Instruction Exception", extension_fault, 
	      ILL_ILLOPC)
DO_ERROR_INFO(SIGILL,"Illegal Instruction/Illegal Instruction Sequence", 
	      instruction_error, ILL_ILLOPC)
DO_ERROR_INFO(SIGBUS,"Access to Invalid Memory",memory_error, BUS_ADRERR)
DO_ERROR_INFO(SIGTRAP, "Breakpoint Set", 
	      trap_s_brkpt, TRAP_BRKPT)


asmlinkage void machine_check(unsigned long cause, unsigned long address,
				   struct pt_regs *regs)
{
	show_regs(regs);
	panic("Machine Check Exception: address = 0x%08lx ecause = 0x%08lx\n", address, cause);
}

void __init trap_init(void)
{
	return;
}

#if 1
asmlinkage void tlb_overlap(unsigned long cause, unsigned long address,
                              struct pt_regs *regs)
{
/**
 * modified by Robert 2008-08-04 Remove volatile, prefetch 3n & 3z to speed up
 * modified by alex 2008/8/1 to speed up
 * modified by wschen 2008-07-30, disable printk 
 * First release by ARC
 */

// Modification by Simon Spooner of ARC
// The machine check exception could possibly be caused by duplicate
// entries getting into the JTLB due to a bug in the hardware that in
// very rare circumstances could allow duplicates.
// If the machine check was caused by a rogue PD in the TLB, delete it and carry on.
// Note that the MMU is switched off during this type of MachineCheck.
// If you don't switch it back on before exiting, then you go off into la la land.

#if 0 //wschen 2008-07-30
    unsigned int tlbpd0[TLB_SIZE];
    unsigned int tlbpd1[TLB_SIZE];
    unsigned int udtlb0[8];
    unsigned int udtlb1[8];
    unsigned int n,z,u;
    unsigned int flags;
    unsigned int erstatus;
#else
    //Robert 2008-08-04 Remove volatile, prefetch 3n & 3z to speed up
    /*volatile*/ unsigned int tlbpd0[TLB_SIZE*3];	// modified by alex 2008/8/1 to speed up
//    volatile unsigned int tlbpd1[TLB_SIZE];
    /*volatile*/ unsigned int n,z,u;
    unsigned int tmp_3n, tmp_3z;
#endif
#if 0 //wschen 2008-07-30
    printk("EV_MachineCheck : Duplicate PD in TLB.  Attempting to correct\n");
#endif

    
    write_new_aux_reg(ARC_REG_PID,0x80000000 | read_new_aux_reg(ARC_REG_PID));


    for(n=0;n<TLB_SIZE;n++)
    {
        write_new_aux_reg(ARC_REG_TLBINDEX,n);
        write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBRead);
        tlbpd0[3*n] = read_new_aux_reg(ARC_REG_TLBPD0);
        tlbpd0[3*n+1] = 0;	// left (smaller) subtree
        tlbpd0[3*n+2] = 0;	// right (bigger) subtree

        if(  !(tlbpd0[3*n] & 0x400) ) // PD is not valid, continue
        {
          continue;
        }

//        tlbpd1[n] = read_new_aux_reg(ARC_REG_TLBPD1);
        tmp_3n = tlbpd0[3*n];
	for (z = 0; z < n;)
	{
                tmp_3z = tlbpd0[3*z];
//		if (tlbpd0[3*z] < tlbpd0[3*n])
		if (tmp_3z < tmp_3n)
		{
			if (tlbpd0[3*z+2] != 0)
				z = tlbpd0[3*z+2];	// goto right subtree
			else
			{
				tlbpd0[3*z+2] = n; z = n;
			}
		}
//		else if (tlbpd0[3*z] > tlbpd0[3*n])
		else if (tmp_3z > tmp_3n)
		{
			if (tlbpd0[3*z+1] != 0)
				z = tlbpd0[3*z+1];	// goto left subtree
			else
			{
				tlbpd0[3*z+1] = n; z = n;
			}
		}
		else // Duplicate TLB
		{
//           		if(tlbpd0[3*n] & 0x400) // PD is valid.
			{
#if 0 //wschen 2008-07-30
                   printk("Duplicate PD's @ %u and %u\n", n,z);
                   printk("TLBPD0[%u] : %08x TLBPD0[%u] : %08x\n",n,tlbpd0[3*n], z, tlbpd0[3*z]);
                   printk("Removing one of the duplicates\n");
#endif
	                   write_new_aux_reg(ARC_REG_TLBPD0,0);
	       	           write_new_aux_reg(ARC_REG_TLBPD1,0);
	                   write_new_aux_reg(ARC_REG_TLBINDEX,n);
	                   write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBWrite);
			}
			z = n;
		}
	}
    }


#if 0	// alexlu 2008-8-1
   for(n=0;n<TLB_SIZE;n++)
   {
       for(z=0;z<TLB_SIZE;z++)
      {
           if(tlbpd0[z] & 0x400) // PD is valid.
           {
               if( (tlbpd0[z] == tlbpd0[n]) && (z!=n))  // Duplicate TLB
               {
#if 0 //wschen 2008-07-30
                   printk("Duplicate PD's @ %u and %u\n", n,z);
                   printk("TLBPD0[%u] : %08x TLBPD0[%u] : %08x\n",n,tlbpd0[n], z, tlbpd0[z]);
                   printk("Removing one of the duplicates\n");
#endif
                   tlbpd0[n]=0;
                   tlbpd1[n]=0;

                   write_new_aux_reg(ARC_REG_TLBPD0,0);
                   write_new_aux_reg(ARC_REG_TLBPD1,0);
                   write_new_aux_reg(ARC_REG_TLBINDEX,n);
                   write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBWrite);

               }
           }
       }
    }
#endif
#if 0 //wschen 2008-07-30
return;
#endif

}
#endif
