/*
      //------------------------------------------------------------------------
      //
      //                  S K Y M E D I   P R O P R I E T A R Y
      //
      //          COPYRIGHT (c)   2003 BY SKYMEDI CORPORATION TAIWAN.
      //                   --  ALL RIGHTS RESERVED  --
      //       SKYMEDI Confidential; Need to Know only.
      //       Protected as an unpublished work.
      //
      //       The computer program listings, specifications and documentation
      //       herein are the property of SKYMEDI CORPORATION and shall not be
      //       reproduced, copied, disclosed, or used in whole or in part for
      //       any reason without the prior express written permission of
      //       SKYMEDI CORPORATION.
      //
      //          File Name:        skyconfig.c
      //
      //          Author:           Jinn Cheng
      //
      //          Created:          May 10, 2008
      //
      //          Revision:         A-1.00
      //
      //------------------------------------------------------------------------
 */

#define __SKYCONFIG_C__

#include <linux/config.h>
#include <linux/autoconf.h>
#include <linux/module.h>

#include <linux/version.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <linux/mm.h>
#include <linux/fs.h>
#include <linux/skyconfig.h>

#include <asm/irq.h>
#include <asm/io.h>
#include <asm/unaligned.h>
#include <asm/uaccess.h>

static SKYCONFIG_DATA_T            skycfg_data;
static SKYCONFIG_DRIVER_T          skycfg_driver;
static struct proc_dir_entry  	*proc_dir;

#define PROC_DATA_POINTER          &skycfg_driver
#define PROC_ENTRY_DIR			"skyconfig_proc"
#define PROC_READ_ENTRY_DIR        "skyconfig_info"

#if PRINT_DRIVER_INFO_FEAUTRE
extern size_t sky_versions_write(const char *buf, size_t count, int cr );
#define PRINT_DRIVER_INFO(_NAME,_TAG) \
     { \
          sky_versions_write( _NAME, sizeof(_NAME), 0); \
          sky_versions_write( " : ", sizeof(" : "), 0); \
          sky_versions_write( _TAG, sizeof(_TAG), 1); \
     }
#else
#define PRINT_DRIVER_INFO(_NAME,_TAG) {}
#endif

#ifdef CONFIG_LOAD_DEFAULT_H
     #include "default.h"
#endif 

const unsigned char default_usb_string_name[36] =
{
     0x00, 0x80, 0x02, 0x02, 0x1F, 0x00, 0x00, 0x00,
      'U',  'S',  'B',  ' ',  '2',  '.',  '0',  ' ',
      'M',  'u',  'l',  't',  'i',  'C',  'a',  'r',
      'd',  ' ',  'R',  'e',  'a',  'd',  'e',  'r',
      '2',  '0',  '0',  '6',
};

volatile unsigned char* pSkycfgData;

/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int cf_get_bus_enable_mode( void ) {
     return skycfg_data.cf.bus_ena_switch_mode;
}
unsigned int cf_get_bus_direction_mode( void ) {
     return skycfg_data.cf.bus_dir_switch_mode;
}
unsigned int hd_get_bus_enable_mode( void ) {
     return skycfg_data.hd.bus_ena_switch_mode;
}
unsigned int hd_get_bus_direction_mode( void ) {
     return skycfg_data.hd.bus_dir_switch_mode;
}
EXPORT_SYMBOL_NOVERS(cf_get_bus_enable_mode);
EXPORT_SYMBOL_NOVERS(cf_get_bus_direction_mode);
EXPORT_SYMBOL_NOVERS(hd_get_bus_enable_mode);
EXPORT_SYMBOL_NOVERS(hd_get_bus_direction_mode);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int cf_get_power_switch_mode( void ) {
     return skycfg_data.cf.power_control_logic;
}
unsigned int hd_get_power_switch_mode( void ) {
     return skycfg_data.hd.power_control_logic;
}
EXPORT_SYMBOL_NOVERS(cf_get_power_switch_mode);
EXPORT_SYMBOL_NOVERS(hd_get_power_switch_mode);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int msx_get_bus_switch_mode( void ) {
     return skycfg_data.msx.bus_switch_mode;
}
EXPORT_SYMBOL_NOVERS(msx_get_bus_switch_mode);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int msx_get_bus_width_sony( void ) {
     return skycfg_data.msx.bus_width_for_sony;
}
unsigned int msx_get_bus_width_non_sony( void ) {
     return skycfg_data.msx.bus_width_for_others;
}
EXPORT_SYMBOL_NOVERS(msx_get_bus_width_sony);
EXPORT_SYMBOL_NOVERS(msx_get_bus_width_non_sony);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int cf_get_max_pio_mode( void ) {
     return skycfg_data.cf.max_pio_mode;
}
unsigned int hd_get_max_pio_mode( void ) {
     return skycfg_data.hd.max_pio_mode;
}
EXPORT_SYMBOL_NOVERS(cf_get_max_pio_mode);
EXPORT_SYMBOL_NOVERS(hd_get_max_pio_mode);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int get_storage_xfer_size( unsigned int _idx ) {
     switch ( _idx ) {
          case SKYCFG_SDMMC:
               return skycfg_data.sdmmc.xfer_sectors;
          case SKYCFG_MSX:
               return skycfg_data.msx.xfer_sectors;
          case SKYCFG_XDSM:
               return skycfg_data.xdsm.xfer_sectors;
          case SKYCFG_CF:
               return skycfg_data.cf.xfer_sectors;
          case SKYCFG_HD:
               return skycfg_data.hd.xfer_sectors;
          case SKYCFG_FLASH:
               return skycfg_data.flash.xfer_sectors;
     }
     return 128;
}
EXPORT_SYMBOL_NOVERS(get_storage_xfer_size);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int get_cd_cfg( unsigned int _idx ) {
     switch ( _idx ) {
          case SKYCFG_SDMMC:
               //MYDEBUG(MYDEBUG_0,"*** %08x ***", skycfg_data.sdmmc.carddetet_mode);
               return skycfg_data.sdmmc.carddetet_mode;
          case SKYCFG_MSX:
               //MYDEBUG(MYDEBUG_0,"*** %08x ***", skycfg_data.msx.carddetet_mode);
               return skycfg_data.msx.carddetet_mode;
          case SKYCFG_XDSM:
               //MYDEBUG(MYDEBUG_0,"*** %08x ***", skycfg_data.xdsm.carddetet_mode);
               return skycfg_data.xdsm.carddetet_mode;
          case SKYCFG_CF:
               //MYDEBUG(MYDEBUG_0,"*** %08x ***", skycfg_data.cf.carddetet_mode);
               return skycfg_data.cf.carddetet_mode;
          case SKYCFG_HD:
               //MYDEBUG(MYDEBUG_0,"*** %08x ***", skycfg_data.hd.carddetet_mode);
               return skycfg_data.hd.carddetet_mode;
          case SKYCFG_FLASH:
               //MYDEBUG(MYDEBUG_0,"*** %08x ***", skycfg_data.flash.carddetet_mode);
               return skycfg_data.flash.carddetet_mode;
     }
     return 0;
}
EXPORT_SYMBOL_NOVERS(get_cd_cfg);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int get_wp_cfg( unsigned int _idx ) {
     switch ( _idx ) {
          case SKYCFG_SDMMC:
               return skycfg_data.sdmmc.writeprotect_mode;
          case SKYCFG_XDSM:
               return skycfg_data.xdsm.writeprotect_mode;
     }
     return 0;
}
EXPORT_SYMBOL_NOVERS(get_wp_cfg);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int get_led_cfg( unsigned int _idx ) {
     switch ( _idx ) {
          case SKYCFG_LED_READ:
               return skycfg_data.io.read_led_mode;
          case SKYCFG_LED_WRITE:
               return skycfg_data.io.write_led_mode;
          case SKYCFG_LED_READWRITE:
               return skycfg_data.io.readwrite_led_mode;
     }
     return 0;
}
EXPORT_SYMBOL_NOVERS(get_led_cfg);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int get_scankey_cfg( void ) {
     return skycfg_data.io.scankey;
}
EXPORT_SYMBOL_NOVERS(get_scankey_cfg);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int* get_gpio_cfg( void ) {
     return skycfg_data.io.gpio;
}
unsigned int* get_epio_cfg( void ) {
     return skycfg_data.io.epio;
}
EXPORT_SYMBOL_NOVERS(get_gpio_cfg);
EXPORT_SYMBOL_NOVERS(get_epio_cfg);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int get_reader_group( unsigned int _idx ) {
     switch ( _idx ) {
          case SKYCFG_SDMMC:
               return skycfg_data.usbdev.sdmmc_disk_idx;
          case SKYCFG_MSX:
               return skycfg_data.usbdev.msx_disk_idx;
          case SKYCFG_XDSM:
               return skycfg_data.usbdev.xdsm_disk_idx;
          case SKYCFG_CF:
               return skycfg_data.usbdev.cf_disk_idx;
          case SKYCFG_HD:
               return skycfg_data.usbdev.hd_disk_idx;
          case SKYCFG_FLASH:
               return skycfg_data.usbdev.flash_disk_idx;
     }
     return SKYCFG_NODISK_GROUP;
}
EXPORT_SYMBOL_NOVERS(get_reader_group);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int get_reader_strlen( void ) {
     return skycfg_data.usb_string_length;
}
EXPORT_SYMBOL_NOVERS(get_reader_strlen);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned char* get_reader_str( void ) {
     return skycfg_data.usb_string_name;
}
EXPORT_SYMBOL_NOVERS(get_reader_str);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int ir_get_ir3_con( void ) {
     return skycfg_data.ir.ir3_con;
}
unsigned int ir_get_ir3conA( void ) {
     return skycfg_data.ir.ir3conA;
}
unsigned int ir_get_ir3conB( void ) {
     return skycfg_data.ir.ir3conB;
}
unsigned int ir_get_ir3conC( void ) {
     return skycfg_data.ir.ir3conC;
}
EXPORT_SYMBOL_NOVERS(ir_get_ir3_con);
EXPORT_SYMBOL_NOVERS(ir_get_ir3conA);
EXPORT_SYMBOL_NOVERS(ir_get_ir3conB);
EXPORT_SYMBOL_NOVERS(ir_get_ir3conC);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int io_get_pad_strength( void ) {
     return skycfg_data.pad_clk.pad_strength;
}
unsigned int io_get_clk_strength( void ) {
     return skycfg_data.pad_clk.clk_strength;
}
EXPORT_SYMBOL_NOVERS(io_get_pad_strength);
EXPORT_SYMBOL_NOVERS(io_get_clk_strength);
/*******************************************************************************
 * fun:
 *******************************************************************************/
unsigned int io_get_toggle_interval( void ) {
     return skycfg_data.io.toggle_interal;
}
EXPORT_SYMBOL_NOVERS(io_get_toggle_interval);
/*******************************************************************************
 * fun:
 *******************************************************************************/
SKYCONFIG_DATA_T* skyconfig_get_data( void ) {
     return &skycfg_data;
}
EXPORT_SYMBOL_NOVERS(skyconfig_get_data);
/*******************************************************************************
 * fun:
 *******************************************************************************/
void skyconfig_init_data( void ) {

     int _i, _iSize;

     _iSize = sizeof( SKYCONFIG_DATA_T );

     memset( &skycfg_data, 0x0, _iSize );

     skycfg_data.sdmmc.xfer_sectors       = 8;
     skycfg_data.sdmmc.carddetet_mode     = ( SKYCFG_CD_ENABLE | SKYCFG_CD_REMOVABLE | SKYCFG_CD_USE_EIOP0 );
     skycfg_data.sdmmc.writeprotect_mode  = ( SKYCFG_WP_ENABLE | SKYCFG_WP_HIGHE_ENABKE | SKYCFG_WP_USE_GPIO_PORT | SKYCFG_WP_USE_GIO8 );

     skycfg_data.msx.xfer_sectors         = 8;
     skycfg_data.msx.carddetet_mode       = ( SKYCFG_CD_ENABLE | SKYCFG_CD_REMOVABLE | SKYCFG_CD_USE_EIOP1 );
     skycfg_data.msx.bus_switch_mode      = ( SKYCFG_MSX_BUS_SWITCH_ENABLE | SKYCFG_MSX_USE_GPIO_PORT | SKYCFG_MSX_BUS_HIGHENABLE | SKYCFG_MSX_USE_GIO7 );
     skycfg_data.msx.bus_width_for_sony   = 4;
     skycfg_data.msx.bus_width_for_others = 4;

     skycfg_data.xdsm.xfer_sectors        = 8;
     skycfg_data.xdsm.carddetet_mode      = ( SKYCFG_CD_ENABLE | SKYCFG_CD_REMOVABLE | SKYCFG_CD_USE_EIOP2 );
     skycfg_data.xdsm.writeprotect_mode   = 0;

     skycfg_data.cf.xfer_sectors          = 8;
     skycfg_data.cf.carddetet_mode        = ( SKYCFG_CD_ENABLE | SKYCFG_CD_REMOVABLE | SKYCFG_CD_USE_EIOP3 );
     skycfg_data.cf.max_pio_mode          = 4;
     skycfg_data.cf.power_control_logic   = ( SKYCFG_CF_PWR_SWITCH_ENABLE | SKYCFG_CF_USE_GPIO_PORT | SKYCFG_CF_PWR_HIGHENABLE | SKYCFG_CF_USE_GIO11 );
     skycfg_data.cf.bus_ena_switch_mode   = ( SKYCFG_CF_BUS_ENA_SWITCH_ENABLE | SKYCFG_CF_USE_GPIO_PORT | SKYCFG_CF_BUS_ENA_LOWENABLE | SKYCFG_CF_USE_GIO9 );
     skycfg_data.cf.bus_dir_switch_mode   = ( SKYCFG_CF_BUS_DIR_SWITCH_ENABLE | SKYCFG_CF_USE_GPIO_PORT | SKYCFG_CF_BUS_DIR_HIGH_IS_WRITE | SKYCFG_HD_USE_GIO10 );

     skycfg_data.hd.xfer_sectors          = 8;
     skycfg_data.hd.carddetet_mode        = ( /*SKYCFG_CD_ENABLE | SKYCFG_CD_NOT_REMOVABLE |*/ SKYCFG_CD_MAP_AS_EIOP4 );
     skycfg_data.hd.max_pio_mode          = 4;
     skycfg_data.hd.power_control_logic   = 0;
     skycfg_data.hd.bus_ena_switch_mode   = 0;
     skycfg_data.hd.bus_dir_switch_mode   = 0;

     skycfg_data.flash.xfer_sectors       = 8;
     skycfg_data.flash.carddetet_mode     = ( SKYCFG_CD_ENABLE | SKYCFG_CD_NOT_REMOVABLE | SKYCFG_CD_MAP_AS_EIOP5 );

     skycfg_data.ir.ir3_con               = 0x00000100;
     skycfg_data.ir.ir3conA               = 0x0000A2C6;
     skycfg_data.ir.ir3conB               = 0x00000067;
     skycfg_data.ir.ir3conC               = 0x00000010;

     skycfg_data.pad_clk.pad_strength     = 4; // 0 ~ 5
     skycfg_data.pad_clk.clk_strength     = 3; // 0~5

     skycfg_data.usbdev.sdmmc_disk_idx    = SKYCFG_DISK_GROUP_A;
     skycfg_data.usbdev.msx_disk_idx      = SKYCFG_DISK_GROUP_A;
     skycfg_data.usbdev.xdsm_disk_idx     = SKYCFG_DISK_GROUP_A;
     skycfg_data.usbdev.cf_disk_idx       = SKYCFG_DISK_GROUP_B;
     skycfg_data.usbdev.hd_disk_idx       = SKYCFG_NODISK_GROUP;
     skycfg_data.usbdev.flash_disk_idx    = SKYCFG_DISK_GROUP_C;

     skycfg_data.io.gpio[0]               = ( SKYCFG_GENRICIO_ENABLE | SKYCFG_GENRICIO_DIR_IS_IN );
     skycfg_data.io.gpio[1]               = ( SKYCFG_GENRICIO_ENABLE | SKYCFG_GENRICIO_DIR_IS_IN );
     skycfg_data.io.gpio[2]               = ( SKYCFG_GENRICIO_ENABLE | SKYCFG_GENRICIO_DIR_IS_IN );
     skycfg_data.io.gpio[3]               = 0;
     skycfg_data.io.gpio[4]               = 0;
     skycfg_data.io.gpio[5]               = 0;
     skycfg_data.io.gpio[6]               = 0;
     skycfg_data.io.gpio[7]               = ( SKYCFG_GENRICIO_ENABLE | SKYCFG_GENRICIO_DIR_IS_OUT | SKYCFG_GENRICIO_DEFAULT_LOGIC_0 );
     skycfg_data.io.gpio[8]               = ( SKYCFG_GENRICIO_ENABLE | SKYCFG_GENRICIO_DIR_IS_IN );
     skycfg_data.io.gpio[9]               = ( SKYCFG_GENRICIO_ENABLE | SKYCFG_GENRICIO_DIR_IS_OUT | SKYCFG_GENRICIO_DEFAULT_LOGIC_1 );
     skycfg_data.io.gpio[10]              = ( SKYCFG_GENRICIO_ENABLE | SKYCFG_GENRICIO_DIR_IS_OUT | SKYCFG_GENRICIO_DEFAULT_LOGIC_1 );
     skycfg_data.io.gpio[11]              = ( SKYCFG_GENRICIO_ENABLE | SKYCFG_GENRICIO_DIR_IS_OUT | SKYCFG_GENRICIO_DEFAULT_LOGIC_0 );
     skycfg_data.io.gpio[12]              = ( SKYCFG_GENRICIO_ENABLE | SKYCFG_GENRICIO_DIR_IS_OUT | SKYCFG_GENRICIO_DEFAULT_LOGIC_0 );
     skycfg_data.io.gpio[13]              = ( SKYCFG_GENRICIO_ENABLE | SKYCFG_GENRICIO_DIR_IS_OUT | SKYCFG_GENRICIO_DEFAULT_LOGIC_0 );
     skycfg_data.io.gpio[14]              = ( SKYCFG_GENRICIO_ENABLE | SKYCFG_GENRICIO_DIR_IS_OUT | SKYCFG_GENRICIO_DEFAULT_LOGIC_0 );
     skycfg_data.io.gpio[15]              = ( SKYCFG_GENRICIO_ENABLE | SKYCFG_GENRICIO_DIR_IS_OUT | SKYCFG_GENRICIO_DEFAULT_LOGIC_0 );

     skycfg_data.io.epio[0]               = 0;
     skycfg_data.io.epio[1]               = 0;
     skycfg_data.io.epio[2]               = 0;
     skycfg_data.io.epio[3]               = 0;
     skycfg_data.io.epio[4]               = 0;
     skycfg_data.io.epio[5]               = 0;

     skycfg_data.io.toggle_interal        = 1000; // ms
     skycfg_data.io.read_led_mode         = 0; // SKYCFG_IO_ENABLE | SKYCFG_IO_USE_GPIO_PORT | SKYCFG_IO_LOW_LIGHT | SKYCFG_IO_USE_GIO15
     skycfg_data.io.write_led_mode        = 0; // SKYCFG_IO_ENABLE | SKYCFG_IO_USE_GPIO_PORT | SKYCFG_IO_LOW_LIGHT | SKYCFG_IO_USE_GIO15
     skycfg_data.io.readwrite_led_mode    = 0; // SKYCFG_IO_ENABLE | SKYCFG_IO_USE_GPIO_PORT | SKYCFG_IO_LOW_LIGHT | SKYCFG_IO_USE_GIO15
     skycfg_data.io.scankey               = 0;

     skycfg_data.usb_string_length        = sizeof(default_usb_string_name);
     for( _i = 0; _i < skycfg_data.usb_string_length; _i++ ) {
          skycfg_data.usb_string_name[_i] = default_usb_string_name[_i];
     }
     
     skycfg_data.cfgid[0] = '2';
     skycfg_data.cfgid[1] = '8';
     skycfg_data.cfgid[2] = '5';
     skycfg_data.cfgid[3] = '0';
     skycfg_data.cfgid[4] = '-';
     skycfg_data.cfgid[5] = 'S';
     skycfg_data.cfgid[6] = '0';
     skycfg_data.cfgid[7] = '0';
}
/*******************************************************************************
 * fun:
 * input:
 * output:
 *******************************************************************************/
static int proc_read_device(char *_page, char **_start, off_t _offset, int _max_count, int *_eof, void *_data)
{
	SKYCONFIG_DRIVER_T* _pSkyConfigDrv;
	char *_p;
	unsigned char* _pConfigData;
	int _i, _j;
	if( _max_count < 1024 ) {
          MYDEBUG(MYDEBUG_1,"Page size not enough!");
          *_eof = 0;
	     return 0;
	}

	_pSkyConfigDrv = (SKYCONFIG_DRIVER_T*) _data;

	// spin_lock(&_pSkyConfigDrv->lock);

     _pConfigData = (unsigned char*) &skycfg_data;

	_p = _page;
	_i = 0;
	_p[_i++] = 'S';
	_p[_i++] = 'k';
	_p[_i++] = 'y';
	_p[_i++] = 'C';
	_p[_i++] = 'o';
	_p[_i++] = 'n';
	_p[_i++] = 'f';
	_p[_i++] = 'i';
	_p[_i++] = 'g';
	_p[_i++] = 0x0D;
	_p[_i++] = 0x0A;

	while( _i < 128 ) {
          _p[_i++] = 0x00;
	}

	// spin_unlock(&_pSkyConfigDrv->lock);

	*_eof = 1;
	return 128;
}
/*******************************************************************************
 * fun:
 * input:
 * output:
 *******************************************************************************/
static int skyconfig_init(void)
{
     int _i, _j, _iSize, _retval = 0;
     unsigned char* _pSource;
     unsigned char* _pTarget;

     // for 2850
     // (1) set isr mode
     // (2) disable isr
     // (3) read isr register in order to clear isr
     // (4) enable isr after run "_request_irq()_" ok

     skyconfig_init_data();
     
     _iSize = 512; // sizeof( SKYCONFIG_DATA_T );

     #ifdef CONFIG_LOAD_DEFAULT_H
          memset( &skycfg_data, 0x0, _iSize );
          _pTarget = (unsigned char*) &skycfg_data;
          _pSource = (unsigned char*) &default_h;                               // load from "default.h"
          for( _i = 0; _i < _iSize; _i++ ) {
               _pTarget[_i] = _pSource[_i];     
          }
     #else
          // search tag from offset 1F8h(504) of base 0x80008200h. "2850-EXX"
          pSkycfgData = (volatile unsigned char*) CONFIG_DATA_BASE;             // load from address "CONFIG_DATA_BASE"
          if( ( pSkycfgData[504] == '2' ) && 
              ( pSkycfgData[505] == '8' ) && 
              ( pSkycfgData[506] == '5' ) && 
              ( pSkycfgData[507] == '0' ) && 
              ( pSkycfgData[508] == '-' ) && 
              ( pSkycfgData[509] == 'E' ) ) {
               memset( &skycfg_data, 0x0, _iSize );
               _pTarget = (unsigned char*) &skycfg_data;
               for( _i = 0; _i < _iSize; _i++ ) {
                    _pTarget[_i] = pSkycfgData[_i];     
               }                   
          }
     #endif
     
     #if 1
          printk("\n");
          for( _i = 0; _i < 32; _i++ ) {
               printk("%08xh:", _i*16);
               for( _j = 0; _j < 16; _j++ ) {
                    printk(" %02x", _pTarget[((16*_i)+_j)]);
               }
               printk("\n");
          }          
          printk("\n");
     #endif
     #if 0
          printk("\n");
          for( _i = 0; _i < 16; _i++ ) {
               printk("%08xh:", _i*32);
               for( _j = 0; _j < 32; _j++ ) {
                    printk(" %02x", _pTarget[((32*_i)+_j)]);
               }
               printk("\n");
          }          
          printk("\n");
     #endif

     // print config version
     printk("[%s:%d] %s config ver. : ", __FILE__, __LINE__, __FUNCTION__);
     for( _i = 0; _i < 8; _i++ ) {
          printk("%c", skycfg_data.cfgid[_i] );
     }
     printk("\n");
     return _retval;
}
/*******************************************************************************
 * fun:
 * input:
 * output:
 ******************************************************************/
static void skyconfig_cleanup(void)
{
     MYDEBUG(MYDEBUG_0,THIS_FUNCTION_NAME);
}
/*******************************************************************************
 * fun: implement global variabl
 * input:
 * output:
 *******************************************************************************/
static SKYCONFIG_DRIVER_T skycfg_driver = {
     owner:     THIS_MODULE,
     name:      SKYMEDI_SKYCONFIG_DRIVER_NAME,
     id:        SKYMEDI_SKYCONFIG_DRIVER_ID,
     init:      skyconfig_init,
     cleanup:   skyconfig_cleanup,
};


/*******************************************************************************
 * fun:
 * input:
 * output:
 *******************************************************************************/
static int __init skyconfig_module_init(void)
{
     int _retval;
     MYDEBUG_NEXTLINE(MYDEBUG_0);

     MYDEBUG(MYDEBUG_1,"TAG:(%s) (%s)", SKYCONFIG_DRIVER_TAG_A, SKYCONFIG_DRIVER_TAG_B );

     PRINT_DRIVER_INFO( SKYCONFIG_DRIVER_NAME_A, SKYCONFIG_DRIVER_TAG_A );
     PRINT_DRIVER_INFO( SKYCONFIG_DRIVER_NAME_B, SKYCONFIG_DRIVER_TAG_B );

     MYDEBUG(MYDEBUG_2,THIS_FUNCTION_NAME);

     // show info. in file "/proc/bus/"PROC_ENTRY_DIR"/"PROC_READ_ENTRY_DIR"

#ifdef CONFIG_PROC_FS
	proc_dir = proc_mkdir( PROC_ENTRY_DIR, proc_bus);
	if ( proc_dir ) {
		create_proc_read_entry( PROC_READ_ENTRY_DIR, 0, proc_dir, proc_read_device, PROC_DATA_POINTER );
		MYDEBUG(MYDEBUG_2,"To create proc-bus-dir(..proc and ..info) ok!");
	}
	else {
		MYDEBUG(MYDEBUG_0,"To create proc-bus-dir(..proc and ..info) fail!");
	}
#else
	MYDEBUG(MYDEBUG_0,"#undef CONFIG_PROC_FS");
#endif

     _retval = skyconfig_init();

#ifdef CONFIG_DEVFS_FS
     // add your code here
     _retval = -1;
     MYDEBUG(MYDEBUG_0,"Kernel not support CONFIG_DEVFS_FS architecture!");
#else
     if( _retval >= 0 ) {
          // register character driver or block driver
     }
#endif
     if( _retval >= 0 ) {
          MYDEBUG(MYDEBUG_2,"Install skyconfig loader success!");
     }
     else {
          MYDEBUG(MYDEBUG_0,"Install skyconfig loader fail!");
          return -1;
     }
     return _retval;
}
/*******************************************************************************
 * fun:
 * input:
 * output:
 *******************************************************************************/
void __exit skyconfig_module_cleanup(void)
{
     MYDEBUG_NEXTLINE(MYDEBUG_0);
     MYDEBUG(MYDEBUG_0,THIS_FUNCTION_NAME);

#ifndef CONFIG_DEVFS_FS
     // free buffer if allocate
     // un-register character or block driver
#endif

     skyconfig_cleanup();

#ifdef CONFIG_PROC_FS
	if ( proc_dir ) {
		MYDEBUG(MYDEBUG_0,"To remove proc-bus-dir(..proc and ..info)");
		remove_proc_entry("PROC_READ_ENTRY_DIR", proc_dir);
	}
	else {
		MYDEBUG(MYDEBUG_0,"Not necessary to remove proc-bus-dir(..proc and ..info)");
	}
     remove_proc_entry("PROC_ENTRY_DIR", proc_bus);
#endif
}
/*******************************************************************************
 *
 *
 *
 *******************************************************************************/
module_init(skyconfig_module_init);
module_exit(skyconfig_module_cleanup);

MODULE_LICENSE("NON-GPL");
MODULE_AUTHOR("Tsair-Jinn Cheng <jinncheng@skymedi.com.tw>");
MODULE_DESCRIPTION("Skymedi Corp. SkyConfig Driver");
