/*
 *	terawins_tcon.h - Register and variable definitions for TeraWins T-CON.
 */
#include "arc_pgu.h"

#ifndef _TERAWINS_TCON_H_
#define _TERAWINS_TCON_H_

/* Terawins T-CON device address. */
#define TERAWINS_PAGE0_ADDR		0x40
#define TERAWINS_PAGE1_ADDR		0x42

enum TerawinsDim
{
	TERAWINS_INIT,
	TERAWINS_640x480,
	TERAWINS_720x480,
	TERAWINS_720x576,
	TERAWINS_800x600,
	TERAWINS_END,
};


/* Definitions for I2C function. */
typedef enum{
	GPIO_DIR = 0x0,
	GPIO_DATA = 0x4,
}SKY_GPIO_REG;

//-- Select EVB board.
#if CONFIG_SOUND_PLATFORM == 0
#define I2C_SCL_MASK			0x00000010		// GPIO 4
#define I2C_SDA_MASK			0x00000020		// GPIO 5
#else
#define I2C_SCL_MASK			0x00000001		// GPIO 0
#define I2C_SDA_MASK			0x00000002		// GPIO 1
#endif

//-- SCLK
#define SCL_INPUT \
{\
	vtval = GIO_R32(GPIO_DIR);\
	vtval = GIO_R32(GPIO_DIR);\
	vtval &= (~I2C_SCL_MASK);\
	GIO_W32(GPIO_DIR, vtval);\
}
#define SCL_OUTPUT \
{ \
	vtval = GIO_R32(GPIO_DIR);\
	vtval = GIO_R32(GPIO_DIR);\
	vtval |= (I2C_SCL_MASK);\
	GIO_W32(GPIO_DIR, vtval);\
}
#define SCL_HI \
{ \
	vtval = GIO_R32(GPIO_DATA);\
	vtval = GIO_R32(GPIO_DATA);\
	vtval |= (I2C_SCL_MASK);\
	GIO_W32(GPIO_DATA, vtval);\
}
#define SCL_LO \
{ \
	vtval = GIO_R32(GPIO_DATA);\
	vtval = GIO_R32(GPIO_DATA);\
	vtval &= (~I2C_SCL_MASK);\
	GIO_W32(GPIO_DATA, vtval);\
}
#define SCL \
{ \
	vtval = GIO_R32(GPIO_DATA);\
	vtval = GIO_R32(GPIO_DATA);\
	vtval &= (I2C_SCL_MASK);\
}

//-- SDA
#define SDA_INPUT \
{\
	vtval = GIO_R32(GPIO_DIR);\
	vtval = GIO_R32(GPIO_DIR);\
	vtval &= (~I2C_SDA_MASK);\
	GIO_W32(GPIO_DIR, vtval);\
}
#define SDA_OUTPUT \
{ \
	vtval = GIO_R32(GPIO_DIR);\
	vtval = GIO_R32(GPIO_DIR);\
	vtval |= I2C_SDA_MASK;\			
	GIO_W32(GPIO_DIR, vtval);\
}
#define SDA_HI \
{ \
	vtval = GIO_R32(GPIO_DATA);\
	vtval = GIO_R32(GPIO_DATA);\
	vtval |= I2C_SDA_MASK;\
	GIO_W32(GPIO_DATA, vtval);\
}
#define SDA_LO \
{ \
	vtval = GIO_R32(GPIO_DATA);\
	vtval = GIO_R32(GPIO_DATA);\
	vtval &= (~I2C_SDA_MASK);\
	GIO_W32(GPIO_DATA, vtval);\
}
#define SDA \
{ \
	vtval = GIO_R32(GPIO_DATA);\
	vtval = GIO_R32(GPIO_DATA);\
	vtval &= (I2C_SDA_MASK);\
}

#define TERAWINS_TIME_1 			1//(833.3KHz)
#define TERAWINS_TIME_2 			1//(833.3KHz)


/* Register definitions for TeraWins OSD control. */
enum{
	OSD_CFG_INDEX 		= 0xA0,
	OSD_CFG_DATA 		= 0xA1,
	OSD_RAM_ADDR_LO 	= 0xA2,
	OSD_RAM_ADDR_HI 	= 0xA3,
	OSD_RAM_DATA 		= 0xA4,
};

enum TERAWINS_OSD_CMD{
	OSD_CTRL			= 0x00,
	CHAR_DLY1			= 0x01,
	CHAR_DLY2			= 0x02,
	CHAR_DLY3			= 0x03,
	CHAR_FONT_SIZE		= 0x04,
	CHAR_RAM_START_ADDR	= 0x05,
	CHAR_BS_CTRL		= 0x06,
	CHAR_BS_COLOR		= 0x07,
	CHAR_HEIGHT_SCALING	= 0x08,
	BLINKING_CTRL		= 0x09,
	BMAP_WIN_SIZE_UPPER	= 0x0A,
	BMAP_WIN_SIZE_WIDTH	= 0x0B,
	BMAP_WIN_SIZE_HEIGHT= 0x0C,	
	BMAP_DOT_ENLARGE	= 0x0D,
	OSD_COLOR_LUT_PORT	= 0x0E,
	CHAR_RAM_ACCESS_CTRL= 0x0F,
	W1_START_ROW_NUM	= 0x10,
	W1_END_ROW_NUM		= 0x11,
	W1_START_COL_NUM	= 0x12,
	W1_END_COL_NUM		= 0x13,	
	W1_SHADOW_SIZE		= 0x14,
	CHAR2BP_IDX_BASE	= 0x19,
	CHAR2BP_BASEADDR_LO	= 0x1A,	
	CHAR2BP_BASEADDR_HI	= 0x1B,	
	ALPHA_BLENDING_CTRL	= 0x1C,	
	REVISION_ID			= 0x1D,
	CHAR_RAM_STOP_ADDR	= 0x1E,
};

struct _terawins_reg
{
	unsigned char osd_ctrl;
	unsigned char dly1;
	unsigned char dly2;
	unsigned char dly3;
	unsigned char font_size;
	unsigned char ram_start;
	unsigned char bs_ctrl;
	unsigned char bs_color;	
	
	unsigned char h_scaling;
	unsigned char blink_ctrl;
	unsigned char bmap_u;
	unsigned char bmap_w;
	unsigned char bmap_h;
	unsigned char bmap_large;
	unsigned char lut_port;
	unsigned char acc_ctrl;	

	unsigned char start_row;
	unsigned char end_row;
	unsigned char start_col;
	unsigned char end_col;
	unsigned char sha_size;
	unsigned char c2bp_idx;
	unsigned char c2bp_addrlo;
	unsigned char c2bp_addrhi;	

	unsigned char alpha_ctrl;	
	unsigned char rev_id;
	unsigned char ram_stop;	
};


/* Data structure for OSD info. */
struct _terawins_line_info
{
	int *char_idx;
	unsigned char char_attr;		// bit[7:5]=BG, bit[4]=blink, bit[3:0]=FG, 
	unsigned char row_attr;			// bit[7]=RGAP_BG(1:show background color, 0:transparent),
									// bit[6:2]=RGAP (Row gap/spacing heigh (4*RGAP).)
									// bit[1]=CHS(char height be double)
									// bit[0]=CWS(char width be double)
};

struct _terawins_window_info
{
	unsigned char row_start;
	unsigned char row_end;
	unsigned char col_start;
	unsigned char col_end;
	unsigned char shadow_size;
};

struct _terawins_menu_info
{
	unsigned short h_start;
	unsigned short v_start;
	unsigned char row_index;
	unsigned char num_rows;
};

enum{
	FONT_10x16 = 0,
	FONT_12x18,
	FONT_16x24,
	FONT_END,
};

struct _font_info
{
	unsigned long font_size;	// register val
	unsigned char width;		// font width
	unsigned char height;		// font height
	unsigned char font_nums;	// total font nums
};


#define TERAWINS_OSD_RAM_SIZE				(6*1024)	// byte
#define CHAR_PER_LINE						30
#define BYTES_PER_LINE_INFO					64

#define FONT_WIDTH							font_info[font_index].width
#define FONT_HEIGHT							font_info[font_index].height
#define FONT_WIDTH_BYTES					((FONT_WIDTH + 7) / 8)

#define CHAR_START_ADDR(num_of_font)		( (((((FONT_WIDTH_BYTES*FONT_HEIGHT*num_of_font)/32)+1)*32)+63)/64 )
#define CHAR_START_RAM_ADDR(num_of_font)	( CHAR_START_ADDR(num_of_font) * 32 )


/* Define the Foreground color and Background color. */
enum FG_color{
	FG_COLOR0 = 0,
	FG_COLOR1,
	FG_COLOR2,
	FG_COLOR3,
	FG_COLOR4,
	FG_COLOR5,
	FG_COLOR6,
	FG_COLOR7,
	FG_COLOR8,
	FG_COLOR9,
	FG_COLOR10,
	FG_COLOR11,
	FG_COLOR12,
	FG_COLOR13,
	FG_COLOR14,
	FG_COLOR15,	
};

enum BG_color{
	BG_COLOR0 = 0,
	BG_COLOR1,
	BG_COLOR2,
	BG_COLOR3,
	BG_COLOR4,
	BG_COLOR5,
	BG_COLOR6,
	BG_COLOR7,
};


/* Function declarations. */
int init_terawins_tcon(void);
int terawins_on_off(unsigned char _on);
int terawins_set_cmd(unsigned char dev_addr, unsigned char reg_addr, unsigned char reg_val);
int terawins_set_resolution(unsigned char idx);
void terawins_clear_osd_ram(void);
unsigned char terawins_load_osd_fonts(unsigned char** _font_array);

void terawins_set_osd_window_info(
				unsigned char r_start, unsigned char r_end,
				unsigned short c_start, unsigned short c_end,
				unsigned char win_en, unsigned char win_int, unsigned char win_color,
				unsigned char sha_en, unsigned char sha_gray, unsigned char sha_color,
				unsigned char sha_w, unsigned char sha_h,
				struct _terawins_window_info *_info );

void terawins_set_osd_menu_info(unsigned short h_start, 
										   unsigned short v_start,
										   unsigned char row_index,
										   unsigned char n_rows,	
						 				   struct _terawins_menu_info *_info);

void terawins_set_osd_line_info(int idx[30], unsigned char c_blink,
						 	unsigned char c_fg_c, unsigned char c_bg_c, 
						 	unsigned char rgap_bg, unsigned char rgap, 
						 	unsigned char r_chs, unsigned char r_cws,
						 	struct _terawins_line_info *_info);

void terawins_set_osd_window(struct _terawins_window_info *_info);
void terawins_set_osd_line(struct _terawins_line_info *_info);
void terawins_clear_osd_line(unsigned char _num);
void terawins_set_osd_menu(struct _terawins_menu_info *_info);
void terawins_show_osd_menu(void);
void terawins_set_osd_alpha_blending(unsigned char mode, unsigned char val);
void terawins_set_osd_blinking(unsigned char blk_freq, unsigned char duty);
void terawins_enable_osd_win(struct _terawins_window_info *_info);
void terawins_disable_osd_win(struct _terawins_window_info *_info);
void terawins_enable_osd(void);
void terawins_disable_osd(void);
void terawins_pwm_on_off(uint8 _on);
void terawins_backlight_on_off(uint8 _on);
void terawins_set_pwm(uint8 _val);
void terawins_set_brightness(uint8 _val);
void terawins_set_contrast(uint8 _val);
void terawins_show_mute_vol(uint32 idx);
void terawins_clear_mute_vol(void);
void terawins_show_icon(uint32 _idx);
void terawins_clear_icon(void);
void terawins_show_subtitle(char _s[][31]);
void terawins_clear_subtitle(void);
void terawins_clear_all_osd_item(void);
void terawins_show_time(char *_t);
void terawins_clear_time(void);
void init_terawins_osd(void);

void test_terawins(unsigned int _val);

#endif

