/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  linux/include/asm-arc/arcregs.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Authors: Amit Bhor
 * Auxiliary register definitions and macros to read and write to them. 
 */

#ifndef	_ASM_ARC_ARCDEFS_H
#define	_ASM_ARC_ARCDEFS_H

/* These masks correspond to the status word(STATUS_32) bits */
#define	STATUS_H_MASK	0x01	/* Mask for Halt bit */
#define	STATUS_E1_MASK	0x02	/* Mask for Inerrupt 1 enable */
#define	STATUS_E2_MASK	0x04	/* Mask for Inerrupt 2 enable */
#define	STATUS_A1_MASK	0x08	/* Interrupt 1 active */
#define	STATUS_A2_MASK	0x10	/* Interrupt 2 active */
#define	STATUS_AE_MASK	0x20	/* Exception active */
#define	STATUS_DE_MASK	0x40	/* PC is in delay slot */
#define STATUS_U_MASK	0x80	/* User/Kernel mode bit */
#define STATUS_L_MASK	0x1000	/* Loop inhibit bit */

/* These masks correspond to the exception cause register (ecause) bits */

#define ECAUSE_VECTOR_MASK     0xff0000
#define ECAUSE_CODE_MASK       0x00ff00
#define ECAUSE_PARAMETER_MASK  0x0000ff


/* Auxiliary registers not supported by the assembler */
#define AUX_IDENTITY    4
#define AUX_IRQ_LEV     0x200   /* interrupt priority level setting */
#define AUX_IRQ_HINT	0x201	/* Aux register for generating software interrupts */
#define AUX_IRQ_LV12	0x43	/* interrupt level register */

#define AUX_IENABLE     0x40c   /* Known to the assembler as auxienable, so don't really need this */
#define AUX_ITRIGGER    0x40d
#define AUX_IRQ_PULSE_CANCEL	0x415

/* Privileged MMU auxiliary register definitions */
#define	ARC_REG_TLBPD0		0x405
#define	ARC_REG_TLBPD1		0x406
#define	ARC_REG_TLBINDEX	0x407	
#define	ARC_REG_TLBCOMMAND	0x408
#define	ARC_REG_PID 		0x409
#define	ARC_REG_SCRATCH_DATA0	0x418

/* Instruction cache related Auxiliary registers */
#define ARC_REG_I_CACHE_BUILD_REG 0x77
#define ARC_REG_IC_IVIC		0x10
#define ARC_REG_IC_CTRL		0x11
#define ARC_REG_IC_IVIL     0x19
/* Data cache related Auxiliary registers */
#define ARC_REG_D_CACHE_BUILD_REG 0x72
#define ARC_REG_DC_IVDC		0x47
#define ARC_REG_DC_CTRL		0x48
#define ARC_REG_DC_IVDL     0x4A
#define ARC_REG_DC_FLSH     0x4B
#define ARC_REG_DC_FLDL     0x4C

/* Timer related Aux registers */
#define ARC_REG_TIMER0_LIMIT 0x23 /* timer 0 limit */
#define ARC_REG_TIMER0_CTRL  0x22 /* timer 0 control */
#define ARC_REG_TIMER0_CNT   0x21 /* timer 0 count */
#define ARC_REG_TIMER1_LIMIT 0x102 /* timer 1 limit */
#define ARC_REG_TIMER1_CTRL  0x101 /* timer 1 control */
#define ARC_REG_TIMER1_CNT   0x100 /* timer 1 count */

#ifdef CONFIG_ARC700_SSRAM
/* Synchronous SRAM controller registers */
#define SSRAM_CTRL_BASE	0xc0fc7000	/* Base of the controller registers */
#define SSRAM_CTRL_ID		SSRAM_CTRL_BASE + 0x0
#define SSRAM_CTRL_B0CNFR1	SSRAM_CTRL_BASE + 0x2
#define SSRAM_CTRL_B0CNFR2	SSRAM_CTRL_BASE + 0x3
#define SSRAM_CTRL_B1CNFR1	SSRAM_CTRL_BASE + 0x6
#define SSRAM_CTRL_B1CNFR2	SSRAM_CTRL_BASE + 0x7
#define SSRAM_CTRL_B2CNFR1	SSRAM_CTRL_BASE + 0xa
#define SSRAM_CTRL_B2CNFR2	SSRAM_CTRL_BASE + 0xb
#endif /* CONFIG_ARC700_SSRAM */

#define TLBPD1_MASK 0xffffe1fc /* Mask of bits to be written to the TLBPD1 */

#ifndef	__ASSEMBLY__
/* Read an auxiliary register */
#if 0 //wschen 2008-07-30
#define	read_aux_reg(reg)						\
({ unsigned int __ret;							\
	__asm__ __volatile__("lr	%0, ["#reg"]":"=r"(__ret));	\
	__ret;								\
})
		
/* Write to an auxiliary register */
#define	write_aux_reg(reg, val)					\
({ 									\
	__asm__ __volatile__ ("sr	%0, ["#reg"]"::"r"(val));		\
})


#define	read_new_aux_reg(reg)						\
({ unsigned int __ret;							\
	__asm__ __volatile__("lr	%0, [%1]":"=r"(__ret):"i"(reg));\
	__ret;								\
})
		
/* Write to an auxiliary register */

#define	write_new_aux_reg(reg, val)					\
({ 									\
	__asm__ __volatile__ ("sr	%0, [%1]"::"r"(val),"i"(reg));		\
})
#else
#define	read_aux_reg(reg)						\
({ volatile unsigned int __ret;							\
	__asm__ __volatile__("lr	%0, ["#reg"]":"=r"(__ret)::"memory");	\
	__ret;								\
})
		
/* Write to an auxiliary register */
#define	write_aux_reg(reg, val)					\
({ 									\
	__asm__ __volatile__ ("sr	%0, ["#reg"]"::"r"(val):"memory");		\
})


#define	read_new_aux_reg(reg)						\
({ volatile unsigned int __ret;							\
	__asm__ __volatile__("lr	%0, [%1]":"=r"(__ret):"i"(reg):"memory");\
	__ret;								\
})
		
/* Write to an auxiliary register */

#define	write_new_aux_reg(reg, val)					\
({ 									\
	__asm__ __volatile__ ("sr	%0, [%1]"::"r"(val),"i"(reg):"memory");		\
})
#endif
#endif	/* __ASEMBLY__ */

#endif	/* _ASM_ARC_ARCDEFS_H */
