#ifndef __ASM_ATAG_H
#define __ASM_ATAG_H

//#include <linux/types.h>

/*
 * The new way of passing information: a list of tagged entries
 */

/* The list ends with an ATAG_NONE node. */

#define ATAG_NONE	0x00000000

struct tag_header {
	unsigned int size;
	unsigned int tag;
};

/* The list must start with an ATAG_CORE node */
#define ATAG_CORE	0x54410001

struct tag_core {
};

/* it is allowed to have multiple ATAG_MEM nodes */
#define ATAG_MEM	0x54410002

struct tag_mem32 {
	unsigned int	size;
};


/* clock speed */
#define ATAG_CLK_SPEED  0x5441003

struct tag_clk_speed {
  unsigned int clk_speed_hz;
};
  


/* describes how the ramdisk will be used in kernel */
#define ATAG_RAMDISK	0x54410004

struct tag_ramdisk {
	unsigned int flags;	/* bit 0 = load, bit 1 = prompt */
	unsigned int size;	/* decompressed ramdisk size in _kilo_ bytes */
	unsigned int start;	/* starting block of floppy-based RAM disk image */
};

/* describes where the compressed ramdisk image lives (virtual address) */
/*
 * this one accidentally used virtual addresses - as such,
 * its depreciated.
 */
#define ATAG_INITRD	0x54410005

/* describes where the compressed ramdisk image lives (physical address) */
#define ATAG_INITRD2	0x54420005

struct tag_initrd {
	unsigned int start;	/* physical start address */
	unsigned int size;	/* size of compressed ramdisk image in bytes */
};


/* configuring cache */ 
#define ATAG_CACHE     0x54420006
struct tag_cache {
         unsigned short icache;
         unsigned short dcache;  
};



/* describes the configuration of serial controller */


#define ATAG_SERIAL 0x54420007
struct tag_serial {
       unsigned int serial_nr;
       unsigned int baudrate;
};

/* describes the configuration of vmac controller */

#define ATAG_VMAC 0x54420008
struct tag_vmac {
       unsigned char addr[8];
};



/* command line: \0 terminated string */
#define ATAG_CMDLINE	0x54410009

struct tag_cmdline {
	char	cmdline[1];	/* this is the minimum size */
};


struct tag {
	struct tag_header hdr;
	union {
		struct tag_core		core;
		struct tag_mem32	mem;
		struct tag_ramdisk	ramdisk;
		struct tag_initrd	initrd;
		struct tag_cmdline	cmdline;
	        struct tag_cache        cache;
                struct tag_clk_speed    clk_speed;
 	        struct tag_serial       serial;
	        struct tag_vmac         vmac;

	} u;
};



#define tag_next(t)	((struct tag *)((unsigned int *)(t) + (t)->hdr.size))
#define tag_size(type)	((sizeof(struct tag_header) + sizeof(struct type)) >> 2)

struct tagtable {
	unsigned int tag;
	int (*parse)(struct tag *);
};

//#define __tag __attribute__((unused, __section__(".taglist")))
#define __tag __attribute_used__ __attribute__ ((__section__(".taglist")))

#define __tagtable(tag, fn) \
static struct tagtable __tagtable_##fn __attribute_used__ __tag = { tag, fn }



#endif /* __ASM_ATAG_H */

