/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  linux/include/asm-arcnommu/delay.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Authors : Amit Bhor
 * Delay routines using pre computed loops_per_jiffy value.
 */

#ifndef __ASM_ARC_UDELAY_H
#define __ASM_ARC_UDELAY_H

extern __inline__ void __delay(unsigned long loops)
{
      __asm__ __volatile__ ( "1: \n\t"
			     "sub.f %0, %1, 1\n\t"
			     "jpnz 1b"
			     : "=r" (loops)
			     : "0" (loops));
}

extern void __bad_udelay(void); /* Undefined to cause compilation failure if constant delay value is insanely large */

/*
 * Use only for very small delays ( < 1 msec).  Should probably use a
 * lookup table, really, as the multiplications take much too long with
 * short delays.  This is a "reasonable" implementation, though (and the
 * first constant multiplications gets optimized away if the delay is
 * a constant)  
 */
static inline void __const_udelay(unsigned long xloops)
{
#if 0
	__asm__ ("mpyhu %0, %1, %2"
		 : "=r" (xloops)
		 : "r" (xloops), "r" (loops_per_jiffy));
       __delay(xloops * HZ);

#else
	unsigned long tmp;

#if HZ <= 1000 && !(1000 % HZ)
        tmp = (xloops + (1000 / HZ) - 1) / (1000 / HZ);
#elif HZ > 1000 && !(HZ % 1000)
        tmp= xloops * (HZ / 1000);
#else
        tmp =  (xloops * HZ + 999) / 1000;
#endif

	__delay(tmp);

	/* FIXME - the original delay loop above appears to go into an indefinite loop.
	   The code above is lifted from another port. */
/* 	__asm__ ("mpyu %2, %0, %1" */
/* 		 : "=r" (xloops),"=r" (tmp) */
/* 		 : "r" (xloops), "1" (loops_per_jiffy)); */
/* 	__delay(tmp); */
#endif
}

static inline void __udelay(unsigned long usecs)
{
	__const_udelay(usecs * 4295);	/* 2**32 / 1000000 */
}

#define udelay(n) (__builtin_constant_p(n) ? \
	((n) > 20000 ? __bad_udelay() : __const_udelay((n) * 4295)) : \
	__udelay(n))


static inline void __ndelay(unsigned long nsecs)
{
	__const_udelay(nsecs * 4);	/* 2**32 / 1000000000 */
}

#define ndelay(n) (__builtin_constant_p(n) ? \
	((n) > 20000 ? __bad_udelay() : __const_udelay((n) * 4)) : \
	__ndelay(n))

#endif	/* __ASM_ARC_UDELAY_H */
