/*
 *	linux/include/asm-arc/ide.h
 *
 *	ARC IDE Interface
 *
 *	Copyright (C) 1994-2005	 Linus Torvalds & authors
 *	ARC IDE functions - ARC International, 2005
 *
 */


#ifndef __ASMARC_IDE_H
#define __ASMARC_IDE_H

#ifdef __KERNEL__

#ifdef MAX_HWIFS
#undef MAX_HWIFS
#endif 
#define MAX_HWIFS				1


/*
 * We always use the new IDE port registering,
 * so these are fixed here.
 */
#define ide_default_io_base(i)					((ide_ioreg_t)0)
#define ide_default_irq(b)							(10)

#include <asm-generic/ide_iops.h>

/* This is what our controller looks like: */

typedef struct {
	volatile unsigned long ID;
	volatile unsigned long timing;
	volatile unsigned long statctrl;
} ARC_IDE_if;

/* Aurora-defined base address */
#define IDE_CONTROLLER_BASE	 0xc0fc9000

/* Then the drive's registers map in at 0x80 above the controller base address */
#define DRIVE_REGISTER_OFFSET 0xA0
#define DRIVE_STATCTRL_OFFSET 0xD8
#define DRIVE_STATCTRL_REL_OFFSET (DRIVE_STATCTRL_OFFSET - DRIVE_REGISTER_OFFSET) /* FIXME there must be a better way of doing this */

/* statctrl bits */
#define IDE_STATCTRL_IS 0x20
#define IDE_STATCTRL_RS 4
#define IDE_STATCTRL_IC 2
#define IDE_STATCTRL_IE 1

#define ARC_IDE_INTERRUPT 13

/*
 * Set up a hw structure for a specified data port, control port and IRQ.
 * This should follow whatever the default interface uses.
 */
static __inline__ void
ide_init_hwif_ports(hw_regs_t *hw, int data_port, int control_port, int* irq)
{
	ide_ioreg_t reg = (ide_ioreg_t) data_port;		 /* ide_ioreg_t is actually an unsigned char */
	int i;

	memset(hw, 0, sizeof(*hw));

	for (i = IDE_DATA_OFFSET; i <= IDE_STATUS_OFFSET; i++) {
		hw->io_ports[i] = reg;
		reg += 4;
	}

	hw->io_ports[IDE_CONTROL_OFFSET] = (ide_ioreg_t) (data_port + DRIVE_STATCTRL_REL_OFFSET);
}

/*
 * This registers the standard ports for this architecture with the IDE
 * driver.
 */
static __inline__ void
ide_init_default_hwifs(void)
{
	hw_regs_t hw;
/*	int tmpint = initializing; */

/*	initializing = 1; */
	printk(KERN_INFO "ARC IDE interface driver, initing drive registers\n");
	ide_init_hwif_ports(&hw, IDE_CONTROLLER_BASE+DRIVE_REGISTER_OFFSET, 0, NULL);
	hw.irq = ARC_IDE_INTERRUPT;
	hw.dma = NO_DMA;
/*	if (ide_register_hw(&hw, NULL) == -1) */
/*		printk("Error registering IDE interface\n"); */
	ide_register_hw(&hw, NULL);
/*	initializing = tmpint; */
}

#endif /* __KERNEL__ */

#endif 
