/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  linux/include/asm-arc/processor.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Amit Bhor
 */

#ifndef __ASM_ARC_PROCESSOR_H
#define __ASM_ARC_PROCESSOR_H

#include <linux/linkage.h>

/*
 * Default implementation of macro that returns current
 * instruction pointer ("program counter").
 * Should the PC register be read instead ? This macro does not seem to
 * be used in many places so this wont be all that bad.
 */
#define current_text_addr() ({ __label__ _l; _l: &&_l;})

typedef unsigned long mm_segment_t;		/* domain register	*/

#ifdef __KERNEL__
#include <linux/config.h>
#include <asm/system.h>
#include <asm/atomic.h>
#include <asm/ptrace.h>
/* FIXME :: 
 * Most of the architectures seem to be keeping some kind of padding between
 * userspace TASK_SIZE and PAGE_OFFSET. i.e TASK_SIZE != PAGE_OFFSET. 
 * I'm not sure why it is so. We will keep this padding for now and remove it
 * later if not required. 
 */
#define ARC_SOME_PADDING	0x10000000
/*
 * User space process size:
 * 2GB - kernel VM size (CONFIG_VMALLOC_SIZE) - ARC_SOME_PADDING.
 * This is hardcoded into a few places,so don't change it unless you know what
 * you are doing.
 */
#define TASK_SIZE	(PAGE_OFFSET - (CONFIG_VMALLOC_SIZE * 1024 * 1024) - ARC_SOME_PADDING)

/* This decides where the kernel will search for a free chunk of vm
 * space during mmap's.
 */
#define TASK_UNMAPPED_BASE      (TASK_SIZE / 3)

/* THREAD_SIZE is the size of the task_struct/kernel_stack union.
 * It is normally defined as 8k ( two 4k pages). The A700 page is 8k so we 
 * define this to page size. No point in keeping this smaller since 
 * the task will be allocated in multiples of pages. 
 */

#define THREAD_SIZE       PAGE_SIZE
#define KERNEL_STACK_SIZE PAGE_SIZE

struct thread_struct {
	unsigned long	ksp;	/* kernel mode stack pointer */
	unsigned long	usp;	/* user mode stack pointer */
	unsigned long	pc;	/* status and PC */
	unsigned long   fault_address; /* fault address when exception occurs */
	unsigned long   error_code;    /* exception cause code value */
};

#define INIT_THREAD  {							\
	ksp: sizeof(init_stack) + (unsigned long) init_stack,	\
	usp:	 0							\
}

/*
 * Return saved PC of a blocked thread.
 */
extern __inline__ unsigned long thread_saved_pc(struct thread_struct *t)
{
	return t->pc;
}

/* Forward declaration, a strange C thing */
struct task_struct;


/* Allocating and freeing of basic task structures */
#define alloc_task_struct()	\
	((struct task_struct *) __get_free_pages(GFP_KERNEL,0))

#define free_task_struct(p)	free_pages((unsigned long) (p), 0)

#define init_task	(init_task_union.task)
#define init_stack	(init_task_union.stack)

#define cpu_relax()    do { } while (0)

/*
 * Create a new kernel thread
 */
extern int arch_kernel_thread(int (*fn)(void *), void *arg, unsigned long flags);

#define copy_segments(tsk, mm)		do { } while (0)
#define release_segments(mm)		do { } while (0)

#define KSTK_EIP(tsk)	tsk->thread.pc
#define KSTK_ESP(tsk)	tsk->thread.usp
	
#define get_task_struct(tsk)	atomic_inc(&virt_to_page(tsk)->count)

/*
 * Do necessary setup to start up a newly executed thread.
 *
 * The ilink reg where we have to store the pc... ret_from_syscall
 * will finally pass control to us in this way.
 * the status32 register has User mode, E1, E2 bits set
 *
 * pass the data segment into user programs if it exists,
 * it can't hurt anything as far as I can tell
 *
 */
#define start_thread(_regs, _pc, _usp)			\
do {							\
	set_fs(USER_DS); /* reads from user space */	\
	(_regs)->ret = (_pc);				\
	/* User mode, E1 and E2 enabled */		\
	(_regs)->status32 = STATUS_U_MASK		\
				 | STATUS_E1_MASK	\
				 | STATUS_E2_MASK;	\
	(_regs)->sp = (_usp);			        \
} while(0)

#endif	/* __KERNEL__ */
#endif	/* __ASM_ARC_PROCESSOR_H */
