#ifndef SKY_OT_H_
#define SKY_OT_H_

#include <asm/types.h>

#define OT_ADDR			0x2C
#define OT_RE_ADDR		0x59
#define OT_WR_ADDR		0x58

/* configuration registers */
#define IF_CFG_HI			0x00
#define IF_CFG_LO			0x00
	#define DATA_EN			0x04
	#define ATTN_HI			0x01
	#define ATTN_EN			0x02

#define GEN_CFG_HI		0x00
#define GEN_CFG_LO		0x01
	#define STR_BTN_OLY		0x20
	#define FIR_BTN_OLY		0x30
#define BTN_EN_HI			0x00
#define BTN_EN_LO		0x04
#define GPIO_CTL_HI		0x00
#define GPIO_CTL_LO		0x0E
#define SEN_10_HI			0x00
#define SEN_10_LO		0x10
#define SEN_32_HI			0x00
#define SEN_32_LO		0x11
#define SEN_54_HI			0x00
#define SEN_54_LO		0x12
#define SEN_76_HI			0x00
#define SEN_76_LO		0x13
#define SEN_98_HI			0x00
#define SEN_98_LO		0x14
#define SEN_BA_HI			0x00
#define SEN_BA_LO		0x15
#define CUR_CTL_EN_HI	0x00
#define CUR_CTL_EN_LO	0x22
#define INTEN_10_HI		0x00
#define INTEN_10_LO		0x23
#define INTEN_32_HI		0x00
#define INTEN_32_LO		0x24

/* data registers */
#define DUMMY_REG_HI	0x01
#define DUMMY_REG_LO	0x06
#define GPIO_STS_HI		0x01
#define GPIO_STS_LO		0x08
	#define FRESH_BIT			0x80
#define BTN_STS_HI		0x01
#define BTN_STS_LO		0x09
#define PRE_VAL_10_HI	0x01
#define PRE_VAL_10_LO	0x0C
#define PRE_VAL_32_HI	0x01
#define PRE_VAL_32_LO	0x0D
#define PRE_VAL_54_HI	0x01
#define PRE_VAL_54_LO	0x0E
#define PRE_VAL_76_HI	0x01
#define PRE_VAL_76_LO	0x0F

/* reset register */
#define RESET_HI			0x03
#define RESET_LO			0x00
#define RST_HI			0x00
#define RST_LO			0x01

typedef struct _OT_CONFIG{
	u8 reg_hi;
	u8 reg_lo;
	u8 data_hi;
	u8 data_lo;
} SKY_OT_CONFIG, *SKY_OT_CONFIG_PTR;

static SKY_OT_CONFIG g_OT_Config[] = 
{
	{IF_CFG_HI, IF_CFG_LO, 0x00, 0x00},
	{IF_CFG_HI, IF_CFG_LO, 0x00, DATA_EN | ATTN_EN},	
	{GEN_CFG_HI, GEN_CFG_LO, 0x00, FIR_BTN_OLY},	
	{0x00, 0x02, 0x00, 0x32},	
	{BTN_EN_HI, BTN_EN_LO, 0x00, 0xFF},
	{0x00, 0x08, 0x00, 0x00},	
	{GPIO_CTL_HI, GPIO_CTL_LO, 0x07, 0x01},	
	{0x00, 0x0F, 0x01, 0x00},	
	{SEN_10_HI, SEN_10_LO, 0xBD, 0xC3},	
	{SEN_32_HI, SEN_32_LO, 0xCC, 0xC7},	
	{SEN_54_HI, SEN_54_LO, 0xCA, 0xCF},	
	{SEN_76_HI, SEN_76_LO, 0xC0, 0xBE},	
	{SEN_98_HI, SEN_98_LO, 0xAA, 0xAA},	
	{SEN_BA_HI, SEN_BA_LO, 0xAA, 0xAA},	
	{IF_CFG_HI, IF_CFG_LO, 0x00, DATA_EN | ATTN_HI | ATTN_EN},
};

#if 0
/* billition parameters 080130*/
static SKY_OT_CONFIG g_OT_Config[] = 
{
	{IF_CFG_HI, IF_CFG_LO, 0x00, 0x00},
	{IF_CFG_HI, IF_CFG_LO, 0x00, DATA_EN | ATTN_EN},
	{GEN_CFG_HI, GEN_CFG_LO, 0x00, FIR_BTN_OLY},	
	{0x00, 0x02, 0x00, 0x32},	
	{BTN_EN_HI, BTN_EN_LO, 0x00, 0xFF},
	{0x00, 0x08, 0x00, 0x00},	
	{GPIO_CTL_HI, GPIO_CTL_LO, 0x07, 0x01},
	{0x00, 0x0F, 0x01, 0x00},	
	{SEN_10_HI, SEN_10_LO, 0xB5, 0xB9},
	{SEN_32_HI, SEN_32_LO, 0xBC, 0xBD},
	{SEN_54_HI, SEN_54_LO, 0xBC, 0xBC},
	{SEN_76_HI, SEN_76_LO, 0xBC, 0xB6},
	{SEN_98_HI, SEN_98_LO, 0xAA, 0xAA},
	{SEN_BA_HI, SEN_BA_LO, 0xAA, 0xAA},
	{IF_CFG_HI, IF_CFG_LO, 0x00, DATA_EN | ATTN_HI | ATTN_EN},
};
#endif

#endif
