#ifndef SKY_OT_H_
#define SKY_OT_H_

#include <asm/types.h>

#define OT_ADDR			0x2D
#define OT_RE_ADDR		0x5B
#define OT_WR_ADDR		0x5A

/* configuration registers */
#define IF_CFG_HI		0x00
#define IF_CFG_LO		0x00
	#define DATA_EN			0x04
	#define ATTN_HI			0x01
	#define ATTN_EN			0x02

#define GEN_CFG_HI		0x00
#define GEN_CFG_LO		0x01
	#define PRXRES			0x01
	#define SLEEP			0x80
	#define GPI_ATTN			0x40

#define BTN_EN_HI		0x00
#define BTN_EN_LO		0x04
#define GPIO_CTL_HI		0x00
#define GPIO_CTL_LO		0x0E
#define SEN_10_HI		0x00
#define SEN_10_LO		0x10
#define SEN_32_HI		0x00
#define SEN_32_LO		0x11
#define SEN_54_HI		0x00
#define SEN_54_LO		0x12
#define SEN_76_HI		0x00
#define SEN_76_LO		0x13
#define BTN_IO_MAP_HI	0x00
#define BTN_IO_MAP_LO	0x1E
#define TIMER_CTL_HI	0x00
#define TIMER_CTL_LO	0x1F
#define LED_EN_HI		0x00
#define LED_EN_LO		0x22
#define LED_EFFECT_HI	0x00
#define LED_EFFECT_LO	0x23
#define LED_10_HI		0x00
#define LED_10_LO		0x24
#define LED_32_HI		0x00
#define LED_32_LO		0x25
#define LED_54_HI		0x00
#define LED_54_LO		0x26
#define LED_6_HI			0x00
#define LED_6_LO		0x27
#define PROXI_EN_HI		0x00
#define PROXI_EN_LO		0x30
#define DEBOUN_EN_HI	0x00
#define DEBOUN_EN_LO	0x32

/* data registers */
#define DUMMY_REG_HI	0x01
#define DUMMY_REG_LO	0x06
#define GPIO_STS_HI		0x01
#define GPIO_STS_LO		0x08
	#define FRESH_BIT		0x80
#define BTN_STS_HI		0x01
#define BTN_STS_LO		0x09
#define TIMER_STS_HI	0x01
#define TIMER_STS_LO	0x0B
#define PRE_VAL_10_HI	0x01
#define PRE_VAL_10_LO	0x0C
#define PRE_VAL_32_HI	0x01
#define PRE_VAL_32_LO	0x0D
#define PRE_VAL_54_HI	0x01
#define PRE_VAL_54_LO	0x0E
#define PRE_VAL_76_HI	0x01
#define PRE_VAL_76_LO	0x0F

/* reset register */
#define RESET_HI			0x03
#define RESET_LO			0x00
#define RST_HI			0x00
#define RST_LO			0x01

typedef struct _OT_CONFIG{
	u8 reg_hi;
	u8 reg_lo;
	u8 data_hi;
	u8 data_lo;
} SKY_OT_CONFIG, *SKY_OT_CONFIG_PTR;

/* 080724 new sensivity valu from eecom */
static SKY_OT_CONFIG g_OT_Config[] = 
{
	{GEN_CFG_HI, GEN_CFG_LO, PRXRES, 0x00},
	{BTN_EN_HI, BTN_EN_LO, 0x00, 0x1F},
	{SEN_10_HI, SEN_10_LO, 0xB0, 0xB0},
	{SEN_32_HI, SEN_32_LO, 0xB7, 0xB6},
	{SEN_54_HI, SEN_54_LO, 0xB7, 0xB2},
	{SEN_76_HI, SEN_76_LO, 0xB0, 0x00},
	{LED_EN_HI, LED_EN_LO, 0x00, 0x1F},
	{LED_10_HI, LED_10_LO, 0x4F, 0x5F},
	{LED_32_HI, LED_32_LO, 0x4F, 0x4F},
	{LED_54_HI, LED_54_LO, 0x00, 0x4F},
	{IF_CFG_HI, IF_CFG_LO, 0x00, DATA_EN | ATTN_HI | ATTN_EN},
};

#if 0
static SKY_OT_CONFIG g_OT_Config[] = 
{
	{GEN_CFG_HI, GEN_CFG_LO, PRXRES, 0x00},
	{BTN_EN_HI, BTN_EN_LO, 0x00, 0x1F},
	{SEN_10_HI, SEN_10_LO, 0x64, 0x64},
	{SEN_32_HI, SEN_32_LO, 0x64, 0x64},
	{SEN_54_HI, SEN_54_LO, 0xDF, 0x64},
	{SEN_76_HI, SEN_76_LO, 0x00, 0x64},
	{LED_EN_HI, LED_EN_LO, 0x00, 0x1F},
	{LED_10_HI, LED_10_LO, 0x4F, 0x5F},
	{LED_32_HI, LED_32_LO, 0x4F, 0x4F},
	{LED_54_HI, LED_54_LO, 0x00, 0x4F},
	{IF_CFG_HI, IF_CFG_LO, 0x00, DATA_EN | ATTN_HI | ATTN_EN},
};
#endif

#if 0
static SKY_OT_CONFIG g_OT_Config[] = 
{
	{IF_CFG_HI, IF_CFG_LO, 0x00, 0x00},
	{IF_CFG_HI, IF_CFG_LO, 0x00, DATA_EN | ATTN_EN},
	{GEN_CFG_HI, GEN_CFG_LO, 0x00, 0x00},
	{GEN_CFG_HI, GEN_CFG_LO, PRXRES, 0x00},
//	{BTN_EN_HI, BTN_EN_LO, 0x00, 0x7F},
	{BTN_EN_HI, BTN_EN_LO, 0x00, 0x1F},
//	{GPIO_CTL_HI, GPIO_CTL_LO, 0x7F, 0x6E},
	{SEN_10_HI, SEN_10_LO, 0x64, 0x64},
	{SEN_32_HI, SEN_32_LO, 0x64, 0x64},
	{SEN_54_HI, SEN_54_LO, 0xDF, 0x64},
	{SEN_76_HI, SEN_76_LO, 0x00, 0x64},
	{LED_EN_HI, LED_EN_LO, 0x00, 0x1F},
	{LED_10_HI, LED_10_LO, 0x4F, 0x5F},
	{LED_32_HI, LED_32_LO, 0x4F, 0x4F},
	{LED_54_HI, LED_54_LO, 0x00, 0x4F},
	{PROXI_EN_HI, PROXI_EN_LO, 0x00, 0x20},
	{DEBOUN_EN_HI, DEBOUN_EN_LO, 0x00, 0x3F},
	{IF_CFG_HI, IF_CFG_LO, 0x00, DATA_EN | ATTN_HI | ATTN_EN},
};
#endif

#if 0
/* billition parameters 080130*/
static SKY_OT_CONFIG g_OT_Config[] = 
{
	{IF_CFG_HI, IF_CFG_LO, 0x00, 0x00},
	{IF_CFG_HI, IF_CFG_LO, 0x00, DATA_EN | ATTN_EN},
	{GEN_CFG_HI, GEN_CFG_LO, 0x00, FIR_BTN_OLY},	
	{0x00, 0x02, 0x00, 0x32},	
	{BTN_EN_HI, BTN_EN_LO, 0x00, 0xFF},
	{0x00, 0x08, 0x00, 0x00},	
	{GPIO_CTL_HI, GPIO_CTL_LO, 0x07, 0x01},
	{0x00, 0x0F, 0x01, 0x00},	
	{SEN_10_HI, SEN_10_LO, 0xB5, 0xB9},
	{SEN_32_HI, SEN_32_LO, 0xBC, 0xBD},
	{SEN_54_HI, SEN_54_LO, 0xBC, 0xBC},
	{SEN_76_HI, SEN_76_LO, 0xBC, 0xB6},
	{SEN_98_HI, SEN_98_LO, 0xAA, 0xAA},
	{SEN_BA_HI, SEN_BA_LO, 0xAA, 0xAA},
	{IF_CFG_HI, IF_CFG_LO, 0x00, DATA_EN | ATTN_HI | ATTN_EN},
};
#endif

#endif
