/*
      //------------------------------------------------------------------------
      //
      //                  S K Y M E D I   P R O P R I E T A R Y
      //
      //          COPYRIGHT (c)   2003 BY SKYMEDI CORPORATION TAIWAN.
      //                   --  ALL RIGHTS RESERVED  --
      //       SKYMEDI Confidential; Need to Know only.
      //       Protected as an unpublished work.
      //
      //       The computer program listings, specifications and documentation
      //       herein are the property of SKYMEDI CORPORATION and shall not be
      //       reproduced, copied, disclosed, or used in whole or in part for
      //       any reason without the prior express written permission of
      //       SKYMEDI CORPORATION.
      //
      //          File Name:        skyconfig.h
      //
      //          Author:           Jinn Cheng
      //
      //          Created:          May 10, 2008
      //
      //          Revision:         B-1.00
      //
      //------------------------------------------------------------------------
 */
#ifndef __SKYCONFIG_H__
#define __SKYCONFIG_H__

#define SKYCONFIG_DRIVER_NAME_A "skyconfig.c"
#define SKYCONFIG_DRIVER_NAME_B "skyconfig.h" 
#define SKYCONFIG_DRIVER_TAG_A "1-1.00"
#define SKYCONFIG_DRIVER_TAG_B "1-1.00"

#include <linux/autoconf.h>

#define TYPE(dev)   (MINOR(dev) >> 4)  /* high nibble */
#define NUM(dev)    (MINOR(dev) & 0xf) /* low  nibble */

typedef struct SDMMC_CFG_S {
     unsigned int        xfer_sectors;
     unsigned int        carddetet_mode;
     unsigned int        writeprotect_mode;
     unsigned int        dummy0;
     unsigned int        dummy1;
     unsigned int        dummy2;
     unsigned int        dummy3;
     unsigned int        dummy4;
} SDMMC_CFG_T;

typedef struct MSX_CFG_S {
     unsigned int        xfer_sectors;
     unsigned int        carddetet_mode;
     unsigned int        bus_switch_mode;
     unsigned int        bus_width_for_sony;
     unsigned int        bus_width_for_others;
     unsigned int        dummy0;
     unsigned int        dummy1;
     unsigned int        dummy2;
} MSX_CFG_T;

typedef struct XDSM_CFG_S {
     unsigned int        xfer_sectors;
     unsigned int        carddetet_mode;
     unsigned int        writeprotect_mode;
     unsigned int        dummy0;
     unsigned int        dummy1;
     unsigned int        dummy2;
     unsigned int        dummy3;
     unsigned int        dummy4;
} XDSM_CFG_T;

typedef struct CF_CFG_S {
     unsigned int        xfer_sectors;
     unsigned int        carddetet_mode;
     unsigned int        max_pio_mode;
     unsigned int        power_control_logic;
     unsigned int        bus_ena_switch_mode;
     unsigned int        bus_dir_switch_mode;
     unsigned int        dummy0;
     unsigned int        dummy1;
} CF_CFG_T;

typedef struct HD_CFG_S {
     unsigned int        xfer_sectors;
     unsigned int        carddetet_mode;
     unsigned int        max_pio_mode;
     unsigned int        power_control_logic;
     unsigned int        bus_ena_switch_mode;
     unsigned int        bus_dir_switch_mode;
     unsigned int        dummy0;
     unsigned int        dummy1;
} HD_CFG_T;

typedef struct FLASH_CFG_S {
     unsigned int        xfer_sectors;
     unsigned int        carddetet_mode;
     unsigned int        dummy0;
     unsigned int        dummy1;
     unsigned int        dummy2;
     unsigned int        dummy3;
     unsigned int        dummy4;
     unsigned int        dummy5;
} FLASH_CFG_T;

typedef struct IR_CFG_S {
     unsigned int        ir3_con;
     unsigned int        ir3conA;
     unsigned int        ir3conB;
     unsigned int        ir3conC;
     unsigned int        dummy0;
     unsigned int        dummy1;
     unsigned int        dummy2;
     unsigned int        dummy3;     
} IR_CFG_T;

typedef struct PAD_CLK_CFG_S {
     unsigned int        pad_strength;
     unsigned int        clk_strength;
     unsigned int        dummy0;
     unsigned int        dummy1;
     unsigned int        dummy2;
     unsigned int        dummy3;
     unsigned int        dummy4;
     unsigned int        dummy5;
} PADSTRENGTH_CFG_T;

typedef struct USB_DEVICE_CFG_S {
     unsigned int        sdmmc_disk_idx;
     unsigned int        msx_disk_idx;
     unsigned int        xdsm_disk_idx;
     unsigned int        cf_disk_idx;
     unsigned int        hd_disk_idx;
     unsigned int        flash_disk_idx;     
     unsigned int        dummy0;
     unsigned int        dummy1;
} USB_DEVICE_CFG_T;

typedef struct IO_CFG_S {
     unsigned int        gpio[16];
     unsigned int        epio[6];
     unsigned int        toggle_interal;
     unsigned int        read_led_mode;
     unsigned int        write_led_mode;
     unsigned int        readwrite_led_mode;
     unsigned int        scankey;
     unsigned int        dummy0;
     unsigned int        dummy1;
     unsigned int        dummy2;
     unsigned int        dummy3;
     unsigned int        dummy4;
} IO_CFG_T;

typedef struct SKYCONFIG_DATA_S {
     SDMMC_CFG_T         sdmmc;                                                 //  8 x 4
     MSX_CFG_T           msx;                                                   //  8 x 4
     XDSM_CFG_T          xdsm;                                                  //  8 x 4
     CF_CFG_T            cf;                                                    //  8 x 4
     HD_CFG_T            hd;                                                    //  8 x 4
     FLASH_CFG_T         flash;                                                 //  8 x 4
     IR_CFG_T            ir;                                                    //  8 x 4
     PADSTRENGTH_CFG_T   pad_clk;                                               //  8 x 4
     USB_DEVICE_CFG_T    usbdev;                                                //  8 x 4
     IO_CFG_T            io;                                                    // 32 x 4
     unsigned int        usb_string_length;                                     //  1 x 4
     unsigned char       usb_string_name[84];                                   // 84 x 1
     unsigned char       cfgid[8];                                                 //  8 x 1
} SKYCONFIG_DATA_T;

typedef struct SKYCONFIG_DRIVER_S {
	struct module  	*owner;
	char           	*name;
	spinlock_t          lock;
	int                 id;
	int  (*init)(void);   
	void (*cleanup)(void);
} SKYCONFIG_DRIVER_T;

#define SKYMEDI_SKYCONFIG_DRIVER_NAME   "skyconfig loader"
#define SKYMEDI_SKYCONFIG_DRIVER_ID     1243

#define SKYCFG_SDMMC                    0
#define SKYCFG_MSX                      1
#define SKYCFG_XDSM                     2
#define SKYCFG_CF                       3
#define SKYCFG_HD                       4
#define SKYCFG_FLASH                    5

#define SKYCFG_LED_READ                 0
#define SKYCFG_LED_WRITE                1
#define SKYCFG_LED_READWRITE            2

#define SKYCFG_DISK_GROUP_A             0
#define SKYCFG_DISK_GROUP_B             1
#define SKYCFG_DISK_GROUP_C             2
#define SKYCFG_DISK_GROUP_D             3
#define SKYCFG_DISK_GROUP_E             4
#define SKYCFG_DISK_GROUP_F             5
#define SKYCFG_DISK_GROUP_G             6
#define SKYCFG_DISK_GROUP_H             7
#define SKYCFG_NODISK_GROUP             9

#define SKYCFG_MSX_BUS_SWITCH_ENABLE    0x80000000
#define SKYCFG_MSX_BUS_HIGHENABLE       0x08000000
#define SKYCFG_MSX_BUS_LOWENABLE        0x04000000
#define SKYCFG_MSX_USE_GPIO_PORT        0x00800000
#define SKYCFG_MSX_USE_EGPIO_PORT       0x00400000
#define SKYCFG_MSX_CHEKC_PORT           0x00C00000
#define SKYCFG_MSX_IO_MASK              0x0000FFFF
#define SKYCFG_MSX_USE_GPIO_MASK        0x0000FFFF
#define SKYCFG_MSX_USE_EGPIO_MASK       0x0000003F
#define SKYCFG_MSX_USE_GIO0             0x00000001
#define SKYCFG_MSX_USE_GIO1             0x00000002
#define SKYCFG_MSX_USE_GIO2             0x00000004
#define SKYCFG_MSX_USE_GIO3             0x00000008
#define SKYCFG_MSX_USE_GIO4             0x00000010
#define SKYCFG_MSX_USE_GIO5             0x00000020
#define SKYCFG_MSX_USE_GIO6             0x00000040
#define SKYCFG_MSX_USE_GIO7             0x00000080
#define SKYCFG_MSX_USE_GIO8             0x00000100
#define SKYCFG_MSX_USE_GIO9             0x00000200
#define SKYCFG_MSX_USE_GIO10            0x00000400
#define SKYCFG_MSX_USE_GIO11            0x00000800
#define SKYCFG_MSX_USE_GIO12            0x00001000
#define SKYCFG_MSX_USE_GIO13            0x00002000
#define SKYCFG_MSX_USE_GIO14            0x00004000
#define SKYCFG_MSX_USE_GIO15            0x00008000
#define SKYCFG_MSX_USE_EIO0             0x00000001
#define SKYCFG_MSX_USE_EIO1             0x00000002
#define SKYCFG_MSX_USE_EIO2             0x00000004
#define SKYCFG_MSX_USE_EIO3             0x00000008
#define SKYCFG_MSX_USE_EIO4             0x00000010
#define SKYCFG_MSX_USE_EIO5             0x00000020

#define SKYCFG_CF_PWR_SWITCH_ENABLE     0x80000000
#define SKYCFG_CF_PWR_HIGHENABLE        0x08000000
#define SKYCFG_CF_PWR_LOWENABLE         0x04000000
#define SKYCFG_CF_USE_GPIO_PORT         0x00800000
#define SKYCFG_CF_USE_EGPIO_PORT        0x00400000
#define SKYCFG_CF_CHEKC_PORT            0x00C00000
#define SKYCFG_CF_IO_MASK               0x0000FFFF
#define SKYCFG_CF_USE_GPIO_MASK         0x0000FFFF
#define SKYCFG_CF_USE_EGPIO_MASK        0x0000003F
#define SKYCFG_CF_USE_GIO0              0x00000001
#define SKYCFG_CF_USE_GIO1              0x00000002
#define SKYCFG_CF_USE_GIO2              0x00000004
#define SKYCFG_CF_USE_GIO3              0x00000008
#define SKYCFG_CF_USE_GIO4              0x00000010
#define SKYCFG_CF_USE_GIO5              0x00000020
#define SKYCFG_CF_USE_GIO6              0x00000040
#define SKYCFG_CF_USE_GIO7              0x00000080
#define SKYCFG_CF_USE_GIO8              0x00000100
#define SKYCFG_CF_USE_GIO9              0x00000200
#define SKYCFG_CF_USE_GIO10             0x00000400
#define SKYCFG_CF_USE_GIO11             0x00000800
#define SKYCFG_CF_USE_GIO12             0x00001000
#define SKYCFG_CF_USE_GIO13             0x00002000
#define SKYCFG_CF_USE_GIO14             0x00004000
#define SKYCFG_CF_USE_GIO15             0x00008000
#define SKYCFG_CF_USE_EIO0              0x00000001
#define SKYCFG_CF_USE_EIO1              0x00000002
#define SKYCFG_CF_USE_EIO2              0x00000004
#define SKYCFG_CF_USE_EIO3              0x00000008
#define SKYCFG_CF_USE_EIO4              0x00000010
#define SKYCFG_CF_USE_EIO5              0x00000020

#define SKYCFG_HD_PWR_SWITCH_ENABLE     0x80000000
#define SKYCFG_HD_PWR_HIGHENABLE        0x08000000
#define SKYCFG_HD_PWR_LOWENABLE         0x04000000
#define SKYCFG_HD_USE_GPIO_PORT         0x00800000
#define SKYCFG_HD_USE_EGPIO_PORT        0x00400000
#define SKYCFG_HD_CHEKC_PORT            0x00C00000
#define SKYCFG_HD_IO_MASK               0x0000FFFF
#define SKYCFG_HD_USE_GPIO_MASK         0x0000FFFF
#define SKYCFG_HD_USE_EGPIO_MASK        0x0000003F
#define SKYCFG_HD_USE_GIO0              0x00000001
#define SKYCFG_HD_USE_GIO1              0x00000002
#define SKYCFG_HD_USE_GIO2              0x00000004
#define SKYCFG_HD_USE_GIO3              0x00000008
#define SKYCFG_HD_USE_GIO4              0x00000010
#define SKYCFG_HD_USE_GIO5              0x00000020
#define SKYCFG_HD_USE_GIO6              0x00000040
#define SKYCFG_HD_USE_GIO7              0x00000080
#define SKYCFG_HD_USE_GIO8              0x00000100
#define SKYCFG_HD_USE_GIO9              0x00000200
#define SKYCFG_HD_USE_GIO10             0x00000400
#define SKYCFG_HD_USE_GIO11             0x00000800
#define SKYCFG_HD_USE_GIO12             0x00001000
#define SKYCFG_HD_USE_GIO13             0x00002000
#define SKYCFG_HD_USE_GIO14             0x00004000
#define SKYCFG_HD_USE_GIO15             0x00008000
#define SKYCFG_HD_USE_EIO0              0x00000001
#define SKYCFG_HD_USE_EIO1              0x00000002
#define SKYCFG_HD_USE_EIO2              0x00000004
#define SKYCFG_HD_USE_EIO3              0x00000008
#define SKYCFG_HD_USE_EIO4              0x00000010
#define SKYCFG_HD_USE_EIO5              0x00000020

#define SKYCFG_CF_BUS_ENA_SWITCH_ENABLE 0x80000000
#define SKYCFG_CF_BUS_ENA_HIGHENABLE    0x08000000
#define SKYCFG_CF_BUS_ENA_LOWENABLE     0x04000000

#define SKYCFG_CF_BUS_DIR_SWITCH_ENABLE 0x80000000
#define SKYCFG_CF_BUS_DIR_HIGH_IS_WRITE 0x08000000
#define SKYCFG_CF_BUS_DIR_HIGH_IS_READ  0x04000000

#define SKYCFG_HD_BUS_ENA_SWITCH_ENABLE 0x80000000
#define SKYCFG_HD_BUS_ENA_HIGHENABLE    0x08000000
#define SKYCFG_HD_BUS_ENA_LOWENABLE     0x04000000

#define SKYCFG_HD_BUS_DIR_SWITCH_ENABLE 0x80000000
#define SKYCFG_HD_BUS_DIR_HIGH_IS_WRITE 0x08000000
#define SKYCFG_HD_BUS_DIR_HIGH_IS_READ  0x04000000

#define SKYCFG_CD_ENABLE                0x80000000
#define SKYCFG_CD_REMOVABLE             0x08000000
#define SKYCFG_CD_NOT_REMOVABLE         0x04000000
#define SKYCFG_CD_MAPIO_MASK            0x00000007
#define SKYCFG_CD_USE_EIOP0             0x00000000                              // numeric type, not bit mapping
#define SKYCFG_CD_USE_EIOP1             0x00000001                              // numeric type, not bit mapping
#define SKYCFG_CD_USE_EIOP2             0x00000002                              // numeric type, not bit mapping
#define SKYCFG_CD_USE_EIOP3             0x00000003                              // numeric type, not bit mapping
#define SKYCFG_CD_USE_EIOP4             0x00000004                              // numeric type, not bit mapping
#define SKYCFG_CD_USE_EIOP5             0x00000005                              // numeric type, not bit mapping
#define SKYCFG_CD_MAP_AS_EIOP0          0x00000000                              // numeric type, not bit mapping
#define SKYCFG_CD_MAP_AS_EIOP1          0x00000001                              // numeric type, not bit mapping
#define SKYCFG_CD_MAP_AS_EIOP2          0x00000002                              // numeric type, not bit mapping
#define SKYCFG_CD_MAP_AS_EIOP3          0x00000003                              // numeric type, not bit mapping
#define SKYCFG_CD_MAP_AS_EIOP4          0x00000004                              // numeric type, not bit mapping
#define SKYCFG_CD_MAP_AS_EIOP5          0x00000005                              // numeric type, not bit mapping

#define SKYCFG_WP_ENABLE                0x80000000
#define SKYCFG_WP_HIGHE_ENABKE          0x08000000
#define SKYCFG_WP_LOW_DISABLE           0x04000000
#define SKYCFG_WP_USE_GPIO_PORT         0x00800000
#define SKYCFG_WP_USE_EGPIO_PORT        0x00400000
#define SKYCFG_WP_MAPIO_MASK            0x0000FFFF
#define SKYCFG_WP_USE_GIO0              0x00000001
#define SKYCFG_WP_USE_GIO1              0x00000002
#define SKYCFG_WP_USE_GIO2              0x00000004
#define SKYCFG_WP_USE_GIO3              0x00000008
#define SKYCFG_WP_USE_GIO4              0x00000010
#define SKYCFG_WP_USE_GIO5              0x00000020
#define SKYCFG_WP_USE_GIO6              0x00000040
#define SKYCFG_WP_USE_GIO7              0x00000080
#define SKYCFG_WP_USE_GIO8              0x00000100
#define SKYCFG_WP_USE_GIO9              0x00000200
#define SKYCFG_WP_USE_GIO10             0x00000400
#define SKYCFG_WP_USE_GIO11             0x00000800
#define SKYCFG_WP_USE_GIO12             0x00001000
#define SKYCFG_WP_USE_GIO13             0x00002000
#define SKYCFG_WP_USE_GIO14             0x00004000
#define SKYCFG_WP_USE_GIO15             0x00008000
#define SKYCFG_WP_USE_EIO0              0x00000001
#define SKYCFG_WP_USE_EIO1              0x00000002
#define SKYCFG_WP_USE_EIO2              0x00000004
#define SKYCFG_WP_USE_EIO3              0x00000008
#define SKYCFG_WP_USE_EIO4              0x00000010
#define SKYCFG_WP_USE_EIO5              0x00000020

#define SKYCFG_IO_ENABLE                0x80000000
#define SKYCFG_IO_HIGHE_LIGHT           0x08000000
#define SKYCFG_IO_LOW_LIGHT             0x04000000
#define SKYCFG_IO_USE_GPIO_PORT         0x00800000
#define SKYCFG_IO_USE_EGPIO_PORT        0x00400000
#define SKYCFG_IO_MAPIO_MASK            0x0000FFFF
#define SKYCFG_IO_USE_GIO0              0x00000001
#define SKYCFG_IO_USE_GIO1              0x00000002
#define SKYCFG_IO_USE_GIO2              0x00000004
#define SKYCFG_IO_USE_GIO3              0x00000008
#define SKYCFG_IO_USE_GIO4              0x00000010
#define SKYCFG_IO_USE_GIO5              0x00000020
#define SKYCFG_IO_USE_GIO6              0x00000040
#define SKYCFG_IO_USE_GIO7              0x00000080
#define SKYCFG_IO_USE_GIO8              0x00000100
#define SKYCFG_IO_USE_GIO9              0x00000200
#define SKYCFG_IO_USE_GIO10             0x00000400
#define SKYCFG_IO_USE_GIO11             0x00000800
#define SKYCFG_IO_USE_GIO12             0x00001000
#define SKYCFG_IO_USE_GIO13             0x00002000
#define SKYCFG_IO_USE_GIO14             0x00004000
#define SKYCFG_IO_USE_GIO15             0x00008000
#define SKYCFG_IO_USE_EIO0              0x00000001
#define SKYCFG_IO_USE_EIO1              0x00000002
#define SKYCFG_IO_USE_EIO2              0x00000004
#define SKYCFG_IO_USE_EIO3              0x00000008
#define SKYCFG_IO_USE_EIO4              0x00000010
#define SKYCFG_IO_USE_EIO5              0x00000020

#define SKYCFG_GENRICIO_ENABLE          0x80000000
#define SKYCFG_GENRICIO_DIR_IS_OUT      0x08000000
#define SKYCFG_GENRICIO_DIR_IS_IN       0x04000000
#define SKYCFG_GENRICIO_DEFAULT_LOGIC_1 0x00800000
#define SKYCFG_GENRICIO_DEFAULT_LOGIC_0 0x00400000

#ifdef __SKYCONFIG_C__

#define THIS_FUNCTION_NAME              ""
#define MYDEBUG_0                       1
#define MYDEBUG_1                       2
#define MYDEBUG_2                       3
#define MYDEBUG_3                       4
#define MYDEBUG_4                       5
#define MYDEBUG_5                       6
#define MYDEBUG_X                       99

#define MYDEBUG(n, format, args...)	\
		{ \
		     if( n <= 2 ) { \
			     printk("[%s:%d] %s ", __FILE__, __LINE__, __FUNCTION__); \
			     printk("\"" format "\"" "\n" , ## args); \
			} \
		}
		
#define MYDEBUG_NEXTLINE(n) \
		{ \
			if( n <= 2 ) { \
			     printk("\n"); \
			} \
		}
		
#else

extern unsigned int cf_get_power_switch_mode( void );
extern unsigned int cf_get_max_pio_mode( void );
extern unsigned int cf_get_bus_enable_mode( void );
extern unsigned int cf_get_bus_direction_mode( void );

extern unsigned int hd_get_power_switch_mode( void );
extern unsigned int hd_get_max_pio_mode( void );
extern unsigned int hd_get_bus_enable_mode( void );
extern unsigned int hd_get_bus_direction_mode( void );

extern unsigned int msx_get_bus_switch_mode( void );
extern unsigned int msx_get_bus_width_sony( void );
extern unsigned int msx_get_bus_width_non_sony( void );

extern unsigned int get_storage_xfer_size( unsigned int _idx );
extern unsigned int get_cd_cfg( unsigned int _idx );
extern unsigned int get_wp_cfg( unsigned int _idx );
extern unsigned int get_led_cfg( unsigned int _idx );
extern unsigned int get_scankey_cfg( void );
extern unsigned int* get_gpio_cfg( void );
extern unsigned int* get_epio_cfg( void );
extern unsigned int get_reader_group( unsigned int _idx );
extern unsigned int get_reader_strlen( void );
extern unsigned char* get_reader_str( void );

extern unsigned int ir_get_ir3_con( void );
extern unsigned int ir_get_ir3conA( void );
extern unsigned int ir_get_ir3conB( void );
extern unsigned int ir_get_ir3conC( void );

extern unsigned int io_get_pad_strength( void );
extern unsigned int io_get_clk_strength( void );
extern unsigned int io_get_toggle_interval( void );

extern SKYCONFIG_DATA_T* skyconfig_get_data( void );

#endif		
		
#endif // __SKYCONFIG_H__
