/*
 ***************************************************************************
 * Ralink Tech Inc.
 * 4F, No. 2 Technology	5th	Rd.
 * Science-based Industrial	Park
 * Hsin-chu, Taiwan, R.O.C.
 *
 * (c) Copyright 2002-2006, Ralink Technology, Inc.
 *
 * All rights reserved.	Ralink's source	code is	an unpublished work	and	the
 * use of a	copyright notice does not imply	otherwise. This	source code
 * contains	confidential trade secret material of Ralink Tech. Any attemp
 * or participation	in deciphering,	decoding, reverse engineering or in	any
 * way altering	the	source code	is stricitly prohibited, unless	the	prior
 * written consent of Ralink Technology, Inc. is obtained.
 ***************************************************************************

	Module Name:
	wpa.c

	Abstract:

	Revision History:
	Who			When			What
	--------	----------		----------------------------------------------
	Paul Lin	06-08-08		Initial
*/
#include "rt_config.h"
#ifdef CONFIG_CRYPTO_HMACXXX
#include <linux/crypto.h>
#include <linux/scatterlist.h>
#endif //CONFIG_CRYPTO_HMAC


// General used field
// The last 6 bytes of UUID_E shall changed to MAC address
UCHAR	Wsc_Uuid_E[16] = {0x57, 0x50, 0x53, 0x45, 0x6E, 0x72, 0x6F, 0x6C, 0x65, 0x65, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06};
UCHAR	Wsc_Manufacture[] = "Ralink Technology, Corp.";
UCHAR	Wsc_Model_Name[] = "Ralink Wireless Adapter";
UCHAR	Wsc_Model_Number[] = MODEL_NUMBER;
UCHAR	Wsc_Model_Serial[] = "12345678";
UCHAR	Wsc_Pri_Dev_Type[8] = {0x00, 0x01, 0x00, 0x50, 0xf2, 0x04, 0x00, 0x01};
UCHAR	Wsc_Dev_Name[] = "Ralink Client"; // default to Registry
UCHAR	Wsc_Personal_String[] =  "Wi-Fi Easy and Secure Key Derivation";
UCHAR	EnrolleeMsg[] = "WFA-SimpleConfig-Enrollee-1-0"; // Ralink-WFA-Enrolee

// Global reusable buffer
UCHAR	OutMsgBuf[1500];		// buffer to create message contents
UCHAR	HmacData[4096];			// temp buffer to store encrypt output data

#ifndef PF_NOFREEZE
#define PF_NOFREEZE  0
#endif


BUILD_TIMER_FUNCTION(WscConnectTimeout);
BUILD_TIMER_FUNCTION(WscScanTimeout);
BUILD_TIMER_FUNCTION(WscPBCTimeout);
BUILD_TIMER_FUNCTION(WscPINTimeout);
BUILD_TIMER_FUNCTION(WscEAPRxTimeout);
BUILD_TIMER_FUNCTION(WscM8WaitEapFailTimeout);
BUILD_TIMER_FUNCTION(WscEapM2Timeout);
BUILD_TIMER_FUNCTION(WscEndTimeout);
BUILD_TIMER_FUNCTION(WscProfileRetryTimeout);

/*	
	==========================================================================
	Description: 
		association	state machine init,	including state	transition and timer init
	Parameters:	
		S -	pointer	to the association state machine
	==========================================================================
 */
VOID WscStateMachineInit(
	IN	PRTMP_ADAPTER		pAd, 
	IN	STATE_MACHINE		*S, 
	OUT	STATE_MACHINE_FUNC	Trans[])	
{
	StateMachineInit(S,	(STATE_MACHINE_FUNC *)Trans, MAX_WSC_STATE, MAX_WSC_MSG, (STATE_MACHINE_FUNC)Drop, WSC_IDLE, WSC_MACHINE_BASE);
	StateMachineSetAction(S, WSC_IDLE, WSC_EAP_REQ_MSG, (STATE_MACHINE_FUNC)WscEAPAction);
	StateMachineSetAction(S, WSC_IDLE, WSC_EAP_RSP_MSG, (STATE_MACHINE_FUNC)WscEAPAction);
	StateMachineSetAction(S, WSC_IDLE, WSC_EAP_FAIL_MSG, (STATE_MACHINE_FUNC)WscEAPAction);

	// initialize the timer
	RTMPInitTimer(pAd, &pAd->PortCfg.WscControl.WscConnectTimer, GET_TIMER_FUNCTION(WscConnectTimeout), pAd, FALSE);
	RTMPInitTimer(pAd, &pAd->PortCfg.WscControl.WscScanTimer, GET_TIMER_FUNCTION(WscScanTimeout), pAd, FALSE);
	RTMPInitTimer(pAd, &pAd->PortCfg.WscControl.WscPBCTimer, GET_TIMER_FUNCTION(WscPBCTimeout), pAd, FALSE);
	RTMPInitTimer(pAd, &pAd->PortCfg.WscControl.WscPINTimer, GET_TIMER_FUNCTION(WscPINTimeout), pAd, FALSE);
	RTMPInitTimer(pAd, &pAd->PortCfg.WscControl.WscEapRxTimer, GET_TIMER_FUNCTION(WscEAPRxTimeout), pAd, FALSE);
	RTMPInitTimer(pAd, &pAd->PortCfg.WscControl.WscM8WaitEapFailTimer, GET_TIMER_FUNCTION(WscM8WaitEapFailTimeout), pAd, FALSE);;
	RTMPInitTimer(pAd, &pAd->PortCfg.WscControl.WscEapM2Timer, GET_TIMER_FUNCTION(WscEapM2Timeout), pAd, FALSE);
	RTMPInitTimer(pAd, &pAd->PortCfg.WscControl.WscEndTimer, GET_TIMER_FUNCTION(WscEndTimeout), pAd, FALSE);; // add by johnli
	RTMPInitTimer(pAd, &pAd->PortCfg.WscControl.WscProfileRetryTimer, GET_TIMER_FUNCTION(WscProfileRetryTimeout), pAd, FALSE);  // add by johnli, fix WPS test plan 5.1.1
	DBGPRINT(RT_DEBUG_WPS,"-----------> WscStateMachineInit ok \n");
}

/*
	==========================================================================
	Description:
		This is	state machine function.	
		When receiving EAPOL packets which is  for 802.1x key management. 
		Use	both in	WPA, and WPAPSK	case. 
		In this	function, further dispatch to different	functions according	to the received	packet.	 3 categories are :	
		  1.  normal 4-way pairwisekey and 2-way groupkey handshake
		  2.  MIC error	(Countermeasures attack)  report packet	from STA.
		  3.  Request for pairwise/group key update	from STA
	Return:
	==========================================================================
*/
VOID WscEAPAction(
	IN	PRTMP_ADAPTER	pAdapter, 
	IN	MLME_QUEUE_ELEM	*Elem) 
{
	UCHAR	MsgType;
	UCHAR	Id;
	BOOLEAN Cancelled;

	DBGPRINT(RT_DEBUG_WPS, "-----> WscEAPAction\n");
	// Receive WSC EAP packet, cancel EAP Rx timer
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer,&Cancelled);

	// 0. Start to process EAP message, finding the exact message type first.
	MsgType = WscRxMsgType(pAdapter, &Id, Elem);	
	DBGPRINT(RT_DEBUG_WPS,"WscEAPAction=%d,MsgType=%d\n",pAdapter->PortCfg.WscControl.WscState,MsgType);

	// Base on state doing the Msg, State change diagram
	// TODO: Add return for success or failure, modify state machine according to the result	
	switch (MsgType)
	{	
		case WSC_MSG_EAP_REQ_ID:
			// This is only valid at WSC_STATE_LINKUP
			if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_LINK_UP)
			{
				// call to process EAP-Req and send out EAP-Rsp
				WscSendEapRspId(pAdapter, Id, Elem);
				// Change the state to next one
				pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_START;
			}
			// resend the rsp-id
			else if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_WAIT_START)
			{
				// call to process EAP-Req and send out EAP-Rsp
				WscSendEapRspId(pAdapter, Id, Elem);
				// Change the state to next one
				pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_START;
			}

			// EAP Rx processed, set new EAP Rx timer
			RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer, WSC_PER_MSG_TIMEOUT);
			
			break;

		case WSC_MSG_EAP_REQ_START:
			// This is only valid at WSC_STATE_WAIT_START
			if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_WAIT_START)
			{	
				// Send out M1
				WscSendEapM1(pAdapter, Id, Elem);
				// Change the state to next one
				pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_M2;

				// Set a new timer more than 5 sec for M2. Protect against multiple M2D received
				// Once re-associate , this will be canceled
				RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapM2Timer,&Cancelled);
				RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscEapM2Timer, WSC_PER_MSG_TIMEOUT);
			}
			// We have to resend M1 if we receive WSC start instead of M2 or M2D
			else if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_WAIT_M2)
			{
				// Send out M1
				WscSendEapM1(pAdapter, Id, Elem);
				// Change the state to next one
				pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_M2;

				// Set a new timer more than 5 sec for M2. Protect against multiple M2D received
				// Once re-associate , this will be canceled
				RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapM2Timer,&Cancelled);
				RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscEapM2Timer, WSC_PER_MSG_TIMEOUT);
			}

			// EAP Rx processed, set new EAP Rx timer
			//RTMPSetTimer(pAdapter, &pAdapter->PortCfg.WscControl.WscEapRxTimer, WSC_PER_MSG_TIMEOUT);

			break;
	
		case WSC_MSG_M2:
			// Cancel waiting M2 timer first
			RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapM2Timer,&Cancelled);
			// Receive M2, if we are at WSC_STATE_WAIT_M2 start, process it immediately
			if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_WAIT_M2)
			{
				// Process M2
				if (WscReceiveEapM2(pAdapter, Id, Elem))    // succeed
				{
					// Send out M3
					WscSendEapM3(pAdapter, Id, Elem);
					// Change the state to next one
					pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_M4;
				}
				else	// fail
				{
					// For a invalid nonce or authenticator:
					// sliently ignore this message and stay at the same state
				}
			}
			// We have to resend M3 if we receive M2 instead of M4
			else if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_WAIT_M4)
			{
				// Process M2
				if (WscReceiveEapM2(pAdapter, Id, Elem))    // succeed
				{
					// Send out M3
					WscSendEapM3(pAdapter, Id, Elem);
					// Change the state to next one
					pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_M4;
				}
				else    // fail
				{
					// For a invalid nonce or authenticator:
					// sliently ignore this message and stay at the same state
				}
			}
			// Received delayed M2 afetr receiving M2D
			else if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_RX_M2D)
			{
				// Process M2
				if (WscReceiveEapM2(pAdapter, Id, Elem))    // succeed
				{
					// Send out M3
					WscSendEapM3(pAdapter, Id, Elem);
					// Change the state to next one
					pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_M4;
				}
				else    // fail
				{
					// For a invalid nonce or authenticator:
					// sliently ignore this message and stay at the same state 
				}

			}
			
			// EAP Rx processed, set new EAP Rx timer
			RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer, WSC_PER_MSG_TIMEOUT);

			break;
			
		case WSC_MSG_M2D:
			// Receive M2D, if we are at WSC_STATE_WAIT_M2 start, process it immediately
			if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_WAIT_M2)
			{
				// Process M2D
				// Check our own ENonce. If mismatch, return false
				if (WscReceiveEapM2D(pAdapter, Id, Elem))
				{
					// Send out EAP-Rsp-Ack
					WscSendEapRspAck(pAdapter, Id, Elem);

					// Change the state to next one
					pAdapter->PortCfg.WscControl.WscState = WSC_STATE_RX_M2D;
				}
			}
			// Multiple M2D
			else if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_RX_M2D)
			{
				// Process M2D
				// Check our own ENonce. If mismatch, return false
				if (WscReceiveEapM2D(pAdapter, Id, Elem))
				{
					// Send out EAP-Rsp-Ack
					WscSendEapRspAck(pAdapter, Id, Elem);
				}
			}
			break;

		case WSC_MSG_M4:
			// Receive M4, if we are at WSC_STATE_WAIT_M4 start, process it immediately
			if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_WAIT_M4)
			{
				// Process M4
				if (WscReceiveEapM4(pAdapter, Id, Elem))    // succeed
				{
					// Send out M5
					WscSendEapM5(pAdapter, Id, Elem);
					// Change the state to next one
					pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_M6;
				}
				else    // fail
				{
					// TODO:  Might need to send out a Nack and change  the state to wait for an EAP-FAIL
					if ((pAdapter->PortCfg.WscControl.WscStatus == STATUS_WSC_PASSWORD_MISMATCH) &&
						(pAdapter->PortCfg.WscControl.WscState != WSC_STATE_WAIT_EAPFAIL))
					{
#ifdef SKY_LITTLE_ENDIAN
    {
    	unsigned short tmp;
    	tmp = WSC_ERROR_DEV_PWD_AUTH_FAIL;
        pAdapter->PortCfg.WscControl.RegData.EnrolleeInfo.ConfigError = cpu2be16((unsigned char *)&tmp);
    }
#else
		pAdapter->PortCfg.WscControl.RegData.EnrolleeInfo.ConfigError = cpu2be16(WSC_ERROR_DEV_PWD_AUTH_FAIL);
#endif
						WscSendEapRspNack(pAdapter, Id, Elem);
						// Change the state to next one
						pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_EAPFAIL;
					}

					// For a invalid nonce or authenticator:
					// sliently ignore this message and stay at the same state
				}

			}
			// We have to resend M5 if we receive M4 instead of M6
			else if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_WAIT_M6)
			{
				// Process M4
				if (WscReceiveEapM4(pAdapter, Id, Elem))    // succeed
				{            
					// Send out M5
					WscSendEapM5(pAdapter, Id, Elem);
					// Change the state to next one
					pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_M6;
				}
				else    // fail
				{
					// TODO:  Might need to send out a Nack and change  the state to wait for an EAP-FAIL
					if ((pAdapter->PortCfg.WscControl.WscStatus == STATUS_WSC_PASSWORD_MISMATCH) &&
						(pAdapter->PortCfg.WscControl.WscState != WSC_STATE_WAIT_EAPFAIL))
					{
#ifdef SKY_LITTLE_ENDIAN
	{
    	unsigned short tmp;
    	tmp = WSC_ERROR_DEV_PWD_AUTH_FAIL;
        pAdapter->PortCfg.WscControl.RegData.EnrolleeInfo.ConfigError = cpu2be16((unsigned char *)&tmp);
    }
#else
		pAdapter->PortCfg.WscControl.RegData.EnrolleeInfo.ConfigError = cpu2be16(WSC_ERROR_DEV_PWD_AUTH_FAIL);
#endif
						WscSendEapRspNack(pAdapter, Id, Elem);
						// Change the state to next one
						pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_EAPFAIL;
					}
					
					// For a invalid nonce or authenticator:
					// sliently ignore this message and stay at the same state 
				}

			}

			// EAP Rx processed, set new EAP Rx timer
			if (pAdapter->PortCfg.WscControl.WscState != WSC_STATE_WAIT_EAPFAIL)
				RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer, WSC_PER_MSG_TIMEOUT);
			
			break;

		case WSC_MSG_M6:
			// Receive M6, if we are at WSC_STATE_WAIT_M6 start, process it immediately
			if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_WAIT_M6)
			{
				// Process M6
				if (WscReceiveEapM6(pAdapter, Id, Elem))    // succeed
				{            
					// Send out M7
					WscSendEapM7(pAdapter, Id, Elem);
					// Change the state to next one
					pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_M8;
				}
				else    // fail
				{
					// TODO:  Might need to send out a Nack and change  the state to wait for an EAP-FAIL
					if ((pAdapter->PortCfg.WscControl.WscStatus == STATUS_WSC_PASSWORD_MISMATCH) &&
						(pAdapter->PortCfg.WscControl.WscState != WSC_STATE_WAIT_EAPFAIL))
					{
#ifdef SKY_LITTLE_ENDIAN
	{
    	unsigned short tmp;
    	tmp = WSC_ERROR_DEV_PWD_AUTH_FAIL;
        pAdapter->PortCfg.WscControl.RegData.EnrolleeInfo.ConfigError = cpu2be16((unsigned char *)&tmp);
    }
#else
		pAdapter->PortCfg.WscControl.RegData.EnrolleeInfo.ConfigError = cpu2be16(WSC_ERROR_DEV_PWD_AUTH_FAIL);
#endif
						WscSendEapRspNack(pAdapter, Id, Elem);
						// Change the state to next one
						pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_EAPFAIL;
					}

					// For a invalid nonce or authenticator:
					// sliently ignore this message and stay at the same state 
				}
                
			}
			// We have to resend M7 if we receive M6 instead of M8
			else if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_WAIT_M8)
			{
				// Process M6
				if (WscReceiveEapM6(pAdapter, Id, Elem))    // succeed
				{            
					// Send out M7
					WscSendEapM7(pAdapter, Id, Elem);
					// Change the state to next one
					pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_M8;
				}
				else    // fail
				{
					// TODO:  Might need to send out a Nack and change  the state to wait for an EAP-FAIL
					if ((pAdapter->PortCfg.WscControl.WscStatus == STATUS_WSC_PASSWORD_MISMATCH) &&
						(pAdapter->PortCfg.WscControl.WscState != WSC_STATE_WAIT_EAPFAIL))
					{
#ifdef SKY_LITTLE_ENDIAN
	{
    	unsigned short tmp;
    	tmp = WSC_ERROR_DEV_PWD_AUTH_FAIL;
        pAdapter->PortCfg.WscControl.RegData.EnrolleeInfo.ConfigError = cpu2be16((unsigned char *)&tmp);
    }
#else
		pAdapter->PortCfg.WscControl.RegData.EnrolleeInfo.ConfigError = cpu2be16(WSC_ERROR_DEV_PWD_AUTH_FAIL);
#endif
						WscSendEapRspNack(pAdapter, Id, Elem);
						// Change the state to next one
						pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_EAPFAIL;
					}
					
					// For a invalid nonce or authenticator:
					// sliently ignore this message and stay at the same state 
				} 
			}

			// EAP Rx processed, set new EAP Rx timer
			if (pAdapter->PortCfg.WscControl.WscState != WSC_STATE_WAIT_EAPFAIL)
				RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer, WSC_PER_MSG_TIMEOUT);
			
			break;

		case WSC_MSG_M8:
			// Receive M8, if we are at WSC_STATE_WAIT_M6 start, process it immediately
			if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_WAIT_M8)
			{
				// Process M8
				if (WscReceiveEapM8(pAdapter, Id, Elem))    // succeed
				{            
					// Send out EAP-Rsp-Done
					WscSendEapRspDone(pAdapter, Id, Elem);
					// Change the state to next one
					pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_DISCONN;

				}
				else    // fail
				{
					// For a invalid nonce or authenticator:
					// sliently ignore this message and stay at the same state 
				}
			}
			
			if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_WAIT_DISCONN)
			{ // Set a new timer. timeout = Per Message Timeout on AP (15 sec)
			  // Regardless of Eap-Fail, this will be configured after timeout.
				RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscM8WaitEapFailTimer,&Cancelled);
				RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscM8WaitEapFailTimer, 15000);
			}
			else if (pAdapter->PortCfg.WscControl.WscState != WSC_STATE_WAIT_EAPFAIL)
			{ // EAP Rx processed, set new EAP Rx timer
				RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer, WSC_PER_MSG_TIMEOUT);
			}
			break;

		case WSC_MSG_EAP_FAIL:
			// Receive Eap-Fail at WSC_STATE_WAIT_DISCONN, process it and go to WSC_STATE_CONFIGURED
			if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_WAIT_DISCONN)
			{
				NDIS_802_11_SSID	Ssid;  // add by johnli
				
				// Receive Eap-Fail, cancel timer
				RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscM8WaitEapFailTimer,&Cancelled);
				
				// call to process Eap-Fail
				WscReceiveEapFail(pAdapter, Id, Elem);
				
#if 1			// reconfigure itself and write back to datFile
				pAdapter->PortCfg.WscControl.WscProfile.ApplyProfileIdx = 0;  // add by johnli, fix WPS test plan 5.1.1
				WscWriteConfToPortCfg(pAdapter);
				pAdapter->WriteWscCfgToDatFile = TRUE;
 				up(&(pAdapter->write_dat_file_semaphore));      
#endif	    
				
				// Update WSC status
				pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_CONFIGURED;	

				// Change the state to next one
				pAdapter->PortCfg.WscControl.WscState = WSC_STATE_OFF;
				
				// stop work
				pAdapter->PortCfg.WscControl.WscTriggered = FALSE;
				
				// add by johnli, Enqueue SSID connection command
				RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscEndTimer, 500);  // 0.5 sec
/*
				if (pAdapter->Mlme.CntlMachine.CurrState != CNTL_IDLE)
				{
					MlmeRestartStateMachine(pAdapter);
					DBGPRINT(RT_DEBUG_WPS, "!!! MLME busy, reset MLME state machine !!!\n");
				}

				NdisZeroMemory(&Ssid, sizeof(NDIS_802_11_SSID));
				NdisMoveMemory(Ssid.Ssid, pAdapter->MlmeAux.Ssid, pAdapter->MlmeAux.SsidLen);
				Ssid.SsidLength = pAdapter->MlmeAux.SsidLen;
				
				pAdapter->bConfigChanged = TRUE;
				MlmeEnqueue(pAdapter, 
							MLME_CNTL_STATE_MACHINE, 
							OID_802_11_SSID,
							sizeof(NDIS_802_11_SSID),
							(PVOID) &Ssid);
				MlmeHandler(pAdapter);
				
				DBGPRINT(RT_DEBUG_WPS, "WscEAPAction:: Enqueue OID_802_11_SSID(SsidLen=%d,Ssid=%s)\n", Ssid.SsidLength, Ssid.Ssid);
				// end johnli
*/
			}
			// edit by johnli
			// Receive EAP fail at M2D state, restart the association
//			else if (pAdapter->PortCfg.WscControl.WscState == WSC_STATE_RX_M2D)
			// Received unexpected EAP-FAIL and expected EAP-FAIL at the state of M2D, restart the association
			else if (pAdapter->PortCfg.WscControl.WscState != WSC_STATE_WAIT_EAPFAIL)
			// end johnli
			{
				// call to process Eap-Fail
				WscReceiveEapFail(pAdapter, Id, Elem);
				
				// re-associate immediately and change to state WSC_STATE_START
				RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer, 0);
				
			}
			// Receive Eap-Fail, it indicate the EAP failed, we have to set the correct state and staus
			else
			{
				// call to process Eap-Fail
				WscReceiveEapFail(pAdapter, Id, Elem);
				
				// Update WSC status
				pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_FAILED;	

				// Change the state to next one
				pAdapter->PortCfg.WscControl.WscState = WSC_STATE_OFF;
				
			}
			DBGPRINT(RT_DEBUG_WPS, "WscEAPAction : Rx EAP-FAIL\n");			

		case WSC_MSG_WSC_NACK:
			// Receive EAP-Req-Nack at entering lock-step (M3~Done)
			if (pAdapter->PortCfg.WscControl.WscState >= WSC_STATE_WAIT_M4)
				WscReceiveEapNack(pAdapter, Id, Elem);
			break;

		default:
			DBGPRINT(RT_DEBUG_WPS, "WscEAPAction : Unsupported Msg Type\n");	   
			break;
	}
	
	DBGPRINT(RT_DEBUG_WPS, "<----- WscEAPAction\n");
}

/*
	========================================================================
	
	Routine Description:
		Classify EAP message type for enrolee

	Arguments:
		pAd         - NIC Adapter pointer
		Elem		- The EAP packet
		
	Return Value:
		Received EAP message type

	IRQL = DISPATCH_LEVEL
	
	Note:
		
	========================================================================
*/
UCHAR	WscRxMsgType(
	IN	PRTMP_ADAPTER		pAdapter,
	OUT	PUCHAR				pId,
	IN	PMLME_QUEUE_ELEM	pElem) 
{
	USHORT				Length;
	PUCHAR				pData;
	PIEEE8021X_FRAME	p802_1x;
	PEAP_FRAME			pEap;
	PWSC_FRAME			pWsc;
	USHORT				WscType, WscLen;

	// Skip the EAP LLC header check since it's been checked many times.
	pData = (PUCHAR) &pElem->Msg[LENGTH_802_11 + LENGTH_802_1_H];
	pData += sizeof(IEEE8021X_FRAME);

	// start of 802.1x frame header
	p802_1x = (PIEEE8021X_FRAME) pData;
#ifdef SKY_LITTLE_ENDIAN
    Length = be2cpu16((unsigned char *)&p802_1x->Length);
#else
	Length = be2cpu16(p802_1x->Length);
#endif
	// Start of EAP frame
	pEap = (PEAP_FRAME) pData;

	// check for EAP-Fail which has length smaller than 5
	if (pEap->Code == EAP_CODE_FAIL)
	{
		// Receive EAP-Fail
		*pId = pEap->Id;
		return(WSC_MSG_EAP_FAIL);
	}

	// check length for EAP-packet WSC type
	if (Length < sizeof(EAP_FRAME))
	{
		DBGPRINT(RT_DEBUG_WPS, "WscRxMsgType : EAP length is too small --> %d; \n", Length);	   
		return(WSC_MSG_UNKNOWN);
	}
	// Length include EAP 5 bytes header
	pData += sizeof(EAP_FRAME);
	Length -= 5;	
	*pId = pEap->Id;
	
	// For EAP request, it include
	//		EAP-Req ID
	//		EAP-Req Start
	//		EAP-Fail ???
	//
	// For EAP response, it include
	//		M2, M2D, M4, M6, M8
	//		EAP-Rsp Done Nack
	//		EAP-Fail ???
	switch (pEap->Code)
	{
		case EAP_CODE_REQ:
			switch (pEap->Type)
			{
				case EAP_TYPE_ID:
					return(WSC_MSG_EAP_REQ_ID);
					break;
					
				case EAP_TYPE_WSC:
					// WSC messages, check for WSC Op-Code, or Messages
					pWsc = (PWSC_FRAME) pData;
					pData += sizeof(WSC_FRAME);
					Length -= sizeof(WSC_FRAME);

					// Verify SMI first
					if (((pWsc->SMI[0] * 256 + pWsc->SMI[1]) * 256 + pWsc->SMI[2]) != WSC_SMI)
					{
						// Wrong WSC SMI Vendor ID, Update WSC status
						pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_REQ_WRONG_SMI;				
						return(WSC_MSG_UNKNOWN);
					}

					// Verify Vendor Type
#ifdef  SKY_LITTLE_ENDIAN
                    if (cpu2be32((unsigned char *)&pWsc->VendorType) != WSC_VENDOR_TYPE)
#else
					if (cpu2be32(pWsc->VendorType) != WSC_VENDOR_TYPE)
#endif
					{
						// Wrong WSC Vendor Type, Update WSC status
						pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_REQ_WRONG_VENDOR_TYPE;				
						return(WSC_MSG_UNKNOWN);
					}

					if (pWsc->OpCode == WSC_OPCODE_START)
					{
						// Receive WSC Start
						pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_REQ_WSC_START;				
						return(WSC_MSG_EAP_REQ_START);
					}

					// Not Wsc Start, We have to look for WSC_IE_MSG_TYPE to classify M2 ~ M8, the remain size must large than 4
					while (Length > 4)
					{
#ifdef SKY_LITTLE_ENDIAN
	{
		WscType = cpu2be16((unsigned char *)pData);
		WscLen  = cpu2be16((unsigned char *)(pData + 2));
	}
#else
		WscType = cpu2be16(*((PUSHORT) pData));
		WscLen  = cpu2be16(*((PUSHORT) (pData + 2)));
	}
#endif
						if (WscType == WSC_IE_MSG_TYPE)
						{
							return(*(pData + 4));	// Found the message type
						}
						else
						{
							pData  += (WscLen + 4);
							Length -= (WscLen + 4);
						}
					}					
					break;
					
				default:
					DBGPRINT(RT_DEBUG_WPS, "WscRxMsgType : Unsupported EAP Code --> %d; \n", pEap->Code);	   
					break;							
			}
			break;
			
		case EAP_CODE_RSP:
			DBGPRINT(RT_DEBUG_WPS, "WscRxMsgType : Not supposed to receive EAP-Rsp\n");	   
			break;
			
		case EAP_CODE_FAIL:
			// Receive EAP-Fail
			return(WSC_MSG_EAP_FAIL);
			break;

		default:
			DBGPRINT(RT_DEBUG_WPS, "WscRxMsgType : Unsupported EAP Code --> %d; \n", pEap->Code);	   
			break;			
	}
	
	return(WSC_MSG_UNKNOWN);
}

/*
	========================================================================
	
	Routine Description:
		Classify WSC message type

	Arguments:
		EAPType		Value of EAP message type
		MsgType		Internal Message definition for MLME state machine
		
	Return Value:
		TRUE		Found appropriate message type
		FALSE		No appropriate message type

	IRQL = DISPATCH_LEVEL
	
	Note:
		All these constants are defined in wpa.h
		For supplicant, there is only EAPOL Key message avaliable
		
	========================================================================
*/
BOOLEAN	WscMsgTypeSubst(
	IN	UCHAR	EAPType,
	IN	UCHAR	EAPCode,
	OUT	ULONG	*MsgType)	
{
	switch (EAPType)
	{
		case EAPPacket:
			switch (EAPCode)
			{
				case EAP_CODE_REQ:
					*MsgType = WSC_EAP_REQ_MSG;
					break;

				case EAP_CODE_RSP:
					*MsgType = WSC_EAP_RSP_MSG;
					break;

				case EAP_CODE_FAIL:
					*MsgType = WSC_EAP_FAIL_MSG;
					break;

				default:
					DBGPRINT(RT_DEBUG_WPS, "WscMsgTypeSubst : unsupported EAP Code; \n");	   
					return FALSE;		
			}
			break;
			
		default:
			DBGPRINT(RT_DEBUG_WPS, "WscMsgTypeSubst : unsupported EAP Type; \n");	   
			return FALSE;		
	}	
	return TRUE;
}

/*
	========================================================================
	
	Routine Description:
		WSC connection time out

	Arguments:
		FunctionContext		NIC Adapter pointer
		
	Return Value:
		None
		
	IRQL = DISPATCH_LEVEL
	
	Note:
		
	========================================================================
*/
VOID WscConnectTimeout(
	IN PVOID SystemSpecific1,
	IN PVOID FunctionContext,
	IN PVOID SystemSpecific2,
	IN PVOID SystemSpecific3)
{
	RTMP_ADAPTER *pAdapter = (RTMP_ADAPTER *)FunctionContext;
	BOOLEAN Cancelled;
	// 1. cancel all timer
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscConnectTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscScanTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscPBCTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscPINTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscM8WaitEapFailTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapM2Timer,&Cancelled);

	pAdapter->PortCfg.WscControl.WscTriggered = FALSE; // disable on connection timeout

	// if link is up, there shall be nothing wrong
	// perhaps we will set another flag to do it
	if ((OPSTATUS_TEST_FLAG(pAdapter, fOP_STATUS_MEDIA_STATE_CONNECTED)) && 
		(pAdapter->PortCfg.WscControl.WscState == WSC_STATE_OFF) &&
		(pAdapter->PortCfg.WscControl.WscStatus == STATUS_WSC_EAP_CONFIGURED))
	{
		DBGPRINT(RT_DEBUG_WPS, "WscConnectTimeout --> Connection OK\n");
	}
	else
	{
		pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_FAILED;	
		pAdapter->PortCfg.WscControl.WscState = WSC_STATE_OFF;
		DBGPRINT(RT_DEBUG_WPS, "WscConnectTimeout --> Fail to connect\n");
	}
}

/*
	========================================================================
	
	Routine Description:
		Exec scan after scan timer expiration 

	Arguments:
		FunctionContext		NIC Adapter pointer
		
	Return Value:
		None
		
	IRQL = DISPATCH_LEVEL
	
	Note:
		
	========================================================================
*/
VOID WscScanTimeout(
	IN PVOID SystemSpecific1,
	IN PVOID FunctionContext,
	IN PVOID SystemSpecific2,
	IN PVOID SystemSpecific3)
{
    RTMP_ADAPTER *pAdapter = (RTMP_ADAPTER *)FunctionContext;

	// call to execute the scan actions
	WscScanExec(pAdapter);

	// register 10 second timer for PBC or PIN connection execution
	if (pAdapter->PortCfg.WscControl.WscMode == WSC_PBC_MODE)
	{
	    // Prevent infinite loop if conncet time out didn't stop the repeat scan
		if (pAdapter->PortCfg.WscControl.WscState != WSC_STATE_OFF)
		    RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscPBCTimer, 10000);
	}
	else if (pAdapter->PortCfg.WscControl.WscMode == WSC_PIN_MODE)
	{
        // Prevent infinite loop if conncet time out didn't stop the repeat scan
		if (pAdapter->PortCfg.WscControl.WscState != WSC_STATE_OFF)
		    RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscPINTimer, 10000);
	}

	DBGPRINT(RT_DEBUG_WPS, "!!! WscScanTimeout !!!\n");
    
	// call Mlme handler to execute it
	MlmeHandler(pAdapter);
    
}

/*
	========================================================================
	
	Routine Description:
		Exec PBC connection verification after PBC timer expiration 

	Arguments:
		FunctionContext		NIC Adapter pointer
		
	Return Value:
		None
		
	IRQL = DISPATCH_LEVEL
	
	Note:
		
	========================================================================
*/
VOID WscPBCTimeout(
	IN PVOID SystemSpecific1,
	IN PVOID FunctionContext,
	IN PVOID SystemSpecific2,
	IN PVOID SystemSpecific3)
{
    RTMP_ADAPTER *pAdapter = (RTMP_ADAPTER *)FunctionContext;

	DBGPRINT(RT_DEBUG_WPS, "!!! WscPBCTimeout !!!\n");
	WscPBCExec(pAdapter);

	// call Mlme handler to execute it
	MlmeHandler(pAdapter);
}

VOID WscPINTimeout(
	IN PVOID SystemSpecific1,
	IN PVOID FunctionContext,
	IN PVOID SystemSpecific2,
	IN PVOID SystemSpecific3)
{
    RTMP_ADAPTER *pAdapter = (RTMP_ADAPTER *)FunctionContext;

	DBGPRINT(RT_DEBUG_WPS, "!!! WscPINTimeout !!!\n");

	WscPINExec(pAdapter);
}

VOID WscEAPRxTimeout(
	IN PVOID SystemSpecific1,
	IN PVOID FunctionContext,
	IN PVOID SystemSpecific2,
	IN PVOID SystemSpecific3)
{
    RTMP_ADAPTER *pAdapter = (RTMP_ADAPTER *)FunctionContext;
	MLME_DISASSOC_REQ_STRUCT   DisassocReq;

	// No EAP receive within WSC_PER_MSG_TIMEOUT, reset the state machine
	// and redo protocols if it's still within 2 min
	// Set WSC state to WSC_STATE_START
	pAdapter->PortCfg.WscControl.WscState = WSC_STATE_START;
	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_START_ASSOC;
	DBGPRINT(RT_DEBUG_WPS, "!!! WscEAPRxTimeout !!!\n");

	if(INFRA_ON(pAdapter))
	{
		DBGPRINT(RT_DEBUG_WPS, "Disassociate with current WPS AP...\n");
		DisassocParmFill(pAdapter, &DisassocReq, pAdapter->PortCfg.Bssid, REASON_DISASSOC_STA_LEAVING);
		MlmeEnqueue(pAdapter, ASSOC_STATE_MACHINE, MT2_MLME_DISASSOC_REQ, 
			sizeof(MLME_DISASSOC_REQ_STRUCT), &DisassocReq);
		pAdapter->Mlme.CntlMachine.CurrState = CNTL_WAIT_DISASSOC;

	}
}

VOID WscM8WaitEapFailTimeout(
	IN PVOID SystemSpecific1,
	IN PVOID FunctionContext,
	IN PVOID SystemSpecific2,
	IN PVOID SystemSpecific3)
{
    RTMP_ADAPTER *pAdapter = (RTMP_ADAPTER *)FunctionContext;

	// It is not important for lost Eap-Fail
	// Wait Eap-Fail until timeout(WSC_PER_MSG_TIMEOUT), set WPS state to complete state
	pAdapter->PortCfg.WscControl.WscState = WSC_STATE_OFF;
	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_CONFIGURED;

	DBGPRINT(RT_DEBUG_WPS, "!!! WscM8WaitEapFailTimeout !!!\n");
}

VOID WscEapM2Timeout(
	IN PVOID SystemSpecific1,
	IN PVOID FunctionContext,
	IN PVOID SystemSpecific2,
	IN PVOID SystemSpecific3)
{
    RTMP_ADAPTER *pAdapter = (RTMP_ADAPTER *)FunctionContext;
	BOOLEAN Cancelled;

	/* 1. M2Timer has started while M1 sent
	 * 2. Cancel it once STA re-associate. Imply that ...
	 * 		->Reset EapRxTimer beyond the state of WSC_STATE_WAIT_M2
	 *		->Received Eap-Fail after Rsp-Ack	
	 * 3. Excute this timeout
	 */
	 
	if ((pAdapter->PortCfg.WscControl.WscState > WSC_STATE_OFF) &&
		(pAdapter->PortCfg.WscControl.WscState < WSC_STATE_WAIT_M4))
	{
		// re-associate immediately and change to state WSC_STATE_START
		RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer,&Cancelled);
		RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer, 0);
		
		DBGPRINT(RT_DEBUG_WPS, "!!! WscEapM2Timeout !!!\n");
	}
}
/*
	========================================================================
	
	Routine Description:
		Push PBC from HW/SW Buttton

	Arguments:
		pAdapter    - NIC Adapter pointer
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		
	========================================================================
*/

VOID	WscPushPBCAction(
	IN	PRTMP_ADAPTER	pAdapter)
{		
	BOOLEAN Cancelled;
	//DBGPRINT(RT_DEBUG_WPS, "-----> WscPushPBCAction\n");

	// 0. PBC mode, disregard the SSID information, we have to get the current AP list 
	//    and check the beacon for Push buttoned AP.
	// 1. Cancel old timer to prevent use push continuously
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscConnectTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscScanTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscPBCTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscPINTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscM8WaitEapFailTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapM2Timer,&Cancelled);

	// Set WSC state to WSC_STATE_INIT
	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_SCAN_AP;

	// Init Registrar pair structures
	WscInitRegistrarPair(pAdapter);

	// For PBC, the PIN is all '0'
	WscGetRegDataPIN(pAdapter, pAdapter->PortCfg.WscControl.WscPinCode);
			
	// 2. Set 2 min timout routine
	RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscConnectTimer, WSC_REG_SESSION_TIMEOUT);
	pAdapter->PortCfg.WscControl.WscTriggered = TRUE;	// start work

	// 3. Call WscScan subroutine
	WscScanExec(pAdapter);

	// 4. Set 10 second timer to invoke PBC connection actions.
	RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscPBCTimer, 10000);
	DBGPRINT(RT_DEBUG_WPS, "<----- WscPushPBCAction\n");	

}

// add by johnli
VOID WscEndTimeout(
	IN PVOID SystemSpecific1,
	IN PVOID FunctionContext,
	IN PVOID SystemSpecific2,
	IN PVOID SystemSpecific3)
{
	RTMP_ADAPTER *pAdapter = (RTMP_ADAPTER *)FunctionContext;
	NDIS_802_11_SSID	Ssid;  // add by johnli
DBGPRINT(RT_DEBUG_WPS,"!!! WscEndTimeout !!!\n");
	if (pAdapter->Mlme.CntlMachine.CurrState != CNTL_IDLE)
	{
		MlmeRestartStateMachine(pAdapter);
		DBGPRINT(RT_DEBUG_WPS, "!!! MLME busy, reset MLME state machine !!!\n");
	}

	NdisZeroMemory(&Ssid, sizeof(NDIS_802_11_SSID));
// edit by johnli, fix WPS connect failed problem with PCI BLW-54CW3 AP
/*
	NdisMoveMemory(Ssid.Ssid, pAdapter->MlmeAux.Ssid, pAdapter->MlmeAux.SsidLen);
	Ssid.SsidLength = pAdapter->MlmeAux.SsidLen;
*/
	NdisMoveMemory(Ssid.Ssid, pAdapter->MlmeAux.AutoReconnectSsid, pAdapter->MlmeAux.AutoReconnectSsidLen);
	Ssid.SsidLength = pAdapter->MlmeAux.AutoReconnectSsidLen;
	// end johnli
	
	pAdapter->bConfigChanged = TRUE;
	MlmeEnqueue(pAdapter, 
				MLME_CNTL_STATE_MACHINE, 
				OID_802_11_SSID,
				sizeof(NDIS_802_11_SSID),
				(PVOID) &Ssid);
	MlmeHandler(pAdapter);
	
	DBGPRINT(RT_DEBUG_WPS, "WscEAPAction:: Enqueue OID_802_11_SSID(SsidLen=%d,Ssid=%s)\n", Ssid.SsidLength, Ssid.Ssid);
}
// end johnli

// add by johnli, fix WPS test plan 5.1.1
VOID WscProfileRetryTimeout(
	IN PVOID SystemSpecific1,
	IN PVOID FunctionContext,
	IN PVOID SystemSpecific2,
	IN PVOID SystemSpecific3)
{
	RTMP_ADAPTER *pAdapter = (RTMP_ADAPTER *)FunctionContext;
	DBGPRINT(RT_DEBUG_WPS, "WscProfileRetryTimeout:: WSC profile retry timeout index: %d\n", pAdapter->PortCfg.WscControl.WscProfile.ApplyProfileIdx);

	if (!INFRA_ON(pAdapter))
	{
		if (pAdapter->PortCfg.WscControl.WscProfile.ApplyProfileIdx < pAdapter->PortCfg.WscControl.WscProfile.ProfileCnt-1)
			pAdapter->PortCfg.WscControl.WscProfile.ApplyProfileIdx++;
		else
			pAdapter->PortCfg.WscControl.WscProfile.ApplyProfileIdx = 0;
			
		WscWriteConfToPortCfg(pAdapter);
		pAdapter->WriteWscCfgToDatFile = TRUE;
		up(&(pAdapter->write_dat_file_semaphore));      
	
		DBGPRINT(RT_DEBUG_WPS, "WscProfileRetryTimeout:: WSC profile retry index: %d\n", pAdapter->PortCfg.WscControl.WscProfile.ApplyProfileIdx);
	}
}
// end johnli

/*
	========================================================================
	
	Routine Description:
		Doing an active scan with empty SSID, the scanened list will
		be processed in PBCexec or PINexec routines

	Arguments:
		pAd         - NIC Adapter pointer
		
	Return Value:
		None
		
	IRQL = DISPATCH_LEVEL
	
	Note:
		
	========================================================================
*/
VOID	WscScanExec(
	IN	PRTMP_ADAPTER	pAdapter) 
{
	MLME_SCAN_REQ_STRUCT	ScanReq;
	UCHAR					ZeroSsid[32];
	// Make sure connect state machine is idle
	if (pAdapter->Mlme.CntlMachine.CurrState != CNTL_IDLE)
	{
		MlmeRestartStateMachine(pAdapter);
		DBGPRINT(RT_DEBUG_WPS, "!!! MLME busy, reset MLME state machine !!!\n");
	}

	// Prevent infinite loop if conncet time out didn't stop the repeat scan
	if (pAdapter->PortCfg.WscControl.WscStatus == STATUS_WSC_EAP_FAILED)
		return;

	DBGPRINT(RT_DEBUG_WPS, "!!! WscScanExec !!!\n");

	BssTableInit(&pAdapter->ScanTab);  // add by johnli, clean up previous SCAN result

	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_SCAN_AP;

	// Prepare Scan parameters and enqueue the MLME command
	NdisZeroMemory(ZeroSsid, 32);
	ScanParmFill(pAdapter, &ScanReq, ZeroSsid, 0, BSS_ANY, SCAN_WSC_ACTIVE);
	MlmeEnqueue(pAdapter, SYNC_STATE_MACHINE, MT2_MLME_SCAN_REQ, sizeof(MLME_SCAN_REQ_STRUCT), &ScanReq);
	pAdapter->Mlme.CntlMachine.CurrState = CNTL_WAIT_OID_LIST_SCAN;
}

/*
	========================================================================
	
	Routine Description:
		Doing PBC conenction verification, it will check current BSS list
		and find the correct number of PBC AP. If only 1 exists, it will
		start to make connection. Otherwise, it will set a scan timer
		to perform another scan for next PBC connection execution.

	Arguments:
		pAd         - NIC Adapter pointer
		
	Return Value:
		None
		
	IRQL = DISPATCH_LEVEL
	
	Note:
		
	========================================================================
*/
VOID	WscPBCExec(
	IN	PRTMP_ADAPTER	pAdapter) 
{
	NDIS_802_11_SSID	WscSSID;

	// 1. Search the qualified SSID from current SSID list
	WscPBCBssTableSort(pAdapter, &pAdapter->PortCfg.WscControl.WscBssTab);
	// 2. Check the qualified AP for connection, if more than 1 AP avaliable, report error.
	if (pAdapter->PortCfg.WscControl.WscBssTab.BssNr != 1)
	{
		// Set WSC state to WSC_FAIL
		pAdapter->PortCfg.WscControl.WscState = WSC_STATE_FAIL;
		if (pAdapter->PortCfg.WscControl.WscBssTab.BssNr == 0)
			pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_PBC_NO_AP;
		else
			pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_PBC_TOO_MANY_AP;			
		DBGPRINT(RT_DEBUG_WPS, "WscPBCExec --> AP list is %d, wait for next time\n", pAdapter->PortCfg.WscControl.WscBssTab.BssNr);

		// 2.1. Set 10 second timer to invoke another scan
		RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscScanTimer, 10000);
		// 2.2 We have to quit for now
		return;
	}

	// 3. Now we got the intend AP, Set the WSC state and enqueue the SSID connection command	
	pAdapter->MlmeAux.CurrReqIsFromNdis = FALSE; 

	if (pAdapter->Mlme.CntlMachine.CurrState != CNTL_IDLE)
	{
		MlmeRestartStateMachine(pAdapter);
		DBGPRINT(RT_DEBUG_WPS, "!!! WscPBCExec --> MLME busy, reset MLME state machine !!!\n");
	}
	// 4. Set WSC state to WSC_STATE_START
	pAdapter->PortCfg.WscControl.WscState = WSC_STATE_START;
	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_START_ASSOC;

	// 5. Copy the entry from BSS table to WscMode structure
	WscSSID.SsidLength = pAdapter->PortCfg.WscControl.WscBssTab.BssEntry[0].SsidLen;
	NdisMoveMemory(WscSSID.Ssid, pAdapter->PortCfg.WscControl.WscBssTab.BssEntry[0].Ssid, WscSSID.SsidLength);

	// 6. Enqueue SSID connection command
	MlmeEnqueue(pAdapter, 
				MLME_CNTL_STATE_MACHINE, 
				OID_802_11_SSID, 
				sizeof(NDIS_802_11_SSID), 
				(PVOID) &WscSSID);
}

VOID	WscPINExec(
	IN	PRTMP_ADAPTER	pAdapter) 
{
}

/*
	========================================================================
	
	Routine Description:
		Find WSC PBC activated AP list

	Arguments:
		pAd         - NIC Adapter pointer
		OutTab		- Qualified AP BSS table
		
	Return Value:
		None
		
	IRQL = DISPATCH_LEVEL
	
	Note:
		All these constants are defined in wsc.h
		
	========================================================================
*/
VOID WscPBCBssTableSort(
	IN	PRTMP_ADAPTER	pAdapter, 
	OUT BSS_TABLE		*OutTab) 
{
	INT					i, j, Len, idx;
	UCHAR				*pData;
	PBSS_ENTRY			pInBss, pOutBss;
	PBEACON_EID_STRUCT	pEid;
	PWSC_IE				pWscIE;
	USHORT				DevicePasswordID;
	BOOLEAN				Found;
	UCHAR				UUID[8][16];
	UCHAR				TmpUUID[16];
	UCHAR				Duplicate;
	UCHAR				zeros16[16]= {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

	// initialize Output BSS Table
	BssTableInit(OutTab);
	for (i = 0; i < pAdapter->ScanTab.BssNr; i++) 
	{
		pInBss  = (PBSS_ENTRY) &pAdapter->ScanTab.BssEntry[i];	// BSS entry for VarIE processing
		pOutBss = (PBSS_ENTRY)&OutTab->BssEntry[OutTab->BssNr];		// avaliable entry for new BSS list
		pData   = (UCHAR *) &pInBss->VarIEs;
		Found   = FALSE;
		NdisZeroMemory(&TmpUUID[0], 16);

		// 1. Check VarIE length
		if (pInBss->VarIELen == 0)
			continue;

		// 2. Search for WSC IE - 0xdd xx 00 50 f2 04
		Len = pInBss->VarIELen;
		while ((Len > 0) && (Found == FALSE))
		{
			pEid = (PBEACON_EID_STRUCT) pData;
			// No match, skip the Eid and move forward, IE_WFA_WSC = 0xdd
			if (pEid->Eid != IE_WFA_WSC)
			{
				// Set the offset and look for next IE
				pData += (pEid->Len + 2);
				Len   -= (pEid->Len + 2);
				continue;
			}
			else
			{
				// Found IE with 0xdd
				// check for WSC OUI -- 00 50 f2 04
#ifndef SKY_LITTLE_ENDIAN       //tyhuang: we can not use this kind of function in our platform
				if (be2cpu32(*(ULONG *)&pEid->Octet) != WSC_OUI)
#else
				 if(be2cpu32(&pEid->Octet[0])!=WSC_OUI)
#endif
				{
					// Set the offset and look for next IE
					pData += (pEid->Len + 2);
					Len   -= (pEid->Len + 2);
					continue;
				}				
			}
			// 3. Found	AP with WSC IE in beacons, skip 6 bytes = 1 + 1 + 4
			pData += 6;
			Len   -= 6;

			// 4. Start to look the PBC type within WSC VarIE
			while (Len > 0)
			{
				// Check for WSC IEs
				pWscIE = (PWSC_IE) pData;

				// Check for device password ID, PBC = 0x0004
#ifdef SKY_LITTLE_ENDIAN
                if (be2cpu16((unsigned char *)&pWscIE->Type) == WSC_IE_DEV_PASS_ID)
#else
				if (be2cpu16(pWscIE->Type) == WSC_IE_DEV_PASS_ID)
#endif
				{
					// Found device password ID
#ifdef SKY_LITTLE_ENDIAN
                    DevicePasswordID = be2cpu16((unsigned char *) &pWscIE->Data);
#else
					DevicePasswordID = be2cpu16(*((USHORT *) &pWscIE->Data));
#endif
					DBGPRINT(RT_DEBUG_WPS, "WscPBCBssTableSort : DevicePasswordID = 0x%04x\n", DevicePasswordID);
					if (DevicePasswordID == DEV_PASS_ID_PBC)	// Check for PBC value
					{
						// Found matching PBC AP in current list, add it into table and add the count
						NdisMoveMemory(pOutBss, pInBss, sizeof(BSS_ENTRY));
						Found = TRUE;
						
						DBGPRINT(RT_DEBUG_WPS, "PBC Found --> \n");
						DBGPRINT(RT_DEBUG_WPS, "#%d  Bssid %02x:%02x:%02x:%02x:%02x:%02x\n", OutTab->BssNr, 							
							pOutBss->Bssid[0], pOutBss->Bssid[1], pOutBss->Bssid[2], pOutBss->Bssid[3], pOutBss->Bssid[4], pOutBss->Bssid[5]);
					}
				}

				// UUID_E is optional for beacons, but mandatory for probe-request
#ifdef SKY_LITTLE_ENDIAN
                if(be2cpu16((unsigned char *)&pWscIE->Type) == WSC_IE_UUID_E)
#else
				if (be2cpu16(pWscIE->Type) == WSC_IE_UUID_E)
#endif
				{
					// Avoid error UUID-E storage from PIN mode
					RTMPMoveMemory(&TmpUUID[0], (UCHAR *)(pData+4), 16);
				}
				
				// Set the offset and look for PBC information
				// Since Type and Length are both short type, we need to offset 4, not 2
#ifdef  SKY_LITTLE_ENDIAN
                pData += (be2cpu16((unsigned char *)&pWscIE->Length) + 4);
				Len   -= (be2cpu16((unsigned char *)&pWscIE->Length) + 4);
#else
				pData += (be2cpu16(pWscIE->Length) + 4);
				Len   -= (be2cpu16(pWscIE->Length) + 4);
#endif
			}
			
			if (Found == TRUE)
			{							
				// Store UUID
				RTMPMoveMemory(&UUID[OutTab->BssNr][0], &TmpUUID[0], 16); 			

				DBGPRINT(RT_DEBUG_WPS, "UUID-E= ");
				for(idx=0; idx<16; idx++)
					DBGPRINT_RAW(RT_DEBUG_WPS, "%02x  ", UUID[OutTab->BssNr][idx]);
				DBGPRINT(RT_DEBUG_WPS, "\n");

				OutTab->BssNr++;				
			}
		}
	}
	Duplicate = 0;
	// Check for concurrent dual band AP
	if (OutTab->BssNr > 1)
	{
		for (i = 0; i < OutTab->BssNr; i++)
		{
			for (j = i + 1; j < OutTab->BssNr; j++)
			{
				if ((RTMPCompareMemory(&UUID[i][0], &UUID[j][0], 16) == 0) &&
					(RTMPCompareMemory(&UUID[i][0], zeros16, 16) != 0))
				{
					// Same UUID at different channel, indicate concurrent AP
					// We can indicate 1 AP only.
					Duplicate++;					
				}
			}
		}
	}
	// Set the finakl AP found, it might not be graceful, snce we might always select 2.4G band
	OutTab->BssNr -= Duplicate;
	DBGPRINT(RT_DEBUG_WPS, "WscPBCBssTableSort : Total %d PBC AP Found\n", OutTab->BssNr);
}

/*
	========================================================================
	
	Routine Description:
		Init WSC MAC header

	Arguments:
		pAdapter	Pointer	to our adapter
		
	Return Value:
		None
		
	Note:
		
	========================================================================
*/
VOID	WscMacHeaderInit(
	IN		PRTMP_ADAPTER	pAdapter, 
	IN OUT	PHEADER_802_11	Hdr, 
	IN		PUCHAR			pAddr1)	
{
	NdisZeroMemory(Hdr,	sizeof(HEADER_802_11));
	Hdr->FC.Type = BTYPE_DATA;
	Hdr->FC.ToDs = 1;
	
	 //	Addr1: DA, Addr2: BSSID, Addr3:	SA
	
	COPY_MAC_ADDR(&Hdr->Addr1, pAddr1);
	COPY_MAC_ADDR(&Hdr->Addr2, &pAdapter->CurrentAddress);
	COPY_MAC_ADDR(&Hdr->Addr3, &pAdapter->PortCfg.Bssid);
	Hdr->Sequence =	pAdapter->Sequence;		
}

/*
	========================================================================
	
	Routine Description:
		WSC_STATE_START action

	Arguments:
		pAd         - NIC Adapter pointer
		
	Return Value:
		None
		
	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after link up
		1. Change the correct parameters
		2. Send EAPOL - START
		
	========================================================================
*/
VOID WscStartAction(
	IN	PRTMP_ADAPTER	pAdapter)
{
	HEADER_802_11		Header_802_11;
	UCHAR				AckRate = RATE_2;
	USHORT				AckDuration = 0;
	IEEE8021X_FRAME		Packet;
	UCHAR				*OutBuffer = NULL;
	ULONG				FrameLen = 0;
	UCHAR				EAPHEAD[8] = {0xaa,	0xaa, 0x03,	0x00, 0x00,	0x00,0x88,0x8e};
	BOOLEAN Cancelled;

	// 1. Change the authentication to open and encryption to none if necessary.

	// 2. Send EAPOL start
	
	// =====================================
	// Use Priority Ring & MiniportMMRequest
	// =====================================
	pAdapter->Sequence = ((pAdapter->Sequence) + 1)	& (MAX_SEQ_NUMBER);
	WscMacHeaderInit(pAdapter, &Header_802_11, pAdapter->PortCfg.Bssid);

	// ACK size	is 14 include CRC, and its rate	is based on real time information
	AckRate = pAdapter->PortCfg.ExpectedACKRate[pAdapter->PortCfg.TxRate];
	AckDuration = RTMPCalcDuration(pAdapter, AckRate, 14);
	Header_802_11.Duration = pAdapter->PortCfg.Dsifs + AckDuration;
	
	// Zero message 2 body
	NdisZeroMemory(&Packet, sizeof(Packet));
	Packet.Version = EAPOL_VER;
	Packet.Type    = EAPOLStart;
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp = 0;
		Packet.Length  = cpu2be16((unsigned char *)&tmp);
	}
#else
	Packet.Length  = cpu2be16(0);
#endif
	// Out buffer for transmitting message 2
	
	OutBuffer = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);  //Get an unused nonpaged memory
	if (OutBuffer == NULL)	
		return;					

	FrameLen = 0;
	// Make	 Transmitting frame
	MakeOutgoingFrame(OutBuffer, &FrameLen,	sizeof(HEADER_802_11), &Header_802_11,
		sizeof(EAPHEAD), EAPHEAD, 
		4, &Packet,
		END_OF_ARGS);
	// Send using priority queue
	MiniportMMRequest(pAdapter,QID_AC_BE,OutBuffer, FrameLen);
	DBGPRINT(RT_DEBUG_WPS, "Send EAPOL-START size = %d\n", FrameLen);

	// Update WSC status
	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAPOL_START_SENT;

	// set new EAP Rx timer
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer,&Cancelled);
	
	RTMPSetTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer, WSC_PER_MSG_TIMEOUT);
	DBGPRINT(RT_DEBUG_WPS,"WscStartAction WscStatus(%d)=%d\n",STATUS_WSC_EAPOL_START_SENT,pAdapter->PortCfg.WscControl.WscStatus);
	kfree(OutBuffer);
}

/*
	========================================================================
	
	Routine Description:
		Initialize WSC registrar pair data structure

	Arguments:
		pAdapter    - NIC Adapter pointer
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		
	========================================================================
*/
VOID	WscInitRegistrarPair(
	IN	PRTMP_ADAPTER		pAdapter)
{
	PWSC_REG_DATA		pReg;
	INT					idx, DH_Len;
	
	DBGPRINT(RT_DEBUG_WPS, "-----> WscInitRegistrarPair\n");

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;

	// remove by johnli, this will zero the dh public key data structure
	//0. Zero the data structure first
//	NdisZeroMemory(&pAdapter->PortCfg.WscControl.RegData, sizeof(WSC_REG_DATA));

	// Since our role is enrollee, fill all the enrollee information.

	// 1. Version
	pReg->EnrolleeInfo.Version = WSC_VERSION;

	// 2. UUID Enrollee, "WPSEnrolee"+"MACAddr"
	RTMPMoveMemory(&pReg->EnrolleeInfo.Uuid[0], &Wsc_Uuid_E[0], 10);
	RTMPMoveMemory(&pReg->EnrolleeInfo.Uuid[10], pAdapter->CurrentAddress, MAC_ADDR_LEN);

	// 3. MAC address
	NdisMoveMemory(pReg->EnrolleeInfo.MacAddr, pAdapter->CurrentAddress, 6);

	// 4. Device Name
	NdisMoveMemory(pReg->EnrolleeInfo.DeviceName, &Wsc_Dev_Name[0], sizeof(Wsc_Dev_Name));
	
	// 5. Manufacture
	NdisMoveMemory(pReg->EnrolleeInfo.Manufacturer, &Wsc_Manufacture[0], sizeof(Wsc_Manufacture));

	// 6. Model Name
	NdisMoveMemory(pReg->EnrolleeInfo.ModelName, &Wsc_Model_Name[0], sizeof(Wsc_Model_Name));
	
	// 7. Model Number
	NdisMoveMemory(pReg->EnrolleeInfo.ModelNumber, &Wsc_Model_Number[0], sizeof(Wsc_Model_Number));
	
	// 8. Serial Number
	NdisMoveMemory(pReg->EnrolleeInfo.SerialNumber, &Wsc_Model_Serial[0], sizeof(Wsc_Model_Serial));
	
	// 9. Authentication Type Flags
#ifdef SKY_LITTLE_ENDIAN
	{
	unsigned short tmp=0x003F;
	pReg->EnrolleeInfo.AuthTypeFlags = cpu2be16((unsigned char *)&tmp);	// Open(=1), WPAPSK(=2),Shared(=4), WPA2PSK(=20)
	}
#else
	// edit by johnli, fix WPS connect failed problem with PCI BLW-54CW3 AP
	// Open(=1), WPAPSK(=2),Shared(=4), WPA2PSK(=20),WPA(=8),WPA2(=10)
	// (0x01 | 0x02 | 0x04 | 0x20 | 0x08 | 0x10) = 0x3F
	// WCN vista logo will check this flags.
	//pReg->EnrolleeInfo.AuthTypeFlags = cpu2be16(0x001B);	// Open(=1), WPAPSK(=2),Shared(=4), WPA2PSK(=20)
	pReg->EnrolleeInfo.AuthTypeFlags = cpu2be16(0x003F);
	// end johnli
#endif
	// 10. Encryption Type Flags
#ifdef SKY_LITTLE_ENDIAN
	{
	unsigned short tmp=0x000F;
	pReg->EnrolleeInfo.EncrTypeFlags  = cpu2be16((unsigned char *)&tmp);
	}
#else
	pReg->EnrolleeInfo.EncrTypeFlags  = cpu2be16(0x000F);	// None(=1), WEP(=2), TKIP(=4), AES(=8)	
#endif
	// 11. Connection Type Flag
	pReg->EnrolleeInfo.ConnTypeFlags = 0x01;				// ESS

	// 12. Associate state
#ifdef SKY_LITTLE_ENDIAN
	pReg->EnrolleeInfo.AssocState = 0x0;
#else
	pReg->EnrolleeInfo.AssocState = cpu2be16(0x0000);		// Not associated
#endif
	// 13. Configure Error
#ifdef SKY_LITTLE_ENDIAN
	{
	unsigned short tmp=WSC_ERROR_NO_ERROR;
	pReg->EnrolleeInfo.ConfigError  = cpu2be16((unsigned char *)&tmp);
	}
#else
	pReg->EnrolleeInfo.ConfigError = cpu2be16(WSC_ERROR_NO_ERROR);	// No error
#endif
	// 14. OS Version
#ifdef SKY_LITTLE_ENDIAN
    {
        unsigned int tmp = 0x80000000;
        pReg->EnrolleeInfo.OsVersion = cpu2be32((unsigned char *)&tmp); 
    }
#else
	pReg->EnrolleeInfo.OsVersion = cpu2be32(0x80000000);	// first bit must be 1
#endif
	// 15. RF Band
	pReg->EnrolleeInfo.RfBand = 0x00;
	if ((pAdapter->PortCfg.PhyMode == PHY_11A) || (pAdapter->PortCfg.PhyMode == PHY_11ABG_MIXED))
		pReg->EnrolleeInfo.RfBand |= 0x02;			// 5.0G
	if (pAdapter->PortCfg.PhyMode != PHY_11A)
		pReg->EnrolleeInfo.RfBand |= 0x01;			// 2.4G

	// 16. Config Method
	// TODO: Might need to differ between PBC and PIN mode
#ifdef SKY_LITTLE_ENDIAN
	{
	unsigned short tmp=0x008c;
	pReg->EnrolleeInfo.ConfigMethods  = cpu2be16((unsigned char *)&tmp);
	}
#else
	pReg->EnrolleeInfo.ConfigMethods = cpu2be16(0x008c);	// Label, Display, PBC
#endif
	// 17. Simple Config State
	pReg->EnrolleeInfo.ScState = 0x01;						// Not Configured

	// 18. Device Password ID
	if (pAdapter->PortCfg.WscControl.WscMode == WSC_PIN_MODE)
	{
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp=DEV_PASS_ID_PIN;
		pReg->EnrolleeInfo.DevPwdId  = cpu2be16((unsigned char *)&tmp);
	}
#else
		pReg->EnrolleeInfo.DevPwdId = cpu2be16(DEV_PASS_ID_PIN);		// PIN mode
#endif
	}
	else
	{
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp=DEV_PASS_ID_PBC;
		pReg->EnrolleeInfo.DevPwdId  = cpu2be16((unsigned char *)&tmp);
	}
#else
		pReg->EnrolleeInfo.DevPwdId = cpu2be16(DEV_PASS_ID_PBC);		// PBC
#endif
	}

	// 19. SSID
	NdisMoveMemory(pReg->EnrolleeInfo.Ssid, pAdapter->PortCfg.Ssid, pAdapter->PortCfg.SsidLen);

	// 15. Primary Device Type
	NdisMoveMemory(pReg->EnrolleeInfo.PriDeviceType, &Wsc_Pri_Dev_Type[0], 8);

	// Other enrollee controlled data in RegData structure
	
	// 1. Enrollee Nonce, first generate and save to Wsc Control Block
	for (idx = 0; idx < 16; idx++)
		pReg->EnrolleeNonce[idx] = RandomByte(pAdapter);
	
	// 2. Enrollee 192 random bytes for DH key generation
	for (idx = 0; idx < 192; idx++)
		pReg->EnrolleeRandom[idx] = RandomByte(pAdapter);

	// 3. Generate DH Public Key
	//TODO: It's moved out (WscGetConf)
	
	// 4. Enrollee 16 byte E-S1 generation
	for (idx = 0; idx < 16; idx++)
		pReg->Es1[idx] = RandomByte(pAdapter);
	
	// 5. Enrollee 16 byte E-S2 generation
	for (idx = 0; idx < 16; idx++)
		pReg->Es2[idx] = RandomByte(pAdapter);
	
	DBGPRINT(RT_DEBUG_WPS, "<----- WscInitRegistrarPair\n");	
}

/*
	========================================================================
	
	Routine Description:
		WSC state actions after receiveing EAP-Req(ID) at LinkUp state

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after Rx EAP-Req(ID)
		1. Change the correct parameters
		2. Send EAP-Rsp(ID)
		
	========================================================================
*/
VOID	WscSendEapRspId(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	HEADER_802_11		Header_802_11;
	UCHAR				AckRate = RATE_2;
	USHORT				AckDuration = 0, Length;
	IEEE8021X_FRAME		Ieee_8021x;
	EAP_FRAME			EapFrame;
	UCHAR				*OutBuffer = NULL;
	ULONG				FrameLen = 0;
	UCHAR				EAPHEAD[8] = {0xaa,	0xaa, 0x03,	0x00, 0x00,	0x00, 0x88, 0x8e};
	
	// 1. Send EAP-Rsp Id
	DBGPRINT(RT_DEBUG_WPS, "-----> WscSendEapRspId\n");
	
	// =====================================
	// Use Priority Ring & MiniportMMRequest
	// =====================================
	pAdapter->Sequence = ((pAdapter->Sequence) + 1)	& (MAX_SEQ_NUMBER);
	WscMacHeaderInit(pAdapter, &Header_802_11, pAdapter->PortCfg.Bssid);

	// ACK size	is 14 include CRC, and its rate	is based on real time information
	AckRate = pAdapter->PortCfg.ExpectedACKRate[pAdapter->PortCfg.TxRate];
	AckDuration = RTMPCalcDuration(pAdapter, AckRate, 14);
	Header_802_11.Duration = pAdapter->PortCfg.Dsifs + AckDuration;

	// Length, -1 NULL pointer of string
	Length = sizeof(EAP_FRAME) + sizeof(EnrolleeMsg) - 1;
	
	// Zero 802.1x body
	NdisZeroMemory(&Ieee_8021x, sizeof(Ieee_8021x));
	Ieee_8021x.Version = EAPOL_VER;
	Ieee_8021x.Type    = EAPPacket;
#ifdef SKY_LITTLE_ENDIAN
	Ieee_8021x.Length  = cpu2be16((unsigned char *)&Length);
#else
	Ieee_8021x.Length  = cpu2be16(Length);
#endif
	// Zero EAP frame
	NdisZeroMemory(&EapFrame, sizeof(EapFrame));
	EapFrame.Code   = EAP_CODE_RSP;
	EapFrame.Id     = Id;
#ifdef SKY_LITTLE_ENDIAN
	EapFrame.Length = cpu2be16((unsigned char *)&Length);
#else
	EapFrame.Length = cpu2be16(Length);
#endif
	EapFrame.Type   = EAP_TYPE_ID;

	
	// Out buffer for transmitting message 2		
	OutBuffer = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);  //Get an unused nonpaged memory
	if (OutBuffer == NULL)	
		return;					

	FrameLen = 0;
	
	// Make	 Transmitting frame
	MakeOutgoingFrame(OutBuffer, &FrameLen,	sizeof(HEADER_802_11), &Header_802_11,
		sizeof(EAPHEAD), EAPHEAD, sizeof(IEEE8021X_FRAME), &Ieee_8021x,
		sizeof(EapFrame), &EapFrame, (sizeof(EnrolleeMsg) - 1), EnrolleeMsg,
		END_OF_ARGS);

	// Send using priority queue
	MiniportMMRequest(pAdapter,QID_AC_BE, OutBuffer, FrameLen);

	// Update WSC status
	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_REQ_ID_SENT;				

	kfree(OutBuffer);
	DBGPRINT(RT_DEBUG_WPS,"<----- WscSendEapRspId\n");
}

/*
	========================================================================
	
	Routine Description:
		Send WSC M1 Message

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after Rx EAP-Req(ID)
		1. Change the correct parameters
		2. Send M1
		
	========================================================================
*/
VOID	WscSendEapM1(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	HEADER_802_11		Header_802_11;
	UCHAR				AckRate = RATE_2;
	USHORT				AckDuration = 0, Length;
	IEEE8021X_FRAME		Ieee_8021x;
	EAP_FRAME			EapFrame;
	WSC_FRAME			WscFrame;
	PWSC_REG_DATA		pReg;	
	UCHAR				*OutBuffer = NULL;
	ULONG				FrameLen = 0;
	UCHAR				EAPHEAD[8] = {0xaa,	0xaa, 0x03,	0x00, 0x00,	0x00, 0x88, 0x8e};
	PUCHAR				pData;
	INT					Len;
	
	// Send M1 message
	DBGPRINT(RT_DEBUG_WPS, "-----> WscSendEapM1\n");

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;

	// Prepare the M1 message body after WSC Frame header
	pData = (PUCHAR) &OutMsgBuf[0];
	Len = 0;
	
	// 1. Version
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_VERSION;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_VERSION);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = pReg->EnrolleeInfo.Version;
	pData += 5;
	Len   += 5;
	// 2. Message Type, M1
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_MSG_TYPE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_MSG_TYPE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = WSC_MSG_M1;
	pData += 5;
	Len   += 5;
	// 3. UUID_E, last 6 bytes use MAC
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_UUID_E;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_UUID_E);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0010);
#endif
	NdisMoveMemory((pData + 4), pReg->EnrolleeInfo.Uuid, 16);
	pData += 20;
	Len   += 20;

	// 4. MAC address
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_MAC_ADDR;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0006;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_MAC_ADDR);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0006);
#endif
	NdisMoveMemory((pData + 4), pReg->EnrolleeInfo.MacAddr, 6);
	pData += 10;
	Len   += 10;
	// 5. Enrollee Nonce, first generate and save to Wsc Control Block
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_EN_NONCE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_EN_NONCE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0010);
#endif
	NdisMoveMemory((pData + 4), pReg->EnrolleeNonce, 16);
	pData += 20;
	Len   += 20;

	// 6. Public Key, 192 bytes
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_PUBLIC_KEY;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 192;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_PUBLIC_KEY);
	*((PUSHORT) (pData + 2)) = cpu2be16(192);
#endif
	NdisMoveMemory((pData + 4), pReg->Pke, 192);
	pData += 196;
	Len   += 196;

	// 7. Authentication Type Flags
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_AUTH_TYPE_FLAG;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0002;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_AUTH_TYPE_FLAG);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0002);
#endif
#ifdef SKY_LITTLE_ENDIAN
	memcpy((pData + 4), (void *)&pReg->EnrolleeInfo.AuthTypeFlags,2);
#else
	*((PUSHORT) (pData + 4)) = pReg->EnrolleeInfo.AuthTypeFlags;		// Open, WPAPSK, Shared, WPA2PSK
#endif
	pData += 6;
	Len   += 6;
	
	// 7. Encryption Type Flags
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_ENCR_TYPE_FLAG;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0002;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_ENCR_TYPE_FLAG);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0002);
#endif
#ifdef SKY_LITTLE_ENDIAN
	memcpy((pData + 4), (void *)&pReg->EnrolleeInfo.EncrTypeFlags,2);
#else
	*((PUSHORT) (pData + 4)) = pReg->EnrolleeInfo.EncrTypeFlags;		// None, WEP, TKIP, AES
#endif
	pData += 6;
	Len   += 6;
	
	// 8. Connection Type Flag
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_CONN_TYPE_FLAG;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_CONN_TYPE_FLAG);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = pReg->EnrolleeInfo.ConnTypeFlags;					// ESS
	pData += 5;
	Len   += 5;

	// 9. Config Method
	// TODO: Might need to differ between PBC and PIN mode
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_CONF_MTHD;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0002;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_CONF_MTHD);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0002);
#endif
#ifdef SKY_LITTLE_ENDIAN
	memcpy((pData + 4), (void *)&pReg->EnrolleeInfo.ConfigMethods,2);
#else
	*((PUSHORT) (pData + 4)) = pReg->EnrolleeInfo.ConfigMethods;		// Label, Display, PBC
#endif
	pData += 6;
	Len   += 6;
	
	// 10. Simple Config State
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_STATE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_STATE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = pReg->EnrolleeInfo.ScState;						// Not Configured
	pData += 5;
	Len   += 5;

	// 11. Manufacture
	NdisZeroMemory(pData, 64 + 4);
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_MANUFACTURER;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0040;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_MANUFACTURER);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0040);	
#endif
	NdisMoveMemory((pData + 4), pReg->EnrolleeInfo.Manufacturer, 64);
	pData += 68;
	Len   += 68;

	// 12. Model Name
	NdisZeroMemory(pData, 32 + 4);
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_MODEL_NAME;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0020;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_MODEL_NAME);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0020);
#endif
	NdisMoveMemory((pData + 4), pReg->EnrolleeInfo.ModelName, 32); 
	pData += 36;
	Len   += 36;
	
	// 13. Model Number
	NdisZeroMemory(pData, 32 + 4);
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_MODEL_NO;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0020;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_MODEL_NO);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0020);	
#endif
	NdisMoveMemory((pData + 4), pReg->EnrolleeInfo.ModelNumber, 32);
	pData += 36;
	Len   += 36;
	
	// 14. Serial Number
	NdisZeroMemory(pData, 32 + 4);
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_SERIAL;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0020;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_SERIAL);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0020);
#endif
	NdisMoveMemory((pData + 4), pReg->EnrolleeInfo.SerialNumber, 32);
	pData += 36;
	Len   += 36;
	
	// 15. Primary Device Type
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_PRI_DEV_TYPE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0008;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_PRI_DEV_TYPE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0008);	
#endif
	NdisMoveMemory((pData + 4), pReg->EnrolleeInfo.PriDeviceType, 8);
	pData += 12;
	Len   += 12;
	
	// 16. Device Name
	NdisZeroMemory(pData, 32 + 4);
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_DEV_NAME;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0020;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_DEV_NAME);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0020);
#endif
	NdisMoveMemory((pData + 4), pReg->EnrolleeInfo.DeviceName, 32);
	pData += 36;
	Len   += 36;
	
	// 17. RF Band
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_RF_BAND;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_RF_BAND);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = pReg->EnrolleeInfo.RfBand;
	pData += 5;
	Len   += 5;

	// 18. Associate state
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_ASSOC_STATE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0002;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_ASSOC_STATE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0002);
#endif
#ifdef SKY_LITTLE_ENDIAN
	memcpy((pData + 4), (void *)&pReg->EnrolleeInfo.AssocState,2);
#else
	*((PUSHORT) (pData + 4)) = pReg->EnrolleeInfo.AssocState;	// Not associated
#endif
	pData += 6;
	Len   += 6;

	// 19. Device Password ID
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_DEV_PASS_ID;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0002;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_DEV_PASS_ID);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0002);
#endif
#ifdef SKY_LITTLE_ENDIAN
	memcpy((pData + 4), (void *)&pReg->EnrolleeInfo.DevPwdId,2);
#else
	*((PUSHORT) (pData + 4)) = pReg->EnrolleeInfo.DevPwdId;
#endif
	pData += 6;
	Len   += 6;

	// 20. Configure Error
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_CONF_ERROR;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0002;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_CONF_ERROR);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0002);
#endif
#ifdef SKY_LITTLE_ENDIAN
	memcpy((pData + 4), (void *)&pReg->EnrolleeInfo.ConfigError,2);
#else
	*((PUSHORT) (pData + 4)) = pReg->EnrolleeInfo.ConfigError;
#endif
	pData += 6;
	Len   += 6;

	// 21. OS Version
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_OS_VER;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0004;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_OS_VER);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0004);
#endif
#ifdef SKY_LITTLE_ENDIAN
	memcpy((pData + 4), (void *)&pReg->EnrolleeInfo.OsVersion,4);
#else
	*((PULONG) (pData + 4)) = pReg->EnrolleeInfo.OsVersion;	
#endif
	pData += 8;
	Len   += 8;

	// Copy the content to Regdata for lasttx information
	pReg->LastTx.Length = Len;
	RTMPMoveMemory(pReg->LastTx.Data, OutMsgBuf, Len);

	// =====================================
	// Use Priority Ring & MiniportMMRequest
	// =====================================
	pAdapter->Sequence = ((pAdapter->Sequence) + 1)	& (MAX_SEQ_NUMBER);
	WscMacHeaderInit(pAdapter, &Header_802_11, pAdapter->PortCfg.Bssid);

	// ACK size	is 14 include CRC, and its rate	is based on real time information
	AckRate = pAdapter->PortCfg.ExpectedACKRate[pAdapter->PortCfg.TxRate];
	AckDuration = RTMPCalcDuration(pAdapter, AckRate, 14);
	Header_802_11.Duration = pAdapter->PortCfg.Dsifs + AckDuration;

	// Length = EAP + WSC_Frame + Payload
	Length = (USHORT)(sizeof(EAP_FRAME) + sizeof(WSC_FRAME) + Len);
	// Zero 802.1x body
	NdisZeroMemory(&Ieee_8021x, sizeof(Ieee_8021x));
	Ieee_8021x.Version = EAPOL_VER;
	Ieee_8021x.Type    = EAPPacket;
#ifdef SKY_LITTLE_ENDIAN
	Ieee_8021x.Length  = cpu2be16((unsigned char *)&Length);
#else
	Ieee_8021x.Length  = cpu2be16(Length);
#endif
	// Zero EAP frame
	NdisZeroMemory(&EapFrame, sizeof(EapFrame));
	EapFrame.Code   = EAP_CODE_RSP;
	EapFrame.Id     = Id;
#ifdef SKY_LITTLE_ENDIAN
	EapFrame.Length = cpu2be16((unsigned char *)&Length);
#else
	EapFrame.Length = cpu2be16(Length);
#endif
	EapFrame.Type   = EAP_TYPE_WSC;

	// Zero WSC Frame
	NdisZeroMemory(&WscFrame, sizeof(WscFrame));
	WscFrame.SMI[0] = 0x00;
	WscFrame.SMI[1] = 0x37;
	WscFrame.SMI[2] = 0x2A;
#ifdef SKY_LITTLE_ENDIAN
    {
        unsigned int tmp = WSC_VENDOR_TYPE;
        WscFrame.VendorType = cpu2be32((unsigned char *)&tmp);
    }
#else
	WscFrame.VendorType = cpu2be32(WSC_VENDOR_TYPE);
#endif
	WscFrame.OpCode = WSC_OPCODE_MSG;
	WscFrame.Flags  = 0x00;
	
	// Out buffer for transmitting message 2		
	OutBuffer = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);  //Get an unused nonpaged memory
	if (OutBuffer == NULL)	
		return;					

	FrameLen = 0;
	
	// Make	 Transmitting frame
	MakeOutgoingFrame(OutBuffer, &FrameLen,	sizeof(HEADER_802_11), &Header_802_11,
		sizeof(EAPHEAD), EAPHEAD, sizeof(IEEE8021X_FRAME), &Ieee_8021x,
		sizeof(EapFrame), &EapFrame, sizeof(WscFrame), &WscFrame, Len, OutMsgBuf,
		END_OF_ARGS);

	// Send using priority queue
	MiniportMMRequest(pAdapter,QID_AC_BE, OutBuffer, FrameLen);
	// Update WSC status
	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_REQ_ID_SENT;	

	kfree(OutBuffer);
	DBGPRINT(RT_DEBUG_WPS, "<----- WscSendEapM1\n");	
}

/*
	========================================================================
	
	Routine Description:
		Send WSC M3 Message

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after Rx EAP-Req(ID)
		1. Change the correct parameters
		2. Send M3
		
	========================================================================
*/
VOID	WscSendEapM3(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	HEADER_802_11		Header_802_11;
	UCHAR				AckRate = RATE_2;
	USHORT				AckDuration = 0, Length;
	IEEE8021X_FRAME		Ieee_8021x;
	EAP_FRAME			EapFrame;
	WSC_FRAME			WscFrame;
	PWSC_REG_DATA		pReg;
	UCHAR				*OutBuffer = NULL;
	ULONG				FrameLen = 0;
	UCHAR				EAPHEAD[8] = {0xaa,	0xaa, 0x03,	0x00, 0x00,	0x00, 0x88, 0x8e};
	UCHAR				PSKBuf[512];
	PUCHAR				pData, pAuth;
	ULONG				HmacLen;
	INT					Len, idx;
	
	DBGPRINT(RT_DEBUG_WPS, "-----> WscSendEapM3\n");

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;

	// Prepare the M3 message body after WSC Frame header
	pData = (PUCHAR) &OutMsgBuf[0];
	Len = 0;
	
	// 1. Version
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_VERSION;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_VERSION);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = pReg->EnrolleeInfo.Version;
	pData += 5;
	Len   += 5;

	// 2. Message Type, M3
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_MSG_TYPE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_MSG_TYPE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = WSC_MSG_M3;
	pData += 5;
	Len   += 5;

	// 3. Registrar Nonce, 16 bytes
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_REG_NONCE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_REG_NONCE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0010);
#endif
	RTMPMoveMemory((pData + 4), pReg->RegistrarNonce, 16);
	pData += 20;
	Len   += 20;

	// 4. E-Hash1
	//
	// Generate PSK1
	hmac_sha(pReg->AuthKey, 32, pReg->PIN, 4, PSKBuf, 32);

	// Copy first 16 bytes to PSK1
	RTMPMoveMemory(pReg->Psk1, PSKBuf, 16);

	// Create input for E-Hash1
	RTMPMoveMemory(PSKBuf, pReg->Es1, 16);
	RTMPMoveMemory(&PSKBuf[16], pReg->Psk1, 16);
	RTMPMoveMemory(&PSKBuf[32], pReg->Pke, 192);
	RTMPMoveMemory(&PSKBuf[224], pReg->Pkr, 192);
	
	// Generate E-Hash1
	hmac_sha(pReg->AuthKey, 32, PSKBuf, 416, pReg->EHash1, 32);
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_E_HASH_1;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0020;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_E_HASH_1);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0020);
#endif
	NdisMoveMemory((pData + 4), pReg->EHash1, 32);
	pData += 36;
	Len   += 36;

	// 5. E-Hash2
	//
	// Generate PSK2
	hmac_sha(pReg->AuthKey, 32, &pReg->PIN[4], 4, PSKBuf, 32);

	// Copy first 16 bytes to PSK2
	RTMPMoveMemory(pReg->Psk2, PSKBuf, 16);
		
	// Create input for E-Hash2
	RTMPMoveMemory(PSKBuf, pReg->Es2, 16);
	RTMPMoveMemory(&PSKBuf[16], pReg->Psk2, 16);
	RTMPMoveMemory(&PSKBuf[32], pReg->Pke, 192);
	RTMPMoveMemory(&PSKBuf[224], pReg->Pkr, 192);
	
	// Generate E-Hash2
	hmac_sha(pReg->AuthKey, 32, PSKBuf, 416, pReg->EHash2, 32);
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_E_HASH_2;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0020;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_E_HASH_2);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0020);
#endif
	NdisMoveMemory((pData + 4), pReg->EHash2, 32);
	pData += 36;
	Len   += 36;

	// Copy the content to Regdata for lasttx information
	pReg->LastTx.Length = Len;
	RTMPMoveMemory(pReg->LastTx.Data, OutMsgBuf, Len);

	// 6. Generate authenticator
	// Combine last TX & RX message contents and validate the HMAC
	HmacLen = pReg->LastTx.Length + pReg->LastRx.Length;
	pAuth = (PUCHAR) &HmacData[0];
	RTMPMoveMemory(pAuth, pReg->LastRx.Data, pReg->LastRx.Length);
	pAuth += pReg->LastRx.Length;
	RTMPMoveMemory(pAuth, pReg->LastTx.Data, pReg->LastTx.Length);

	// Validate HMAC, reuse KDK buffer
	DBGPRINT(RT_DEBUG_WPS, "HmacLen = %d\n", HmacLen);
	DBGPRINT(RT_DEBUG_WPS, "HmacData --> ");
	
	for (idx = 0; idx < 64; idx++)
	{
		DBGPRINT_RAW(RT_DEBUG_WPS, "%02x-", HmacData[idx]);
	}
	DBGPRINT(RT_DEBUG_WPS, "\n");		

	hmac_sha(pReg->AuthKey, 32, HmacData, HmacLen, PSKBuf, 32);
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_AUTHENTICATOR;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0008;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_AUTHENTICATOR);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0008);
#endif
	NdisMoveMemory((pData + 4), PSKBuf, 8);

	// Append the authenticator to last tx buffer for future HMAC calculation
	RTMPMoveMemory(&pReg->LastTx.Data[pReg->LastTx.Length], pData, 12);
	pReg->LastTx.Length += 12;
	pData += 12;
	Len   += 12;
	
	// =====================================
	// Use Priority Ring & MiniportMMRequest
	// =====================================
	pAdapter->Sequence = ((pAdapter->Sequence) + 1)	& (MAX_SEQ_NUMBER);
	WscMacHeaderInit(pAdapter, &Header_802_11, pAdapter->PortCfg.Bssid);

	// ACK size	is 14 include CRC, and its rate	is based on real time information
	AckRate = pAdapter->PortCfg.ExpectedACKRate[pAdapter->PortCfg.TxRate];
	AckDuration = RTMPCalcDuration(pAdapter, AckRate, 14);
	Header_802_11.Duration = pAdapter->PortCfg.Dsifs + AckDuration;

	// Length = EAP + WSC_Frame + Payload
	Length = (USHORT)(sizeof(EAP_FRAME) + sizeof(WSC_FRAME) + Len);
	
	// Zero 802.1x body
	NdisZeroMemory(&Ieee_8021x, sizeof(Ieee_8021x));
	Ieee_8021x.Version = EAPOL_VER;
	Ieee_8021x.Type    = EAPPacket;
#ifdef SKY_LITTLE_ENDIAN
	Ieee_8021x.Length = cpu2be16((unsigned char *)&Length);
#else
	Ieee_8021x.Length  = cpu2be16(Length);
#endif
	// Zero EAP frame
	NdisZeroMemory(&EapFrame, sizeof(EapFrame));
	EapFrame.Code   = EAP_CODE_RSP;
	EapFrame.Id     = Id;
#ifdef SKY_LITTLE_ENDIAN
	EapFrame.Length = cpu2be16((unsigned char *)&Length);
#else
	EapFrame.Length = cpu2be16(Length);
#endif
	EapFrame.Type   = EAP_TYPE_WSC;

	// Zero WSC Frame
	NdisZeroMemory(&WscFrame, sizeof(WscFrame));
	WscFrame.SMI[0] = 0x00;
	WscFrame.SMI[1] = 0x37;
	WscFrame.SMI[2] = 0x2A;
#ifdef SKY_LITTLE_ENDIAN
    {
        unsigned int tmp = WSC_VENDOR_TYPE;
        WscFrame.VendorType = cpu2be32((unsigned char *)&tmp);
    }
#else
	WscFrame.VendorType = cpu2be32(WSC_VENDOR_TYPE);
#endif
	WscFrame.OpCode = WSC_OPCODE_MSG;
	WscFrame.Flags  = 0x00;
	
	// Out buffer for transmitting message 2		
	OutBuffer = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);  //Get an unused nonpaged memory
	if (OutBuffer == NULL)	
		return;					

	FrameLen = 0;
	
	// Make	 Transmitting frame
	MakeOutgoingFrame(OutBuffer, &FrameLen,	sizeof(HEADER_802_11), &Header_802_11,
		sizeof(EAPHEAD), EAPHEAD, sizeof(IEEE8021X_FRAME), &Ieee_8021x,
		sizeof(EapFrame), &EapFrame, sizeof(WscFrame), &WscFrame, Len, OutMsgBuf,
		END_OF_ARGS);

	// Send using priority queue
	MiniportMMRequest(pAdapter,QID_AC_BE, OutBuffer, FrameLen);

	// Update WSC status
	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_M3_SENT;	

	kfree(OutBuffer);
	DBGPRINT(RT_DEBUG_WPS, "<----- WscSendEapM3\n");	
}

/*
	========================================================================
	
	Routine Description:
		Send WSC M5 Message

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after Rx EAP-Req(ID)
		1. Change the correct parameters
		2. Send M5
		
	========================================================================
*/
VOID	WscSendEapM5(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	HEADER_802_11		Header_802_11;
	UCHAR				AckRate = RATE_2;
	USHORT				AckDuration = 0, Length;
	IEEE8021X_FRAME		Ieee_8021x;
	EAP_FRAME			EapFrame;
	WSC_FRAME			WscFrame;
	PWSC_REG_DATA		pReg;
	UCHAR				*OutBuffer = NULL;
	ULONG				FrameLen = 0;
	UCHAR				EAPHEAD[8] = {0xaa,	0xaa, 0x03,	0x00, 0x00,	0x00, 0x88, 0x8e};
	PUCHAR				pData, pAuth, pPlain;
	UCHAR				Plain[128], Tmp[128], IV[16], EncrData[128];
	ULONG				HmacLen;
	INT					Len, idx, PlainLen, EncrLen;

	DBGPRINT(RT_DEBUG_WPS, "-----> WscSendEapM5\n");

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;

	// Prepare the M5 message body after WSC Frame header
	pData = (PUCHAR) &OutMsgBuf[0];
	Len = 0;
	
	// 1. Version
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_VERSION;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_VERSION);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = pReg->EnrolleeInfo.Version;
	pData += 5;
	Len   += 5;

	// 2. Message Type, M5
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_MSG_TYPE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_MSG_TYPE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = WSC_MSG_M5;
	pData += 5;
	Len   += 5;

	// 3. Registrar Nonce, 16 bytes
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_REG_NONCE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_REG_NONCE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0010);
#endif
	RTMPMoveMemory((pData + 4), pReg->RegistrarNonce, 16);
	pData += 20;
	Len   += 20;

	// 4. Encrypted E-S1
	//    ================================================================================
	//    Prepare plain text
	pPlain = (PUCHAR) Plain;
	PlainLen = 0;
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_E_SNonce_1;
		PUTU16(pPlain, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pPlain+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pPlain) = cpu2be16(WSC_IE_E_SNonce_1);
	*((PUSHORT) (pPlain + 2)) = cpu2be16(0x0010);
#endif
	RTMPMoveMemory((pPlain + 4), pReg->Es1, 16);
	pPlain += 20;
	PlainLen += 20;

	DBGPRINT(RT_DEBUG_WPS, "Plain text --> ");	
	for (idx = 0; idx < PlainLen; idx++)
	{
		DBGPRINT_RAW(RT_DEBUG_WPS, "%02x-", Plain[idx]);
	}
	DBGPRINT(RT_DEBUG_WPS, "\n");

	// Generate HMAC
	hmac_sha(pReg->AuthKey, 32, Plain, PlainLen, Tmp, 32);
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_KWRAP_AUTH;
		PUTU16(pPlain, (unsigned char *)&tmp);
		tmp = 0x0008;
		PUTU16(pPlain+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pPlain) = cpu2be16(WSC_IE_KWRAP_AUTH);
	*((PUSHORT) (pPlain + 2)) = cpu2be16(0x0008);
#endif
	RTMPMoveMemory((pPlain + 4), Tmp, 8);
	pPlain += 12;
	PlainLen += 12;

	DBGPRINT(RT_DEBUG_WPS, "Plain text Hmac --> ");	
	for (idx = 0; idx < 8; idx++)
	{
		DBGPRINT_RAW(RT_DEBUG_WPS, "%02x-", Tmp[idx]);
	}
	DBGPRINT(RT_DEBUG_WPS, "\n");

	// generate a random IV with 16 bytes
	for (idx = 0; idx < 16; idx++)
		IV[idx] = RandomByte(pAdapter);
    
	// Encrypt data
	WscEncryptData(Plain, PlainLen, pReg->KeyWrapKey, IV, EncrData, &EncrLen);

	//    ================================================================================
	//    End of Encrypted Setting

	// 5. Encrypted Settings
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_ENCR_SETTING;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = (16 + EncrLen);
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_ENCR_SETTING);
	*((PUSHORT) (pData + 2)) = cpu2be16(16 + EncrLen);
#endif
	RTMPMoveMemory((pData + 4), IV, 16);
	RTMPMoveMemory((pData + 20), EncrData, EncrLen);
	pData += (20 + EncrLen);
	Len   += (20 + EncrLen);

	// Copy the content to Regdata for lasttx information
	pReg->LastTx.Length = Len;
	RTMPMoveMemory(pReg->LastTx.Data, OutMsgBuf, Len);

	// 6. Generate authenticator
	// Combine last TX & RX message contents and validate the HMAC
	HmacLen = pReg->LastTx.Length + pReg->LastRx.Length;
	pAuth = (PUCHAR) &HmacData[0];
	RTMPMoveMemory(pAuth, pReg->LastRx.Data, pReg->LastRx.Length);
	pAuth += pReg->LastRx.Length;
	RTMPMoveMemory(pAuth, pReg->LastTx.Data, pReg->LastTx.Length);

	// Validate HMAC

	DBGPRINT(RT_DEBUG_WPS, "HmacLen = %d\n", HmacLen);
	DBGPRINT(RT_DEBUG_WPS, "HmacData --> ");	
	for (idx = 0; idx < (INT) HmacLen; idx++)
	{
		DBGPRINT_RAW(RT_DEBUG_WPS, "%02x-", HmacData[idx]);
	}
	DBGPRINT(RT_DEBUG_WPS, "\n");
	
	DBGPRINT(RT_DEBUG_WPS, "AuthKey --> ");	
	for (idx = 0; idx < 32; idx++)
	{
		DBGPRINT_RAW(RT_DEBUG_WPS, "%02x-", pReg->AuthKey[idx]);
	}
	DBGPRINT(RT_DEBUG_WPS, "\n");
	
	hmac_sha(pReg->AuthKey, 32, HmacData, HmacLen, Tmp, 32);
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_AUTHENTICATOR;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0008;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_AUTHENTICATOR);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0008);
#endif
	NdisMoveMemory((pData + 4), Tmp, 8);

	// Append the authenticator to last tx buffer for future HMAC calculation
	RTMPMoveMemory(&pReg->LastTx.Data[pReg->LastTx.Length], pData, 12);
	pReg->LastTx.Length += 12;
	pData += 12;
	Len   += 12;
	
	// =====================================
	// Use Priority Ring & MiniportMMRequest
	// =====================================
	pAdapter->Sequence = ((pAdapter->Sequence) + 1)	& (MAX_SEQ_NUMBER);
	WscMacHeaderInit(pAdapter, &Header_802_11, pAdapter->PortCfg.Bssid);

	// ACK size	is 14 include CRC, and its rate	is based on real time information
	AckRate = pAdapter->PortCfg.ExpectedACKRate[pAdapter->PortCfg.TxRate];
	AckDuration = RTMPCalcDuration(pAdapter, AckRate, 14);
	Header_802_11.Duration = pAdapter->PortCfg.Dsifs + AckDuration;

	// Length = EAP + WSC_Frame + Payload
	Length = (USHORT)(sizeof(EAP_FRAME) + sizeof(WSC_FRAME) + Len);
	
	// Zero 802.1x body
	NdisZeroMemory(&Ieee_8021x, sizeof(Ieee_8021x));
	Ieee_8021x.Version = EAPOL_VER;
	Ieee_8021x.Type    = EAPPacket;
#ifdef SKY_LITTLE_ENDIAN
	Ieee_8021x.Length  = cpu2be16((unsigned char *)&Length);
#else
	Ieee_8021x.Length  = cpu2be16(Length);
#endif

	// Zero EAP frame
	NdisZeroMemory(&EapFrame, sizeof(EapFrame));
	EapFrame.Code   = EAP_CODE_RSP;
	EapFrame.Id     = Id;
#ifdef SKY_LITTLE_ENDIAN
	EapFrame.Length = cpu2be16((unsigned char *)&Length);
#else
	EapFrame.Length = cpu2be16(Length);
#endif
	EapFrame.Type   = EAP_TYPE_WSC;

	// Zero WSC Frame
	NdisZeroMemory(&WscFrame, sizeof(WscFrame));
	WscFrame.SMI[0] = 0x00;
	WscFrame.SMI[1] = 0x37;
	WscFrame.SMI[2] = 0x2A;
#ifdef SKY_LITTLE_ENDIAN
    {
        unsigned int tmp = WSC_VENDOR_TYPE;
        WscFrame.VendorType = cpu2be32((unsigned char *)&tmp);
    }
#else
	WscFrame.VendorType = cpu2be32(WSC_VENDOR_TYPE);
#endif
	WscFrame.OpCode = WSC_OPCODE_MSG;
	WscFrame.Flags  = 0x00;
	
	// Out buffer for transmitting message 2		
	OutBuffer = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);   //Get an unused nonpaged memory
	if (OutBuffer == NULL)	
		return;					

	FrameLen = 0;
	
	// Make	 Transmitting frame
	MakeOutgoingFrame(OutBuffer, &FrameLen,	sizeof(HEADER_802_11), &Header_802_11,
		sizeof(EAPHEAD), EAPHEAD, sizeof(IEEE8021X_FRAME), &Ieee_8021x,
		sizeof(EapFrame), &EapFrame, sizeof(WscFrame), &WscFrame, Len, OutMsgBuf,
		END_OF_ARGS);

	// Send using priority queue
	MiniportMMRequest(pAdapter,QID_AC_BE, OutBuffer, FrameLen);

	// Update WSC status
	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_M5_SENT;	

	kfree(OutBuffer);
	DBGPRINT(RT_DEBUG_WPS, "<----- WscSendEapM5\n");	
}

/*
	========================================================================
	
	Routine Description:
		Send WSC M7 Message

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after Rx EAP-Req(ID)
		1. Change the correct parameters
		2. Send M7
		
	========================================================================
*/
VOID	WscSendEapM7(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	HEADER_802_11		Header_802_11;
	UCHAR				AckRate = RATE_2;
	USHORT				AckDuration = 0, Length;
	IEEE8021X_FRAME		Ieee_8021x;
	EAP_FRAME			EapFrame;
	WSC_FRAME			WscFrame;
	PWSC_REG_DATA		pReg;
	UCHAR				*OutBuffer = NULL;
	ULONG				FrameLen = 0;
	UCHAR				EAPHEAD[8] = {0xaa,	0xaa, 0x03,	0x00, 0x00,	0x00, 0x88, 0x8e};
	PUCHAR				pData, pAuth, pPlain;
	UCHAR				Plain[128], Tmp[128], IV[16], EncrData[128];
	ULONG				HmacLen;
	INT					Len, idx, PlainLen, EncrLen;

	DBGPRINT(RT_DEBUG_WPS, "-----> WscSendEapM7\n");

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;

	// Prepare the M5 message body after WSC Frame header
	pData = (PUCHAR) &OutMsgBuf[0];
	Len = 0;
	
	// 1. Version
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_VERSION;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_VERSION);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = pReg->EnrolleeInfo.Version;
	pData += 5;
	Len   += 5;

	// 2. Message Type, M
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_MSG_TYPE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_MSG_TYPE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = WSC_MSG_M7;
	pData += 5;
	Len   += 5;

	// 3. Registrar Nonce, 16 bytes
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_REG_NONCE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_REG_NONCE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0010);
#endif
	RTMPMoveMemory((pData + 4), pReg->RegistrarNonce, 16);
	pData += 20;
	Len   += 20;

	// 4. Encrypted E-S1
	//    ================================================================================
	//    Prepare plain text
	pPlain = (PUCHAR) Plain;
	PlainLen = 0;
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_E_SNonce_2;
		PUTU16(pPlain, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pPlain+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pPlain) = cpu2be16(WSC_IE_E_SNonce_2);
	*((PUSHORT) (pPlain + 2)) = cpu2be16(0x0010);
#endif
	RTMPMoveMemory((pPlain + 4), pReg->Es2, 16);
	pPlain += 20;
	PlainLen += 20;

	DBGPRINT(RT_DEBUG_WPS, "Plain text --> ");	
	for (idx = 0; idx < PlainLen; idx++)
	{
		DBGPRINT_RAW(RT_DEBUG_WPS, "%02x-", Plain[idx]);
	}
	DBGPRINT(RT_DEBUG_WPS, "\n");

	// Generate HMAC
	hmac_sha(pReg->AuthKey, 32, Plain, PlainLen, Tmp, 32);
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_KWRAP_AUTH;
		PUTU16(pPlain, (unsigned char *)&tmp);
		tmp = 0x0008;
		PUTU16(pPlain+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pPlain) = cpu2be16(WSC_IE_KWRAP_AUTH);
	*((PUSHORT) (pPlain + 2)) = cpu2be16(0x008);
#endif
	RTMPMoveMemory((pPlain + 4), Tmp, 8);
	pPlain += 12;
	PlainLen += 12;

	DBGPRINT(RT_DEBUG_WPS, "Plain text Hmac --> ");	
	for (idx = 0; idx < 8; idx++)
	{
		DBGPRINT_RAW(RT_DEBUG_WPS, "%02x-", Tmp[idx]);
	}
	DBGPRINT(RT_DEBUG_WPS, "\n");

	// generate a random IV with 16 bytes
	for (idx = 0; idx < 16; idx++)
		IV[idx] = RandomByte(pAdapter);
    
	// Encrypt data
	WscEncryptData(Plain, PlainLen, pReg->KeyWrapKey, IV, EncrData, &EncrLen);

	//    ================================================================================
	//    End of Encrypted Setting

	// 5. Encrypted Settings
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_ENCR_SETTING;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = (16 + EncrLen);
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_ENCR_SETTING);
	*((PUSHORT) (pData + 2)) = cpu2be16(16 + EncrLen);	
#endif
	RTMPMoveMemory((pData + 4), IV, 16);
	RTMPMoveMemory((pData + 20), EncrData, EncrLen);
	pData += (20 + EncrLen);
	Len   += (20 + EncrLen);

	// Copy the content to Regdata for lasttx information
	pReg->LastTx.Length = Len;
	RTMPMoveMemory(pReg->LastTx.Data, OutMsgBuf, Len);

	// 6. Generate authenticator
	// Combine last TX & RX message contents and validate the HMAC
	HmacLen = pReg->LastTx.Length + pReg->LastRx.Length;
	pAuth = (PUCHAR) &HmacData[0];
	RTMPMoveMemory(pAuth, pReg->LastRx.Data, pReg->LastRx.Length);
	pAuth += pReg->LastRx.Length;
	RTMPMoveMemory(pAuth, pReg->LastTx.Data, pReg->LastTx.Length);

	// Validate HMAC

	DBGPRINT(RT_DEBUG_WPS, "HmacLen = %d\n", HmacLen);
	DBGPRINT(RT_DEBUG_WPS, "HmacData --> ");	
	for (idx = 0; idx < (INT) HmacLen; idx++)
	{
		DBGPRINT_RAW(RT_DEBUG_WPS, "%02x-", HmacData[idx]);
	}
	DBGPRINT(RT_DEBUG_WPS, "\n");
	
	DBGPRINT(RT_DEBUG_WPS, "AuthKey --> ");	
	for (idx = 0; idx < 32; idx++)
	{
		DBGPRINT_RAW(RT_DEBUG_WPS, "%02x-", pReg->AuthKey[idx]);
	}
	DBGPRINT(RT_DEBUG_WPS, "\n");
	
	hmac_sha(pReg->AuthKey, 32, HmacData, HmacLen, Tmp, 32);
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_AUTHENTICATOR;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0008;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_AUTHENTICATOR);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0008);
#endif
	NdisMoveMemory((pData + 4), Tmp, 8);

	// Append the authenticator to last tx buffer for future HMAC calculation
	RTMPMoveMemory(&pReg->LastTx.Data[pReg->LastTx.Length], pData, 12);
	pReg->LastTx.Length += 12;
	pData += 12;
	Len   += 12;
	
	// =====================================
	// Use Priority Ring & MiniportMMRequest
	// =====================================
	pAdapter->Sequence = ((pAdapter->Sequence) + 1)	& (MAX_SEQ_NUMBER);
	WscMacHeaderInit(pAdapter, &Header_802_11, pAdapter->PortCfg.Bssid);

	// ACK size	is 14 include CRC, and its rate	is based on real time information
	AckRate = pAdapter->PortCfg.ExpectedACKRate[pAdapter->PortCfg.TxRate];
	AckDuration = RTMPCalcDuration(pAdapter, AckRate, 14);
	Header_802_11.Duration = pAdapter->PortCfg.Dsifs + AckDuration;

	// Length = EAP + WSC_Frame + Payload
	Length = (USHORT)(sizeof(EAP_FRAME) + sizeof(WSC_FRAME) + Len);
	
	// Zero 802.1x body
	NdisZeroMemory(&Ieee_8021x, sizeof(Ieee_8021x));
	Ieee_8021x.Version = EAPOL_VER;
	Ieee_8021x.Type    = EAPPacket;
#ifdef SKY_LITTLE_ENDIAN
	Ieee_8021x.Length  = cpu2be16((unsigned char *)&Length);
#else
	Ieee_8021x.Length  = cpu2be16(Length);
#endif
	// Zero EAP frame
	NdisZeroMemory(&EapFrame, sizeof(EapFrame));
	EapFrame.Code   = EAP_CODE_RSP;
	EapFrame.Id     = Id;
#ifdef SKY_LITTLE_ENDIAN
	EapFrame.Length = cpu2be16((unsigned char *)&Length);
#else
	EapFrame.Length = cpu2be16(Length);
#endif
	EapFrame.Type   = EAP_TYPE_WSC;

	// Zero WSC Frame
	NdisZeroMemory(&WscFrame, sizeof(WscFrame));
	WscFrame.SMI[0] = 0x00;
	WscFrame.SMI[1] = 0x37;
	WscFrame.SMI[2] = 0x2A;
#ifdef SKY_LITTLE_ENDIAN
    {
        unsigned int tmp = WSC_VENDOR_TYPE;
        WscFrame.VendorType = cpu2be32((unsigned char *)&tmp);
    }
#else
	WscFrame.VendorType = cpu2be32(WSC_VENDOR_TYPE);
#endif
	WscFrame.OpCode = WSC_OPCODE_MSG;
	WscFrame.Flags  = 0x00;
	
	// Out buffer for transmitting message 2		
	OutBuffer = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);  //Get an unused nonpaged memory
	if (OutBuffer == NULL)	
		return;					

	FrameLen = 0;
	
	// Make	 Transmitting frame
	MakeOutgoingFrame(OutBuffer, &FrameLen,	sizeof(HEADER_802_11), &Header_802_11,
		sizeof(EAPHEAD), EAPHEAD, sizeof(IEEE8021X_FRAME), &Ieee_8021x,
		sizeof(EapFrame), &EapFrame, sizeof(WscFrame), &WscFrame, Len, OutMsgBuf,
		END_OF_ARGS);

	// Send using priority queue
	MiniportMMRequest(pAdapter,QID_AC_BE, OutBuffer, FrameLen);

	// Update WSC status
	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_M7_SENT;	

	kfree(OutBuffer);
	DBGPRINT(RT_DEBUG_WPS, "<----- WscSendEapM7\n");	
}

/*
	========================================================================
	
	Routine Description:
		Send WSC Eap-Rsp-Ack

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after Rx EAP-Req(ID)
		1. Change the correct parameters
		2. Send Eap-Rsp-Ack
		
	========================================================================
*/
VOID	WscSendEapRspAck(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	HEADER_802_11		Header_802_11;
	UCHAR				AckRate = RATE_2;
	USHORT				AckDuration = 0;
	IEEE8021X_FRAME		Ieee_8021x;
	EAP_FRAME			EapFrame;
	WSC_FRAME			WscFrame;
	UCHAR				*OutBuffer = NULL;
	ULONG				FrameLen = 0;
	USHORT				Length;
	UCHAR				EAPHEAD[8] = {0xaa,	0xaa, 0x03,	0x00, 0x00,	0x00,0x88,0x8e};
	PWSC_REG_DATA		pReg;
	PUCHAR				pData;
	INT					Len;
        
	// Send EAPOL-Response-Ack
	DBGPRINT(RT_DEBUG_WPS, "-----> WscSendEapRspAck\n");

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;
    
    
	// Prepare ACK message body after WSC Frame header
	pData = (PUCHAR) &OutMsgBuf[0];
	Len = 0;

	// 1. Version
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_VERSION;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_VERSION);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = pReg->EnrolleeInfo.Version;
	pData += 5;
	Len   += 5;

	// 2. Message Type, M
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_MSG_TYPE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_MSG_TYPE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = WSC_MSG_WSC_ACK;
	pData += 5;
	Len   += 5;

	// 3. Enrollee Nonce, 16 bytes
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_EN_NONCE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_EN_NONCE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0010);
#endif
	NdisMoveMemory((pData + 4), pReg->EnrolleeNonce, 16);
	pData += 20;
	Len   += 20;

	// 4. Registrar Nonce, 16 bytes
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_REG_NONCE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_REG_NONCE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0010);
#endif
	RTMPMoveMemory((pData + 4), pReg->M2DRegistrarNonce, 16);
	pData += 20;
	Len   += 20;


	// =====================================
	// Use Priority Ring & MiniportMMRequest
	// =====================================
	pAdapter->Sequence = ((pAdapter->Sequence) + 1)	& (MAX_SEQ_NUMBER);
	WscMacHeaderInit(pAdapter, &Header_802_11, pAdapter->PortCfg.Bssid);

	// ACK size	is 14 include CRC, and its rate	is based on real time information
	AckRate = pAdapter->PortCfg.ExpectedACKRate[pAdapter->PortCfg.TxRate];
	AckDuration = RTMPCalcDuration(pAdapter, AckRate, 14);
	Header_802_11.Duration = pAdapter->PortCfg.Dsifs + AckDuration;


	// Length = EAP_FRAME + WSC_FRAME size + Payload
	Length = (USHORT)(sizeof(EAP_FRAME) + sizeof(WSC_FRAME) + Len);
	
	// Zero 802.1x body
	NdisZeroMemory(&Ieee_8021x, sizeof(Ieee_8021x));
	Ieee_8021x.Version = EAPOL_VER;
	Ieee_8021x.Type    = EAPPacket;
#ifdef SKY_LITTLE_ENDIAN
	Ieee_8021x.Length  = cpu2be16((unsigned char *)&Length);
#else
	Ieee_8021x.Length  = cpu2be16(Length);
#endif
	// Zero EAP frame
	NdisZeroMemory(&EapFrame, sizeof(EapFrame));
	EapFrame.Code   = EAP_CODE_RSP;
	EapFrame.Id     = Id;
#ifdef SKY_LITTLE_ENDIAN
	EapFrame.Length = cpu2be16((unsigned char *)&Length);
#else
	EapFrame.Length = cpu2be16(Length);
#endif
	EapFrame.Type   = EAP_TYPE_WSC;
	
	// Zero WSC Frame
	NdisZeroMemory(&WscFrame, sizeof(WscFrame));
	WscFrame.SMI[0] = 0x00;
	WscFrame.SMI[1] = 0x37;
	WscFrame.SMI[2] = 0x2A;
#ifdef SKY_LITTLE_ENDIAN
    {
        unsigned int tmp = WSC_VENDOR_TYPE;
        WscFrame.VendorType = cpu2be32((unsigned char *)&tmp);
    }
#else
	WscFrame.VendorType = cpu2be32(WSC_VENDOR_TYPE);
#endif
	WscFrame.OpCode = WSC_OPCODE_ACK;
	WscFrame.Flags  = 0x00;
    
	// Out buffer for transmitting message 2		
	OutBuffer = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG); //Get an unused nonpaged memory
	if (OutBuffer == NULL)	
		return;					

	FrameLen = 0;
	
	// Make	 Transmitting frame
	MakeOutgoingFrame(OutBuffer, &FrameLen,	sizeof(HEADER_802_11), &Header_802_11,
		sizeof(EAPHEAD), EAPHEAD, sizeof(IEEE8021X_FRAME), &Ieee_8021x,
		sizeof(EapFrame), &EapFrame, sizeof(WscFrame), &WscFrame, Len, OutMsgBuf,
		END_OF_ARGS);

	// Send using priority queue
	MiniportMMRequest(pAdapter,QID_AC_BE, OutBuffer, FrameLen);

	// Update WSC status
	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_RSP_ACK_SENT;	

	kfree(OutBuffer);
	DBGPRINT(RT_DEBUG_WPS, "<----- WscSendEapRspAck\n");	
}

/*
	========================================================================
	
	Routine Description:
		Send WSC Eap-Rsp-Nack

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		If the supplicant encounters an error authenticating or 
		processing a message, send Eap-Rsp-Nack to the authenticator
		
	========================================================================
*/
VOID	WscSendEapRspNack(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	HEADER_802_11		Header_802_11;
	UCHAR				AckRate = RATE_2;
	USHORT				AckDuration = 0;
	IEEE8021X_FRAME		Ieee_8021x;
	EAP_FRAME			EapFrame;
	WSC_FRAME			WscFrame;
	UCHAR				*OutBuffer = NULL;
	PWSC_REG_DATA		pReg;
	ULONG				FrameLen = 0;
	PUCHAR				pData;
	INT					Len;
	USHORT				Length;
	UCHAR				EAPHEAD[8] = {0xaa,	0xaa, 0x03,	0x00, 0x00,	0x00,0x88,0x8e};
	
	// Send EAPOL-Response-Nack
	DBGPRINT(RT_DEBUG_WPS, "-----> WscSendEapRspNack\n");

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;

	// Prepare the M5 message body after WSC Frame header
	pData = (PUCHAR) &OutMsgBuf[0];
	Len = 0;
	
	// 1. Version
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_VERSION;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_VERSION);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = pReg->EnrolleeInfo.Version;
	pData += 5;
	Len   += 5;

	// 2. Message Type, M
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_MSG_TYPE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_MSG_TYPE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = WSC_MSG_WSC_NACK;
	pData += 5;
	Len   += 5;

	// 3. Enrollee Nonce, 16 bytes
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_EN_NONCE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_EN_NONCE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0010);
#endif
	RTMPMoveMemory((pData + 4), pReg->EnrolleeNonce, 16);
	pData += 20;
	Len   += 20;
	
	// 4. Registrar Nonce, 16 bytes
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_REG_NONCE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_REG_NONCE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0010);
#endif
	RTMPMoveMemory((pData + 4), pReg->RegistrarNonce, 16);
	pData += 20;
	Len   += 20;

	// 5. Configure Error
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_CONF_ERROR;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0002;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_CONF_ERROR);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0002);
#endif
#ifdef SKY_LITTLE_ENDIAN
	memcpy((pData + 4), (void *)&pReg->EnrolleeInfo.ConfigError,2);
#else
	*((PUSHORT) (pData + 4)) = pReg->EnrolleeInfo.ConfigError;	// send error code to the peer
#endif
	pData += 6;
	Len   += 6;

	// =====================================
	// Use Priority Ring & MiniportMMRequest
	// =====================================
	pAdapter->Sequence = ((pAdapter->Sequence) + 1)	& (MAX_SEQ_NUMBER);
	WscMacHeaderInit(pAdapter, &Header_802_11, pAdapter->PortCfg.Bssid);

	// ACK size	is 14 include CRC, and its rate	is based on real time information
	AckRate = pAdapter->PortCfg.ExpectedACKRate[pAdapter->PortCfg.TxRate];
	AckDuration = RTMPCalcDuration(pAdapter, AckRate, 14);
	Header_802_11.Duration = pAdapter->PortCfg.Dsifs + AckDuration;

	// Length = EAP_FRAME + WSC_FRAME size
	Length = (USHORT)(sizeof(EAP_FRAME) + sizeof(WSC_FRAME) + Len);
	
	// Zero 802.1x body
	NdisZeroMemory(&Ieee_8021x, sizeof(Ieee_8021x));
	Ieee_8021x.Version = EAPOL_VER;
	Ieee_8021x.Type    = EAPPacket;
#ifdef SKY_LITTLE_ENDIAN
	Ieee_8021x.Length  = cpu2be16((unsigned char *)&Length);
#else
	Ieee_8021x.Length  = cpu2be16(Length);
#endif
	// Zero EAP frame
	NdisZeroMemory(&EapFrame, sizeof(EapFrame));
	EapFrame.Code   = EAP_CODE_RSP;
	EapFrame.Id     = Id;
#ifdef SKY_LITTLE_ENDIAN
	EapFrame.Length = cpu2be16((unsigned char *)&Length);
#else
	EapFrame.Length = cpu2be16(Length);
#endif
	EapFrame.Type   = EAP_TYPE_WSC;
	
	// Zero WSC Frame
	NdisZeroMemory(&WscFrame, sizeof(WscFrame));
	WscFrame.SMI[0] = 0x00;
	WscFrame.SMI[1] = 0x37;
	WscFrame.SMI[2] = 0x2A;
#ifdef SKY_LITTLE_ENDIAN
    {
        unsigned int tmp = WSC_VENDOR_TYPE;
        WscFrame.VendorType = cpu2be32((unsigned char *)&tmp);
    }
#else
	WscFrame.VendorType = cpu2be32(WSC_VENDOR_TYPE);
#endif
	WscFrame.OpCode = WSC_OPCODE_NACK;
	WscFrame.Flags  = 0x00;
	
	// Out buffer for transmitting message 2		
	OutBuffer = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);   //Get an unused nonpaged memory
	if (OutBuffer == NULL)	
		return;					

	FrameLen = 0;
	
	// Make	 Transmitting frame
	MakeOutgoingFrame(OutBuffer, &FrameLen,	sizeof(HEADER_802_11), &Header_802_11,
		sizeof(EAPHEAD), EAPHEAD, sizeof(IEEE8021X_FRAME), &Ieee_8021x,
		sizeof(EapFrame), &EapFrame, sizeof(WscFrame), &WscFrame, Len, OutMsgBuf,
		END_OF_ARGS);

	// Send using priority queue
	MiniportMMRequest(pAdapter,QID_AC_BE,OutBuffer, FrameLen);

	kfree(OutBuffer);
	DBGPRINT(RT_DEBUG_WPS, "<----- WscSendEapRspNack\n");	
}

/*
	========================================================================
	
	Routine Description:
		Send WSC Eap-Rsp-Done

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after Rx EAP-Req(ID)
		1. Change the correct parameters
		2. Send Eap-Rsp-Done
		
	========================================================================
*/
VOID	WscSendEapRspDone(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	HEADER_802_11		Header_802_11;
	UCHAR				AckRate = RATE_2;
	USHORT				AckDuration = 0;
	IEEE8021X_FRAME		Ieee_8021x;
	EAP_FRAME			EapFrame;
	WSC_FRAME			WscFrame;
	UCHAR				*OutBuffer = NULL;
	PWSC_REG_DATA		pReg;
	ULONG				FrameLen = 0;
	PUCHAR				pData;
	INT					Len;
	USHORT				Length;
	UCHAR				EAPHEAD[8] = {0xaa,	0xaa, 0x03,	0x00, 0x00,	0x00,0x88,0x8e};
	
	// Send EAPOL-Response-Done
	DBGPRINT(RT_DEBUG_WPS, "-----> WscSendEapRspDone\n");

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;

	// Prepare the M5 message body after WSC Frame header
	pData = (PUCHAR) &OutMsgBuf[0];
	Len = 0;
	
	// 1. Version
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_VERSION;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_VERSION);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = pReg->EnrolleeInfo.Version;
	pData += 5;
	Len   += 5;

	// 2. Message Type, M
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_MSG_TYPE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_MSG_TYPE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = WSC_MSG_WSC_DONE;
	pData += 5;
	Len   += 5;

	// 3. Enrollee Nonce, 16 bytes
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_EN_NONCE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_EN_NONCE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0010);
#endif
	RTMPMoveMemory((pData + 4), pReg->EnrolleeNonce, 16);
	pData += 20;
	Len   += 20;
	
	// 4. Registrar Nonce, 16 bytes
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_REG_NONCE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_REG_NONCE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0010);
#endif
	RTMPMoveMemory((pData + 4), pReg->RegistrarNonce, 16);
	pData += 20;
	Len   += 20;

	// =====================================
	// Use Priority Ring & MiniportMMRequest
	// =====================================
	pAdapter->Sequence = ((pAdapter->Sequence) + 1)	& (MAX_SEQ_NUMBER);
	WscMacHeaderInit(pAdapter, &Header_802_11, pAdapter->PortCfg.Bssid);

	// ACK size	is 14 include CRC, and its rate	is based on real time information
	AckRate = pAdapter->PortCfg.ExpectedACKRate[pAdapter->PortCfg.TxRate];
	AckDuration = RTMPCalcDuration(pAdapter, AckRate, 14);
	Header_802_11.Duration = pAdapter->PortCfg.Dsifs + AckDuration;

	// Length = EAP_FRAME + WSC_FRAME size
	Length = (USHORT)(sizeof(EAP_FRAME) + sizeof(WSC_FRAME) + Len);
	
	// Zero 802.1x body
	NdisZeroMemory(&Ieee_8021x, sizeof(Ieee_8021x));
	Ieee_8021x.Version = EAPOL_VER;
	Ieee_8021x.Type    = EAPPacket;
#ifdef SKY_LITTLE_ENDIAN
	Ieee_8021x.Length  = cpu2be16((unsigned char *)&Length);
#else
	Ieee_8021x.Length  = cpu2be16(Length);
#endif
	// Zero EAP frame
	NdisZeroMemory(&EapFrame, sizeof(EapFrame));
	EapFrame.Code   = EAP_CODE_RSP;
	EapFrame.Id     = Id;
#ifdef SKY_LITTLE_ENDIAN
	EapFrame.Length = cpu2be16((unsigned char *)&Length);
#else
	EapFrame.Length = cpu2be16(Length);
#endif
	EapFrame.Type   = EAP_TYPE_WSC;
	
	// Zero WSC Frame
	NdisZeroMemory(&WscFrame, sizeof(WscFrame));
	WscFrame.SMI[0] = 0x00;
	WscFrame.SMI[1] = 0x37;
	WscFrame.SMI[2] = 0x2A;
#ifdef SKY_LITTLE_ENDIAN
    {
        unsigned int tmp = WSC_VENDOR_TYPE;
        WscFrame.VendorType = cpu2be32((unsigned char *)&tmp);
    }
#else
	WscFrame.VendorType = cpu2be32(WSC_VENDOR_TYPE);
#endif
	WscFrame.OpCode = WSC_OPCODE_DONE;
	WscFrame.Flags  = 0x00;
	
	// Out buffer for transmitting message 2		
	OutBuffer = kmalloc(MAX_LEN_OF_MLME_BUFFER, MEM_ALLOC_FLAG);   //Get an unused nonpaged memory
	if (OutBuffer == NULL)	
		return;					

	FrameLen = 0;
	
	// Make	 Transmitting frame
	MakeOutgoingFrame(OutBuffer, &FrameLen,	sizeof(HEADER_802_11), &Header_802_11,
		sizeof(EAPHEAD), EAPHEAD, sizeof(IEEE8021X_FRAME), &Ieee_8021x,
		sizeof(EapFrame), &EapFrame, sizeof(WscFrame), &WscFrame, Len, OutMsgBuf,
		END_OF_ARGS);

	// Send using priority queue
	MiniportMMRequest(pAdapter,QID_AC_BE, OutBuffer, FrameLen);

	kfree(OutBuffer);
	DBGPRINT(RT_DEBUG_WPS, "<----- WscSendEapRspDone\n");	
}

/*
	========================================================================
	
	Routine Description:
		Receive M2D message

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		TRUE/FALSE

	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after Rx M2
		1. Verify the contents
		2. Send M3
		
	========================================================================
*/
BOOLEAN	WscReceiveEapM2(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	USHORT				Length;
	ULONG				HmacLen;
	PUCHAR				pData, pAuth;
	UCHAR				Hmac[8], DHKey[32], KDK[32], KdkInput[38], KdfKey[80];
	PIEEE8021X_FRAME	p802_1x;
	PWSC_REG_DATA		pReg;
	USHORT				WscType, WscLen;
	INT					DH_Len;

	DBGPRINT(RT_DEBUG_WPS, "-----> WscReceiveEapM2\n");

// <<Low Power CPU>> ++, add by johnli
	// Update WSC status
	if (pAdapter->PortCfg.WscControl.WscStatus == STATUS_WSC_EAP_M2_RECEIVED)
		return FALSE;
	else
		pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_M2_RECEIVED;	
//<<Low Power CPU>> --

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;

	// Skip all the EAP headers since we already parse the correct M2 type earlier
	// Offset the pointer to start of WSC IEs, but we have to store the EAP length first

	// Skip the EAP LLC header check since it's been checked many times.
	pData = (PUCHAR) &pElem->Msg[LENGTH_802_11 + LENGTH_802_1_H];
	pData += sizeof(IEEE8021X_FRAME);

	// start of 802.1x frame header
	p802_1x = (PIEEE8021X_FRAME) pData;
#ifdef SKY_LITTLE_ENDIAN
    Length = be2cpu16((unsigned char *)&p802_1x->Length);
#else
	Length = be2cpu16(p802_1x->Length);
#endif
	// Length include EAP 5 bytes header
	Length -= sizeof(EAP_FRAME);

	// Skip WSC Frame fixed header
	Length -= sizeof(WSC_FRAME);
	pData += (sizeof(EAP_FRAME) + sizeof(WSC_FRAME));

	// Copy the content to Regdata for lastRx information
	// Length must include authenticator IE size
	if (Length >= WSC_MAX_RX_BUFFER)
	{
		//DBGPRINT(RT_DEBUG_ERROR, "ERROR: EapM2 is too long (=%d)\n", be2cpu16(p802_1x->Length));
		printk("ERROR: EapM2 is too long\n");
		return FALSE;
	}
	pReg->LastRx.Length = Length;		
	RTMPMoveMemory(pReg->LastRx.Data, pData, Length);

	// Start to process WSC IEs
	while (Length > 4)
	{
#ifdef SKY_LITTLE_ENDIAN
		WscType = cpu2be16((unsigned char *)pData);
		WscLen  = cpu2be16((unsigned char *)(pData + 2));
#else
		WscType = cpu2be16(*((PUSHORT) pData));
		WscLen  = cpu2be16(*((PUSHORT) (pData + 2)));
#endif
		pData  += 4;
		Length -= 4;
		// Parse M2 WSC type and store to RegData structure
		switch (WscType)
		{
			case WSC_IE_VERSION:
				pReg->RegistrarInfo.Version = *pData;
				break;
				
			case WSC_IE_EN_NONCE:
				// for verification with our enrollee nonce
				if (RTMPCompareMemory(&pReg->EnrolleeNonce, pData, 16) != 0)
				{
					DBGPRINT(RT_DEBUG_WPS, "Rx M2 Compare enrollee nonce mismatched \n");	   
					pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_NONCE_MISMATCH;	
					return FALSE;
				}
				break;
				
			case WSC_IE_REG_NONCE:
				RTMPMoveMemory(pReg->RegistrarNonce, pData, 16);
				break;
				
			case WSC_IE_UUID_R:
				RTMPMoveMemory(pReg->RegistrarInfo.Uuid, pData, 16);
				break;
				
			case WSC_IE_MSG_TYPE:
				// Do nothing since we already parse it before 
				break;
				
			case WSC_IE_PUBLIC_KEY:
				// Get Registrar Public Key
				RTMPMoveMemory(pReg->Pkr, pData, 192);
				break;
				
			case WSC_IE_AUTH_TYPE_FLAG:
#ifdef SKY_LITTLE_ENDIAN
				memcpy((void *)&pReg->RegistrarInfo.AuthTypeFlags,pData,2);
#else
				pReg->RegistrarInfo.AuthTypeFlags = *((PUSHORT) pData);
#endif
				break;
				
			case WSC_IE_ENCR_TYPE_FLAG:
#ifdef SKY_LITTLE_ENDIAN
				memcpy((void *)&pReg->RegistrarInfo.EncrTypeFlags,pData,2);
#else
				pReg->RegistrarInfo.EncrTypeFlags = *((PUSHORT) pData);
#endif
				break;
				
			case WSC_IE_CONN_TYPE_FLAG:
				pReg->RegistrarInfo.ConnTypeFlags = *pData;
				break;
				
			case WSC_IE_CONF_MTHD:
#ifdef SKY_LITTLE_ENDIAN
				memcpy((void *)&pReg->RegistrarInfo.ConfigMethods,pData,2);
#else
				pReg->RegistrarInfo.ConfigMethods = *((PUSHORT) pData);
#endif
				break;
				
			case WSC_IE_MANUFACTURER:
				RTMPMoveMemory(pReg->RegistrarInfo.Manufacturer, pData, 64);
				break;
				
			case WSC_IE_MODEL_NAME:
				RTMPMoveMemory(pReg->RegistrarInfo.ModelName, pData, 32);
				break;
				
			case WSC_IE_MODEL_NO:
				RTMPMoveMemory(pReg->RegistrarInfo.ModelNumber, pData, 32);
				break;
				
			case WSC_IE_SERIAL:
				RTMPMoveMemory(pReg->RegistrarInfo.SerialNumber, pData, 32);
				break;
				
			case WSC_IE_PRI_DEV_TYPE:
				RTMPMoveMemory(pReg->RegistrarInfo.PriDeviceType, pData, 8);
				break;
				
			case WSC_IE_DEV_NAME:
				RTMPMoveMemory(pReg->RegistrarInfo.DeviceName, pData, 32);
				break;
				
			case WSC_IE_RF_BAND:
				pReg->RegistrarInfo.RfBand = *pData;
				break;
				
			case WSC_IE_ASSOC_STATE:
#ifdef SKY_LITTLE_ENDIAN
	memcpy((void *)&pReg->RegistrarInfo.AssocState,pData,2);
#else
				pReg->RegistrarInfo.AssocState = *((PUSHORT) pData);
#endif
				break;
				
			case WSC_IE_CONF_ERROR:
#ifdef SKY_LITTLE_ENDIAN
	memcpy((void *)&pReg->RegistrarInfo.ConfigError,pData,2);
#else
				pReg->RegistrarInfo.ConfigError = *((PUSHORT) pData);
#endif
#ifdef SKY_LITTLE_ENDIAN
				{
					unsigned short tmp;
					tmp = WSC_ERROR_NO_ERROR;
					if (pReg->RegistrarInfo.ConfigError!=tmp)
						DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapM2 --> ConfigError 0x%04x\n", pReg->RegistrarInfo.ConfigError);
				}
#else
				if (pReg->RegistrarInfo.ConfigError != cpu2be16(WSC_ERROR_NO_ERROR))
					DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapM2 --> ConfigError 0x%04x\n", pReg->RegistrarInfo.ConfigError);
#endif
				break;
				
			case WSC_IE_DEV_PASS_ID:
				break;
				
			case WSC_IE_OS_VER:
#ifdef SKY_LITTLE_ENDIAN
	memcpy((void *)&pReg->RegistrarInfo.OsVersion,pData,4);
#else
				pReg->RegistrarInfo.OsVersion = *((PULONG) pData);
#endif
				break;
				
			case WSC_IE_AUTHENTICATOR:
				RTMPMoveMemory(Hmac, pData, 8);
				break;

			default:
				DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapM2 --> Unknown IE 0x%04x\n", WscType);	   
				break;				
		}

		// Offset to net WSC Ie
		pData  += WscLen;
		Length -= WscLen;
	}					

	// Let's start to generate keys and verify the data payload
	// Generate Secret Key
	GenerateDHSecreteKey(pAdapter->PortCfg.WscControl.pSecKeyMem, &pReg->EnrolleeRandom[0], 192, pReg->Pkr, 192, &pReg->SecretKey[0], &DH_Len);

#ifndef CONFIG_CRYPTO_HMACXXX
	// Compute the DHKey based on the DH secret
	sha256(&DHKey[0], &pReg->SecretKey[0], 192);
#else
{
	struct crypto_tfm *tfm;
	unsigned int klen = 192;
	struct scatterlist sg[1];
	char result[64];

	sg_set_buf(&sg[0], "sha256", 6);
	tfm = crypto_alloc_tfm("sha256", 0);
	if (tfm == NULL) {
		printk("failed to load transform for sha256\n");
		return;
	}
	//crypto_tfm_alg_digestsize(tfm);
	crypto_hmac(tfm, &pReg->SecretKey[0], &klen, sg, 1, result);
}
#endif //CONFIG_CRYPTO_HMAC

	// Create KDK input data
	RTMPMoveMemory(&KdkInput[0], pReg->EnrolleeNonce, 16);
		
	RTMPMoveMemory(&KdkInput[16], pReg->EnrolleeInfo.MacAddr, 6);
		
	RTMPMoveMemory(&KdkInput[22], pReg->RegistrarNonce, 16);
	
	// Generate the KDK
	hmac_sha(DHKey, 32,  KdkInput, 38, KDK, 32);				

	// KDF
	WscDeriveKey(KDK, 32, Wsc_Personal_String, (sizeof(Wsc_Personal_String) - 1), KdfKey, 640);

	// Assign Key from KDF
	RTMPMoveMemory(pReg->AuthKey, &KdfKey[0], 32);
	RTMPMoveMemory(pReg->KeyWrapKey, &KdfKey[32], 16);
	RTMPMoveMemory(pReg->Emsk, &KdfKey[48], 32);

	// Combine last TX & RX message contents and validate the HMAC
	// We have to exclude last 12 bytes from last receive since it's authenticator value
	HmacLen = pReg->LastTx.Length + pReg->LastRx.Length - 12;
	pAuth = (PUCHAR) &HmacData[0];
	RTMPMoveMemory(pAuth, pReg->LastTx.Data, pReg->LastTx.Length);
	pAuth += pReg->LastTx.Length;
	RTMPMoveMemory(pAuth, pReg->LastRx.Data, pReg->LastRx.Length - 12);

	// Validate HMAC, reuse KDK buffer
	hmac_sha(pReg->AuthKey, 32, HmacData, HmacLen, KDK, 32);

	if (RTMPEqualMemory(Hmac, KDK, 8) != 1)
	{
		DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapM2 --> HMAC not match\n");
		pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_INVALID_DATA;	
		return FALSE;
	}
	// Update WSC status
// <<Low Power CPU>> ++, remove by johnli
//	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_M2_RECEIVED;	
// <<Low Power CPU>> --
	DBGPRINT(RT_DEBUG_WPS, "<----- WscReceiveEapM2\n");		
	return TRUE;
	
}

/*
	========================================================================
	
	Routine Description:
		Receive M2D message

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		TRUE/FALSE

	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after Rx M2D
		1. Verify the contents
		2. Send Eap-Rsp-Ack
		3. go back to wait for M2 state
		
	========================================================================
*/
BOOLEAN	WscReceiveEapM2D(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	USHORT				Length;
	PUCHAR				pData;
	PIEEE8021X_FRAME	p802_1x;
	PWSC_REG_DATA		pReg;
	USHORT				WscType, WscLen;

	DBGPRINT(RT_DEBUG_WPS, "-----> WscReceiveEapM2D\n");

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;

	// Skip all the EAP headers since we already parse the correct M2 type earlier
	// Offset the pointer to start of WSC IEs, but we have to store the EAP length first

	// Skip the EAP LLC header check since it's been checked many times.
	pData = (PUCHAR) &pElem->Msg[LENGTH_802_11 + LENGTH_802_1_H];
	pData += sizeof(IEEE8021X_FRAME);

	// start of 802.1x frame header
	p802_1x = (PIEEE8021X_FRAME) pData;
#ifdef SKY_LITTLE_ENDIAN
    Length = be2cpu16((unsigned char *)&p802_1x->Length);
#else
	Length = be2cpu16(p802_1x->Length);
#endif
	// Length include EAP 5 bytes header
	Length -= sizeof(EAP_FRAME);

	// Skip WSC Frame fixed header
	Length -= sizeof(WSC_FRAME);
	pData += (sizeof(EAP_FRAME) + sizeof(WSC_FRAME));

	// Start to process WSC IEs
	while (Length > 4)
	{
#ifdef SKY_LITTLE_ENDIAN
		WscType = cpu2be16((unsigned char *)pData);
		WscLen  = cpu2be16((unsigned char *)(pData + 2));
#else
		WscType = cpu2be16(*((PUSHORT) pData));
		WscLen  = cpu2be16(*((PUSHORT) (pData + 2)));
#endif
		pData  += 4;
		Length -= 4;

		// Parse M2D WSC type and store to RegData structure
		switch (WscType)
		{		
			case WSC_IE_EN_NONCE:
				// for verification with our enrollee nonce
				if (RTMPCompareMemory(&pReg->EnrolleeNonce, pData, 16) != 0)
				{
					DBGPRINT(RT_DEBUG_WPS, "Rx M2D: It's not our own Enrollee Nonce!!! \n");
					return FALSE;
				}
				break;
				
			case WSC_IE_REG_NONCE:
				RTMPMoveMemory(pReg->M2DRegistrarNonce, pData, 16);
				break;
				
			case WSC_IE_MSG_TYPE:
				// Do nothing since we already parse it before 
				break;
				
			// M2D might be not the candidate Registrar we want to connect
			// Parse them on M2 later !!
			case WSC_IE_VERSION:
			case WSC_IE_UUID_R:				
			case WSC_IE_AUTH_TYPE_FLAG:
			case WSC_IE_ENCR_TYPE_FLAG:
			case WSC_IE_CONN_TYPE_FLAG:
			case WSC_IE_CONF_MTHD:
			case WSC_IE_MANUFACTURER:
			case WSC_IE_MODEL_NAME:
			case WSC_IE_MODEL_NO:
			case WSC_IE_SERIAL:
			case WSC_IE_PRI_DEV_TYPE:
			case WSC_IE_DEV_NAME:
			case WSC_IE_RF_BAND:
			case WSC_IE_ASSOC_STATE:
			case WSC_IE_CONF_ERROR:				
			case WSC_IE_DEV_PASS_ID:				
			case WSC_IE_OS_VER:
				break;
				
			case WSC_IE_PUBLIC_KEY:
				// There shall be no Public transmitted in M2D
				break;
				
			case WSC_IE_AUTHENTICATOR:
				// No authenticator in M2D
				break;

			default:
				DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapM2D --> Unknown IE 0x%04x\n", WscType);	   
				break;
				
		}

		// Offset to net WSC Ie
		pData  += WscLen;
		Length -= WscLen;
	}					

	// Update WSC status
	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_M2D_RECEIVED;	

	DBGPRINT(RT_DEBUG_WPS, "<----- WscReceiveEapM2D\n");		

	return TRUE;
}

/*
	========================================================================
	
	Routine Description:
		Receive M4 message

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		TRUE/FALSE

	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after Rx M4
		1. Verify the contents
		2. Send M5
		
	========================================================================
*/
BOOLEAN WscReceiveEapM4(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	USHORT				Length;
	ULONG				HmacLen;
	PUCHAR				pData, pAuth;
	PIEEE8021X_FRAME	p802_1x;
	PWSC_REG_DATA		pReg;
	UCHAR				Hmac[8], Tmp[512], RHash[32];
	INT					EncrLen, idx;
	USHORT				WscType, WscLen;

	DBGPRINT(RT_DEBUG_WPS, "-----> WscReceiveEapM4\n");

// <<Low Power CPU>> ++, add by johnli
	// Update WSC status
	if (pAdapter->PortCfg.WscControl.WscStatus == STATUS_WSC_EAP_M4_RECEIVED)
		return FALSE;
	else
		pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_M4_RECEIVED;	
//<<Low Power CPU>> --

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;

	// Skip all the EAP headers since we already parse the correct M2 type earlier
	// Offset the pointer to start of WSC IEs, but we have to store the EAP length first

	// Skip the EAP LLC header check since it's been checked many times.
	pData = (PUCHAR) &pElem->Msg[LENGTH_802_11 + LENGTH_802_1_H];
	pData += sizeof(IEEE8021X_FRAME);

	// start of 802.1x frame header
	p802_1x = (PIEEE8021X_FRAME) pData;
#ifdef SKY_LITTLE_ENDIAN
    Length = be2cpu16((unsigned char *)&p802_1x->Length);
#else
	Length = be2cpu16(p802_1x->Length);
#endif
	// Length include EAP 5 bytes header
	Length -= sizeof(EAP_FRAME);

	// Skip WSC Frame fixed header
	Length -= sizeof(WSC_FRAME);
	pData += (sizeof(EAP_FRAME) + sizeof(WSC_FRAME));

	// Copy the content to Regdata for lastRx information
	// Length must include authenticator IE size
	if (Length >= WSC_MAX_RX_BUFFER)
	{
		//DBGPRINT(RT_DEBUG_ERROR, "ERROR: EapM4 is too long (=%d)\n", be2cpu16(p802_1x->Length));
		printk("ERROR: EapM4 is too long\n");
		return FALSE;
	}
	pReg->LastRx.Length = Length;		
	RTMPMoveMemory(pReg->LastRx.Data, pData, Length);

	// Start to process WSC IEs
	while (Length > 4)
	{
#ifdef SKY_LITTLE_ENDIAN
		WscType = cpu2be16((unsigned char *)pData);
		WscLen  = cpu2be16((unsigned char *)(pData + 2));
#else
		WscType = cpu2be16(*((PUSHORT) pData));
		WscLen  = cpu2be16(*((PUSHORT) (pData + 2)));
#endif
		pData  += 4;
		Length -= 4;

		// Parse M4 WSC type and store to RegData structure
		switch (WscType)
		{
			case WSC_IE_VERSION:
				pReg->RegistrarInfo.Version = *pData;
				break;
				
			case WSC_IE_EN_NONCE:
				// for verification with our enrollee nonce
				if (RTMPCompareMemory(&pReg->EnrolleeNonce, pData, 16) != 0)
				{
					DBGPRINT(RT_DEBUG_WPS, "Rx M4 Compare enrollee nonce mismatched \n");	   
					pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_NONCE_MISMATCH;
					return FALSE;  //  <<Low Power CPU>> ++, add by johnli
				}
				break;
				
			case WSC_IE_R_HASH_1:
				RTMPMoveMemory(pReg->RHash1, pData, 32);				
				break;
				
			case WSC_IE_R_HASH_2:
				RTMPMoveMemory(pReg->RHash2, pData, 32);				
				break;

			case WSC_IE_ENCR_SETTING:
				// There shall have smoe kind of length check
				if (WscLen <= 16)
					break;
				WscDecryptData((pData + 16), (WscLen - 16), pReg->KeyWrapKey, pData, pReg->ApEncrSettings, &EncrLen);
				DBGPRINT(RT_DEBUG_WPS, "ApEncrSettings len = %d, data --> ", EncrLen);
				
				for (idx = 0; idx < EncrLen; idx++)
				{
					DBGPRINT_RAW(RT_DEBUG_WPS, "%02x-", pReg->ApEncrSettings[idx]);
				}
				DBGPRINT(RT_DEBUG_WPS, "\n");		

				// Parse encryption settings
				WscParseEncrSettings(pAdapter, pReg->ApEncrSettings, EncrLen);
				break;
				
			case WSC_IE_AUTHENTICATOR:
				RTMPMoveMemory(Hmac, pData, 8);
				break;

			case WSC_IE_MSG_TYPE:
				// Do nothing since we already parse it before 
				break;
				
			default:
				DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapM4 --> Unknown IE 0x%04x\n", WscType);	   
				break;
		}

		// Offset to net WSC Ie
		pData  += WscLen;
		Length -= WscLen;
	}

	// Verify R-Hash1
	// Create input for R-Hash1
	RTMPMoveMemory(Tmp, pReg->Rs1, 16);
	RTMPMoveMemory(&Tmp[16], pReg->Psk1, 16);
	RTMPMoveMemory(&Tmp[32], pReg->Pke, 192);
	RTMPMoveMemory(&Tmp[224], pReg->Pkr, 192);
	
	// Generate R-Hash1
	hmac_sha(pReg->AuthKey, 32, Tmp, 416, RHash, 32);
	
	if (RTMPCompareMemory(pReg->RHash1, RHash, 32) != 0)
	{
		DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapM4 --> RHash1 not matched\n");	   

		// Error Psk1 (device password related)  
		pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_PASSWORD_MISMATCH;
		return FALSE;
	}
	
	// Combine last TX & RX message contents and validate the HMAC
	// We have to exclude last 12 bytes from last receive since it's authenticator value
	HmacLen = pReg->LastTx.Length + pReg->LastRx.Length - 12;
	pAuth = (PUCHAR) &HmacData[0];
	RTMPMoveMemory(pAuth, pReg->LastTx.Data, pReg->LastTx.Length);
	pAuth += pReg->LastTx.Length;
	RTMPMoveMemory(pAuth, pReg->LastRx.Data, pReg->LastRx.Length - 12);

	// Validate HMAC, reuse Tmp buffer
	hmac_sha(pReg->AuthKey, 32, HmacData, HmacLen, Tmp, 32);
	
	if (RTMPEqualMemory(Hmac, Tmp, 8) != 1)
	{
		DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapM4 --> HMAC not match\n");
		pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_INVALID_DATA;
		return FALSE;  //  <<Low Power CPU>> ++, add by johnli
	}
	
// <<Low Power CPU>> ++, edit by johnli
	// Check return value
/*
	if (pAdapter->PortCfg.WscControl.WscStatus & 0x0F00)
		return FALSE;
	else
	{
		// Update WSC status
		pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_M4_RECEIVED;
		DBGPRINT(RT_DEBUG_WPS, "<----- WscReceiveEapM4\n");
		return TRUE;
	}
*/

	DBGPRINT(RT_DEBUG_WPS, "<----- WscReceiveEapM4\n");
	return TRUE;
// <<Low Power CPU>> --

}

/*
	========================================================================
	
	Routine Description:
		Receive M6 message

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		TRUE/FALSE

	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after Rx M6
		1. Verify the contents
		2. Send M7
		
	========================================================================
*/
BOOLEAN WscReceiveEapM6(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	USHORT				Length;
	PUCHAR				pData, pAuth;
	PIEEE8021X_FRAME	p802_1x;
	PWSC_REG_DATA		pReg;
	INT					EncrLen, idx;
	UCHAR				Hmac[8], Tmp[512], RHash[32];
	ULONG				HmacLen;
	USHORT				WscType, WscLen;

	DBGPRINT(RT_DEBUG_WPS, "-----> WscReceiveEapM6\n");

// <<Low Power CPU>> ++, add by johnli
	// Update WSC status
	if (pAdapter->PortCfg.WscControl.WscStatus == STATUS_WSC_EAP_M6_RECEIVED)
		return FALSE;
	else
		pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_M6_RECEIVED;	
//<<Low Power CPU>> --

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;

	// Skip all the EAP headers since we already parse the correct M2 type earlier
	// Offset the pointer to start of WSC IEs, but we have to store the EAP length first

	// Skip the EAP LLC header check since it's been checked many times.
	pData = (PUCHAR) &pElem->Msg[LENGTH_802_11 + LENGTH_802_1_H];
	pData += sizeof(IEEE8021X_FRAME);

	// start of 802.1x frame header
	p802_1x = (PIEEE8021X_FRAME) pData;
#ifdef SKY_LITTLE_ENDIAN
    Length = be2cpu16((unsigned char *)&p802_1x->Length);
#else
	Length = be2cpu16(p802_1x->Length);
#endif
	// Length include EAP 5 bytes header
	Length -= sizeof(EAP_FRAME);

	// Skip WSC Frame fixed header
	Length -= sizeof(WSC_FRAME);
	pData += (sizeof(EAP_FRAME) + sizeof(WSC_FRAME));

	// Copy the content to Regdata for lastRx information
	// Length must include authenticator IE size
	if (Length >= WSC_MAX_RX_BUFFER)
	{
		//DBGPRINT(RT_DEBUG_ERROR, "ERROR: EapM6 is too long (=%d)\n", be2cpu16(p802_1x->Length));
		printk("ERROR: EapM6 is too long\n");
		return FALSE;
	}
	pReg->LastRx.Length = Length;		
	RTMPMoveMemory(pReg->LastRx.Data, pData, Length);

	// Start to process WSC IEs
	while (Length > 4)
	{
#ifdef SKY_LITTLE_ENDIAN
		WscType = cpu2be16((unsigned char *)pData);
		WscLen  = cpu2be16((unsigned char *)(pData + 2));
#else
		WscType = cpu2be16(*((PUSHORT) pData));
		WscLen  = cpu2be16(*((PUSHORT) (pData + 2)));
#endif
		pData  += 4;
		Length -= 4;

		// Parse M6 WSC type and store to RegData structure
		switch (WscType)
		{
			case WSC_IE_VERSION:
				pReg->RegistrarInfo.Version = *pData;
				break;
				
			case WSC_IE_EN_NONCE:
				// for verification with our enrollee nonce
				if (RTMPCompareMemory(&pReg->EnrolleeNonce, pData, 16) != 0)
				{
					DBGPRINT(RT_DEBUG_WPS, "Rx M6 Compare enrollee nonce mismatched \n");	   
					pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_NONCE_MISMATCH;	
					return FALSE;  //  <<Low Power CPU>> ++, add by johnli
				}
				break;
				
			case WSC_IE_ENCR_SETTING:
				// There shall have smoe kind of length check
				if (WscLen <= 16)
					break;
				WscDecryptData((pData + 16), (WscLen - 16), pReg->KeyWrapKey, pData, pReg->ApEncrSettings, &EncrLen);
				DBGPRINT(RT_DEBUG_WPS, "ApEncrSettings len = %d, data --> ", EncrLen);
				
				for (idx = 0; idx < EncrLen; idx++)
				{
					DBGPRINT_RAW(RT_DEBUG_WPS, "%02x-", pReg->ApEncrSettings[idx]);
				}
				DBGPRINT(RT_DEBUG_WPS, "\n");		

				// Parse encryption settings
				WscParseEncrSettings(pAdapter, pReg->ApEncrSettings, EncrLen);
				break;
				
			case WSC_IE_AUTHENTICATOR:
				RTMPMoveMemory(Hmac, pData, 8);
				break;

			case WSC_IE_MSG_TYPE:
				// Do nothing since we already parse it before 
				break;
				
			default:
				DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapM6 --> Unknown IE 0x%04x\n", WscType);	   
				break;
		}

		// Offset to net WSC Ie
		pData  += WscLen;
		Length -= WscLen;
	}

	// Verify R-Hash2
	// Create input for R-Hash1
	RTMPMoveMemory(Tmp, pReg->Rs2, 16);
	RTMPMoveMemory(&Tmp[16], pReg->Psk2, 16);
	RTMPMoveMemory(&Tmp[32], pReg->Pke, 192);
	RTMPMoveMemory(&Tmp[224], pReg->Pkr, 192);
	
	// Generate R-Hash2
	hmac_sha(pReg->AuthKey, 32, Tmp, 416, RHash, 32);
	
	if (RTMPCompareMemory(pReg->RHash2, RHash, 32) != 0)
	{
		DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapM6 --> RHash2 not matched\n");	   

		// Error Psk2 (device password related)
		pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_PASSWORD_MISMATCH;
		return FALSE;
	}
	
	// Combine last TX & RX message contents and validate the HMAC
	// We have to exclude last 12 bytes from last receive since it's authenticator value
	HmacLen = pReg->LastTx.Length + pReg->LastRx.Length - 12;
	pAuth = (PUCHAR) &HmacData[0];
	RTMPMoveMemory(pAuth, pReg->LastTx.Data, pReg->LastTx.Length);
	pAuth += pReg->LastTx.Length;
	RTMPMoveMemory(pAuth, pReg->LastRx.Data, pReg->LastRx.Length - 12);

	// Validate HMAC, reuse Tmp buffer
	hmac_sha(pReg->AuthKey, 32, HmacData, HmacLen, Tmp, 32);
	
	if (RTMPEqualMemory(Hmac, Tmp, 8) != 1)
	{
		DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapM6 --> HMAC not match\n");	
		pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_INVALID_DATA;
		return FALSE;  //  <<Low Power CPU>> ++, add by johnli
	}
	

// <<Low Power CPU>> ++, edit by johnli
	// Check return value
/*
	if (pAdapter->PortCfg.WscControl.WscStatus & 0x0F00)
		return FALSE;
	else
	{	
		// Update WSC status
		pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_M6_RECEIVED;
		DBGPRINT(RT_DEBUG_WPS, "<----- WscReceiveEapM6\n");
		return TRUE;
	}
*/

	DBGPRINT(RT_DEBUG_WPS, "<----- WscReceiveEapM6\n");
	return TRUE;
// <<Low Power CPU>> --

}

/*
	========================================================================
	
	Routine Description:
		Receive M8 message

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		TRUE/FALSE

	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after Rx M8
		1. Verify the contents
		2. Send Eap-Rsp-Done
		
	========================================================================
*/
BOOLEAN WscReceiveEapM8(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	USHORT				Length;
	PUCHAR				pData, pAuth;
	PIEEE8021X_FRAME	p802_1x;
	PWSC_REG_DATA		pReg;
	INT					EncrLen, idx;
	UCHAR				Hmac[8], Tmp[512];//, RHash[32];
	ULONG				HmacLen;
	USHORT				WscType, WscLen;

	DBGPRINT(RT_DEBUG_WPS, "-----> WscReceiveEapM8\n");

// <<Low Power CPU>> ++, add by johnli
	// Update WSC status
	if (pAdapter->PortCfg.WscControl.WscStatus == STATUS_WSC_EAP_M8_RECEIVED)
		return FALSE;
	else
		pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_M8_RECEIVED;	
//<<Low Power CPU>> --

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;

	// Skip all the EAP headers since we already parse the correct M2 type earlier
	// Offset the pointer to start of WSC IEs, but we have to store the EAP length first

	// Skip the EAP LLC header check since it's been checked many times.
	pData = (PUCHAR) &pElem->Msg[LENGTH_802_11 + LENGTH_802_1_H];
	pData += sizeof(IEEE8021X_FRAME);

	// start of 802.1x frame header
	p802_1x = (PIEEE8021X_FRAME) pData;
#ifdef SKY_LITTLE_ENDIAN
    Length = be2cpu16((unsigned char *)&p802_1x->Length);
#else
	Length = be2cpu16(p802_1x->Length);
#endif
	// Length include EAP 5 bytes header
	Length -= sizeof(EAP_FRAME);

	// Skip WSC Frame fixed header
	Length -= sizeof(WSC_FRAME);
	pData += (sizeof(EAP_FRAME) + sizeof(WSC_FRAME));

	// Copy the content to Regdata for lastRx information
	// Length must include authenticator IE size
	if (Length >= WSC_MAX_RX_BUFFER)
	{
		//DBGPRINT(RT_DEBUG_ERROR, "ERROR: EapM8 is too long (=%d)\n", be2cpu16(p802_1x->Length));
		printk("ERROR: EapM8 is too long\n");
		return FALSE;
	}

	pReg->LastRx.Length = Length;		
	RTMPMoveMemory(pReg->LastRx.Data, pData, Length);

	// Start to process WSC IEs
	while (Length > 4)
	{
#ifdef SKY_LITTLE_ENDIAN
		WscType = cpu2be16((unsigned char *)pData);
		WscLen  = cpu2be16((unsigned char *)(pData + 2));
#else
		WscType = cpu2be16(*((PUSHORT) pData));
		WscLen  = cpu2be16(*((PUSHORT) (pData + 2)));
#endif
		pData  += 4;
		Length -= 4;

		// Parse M8 WSC type and store to RegData structure
		switch (WscType)
		{
			case WSC_IE_VERSION:
				pReg->RegistrarInfo.Version = *pData;
				break;
				
			case WSC_IE_EN_NONCE:
				// for verification with our enrollee nonce
				if (RTMPCompareMemory(&pReg->EnrolleeNonce, pData, 16) != 0)
				{
					DBGPRINT(RT_DEBUG_WPS, "Rx M8 Compare enrollee nonce mismatched \n");	   
					pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_NONCE_MISMATCH;	
					return FALSE;
				}
				break;
				
			case WSC_IE_ENCR_SETTING:
				// There shall have smoe kind of length check
				if (WscLen <= 16)
					break;
				if (WscLen > 1024)
				{
					// ApEncrSetting is not enough
					DBGPRINT(RT_DEBUG_WPS, "ApEncrSettings array size is not enough, require %d\n", WscLen);	   					
				}
				WscDecryptData((pData + 16), (WscLen - 16), pReg->KeyWrapKey, pData, pReg->ApEncrSettings, &EncrLen);
				DBGPRINT(RT_DEBUG_WPS, "ApEncrSettings len = %d, data --> ", EncrLen);
				
				for (idx = 0; idx < EncrLen; idx++)
				{
					DBGPRINT_RAW(RT_DEBUG_WPS, "%02x-", pReg->ApEncrSettings[idx]);
				}
				DBGPRINT(RT_DEBUG_WPS, "\n");		

				// Parse encryption settings
				WscProcessCredential(pAdapter, pReg->ApEncrSettings, EncrLen);
				break;
				
			case WSC_IE_AUTHENTICATOR:
				RTMPMoveMemory(Hmac, pData, 8);
				break;

			case WSC_IE_MSG_TYPE:
				// Do nothing since we already parse it before 
				break;
				
			default:
				DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapM8 --> Unknown IE 0x%04x\n", WscType);	   
				break;
		}

		// Offset to net WSC Ie
		pData  += WscLen;
		Length -= WscLen;
	}

	// Combine last TX & RX message contents and validate the HMAC
	// We have to exclude last 12 bytes from last receive since it's authenticator value
	HmacLen = pReg->LastTx.Length + pReg->LastRx.Length - 12;
	pAuth = (PUCHAR) &HmacData[0];
	RTMPMoveMemory(pAuth, pReg->LastTx.Data, pReg->LastTx.Length);
	pAuth += pReg->LastTx.Length;
	RTMPMoveMemory(pAuth, pReg->LastRx.Data, pReg->LastRx.Length - 12);

	// Validate HMAC, reuse Tmp buffer
	hmac_sha(pReg->AuthKey, 32, HmacData, HmacLen, Tmp, 32);
	
	if (RTMPEqualMemory(Hmac, Tmp, 8) != 1)
	{
		DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapM8 --> HMAC not match\n");	
		pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_INVALID_DATA;
		return FALSE;
	}
	
	// Update WSC status
// <<Low Power CPU>> ++, remove by johnli
//	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_M8_RECEIVED;	
// <<Low Power CPU>> --

	DBGPRINT(RT_DEBUG_WPS, "<----- WscReceiveEapM8\n");	

	return TRUE;
}

/*
	========================================================================
	
	Routine Description:
		Receive Eap Fail frame

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		Actions after Rx EAP-Fail
		1. Drop the connection
		2. Save the config file.
		3. Set the correct parameters
		4. Use Auto reconnect to make connection.
		
	========================================================================
*/
VOID	WscReceiveEapFail(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{

	DBGPRINT(RT_DEBUG_WPS, "-----> WscReceiveEapFail\n");
	
	// Skip all the EAP headers since we already parse the correct M2 type earlier
	// Offset the pointer to start of WSC IEs, but we have to store the EAP length first

	// Update WSC status
	// pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_FAIL_RECEIVED;	

	DBGPRINT(RT_DEBUG_WPS, "<----- WscReceiveEapFail\n");	
}

/*
	========================================================================
	
	Routine Description:
		Receive Nack message

	Arguments:
		pAdapter    - NIC Adapter pointer
		Id			- ID between EAP-Req and EAP-Rsp pair
		Elem		- The EAP packet
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		1. Verify the contents
		2. Error handling
		
	========================================================================
*/
VOID WscReceiveEapNack(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	UCHAR				Id,
	IN	PMLME_QUEUE_ELEM	pElem)
{
	USHORT				Length;
	PUCHAR				pData;
	PIEEE8021X_FRAME	p802_1x;
	PWSC_REG_DATA		pReg;
	USHORT				WscType, WscLen;
	USHORT				ConfigError;
	BOOLEAN Cancelled;
	
	DBGPRINT(RT_DEBUG_WPS, "-----> WscReceiveEapNack\n");

	pReg = (PWSC_REG_DATA) &pAdapter->PortCfg.WscControl.RegData;

	// Skip all the EAP headers since we already parse the correct M2 type earlier
	// Offset the pointer to start of WSC IEs, but we have to store the EAP length first

	// Skip the EAP LLC header check since it's been checked many times.
	pData = (PUCHAR) &pElem->Msg[LENGTH_802_11 + LENGTH_802_1_H];
	pData += sizeof(IEEE8021X_FRAME);

	// start of 802.1x frame header
	p802_1x = (PIEEE8021X_FRAME) pData;
#ifdef SKY_LITTLE_ENDIAN
    Length = be2cpu16((unsigned char *)&p802_1x->Length);
#else
	Length = be2cpu16(p802_1x->Length);
#endif
	// Length include EAP 5 bytes header
	Length -= sizeof(EAP_FRAME);

	// Skip WSC Frame fixed header
	Length -= sizeof(WSC_FRAME);
	pData += (sizeof(EAP_FRAME) + sizeof(WSC_FRAME));

	// Init ConfigError to zero
#ifdef SKY_LITTLE_ENDIAN
{
	unsigned short tmp = 0;
	ConfigError = cpu2be16((unsigned char *)&tmp);
}
#else
	ConfigError = cpu2be16(0x0000);
#endif
	// Start to process WSC IEs
	while (Length > 4)
	{
#ifdef SKY_LITTLE_ENDIAN
		WscType = cpu2be16((unsigned char *)pData);
		WscLen  = cpu2be16((unsigned char *)(pData + 2));
#else
		WscType = cpu2be16(*((PUSHORT) pData));
		WscLen  = cpu2be16(*((PUSHORT) (pData + 2)));
#endif
		pData  += 4;
		Length -= 4;

		switch (WscType)
		{
			case WSC_IE_VERSION:
				pReg->RegistrarInfo.Version = *pData;
				break;
				
			case WSC_IE_MSG_TYPE:
				// Do nothing since we already parse it before 
				break;
				
			case WSC_IE_EN_NONCE:
				// for verification with our enrollee nonce
				if (RTMPCompareMemory(&pReg->EnrolleeNonce, pData, 16) != 0)
					DBGPRINT(RT_DEBUG_WPS, "Rx Nack Compare enrollee nonce mismatched \n");	
				break;
				
			case WSC_IE_REG_NONCE:
				RTMPMoveMemory(pReg->RegistrarNonce, pData, 16);
				break;
	
			case WSC_IE_CONF_ERROR:
#ifdef SKY_LITTLE_ENDIAN
	memcpy((void *)&pReg->RegistrarInfo.ConfigError,pData,2);
#else
				pReg->RegistrarInfo.ConfigError = *((PUSHORT) pData);
#endif
#ifdef SKY_LITTLE_ENDIAN
				ConfigError = cpu2be16((unsigned char *) pData);
#else
				ConfigError = cpu2be16(*((PUSHORT) pData));
#endif
				DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapNack --> ConfigError 0x%04x\n", pReg->RegistrarInfo.ConfigError);
				break;
				
			default:
				DBGPRINT(RT_DEBUG_WPS, "WscReceiveEapNack --> Unknown IE 0x%04x\n", WscType);	   
				break;
		}

		// Offset to net WSC Ie
		pData  += WscLen;
		Length -= WscLen;
	}


	//  Error handling
	switch(ConfigError)
	{
		// connection timeout at the peer			
		case WSC_ERROR_REG_SESSION_TIMEOUT:
			if (pAdapter->PortCfg.WscControl.WscState != WSC_STATE_OFF)
			{
				
				// cancel all timer
				RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscConnectTimer,&Cancelled);
				RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscScanTimer,&Cancelled);
				RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscPBCTimer,&Cancelled);
				RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscPINTimer,&Cancelled);
				RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer,&Cancelled);
				RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscM8WaitEapFailTimer,&Cancelled);
				RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapM2Timer,&Cancelled);

				pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_EAP_FAILED;	
				pAdapter->PortCfg.WscControl.WscState = WSC_STATE_OFF;
			}	
			break;
			
		case WSC_ERROR_DEV_PWD_AUTH_FAIL:
			// Change the state to wait EAP-FAIL
			if (pAdapter->PortCfg.WscControl.WscState != WSC_STATE_WAIT_EAPFAIL)
				pAdapter->PortCfg.WscControl.WscState = WSC_STATE_WAIT_EAPFAIL;
			break;
			
		default:
			break;
	}
	
	DBGPRINT(RT_DEBUG_WPS, "<----- WscReceiveEapNack\n");	
}

/*
	========================================================================
	
	Routine Description:
		Process elements within AP encryption settings

	Arguments:
		pAdapter    - NIC Adapter pointer
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		2. Send M3
		
	========================================================================
*/
VOID	WscParseEncrSettings(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	PUCHAR				pPlainData,
	IN	INT					PlainLength)
{
	USHORT	WscType, WscLen, Length, HmacLen;
	PUCHAR	pData;//, pAuth;
	UCHAR	Hmac[8], Temp[32];

	Length = (USHORT) PlainLength;
	HmacLen = Length - 12;
	pData  = pPlainData;
	
	// Start to process WSC IEs
	while (Length > 4)
	{
#ifdef SKY_LITTLE_ENDIAN
		WscType = cpu2be16((unsigned char *)pData);
		WscLen  = cpu2be16((unsigned char *)(pData + 2));
#else
		WscType = cpu2be16(*((PUSHORT) pData));
		WscLen  = cpu2be16(*((PUSHORT) (pData + 2)));
#endif
		pData  += 4;
		Length -= 4;

		// Parse WSC type and store to RegData structure
		switch (WscType)
		{
			case WSC_IE_R_SNONCE_1:
				// for verification with our enrollee nonce
				RTMPMoveMemory(pAdapter->PortCfg.WscControl.RegData.Rs1, pData, 16);
				break;
								
			case WSC_IE_R_SNONCE_2:
				// for verification with our enrollee nonce
				RTMPMoveMemory(pAdapter->PortCfg.WscControl.RegData.Rs2, pData, 16);
				break;
								
			case WSC_IE_KWRAP_AUTH:
				RTMPMoveMemory(Hmac, pData, 8);
				break;

			default:
				DBGPRINT(RT_DEBUG_WPS, "WscParseEncrSettings --> Unknown IE 0x%04x\n", WscType);	   
				break;
		}

		// Offset to net WSC Ie
		pData  += WscLen;
		Length -= WscLen;
	}
	// Validate HMAC, reuse KDK buffer
	hmac_sha(pAdapter->PortCfg.WscControl.RegData.AuthKey, 32, pPlainData, HmacLen, Temp, 32);
	
	if (RTMPEqualMemory(Hmac, Temp, 8) != 1)
	{
		DBGPRINT(RT_DEBUG_WPS, "WscParseEncrSettings --> HMAC not match\n");
	}
}


/*
	========================================================================
	
	Routine Description:
		Process credentials within AP encryption settings

	Arguments:
		pAdapter    - NIC Adapter pointer
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		
	========================================================================
*/
VOID	WscProcessCredential(
	IN	PRTMP_ADAPTER		pAdapter, 
	IN	PUCHAR				pPlainData,
	IN	INT					PlainLength)
{
	USHORT			WscType, WscLen, Length, Cnt, CurrentIdx = 0, Idx;
	PUCHAR			pData, pTmp;
	PWSC_PROFILE	pProfile;

	Length = (USHORT) PlainLength;
	pData  = pPlainData;

	// Cleanup Old contents
	NdisZeroMemory(&pAdapter->PortCfg.WscControl.WscProfile, sizeof(WSC_PROFILE));
	
	pProfile = (PWSC_PROFILE) &pAdapter->PortCfg.WscControl.WscProfile;
	
	// Init Profile number
	Cnt = 0;
	
	// Start to process WSC IEs within credential
	while (Length > 4)
	{
#ifdef SKY_LITTLE_ENDIAN
		WscType = cpu2be16((unsigned char *)pData);
		WscLen  = cpu2be16((unsigned char *)(pData + 2));
#else
		WscType = cpu2be16(*((PUSHORT) pData));
		WscLen  = cpu2be16(*((PUSHORT) (pData + 2)));
#endif
		pData  += 4;
		Length -= 4;

		// Parse WSC type and store to RegData structure
		switch (WscType)
		{
			case WSC_IE_NETWORK_IDX:
				// A new profile, add the cnt and save to database
				CurrentIdx = Cnt;	// since the index start from 0, we have to minus 1
				Cnt++;				
				break;

			case WSC_IE_SSID:
				// Find the exact length of SSID without null terminator
				pTmp = pData;
				for (Idx = 0; Idx < WscLen; Idx++)
				{
					if (*pTmp++ == 0x0)
						break;
				}
				pProfile->Profile[CurrentIdx].SSID.SsidLength = Idx;
				RTMPMoveMemory(pProfile->Profile[CurrentIdx].SSID.Ssid, pData, pProfile->Profile[CurrentIdx].SSID.SsidLength);
				break;
								
			case WSC_IE_AUTH_TYPE:
#ifdef SKY_LITTLE_ENDIAN
				pProfile->Profile[CurrentIdx].AuthType = cpu2be16((unsigned char *)pData);
#else
				pProfile->Profile[CurrentIdx].AuthType = cpu2be16(*((PUSHORT) pData));
#endif
				break;
								
			case WSC_IE_ENCR_TYPE:
#ifdef SKY_LITTLE_ENDIAN
				pProfile->Profile[CurrentIdx].EncrType = cpu2be16((unsigned char *)pData);
#else
				pProfile->Profile[CurrentIdx].EncrType = cpu2be16(*((PUSHORT) pData));
#endif
				break;

			case WSC_IE_NETWORK_KEY_IDX:
				// Our key index start from 0 internally
//				pProfile->Profile[CurrentIdx].KeyIndex = (*pData) - 1;
				pProfile->Profile[CurrentIdx].KeyIndex = (*pData);  // edit by johnli, internal key index will subtract 1
				//printk("CurrentIdx=%d,Index=%d\n",CurrentIdx,pProfile->Profile[CurrentIdx].KeyIndex);
				break;
				
			case WSC_IE_NETWORK_KEY:
				pProfile->Profile[CurrentIdx].KeyLength = WscLen;
				RTMPMoveMemory(pProfile->Profile[CurrentIdx].Key, pData, pProfile->Profile[CurrentIdx].KeyLength);				
				break;
				
			case WSC_IE_MAC_ADDR:
				RTMPMoveMemory(pProfile->Profile[CurrentIdx].MacAddr, pData, 6);				
				break;
				
			case WSC_IE_KWRAP_AUTH:
				// Not used here, since we already verify it at decryption
				break;

			case WSC_IE_CREDENTIAL:
				// Credential IE, The WscLen include all length within profile, we need to modify it
				// to be able to parse all profile fields
				WscLen = 0;
				break;

			default:
				DBGPRINT(RT_DEBUG_WPS, "WscProcessCredential --> Unknown IE 0x%04x\n", WscType);	   
				break;
		}

		// Offset to net WSC Ie
		pData  += WscLen;
		Length -= WscLen;
	}

	// Svae the total number
	pProfile->ProfileCnt = Cnt;
	DBGPRINT(RT_DEBUG_WPS, "WscProcessCredential --> %d profile retrieved from credential\n", Cnt);	   
}

/*
	========================================================================
	
	Routine Description:
		Make WSC IE for the ProbeReq frame

	Arguments:
		pAdapter    - NIC Adapter pointer
		pOutBuf		- all of WSC IE field 
		pIeLen		- length
		
	Return Value:
		None

	IRQL = DISPATCH_LEVEL
	
	Note:
		None
		
	========================================================================
*/
VOID WscMakeProbeReqIE(
	IN	PRTMP_ADAPTER	pAd,
	OUT	PUCHAR			pOutBuf,
	OUT	PUCHAR			pIeLen)
{
	UCHAR			WscIEFixed[] = {0xdd, 0x0e, 0x00, 0x50, 0xf2, 0x04};	// length will modify later
	UCHAR			Len;
	PUCHAR			pData;
	PWSC_REG_DATA	pReg;

	DBGPRINT(RT_DEBUG_INFO, "-----> WscMakeProbeReqIE\n");

	pReg = (PWSC_REG_DATA) &pAd->PortCfg.WscControl.RegData;
	pData = (PUCHAR) &OutMsgBuf[0];
	Len = 0;
	*pIeLen = 0;
	
	// 0. WSC fixed IE
	RTMPMoveMemory(pData, &WscIEFixed[0], 6);
	pData += 6;
	Len += 6;
	// 1. Version
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_VERSION;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_VERSION);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = pReg->EnrolleeInfo.Version;
	pData += 5;
	Len   += 5;

	// 2. Request Type, Enrollee, 802.1x
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_REQ_TYPE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_REQ_TYPE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = 0x01;
	pData += 5;
	Len   += 5;

	// 3. Config method
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_CONF_MTHD;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0002;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_CONF_MTHD);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0002);
#endif
#ifdef SKY_LITTLE_ENDIAN
	memcpy((pData + 4), (void *)&pReg->EnrolleeInfo.ConfigMethods,2);
#else
	*((PUSHORT) (pData + 4)) = pReg->EnrolleeInfo.ConfigMethods;		// Label, Display, PBC
#endif
	pData += 6;
	Len   += 6;

	// 4. UUID
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_UUID_E;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0010;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_UUID_E);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0010);
#endif
	NdisMoveMemory((pData + 4), pReg->EnrolleeInfo.Uuid, 16);
	pData += 20;
	Len   += 20;

	// 5. Primary device type
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_PRI_DEV_TYPE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0008;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_PRI_DEV_TYPE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0008);
#endif
	NdisMoveMemory((pData + 4), pReg->EnrolleeInfo.PriDeviceType, 8);
	pData += 12;
	Len   += 12;

	// 6. RF band, shall change based on current channel
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_RF_BAND;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0001;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_RF_BAND);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0001);
#endif
	*(pData + 4) = pReg->EnrolleeInfo.RfBand;
	pData += 5;
	Len   += 5;

	// 7. Associate state
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_ASSOC_STATE;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0002;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_ASSOC_STATE);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0002);
#endif
#ifdef SKY_LITTLE_ENDIAN
	memcpy((pData + 4), (void *)&pReg->EnrolleeInfo.AssocState,2);
#else
	*((PUSHORT) (pData + 4)) = pReg->EnrolleeInfo.AssocState;	// Not associated	
#endif
	pData += 6;
	Len   += 6;
				
	// 8. Config error
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_CONF_ERROR;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0002;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_CONF_ERROR);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0002);
#endif
#ifdef SKY_LITTLE_ENDIAN
	memcpy((pData + 4), (void *)&pReg->EnrolleeInfo.ConfigError,2);
#else
	*((PUSHORT) (pData + 4)) = pReg->EnrolleeInfo.ConfigError;	// No error
#endif
	pData += 6;
	Len   += 6;

	// 9. Device password ID
#ifdef SKY_LITTLE_ENDIAN
	{
		unsigned short tmp;
		tmp = WSC_IE_DEV_PASS_ID;
		PUTU16(pData, (unsigned char *)&tmp);
		tmp = 0x0002;
		PUTU16(pData+2, (unsigned char *)&tmp);
	}
#else
	*((PUSHORT) pData) = cpu2be16(WSC_IE_DEV_PASS_ID);
	*((PUSHORT) (pData + 2)) = cpu2be16(0x0002);
#endif
#ifdef SKY_LITTLE_ENDIAN
	memcpy((pData + 4), (void *)&pReg->EnrolleeInfo.DevPwdId,2);
#else
	*((PUSHORT) (pData + 4)) = pReg->EnrolleeInfo.DevPwdId;
#endif
	pData += 6;
	Len   += 6;

	// update the total length in vendor specific IE
	OutMsgBuf[1] = Len - 2;

	// fill in output buffer
	*pIeLen = Len;
	NdisMoveMemory(pOutBuf, &OutMsgBuf[0], *pIeLen);
	
	DBGPRINT(RT_DEBUG_INFO, "<----- WscMakeProbeReqIE\n");

}

VOID	WscStop(
	IN	PRTMP_ADAPTER	pAdapter)
{
	//
	// Stop all of WPS actions including timer, EAP exchange and scanning
	//
	BOOLEAN Cancelled;
	// Clear all of timer
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscConnectTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscScanTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscPBCTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscPINTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapRxTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscM8WaitEapFailTimer,&Cancelled);
	RTMPCancelTimer(&pAdapter->PortCfg.WscControl.WscEapM2Timer,&Cancelled);


	// Reset state and status
	pAdapter->PortCfg.WscControl.WscState = WSC_STATE_OFF;
	pAdapter->PortCfg.WscControl.WscStatus = STATUS_WSC_NOTUSED;
	pAdapter->PortCfg.WscControl.WscTriggered = FALSE;	// stop work

//	printk("WscStop !!! \n");

}

INT ComputeChecksum(
	IN UINT PIN)
{
	INT digit_s;
    UINT accum = 0;

	PIN *= 10;
	accum += 3 * ((PIN / 10000000) % 10); 
	accum += 1 * ((PIN / 1000000) % 10); 
	accum += 3 * ((PIN / 100000) % 10); 
	accum += 1 * ((PIN / 10000) % 10); 
	accum += 3 * ((PIN / 1000) % 10); 
	accum += 1 * ((PIN / 100) % 10); 
	accum += 3 * ((PIN / 10) % 10); 

	digit_s = (accum % 10);
	return ((10 - digit_s) % 10);
} // ComputeChecksum

BOOLEAN ValidateChecksum(
	IN UINT PIN)
{
	UINT accum = 0;

	accum += 3 * ((PIN / 10000000) % 10); 
	accum += 1 * ((PIN / 1000000) % 10); 
	accum += 3 * ((PIN / 100000) % 10); 
	accum += 1 * ((PIN / 10000) % 10); 
	accum += 3 * ((PIN / 1000) % 10); 
	accum += 1 * ((PIN / 100) % 10); 
	accum += 3 * ((PIN / 10) % 10); 
	accum += 1 * ((PIN / 1) % 10); 
	
    return (0 == (accum % 10));
} // ValidateChecksum

UINT WscGeneratePinCode(
	IN	PRTMP_ADAPTER	pAd)
{
	UCHAR	macAddr[MAC_ADDR_LEN];
	UINT 	iPin;
	UINT	checksum;

	NdisMoveMemory(&macAddr[0], pAd->CurrentAddress, MAC_ADDR_LEN);

	iPin = macAddr[3] * 256 * 256 + macAddr[4] * 256 + macAddr[5];
	iPin = iPin % 10000000;
	checksum = ComputeChecksum( iPin );
	iPin = iPin*10 + checksum;

	return iPin;
}

VOID    WscGetRegDataPIN(
    IN  PRTMP_ADAPTER   pAdapter,
    IN  UINT            PinCode)
{
	UCHAR	tempPIN[9] = {0};
	
	pAdapter->PortCfg.WscControl.WscPinCode = PinCode;

	memset(pAdapter->PortCfg.WscControl.RegData.PIN, 0, 8);
	if (pAdapter->PortCfg.WscControl.WscMode == WSC_PBC_MODE)
		sprintf(tempPIN, "00000000");
	else
		sprintf(tempPIN, "%8u", pAdapter->PortCfg.WscControl.WscPinCode);
	memcpy(pAdapter->PortCfg.WscControl.RegData.PIN, tempPIN, 8);
}

USHORT WscGetAuthType(
    IN NDIS_802_11_AUTHENTICATION_MODE authType)
{
	switch(authType)
	{
		case Ndis802_11AuthModeOpen:
			return WSC_AUTHTYPE_OPEN;
		case Ndis802_11AuthModeWPAPSK:
			return WSC_AUTHTYPE_WPAPSK;
		case Ndis802_11AuthModeShared:
			return WSC_AUTHTYPE_SHARED;
		case Ndis802_11AuthModeWPA:
			return WSC_AUTHTYPE_WPA;
		case Ndis802_11AuthModeWPA2:
			return WSC_AUTHTYPE_WPA2;
		default:
		case Ndis802_11AuthModeWPA2PSK:
			return WSC_AUTHTYPE_WPA2PSK;
	}
}

USHORT WscGetEncryType(
    IN NDIS_802_11_WEP_STATUS encryType)
{
	switch(encryType)
	{
		case Ndis802_11WEPDisabled:
			return WSC_ENCRTYPE_NONE;
		case Ndis802_11WEPEnabled:
			return WSC_ENCRTYPE_WEP;
		default:
		case Ndis802_11Encryption2Enabled:
			return WSC_ENCRTYPE_TKIP;
		case Ndis802_11Encryption3Enabled:
			return WSC_ENCRTYPE_AES;
	}
}

PCHAR   WscGetAuthTypeStr(
    IN  USHORT authFlag)
{
	switch(authFlag)
	{
		case WSC_AUTHTYPE_OPEN:
			return "OPEN";
		default:
		case WSC_AUTHTYPE_WPAPSK:
			return "WPAPSK";
		case WSC_AUTHTYPE_SHARED:
			return "SHARED";
		case WSC_AUTHTYPE_WPA:
			return "WPA";
		case WSC_AUTHTYPE_WPA2:
			return "WPA2";
		case WSC_AUTHTYPE_WPA2PSK:
			return "WPA2PSK";
	}
}

PCHAR   WscGetEncryTypeStr(
    IN  USHORT encryFlag)
{
	switch(encryFlag)
	{
		case WSC_ENCRTYPE_NONE:
			return "NONE";
		case WSC_ENCRTYPE_WEP:
			return "WEP";
		default:
		case WSC_ENCRTYPE_TKIP:
			return "TKIP";
		case WSC_ENCRTYPE_AES:
			return "AES";
	}
}

INT	WscSetAuthMode(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{

	if ((strcmp(arg, "WEPAUTO") == 0) || (strcmp(arg, "wepauto") == 0))
		pAd->PortCfg.AuthMode = Ndis802_11AuthModeAutoSwitch;
	else if ((strcmp(arg, "OPEN") == 0) || (strcmp(arg, "open") == 0))
		pAd->PortCfg.AuthMode = Ndis802_11AuthModeOpen;
	else if ((strcmp(arg, "SHARED") == 0) || (strcmp(arg, "shared") == 0))
		pAd->PortCfg.AuthMode = Ndis802_11AuthModeShared;
	else if ((strcmp(arg, "WPAPSK") == 0) || (strcmp(arg, "wpapsk") == 0))
		pAd->PortCfg.AuthMode = Ndis802_11AuthModeWPAPSK;
	else if ((strcmp(arg, "WPA2PSK") == 0) || (strcmp(arg, "wpa2psk") == 0))
		pAd->PortCfg.AuthMode = Ndis802_11AuthModeWPA2PSK;

	pAd->PortCfg.PortSecured = WPA_802_1X_PORT_NOT_SECURED;

	DBGPRINT(RT_DEBUG_WPS, "Set_AuthMode_Proc::(AuthMode=%d)\n", pAd->PortCfg.AuthMode);

	return TRUE;
}

INT	WscSetEncrypType(
	IN	PRTMP_ADAPTER	pAd, 
	IN	PUCHAR			arg)
{
    
	if ((strcmp(arg, "NONE") == 0) || (strcmp(arg, "none") == 0))
		pAd->PortCfg.WepStatus = Ndis802_11WEPDisabled;
	else if ((strcmp(arg, "WEP") == 0) || (strcmp(arg, "wep") == 0))
		pAd->PortCfg.WepStatus = Ndis802_11WEPEnabled;
	else if ((strcmp(arg, "TKIP") == 0) || (strcmp(arg, "tkip") == 0))
		pAd->PortCfg.WepStatus = Ndis802_11Encryption2Enabled;
	else if ((strcmp(arg, "AES") == 0) || (strcmp(arg, "aes") == 0))
		pAd->PortCfg.WepStatus = Ndis802_11Encryption3Enabled;
		
	pAd->PortCfg.PairCipher    = pAd->PortCfg.WepStatus;
	pAd->PortCfg.GroupCipher   = pAd->PortCfg.WepStatus;

	RTMPMakeRSNIE(pAd, pAd->PortCfg.GroupCipher);

	return TRUE;
}

void    WscWriteConfToPortCfg(
    IN  PRTMP_ADAPTER   pAd)
{
	PWSC_PROFILE	    pProfile;
	PWSC_CTRL           pWscControl;
	INT					idx;
//	NDIS_802_11_SSID	Ssid;  remove by johnli

	DBGPRINT(RT_DEBUG_WPS, "-----> WscWriteConfToPortCfg\n");
    
	pWscControl = &pAd->PortCfg.WscControl;
    pWscControl->WscKeyType = 0;	//default
    
	//TODO: always save the first profile.
	{
		pProfile = (PWSC_PROFILE) &pAd->PortCfg.WscControl.WscProfile;
		/* set SSID */
		NdisZeroMemory(pAd->MlmeAux.Ssid, MAX_LEN_OF_SSID);
		NdisMoveMemory(pAd->MlmeAux.Ssid, pProfile->Profile[pProfile->ApplyProfileIdx].SSID.Ssid, pProfile->Profile[pProfile->ApplyProfileIdx].SSID.SsidLength);
		pAd->MlmeAux.SsidLen = pProfile->Profile[pProfile->ApplyProfileIdx].SSID.SsidLength;

		// add by johnli, fix WPS test plan 5.1.1, update Reconnect Ssid, that user desired to connect.
		NdisZeroMemory(pAd->MlmeAux.AutoReconnectSsid, MAX_LEN_OF_SSID);
		NdisMoveMemory(pAd->MlmeAux.AutoReconnectSsid, pAd->MlmeAux.Ssid, pAd->MlmeAux.SsidLen);
		pAd->MlmeAux.AutoReconnectSsidLen = pAd->MlmeAux.SsidLen;
		// end johnli

		// auth+enc sanity check
		DBGPRINT(RT_DEBUG_WPS, "AuthType: %d, EncrType: %d\n", pProfile->Profile[pProfile->ApplyProfileIdx].AuthType, pProfile->Profile[pProfile->ApplyProfileIdx].EncrType);
		if ((pProfile->Profile[pProfile->ApplyProfileIdx].AuthType == WSC_AUTHTYPE_WPAPSK) || 
			(pProfile->Profile[pProfile->ApplyProfileIdx].AuthType == WSC_AUTHTYPE_WPA2PSK))
		{
			if ((pProfile->Profile[pProfile->ApplyProfileIdx].EncrType != WSC_ENCRTYPE_TKIP) && (pProfile->Profile[pProfile->ApplyProfileIdx].EncrType != WSC_ENCRTYPE_AES))
			{
				DBGPRINT(RT_DEBUG_ERROR, "AuthType is WPAPSK or WPA2PAK.\n"
											"Get illegal EncrType(%d) from the Registrar, set EncrType to TKIP\n", 
											pProfile->Profile[pProfile->ApplyProfileIdx].EncrType);
				pProfile->Profile[pProfile->ApplyProfileIdx].EncrType = WSC_ENCRTYPE_TKIP;
			}
		}
		else if ((pProfile->Profile[pProfile->ApplyProfileIdx].AuthType == WSC_AUTHTYPE_OPEN) || 
				(pProfile->Profile[pProfile->ApplyProfileIdx].AuthType == WSC_AUTHTYPE_SHARED))
		{
			if ((pProfile->Profile[pProfile->ApplyProfileIdx].EncrType == WSC_ENCRTYPE_TKIP) || (pProfile->Profile[pProfile->ApplyProfileIdx].EncrType == WSC_ENCRTYPE_AES))
			{
				DBGPRINT(RT_DEBUG_ERROR, "AuthType is OPEN or SHARED.\n"
											"Get illegal EncrType(%d) from the Registrar, set EncrType to None\n", 
											pProfile->Profile[pProfile->ApplyProfileIdx].EncrType);
				pProfile->Profile[pProfile->ApplyProfileIdx].EncrType = WSC_ENCRTYPE_NONE;
			}
		}

		
		/* set Auth Mode */
		WscSetAuthMode(pAd, WscGetAuthTypeStr(pProfile->Profile[pProfile->ApplyProfileIdx].AuthType));
		/* set Enc Type*/
		WscSetEncrypType(pAd, WscGetEncryTypeStr(pProfile->Profile[pProfile->ApplyProfileIdx].EncrType));

		/* set Key */
		// init key buffer
		memset(pWscControl->WscKey, 0, 64);
		pWscControl->WscKeyLen = 0;
		for(idx=0; idx < 3; idx++)
		{
			pAd->SharedKey[idx].CipherAlg = CIPHER_NONE;
			pAd->SharedKey[idx].KeyLen = 0;
		}

		if (pProfile->Profile[pProfile->ApplyProfileIdx].EncrType != WSC_ENCRTYPE_NONE)
		{
			if ((pProfile->Profile[pProfile->ApplyProfileIdx].EncrType == WSC_ENCRTYPE_TKIP) ||
				(pProfile->Profile[pProfile->ApplyProfileIdx].EncrType == WSC_ENCRTYPE_AES))
			{
				pAd->PortCfg.DefaultKeyId = 1;
                
				if (pProfile->Profile[pProfile->ApplyProfileIdx].KeyLength >= 8 && pProfile->Profile[pProfile->ApplyProfileIdx].KeyLength <= 64)
				{
					pWscControl->WscKeyLen = pProfile->Profile[pProfile->ApplyProfileIdx].KeyLength;
					memcpy(pWscControl->WscKey, pProfile->Profile[pProfile->ApplyProfileIdx].Key, pWscControl->WscKeyLen);

					if (pWscControl->WscKeyLen == 64)
					{
						AtoH(pWscControl->WscKey, pAd->PortCfg.PskKey.Key, 32);
					}
					else
					{
						UCHAR       keyMaterial[40] = {0};
						PasswordHash((CHAR *)pWscControl->WscKey, pAd->MlmeAux.Ssid, pAd->MlmeAux.SsidLen, keyMaterial);
						NdisMoveMemory(pAd->PortCfg.PskKey.Key, keyMaterial, 32);
					}
					DBGPRINT(RT_DEBUG_WPS, "WpaPsk len = %d, key=%s\n", pWscControl->WscKeyLen, pWscControl->WscKey);
				}
				else
				{
					DBGPRINT(RT_DEBUG_ERROR, "WPAPSK: Invalid Key Length (%d)\n", pProfile->Profile[pProfile->ApplyProfileIdx].KeyLength);
				}
                
				RTMPMakeRSNIE(pAd, pAd->PortCfg.GroupCipher);
				// Start STA supplicant state machine
				pAd->PortCfg.WpaState = SS_START;
			}
			else if (pProfile->Profile[pProfile->ApplyProfileIdx].EncrType == WSC_ENCRTYPE_WEP)
			{
				UCHAR   WepKeyId = 0;
				USHORT  WepKeyLen = pProfile->Profile[pProfile->ApplyProfileIdx].KeyLength;

				// For datFile used. 
				pWscControl->WscKeyLen = WepKeyLen;
				memcpy(pWscControl->WscKey, pProfile->Profile[pProfile->ApplyProfileIdx].Key, WepKeyLen);

				if (pProfile->Profile[pProfile->ApplyProfileIdx].KeyIndex == 0)
					WepKeyId = 0;
				else
					WepKeyId = pProfile->Profile[pProfile->ApplyProfileIdx].KeyIndex - 1;
					
				if (WepKeyId <= 3)
				{
					pAd->PortCfg.DefaultKeyId = WepKeyId;
					// 5 or 13 ASCII characters
					// 10 or 26 Hex characters
					if (WepKeyLen == 5 || WepKeyLen == 13 || WepKeyLen == 10 || WepKeyLen == 26)
					{
						if (WepKeyLen == 5 || WepKeyLen == 13)
						{
							pAd->SharedKey[WepKeyId].KeyLen = WepKeyLen;
							memcpy(pAd->SharedKey[WepKeyId].Key, pProfile->Profile[pProfile->ApplyProfileIdx].Key, WepKeyLen);
							pWscControl->WscKeyType = 1;	// ASSIC

							if (WepKeyLen == 5)
							{
								pAd->SharedKey[WepKeyId].CipherAlg = CIPHER_WEP64;
							}
							else
								pAd->SharedKey[WepKeyId].CipherAlg = CIPHER_WEP128;
						}
						else
						{
							pAd->SharedKey[WepKeyId].KeyLen = WepKeyLen/2;
							AtoH(pProfile->Profile[pProfile->ApplyProfileIdx].Key, pAd->SharedKey[WepKeyId].Key, WepKeyLen/2);
							pWscControl->WscKeyType = 0;	// Hex
							
							if (WepKeyLen == 10)
								pAd->SharedKey[WepKeyId].CipherAlg = CIPHER_WEP64;
							else
								pAd->SharedKey[WepKeyId].CipherAlg = CIPHER_WEP128;
						}
						//printk("wep idx=%d, len = %d, key=%s\n", WepKeyId+1, pWscControl->WscKeyLen, pWscControl->WscKey);
					}
					else
					{
						DBGPRINT(RT_DEBUG_ERROR, "WEP: Invalid Key Length (%d)\n", pProfile->Profile[pProfile->ApplyProfileIdx].KeyLength);
						pWscControl->WscKeyLen = 0;
						pAd->SharedKey[WepKeyId].KeyLen = 0;
					}    
				}
				else
				{
					DBGPRINT(RT_DEBUG_ERROR, "Unsupport default key index (%d), use key Index 1.\n", WepKeyId);
					pAd->PortCfg.DefaultKeyId = 0;
					WepKeyId = 0;
					pWscControl->WscKeyLen = 0;
					pAd->SharedKey[WepKeyId].KeyLen = 0;
				}
			}
		}
	}


	// Set key into Asic
    if ((pAd->PortCfg.WepStatus == Ndis802_11WEPEnabled) && 
		(pAd->SharedKey[pAd->PortCfg.DefaultKeyId].KeyLen != 0))
	{
		AsicAddSharedKeyEntry(pAd, 
								0, 
//								0, 
								pAd->PortCfg.DefaultKeyId,   // edit by johnli, bug fix: set WEP key failed
								pAd->SharedKey[pAd->PortCfg.DefaultKeyId].CipherAlg, 
								pAd->SharedKey[pAd->PortCfg.DefaultKeyId].Key, 
								NULL,
								NULL);
	}

	// add by johnli, fix WPS test plan 5.1.1
	if (pProfile->ProfileCnt > 1)
		RTMPSetTimer(&pAd->PortCfg.WscControl.WscProfileRetryTimer, WSC_PROFILE_RETRY_TIMEOUT);
	// end johnli

    DBGPRINT(RT_DEBUG_WPS, "<----- WscWriteConfToPortCfg\n");
}

void    WscWriteConfToDatFile(
    IN  PRTMP_ADAPTER pAd)
{
	PCHAR				fileName, cfgData = 0;
	struct file			*file_r, *file_w;
	INT 				orgfsuid, orgfsgid;
	mm_segment_t		orgfs;
	PWSC_PROFILE	    pProfile;
	ssize_t             rv, fileLen = 0;
	//CHAR                *offset = 0;
	int offset = 0;
	int len = 0;
	CHAR                *pTempStr = 0;
	char				key_str[16]={0}, key_type[16]={0};
	int					idx;

	
	DBGPRINT(RT_DEBUG_WPS, "-----> WscWriteConfToDatFile\n");
	pProfile = (PWSC_PROFILE) &pAd->PortCfg.WscControl.WscProfile;
    
	fileName = PROFILE_PATH;

	// Save uid and gid used for filesystem access.
	// Set user and group to 0 (root)	
	orgfsuid = current->fsuid;
	orgfsgid = current->fsgid;
	current->fsuid = current->fsgid = 0;    
	orgfs = get_fs();
	set_fs(KERNEL_DS);

	file_w = filp_open(fileName, O_CREAT|O_RDWR, 0777);
	if (IS_ERR(file_w)) 
	{
		printk("-->2) %s: Error %ld opening %s\n", __FUNCTION__, -PTR_ERR(file_w), fileName);		  
		goto WriteFileOpenErr;
	}
	else 
	{
		if (!file_w->f_op)
		{
			printk("File Operation Method Error\n");
			filp_close(file_w, NULL);
			goto WriteFileOpenErr;
		}
		else
		{
			cfgData = kmalloc(2048, MEM_ALLOC_FLAG);
			pTempStr = kmalloc(128, MEM_ALLOC_FLAG);
			if (!pTempStr)
			{
				printk("WscWrite kmalloc fail. (512)\n");
				filp_close(file_w, NULL);
				return;
			}
			NdisZeroMemory(cfgData, 2048);
			len = strlen("[Default]\n");
			memcpy(cfgData,"[Default]\n",len);
			offset += len;

			NdisZeroMemory(pTempStr, 128);
			sprintf(pTempStr, "SSID=");
			if (pAd->MlmeAux.SsidLen)
			{
				sprintf(pTempStr, "%s%s\n", pTempStr, pAd->MlmeAux.Ssid);
			}
			len = strlen(pTempStr);
			memcpy(cfgData+offset,pTempStr,len);
			offset += len;
			
			NdisZeroMemory(pTempStr, 128);
			sprintf(pTempStr, "EncrypType=");
			if (pAd->MlmeAux.SsidLen)
			{
				USHORT encrypt_flag = WscGetEncryType(pAd->PortCfg.WepStatus);
				sprintf(pTempStr, "%s%s\n", pTempStr, WscGetEncryTypeStr(encrypt_flag));
			}
			len = strlen(pTempStr);
 			memcpy(cfgData+offset,pTempStr,len);
			offset += len;
			
			if(pAd->PortCfg.AuthMode >= Ndis802_11AuthModeWPA)
			{
				NdisZeroMemory(pTempStr, 128);
				sprintf(pTempStr, "AuthMode=");
				if (pAd->MlmeAux.SsidLen)
				{
					USHORT auth_flag = WscGetAuthType(pAd->PortCfg.AuthMode);
					sprintf(pTempStr, "%s%s\n", pTempStr, WscGetAuthTypeStr(auth_flag));
 				}
 				len = strlen(pTempStr);
 				memcpy(cfgData+offset,pTempStr,len);
				offset += len;
				
				NdisZeroMemory(pTempStr, 128);
				sprintf(pTempStr, "WPAPSK=");
				if (pAd->PortCfg.WscControl.WscKeyLen >=8 && pAd->PortCfg.WscControl.WscKeyLen <=64)
					NdisMoveMemory(pTempStr+7, pAd->PortCfg.WscControl.WscKey, pAd->PortCfg.WscControl.WscKeyLen);
				sprintf(pTempStr,"%s\n",pTempStr);
				len = strlen(pTempStr);
 				memcpy(cfgData+offset,pTempStr,len);
				offset += len;
			}			

			
			if(pAd->PortCfg.WepStatus == Ndis802_11WEPEnabled)
			{
				NdisZeroMemory(pTempStr, 128);
				sprintf(pTempStr, "DefaultKeyID=%d\n", (pAd->PortCfg.DefaultKeyId+1));
				len = strlen(pTempStr);
 				memcpy(cfgData+offset,pTempStr,len);
				offset += len;
			
				NdisZeroMemory(pTempStr, 128);
				sprintf(pTempStr, "AuthMode=WEPAUTO\n");	//tyhuang: we set wepauto always if wep is running
 				len = strlen(pTempStr);
 				memcpy(cfgData+offset,pTempStr,len);
				offset += len;
				
				//for(idx=0;idx<4;idx++)
				{
					idx = (pAd->PortCfg.DefaultKeyId+1);
					NdisZeroMemory(pTempStr, 128);					
					if (pAd->PortCfg.WscControl.WscKeyLen == 5 || pAd->PortCfg.WscControl.WscKeyLen == 13 ||
					pAd->PortCfg.WscControl.WscKeyLen == 10 || pAd->PortCfg.WscControl.WscKeyLen == 26)
					{
						// Not display non-printable key (Assic to Hex)
						if (pAd->PortCfg.WscControl.WscKeyLen == 5)
						{
							sprintf(pTempStr, "Key%dType=1\nKey%dStr=",idx,idx);
							len = strlen(pTempStr);							
							sprintf(pTempStr+len, "%x%x%x%x%x\n", 
							pAd->PortCfg.WscControl.WscKey[0], pAd->PortCfg.WscControl.WscKey[1], 
							pAd->PortCfg.WscControl.WscKey[2], pAd->PortCfg.WscControl.WscKey[3], 
							pAd->PortCfg.WscControl.WscKey[4]);
						}
						else if (pAd->PortCfg.WscControl.WscKeyLen == 13)
						{
							sprintf(pTempStr, "Key%dType=1\nKey%dStr=",idx,idx);
							len = strlen(pTempStr);							
							sprintf(pTempStr+len, "%x%x%x%x%x%x%x%x%x%x%x%x%x\n",
							pAd->PortCfg.WscControl.WscKey[0], pAd->PortCfg.WscControl.WscKey[1],
							pAd->PortCfg.WscControl.WscKey[2], pAd->PortCfg.WscControl.WscKey[3],
							pAd->PortCfg.WscControl.WscKey[4], pAd->PortCfg.WscControl.WscKey[5], 
							pAd->PortCfg.WscControl.WscKey[6], pAd->PortCfg.WscControl.WscKey[7],
							pAd->PortCfg.WscControl.WscKey[8], pAd->PortCfg.WscControl.WscKey[9],
							pAd->PortCfg.WscControl.WscKey[10], pAd->PortCfg.WscControl.WscKey[11],
							pAd->PortCfg.WscControl.WscKey[12]);
						}
						else
						{
							sprintf(pTempStr, "Key%dType=0\nKey%dStr=",idx,idx);
							len = strlen(pTempStr);
							NdisMoveMemory(pTempStr+len, pAd->PortCfg.WscControl.WscKey, pAd->PortCfg.WscControl.WscKeyLen);
							sprintf(pTempStr,"%s\n",pTempStr);
						}
					}
					len = strlen(pTempStr);
 					memcpy(cfgData+offset,pTempStr,len);
					offset += len;
				}
			}
 			
			file_w->f_op->write(file_w, cfgData, strlen(cfgData), &file_w->f_pos);
			if (pTempStr)
				kfree(pTempStr);
			if (cfgData)
				kfree(cfgData);
			filp_close(file_w, NULL);
			set_fs(orgfs);
		    current->fsuid = orgfsuid;
			current->fsgid = orgfsgid;
			return;
		}
	}
WriteFileOpenErr:
	set_fs(orgfs);
	current->fsuid = orgfsuid;
	current->fsgid = orgfsgid;	
    return;
}

void start_write_dat_file_thread(
    IN  PRTMP_ADAPTER pAd)
{
	DBGPRINT(RT_DEBUG_WPS, "-----> start_write_dat_file_thread\n");
	pAd->write_dat_file_pid = -1;
	pAd->time_to_die = 0;
	pAd->write_dat_file_pid = kernel_thread(write_dat_file_thread, pAd, CLONE_VM);
	if (pAd->write_dat_file_pid < 0) {
		printk (KERN_WARNING "unable to start kernel write dat file thread\n");
	}
	wait_for_completion (&pAd->write_dat_file_notify);
	DBGPRINT(RT_DEBUG_WPS, "<----- start_write_dat_file_thread\n");
}

INT write_dat_file_thread (
    IN void *Context)
{
	PRTMP_ADAPTER   pAd = (PRTMP_ADAPTER)Context;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
	daemonize("%s", pAd->net_dev->name);
#else
	daemonize();
#endif
	current->flags |= PF_NOFREEZE;
    /* signal that we've started the thread */
	complete(&(pAd->write_dat_file_notify));
#if 1    
	while (1) {
        /* lock the device pointers */
		down(&(pAd->write_dat_file_semaphore));
        
		if (pAd->time_to_die)
			break;

        /* device had been closed */
    	if (RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_REMOVE_IN_PROGRESS))
    		continue;

        if (pAd->WriteWscCfgToDatFile)
        {
            WscWriteConfToDatFile(pAd);
#if 1		//tyhuang: try to initialize all RF and driver parameters
			LinkDown(pAd,FALSE);
#endif
            pAd->WriteWscCfgToDatFile = FALSE;
        }
	}
#endif
    complete_and_exit (&pAd->write_dat_file_notify, 0);
    return 0;
}

